@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,12 +12,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpwidgets/gimpwidgets.h"
@@ -32,6 +32,7 @@
 #include "widgets/gimpactiongroup.h"
 #include "widgets/gimpgradienteditor.h"
 #include "widgets/gimphelp-ids.h"
+#include "widgets/gimpradioaction.h"
 
 #include "data-editor-commands.h"
 #include "gradient-editor-actions.h"
@@ -42,88 +43,87 @@
 
 static const GimpActionEntry gradient_editor_actions[] =
 {
-  { "gradient-editor-popup", GIMP_STOCK_GRADIENT,
-    N_("Gradient Editor Menu"), NULL, NULL, NULL,
-    GIMP_HELP_GRADIENT_EDITOR_DIALOG },
-
   { "gradient-editor-left-color-type", NULL,
-    N_("Left Color Type") },
-  { "gradient-editor-load-left-color", GTK_STOCK_REVERT_TO_SAVED,
-    N_("_Load Left Color From") },
-  { "gradient-editor-save-left-color", GTK_STOCK_SAVE,
-    N_("_Save Left Color To") },
+    NC_("gradient-editor-action", "Left Color Type") },
+  { "gradient-editor-load-left-color", GIMP_ICON_DOCUMENT_REVERT,
+    NC_("gradient-editor-action", "_Load Left Color From") },
+  { "gradient-editor-save-left-color", GIMP_ICON_DOCUMENT_SAVE,
+    NC_("gradient-editor-action", "_Save Left Color To") },
 
   { "gradient-editor-right-color-type", NULL,
-    N_("Right Color Type") },
-  { "gradient-editor-load-right-color", GTK_STOCK_REVERT_TO_SAVED,
-    N_("Load Right Color Fr_om") },
-  { "gradient-editor-save-right-color", GTK_STOCK_SAVE,
-    N_("Sa_ve Right Color To") },
+    NC_("gradient-editor-action", "Right Color Type") },
+  { "gradient-editor-load-right-color", GIMP_ICON_DOCUMENT_REVERT,
+    NC_("gradient-editor-action", "Load Right Color Fr_om") },
+  { "gradient-editor-save-right-color", GIMP_ICON_DOCUMENT_SAVE,
+    NC_("gradient-editor-action", "Sa_ve Right Color To") },
 
+  /* TODO GMenu: remove these 2 bogus actions once old style XML menu files are
+   * deleted.
+   */
   { "gradient-editor-blending-func", NULL, "blending-function" },
   { "gradient-editor-coloring-type", NULL, "coloring-type"     },
 
   { "gradient-editor-left-color", NULL,
-    N_("L_eft Endpoint's Color..."), NULL, NULL,
-    G_CALLBACK (gradient_editor_left_color_cmd_callback),
+    NC_("gradient-editor-action", "L_eft Endpoint's Color..."), NULL, { NULL }, NULL,
+    gradient_editor_left_color_cmd_callback,
     GIMP_HELP_GRADIENT_EDITOR_LEFT_COLOR },
 
   { "gradient-editor-right-color", NULL,
-    N_("R_ight Endpoint's Color..."), NULL, NULL,
-    G_CALLBACK (gradient_editor_right_color_cmd_callback),
+    NC_("gradient-editor-action", "R_ight Endpoint's Color..."), NULL, { NULL }, NULL,
+    gradient_editor_right_color_cmd_callback,
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_COLOR },
 
-  { "gradient-editor-flip", GIMP_STOCK_FLIP_HORIZONTAL,
-    "flip", NULL, NULL,
-    G_CALLBACK (gradient_editor_flip_cmd_callback),
+  { "gradient-editor-flip", GIMP_ICON_OBJECT_FLIP_HORIZONTAL,
+    NC_("gradient-editor-action", "Flip"), NULL, { NULL }, NULL,
+    gradient_editor_flip_cmd_callback,
     GIMP_HELP_GRADIENT_EDITOR_FLIP },
 
-  { "gradient-editor-replicate", GIMP_STOCK_DUPLICATE,
-    "replicate", NULL, NULL,
-    G_CALLBACK (gradient_editor_replicate_cmd_callback),
+  { "gradient-editor-replicate", GIMP_ICON_OBJECT_DUPLICATE,
+    NC_("gradient-editor-action", "Replicate"), NULL, { NULL }, NULL,
+    gradient_editor_replicate_cmd_callback,
     GIMP_HELP_GRADIENT_EDITOR_FLIP },
 
   { "gradient-editor-split-midpoint", NULL,
-    "splitmidpoint", NULL, NULL,
-    G_CALLBACK (gradient_editor_split_midpoint_cmd_callback),
+    NC_("gradient-editor-action", "Split Midpoint"), NULL, { NULL }, NULL,
+    gradient_editor_split_midpoint_cmd_callback,
     GIMP_HELP_GRADIENT_EDITOR_SPLIT_MIDPOINT },
 
   { "gradient-editor-split-uniform", NULL,
-    "splituniform", NULL, NULL,
-    G_CALLBACK (gradient_editor_split_uniformly_cmd_callback),
+    NC_("gradient-editor-action", "Split Uniformly"), NULL, { NULL }, NULL,
+    gradient_editor_split_uniformly_cmd_callback,
     GIMP_HELP_GRADIENT_EDITOR_SPLIT_UNIFORM },
 
-  { "gradient-editor-delete", GTK_STOCK_DELETE,
-    "delete", "", NULL,
-    G_CALLBACK (gradient_editor_delete_cmd_callback),
+  { "gradient-editor-delete", GIMP_ICON_EDIT_DELETE,
+    NC_("gradient-editor-action", "Delete"), NULL, { NULL }, NULL,
+    gradient_editor_delete_cmd_callback,
     GIMP_HELP_GRADIENT_EDITOR_DELETE },
 
   { "gradient-editor-recenter", NULL,
-    "recenter", NULL, NULL,
-    G_CALLBACK (gradient_editor_recenter_cmd_callback),
+    NC_("gradient-editor-action", "Recenter"), NULL, { NULL }, NULL,
+    gradient_editor_recenter_cmd_callback,
     GIMP_HELP_GRADIENT_EDITOR_RECENTER },
 
   { "gradient-editor-redistribute", NULL,
-    "redistribute", NULL, NULL,
-    G_CALLBACK (gradient_editor_redistribute_cmd_callback),
+    NC_("gradient-editor-action", "Redistribute"), NULL, { NULL }, NULL,
+    gradient_editor_redistribute_cmd_callback,
     GIMP_HELP_GRADIENT_EDITOR_REDISTRIBUTE },
 
   { "gradient-editor-blend-color", NULL,
-    N_("Ble_nd Endpoints' Colors"), NULL, NULL,
-    G_CALLBACK (gradient_editor_blend_color_cmd_callback),
+    NC_("gradient-editor-action", "Ble_nd Endpoints' Colors"), NULL, { NULL }, NULL,
+    gradient_editor_blend_color_cmd_callback,
     GIMP_HELP_GRADIENT_EDITOR_BLEND_COLOR },
 
   { "gradient-editor-blend-opacity", NULL,
-    N_("Blend Endpoints' Opacit_y"), NULL, NULL,
-    G_CALLBACK (gradient_editor_blend_opacity_cmd_callback),
+    NC_("gradient-editor-action", "Blend Endpoints' Opacit_y"), NULL, { NULL }, NULL,
+    gradient_editor_blend_opacity_cmd_callback,
     GIMP_HELP_GRADIENT_EDITOR_BLEND_OPACITY }
 };
 
 static const GimpToggleActionEntry gradient_editor_toggle_actions[] =
 {
-  { "gradient-editor-edit-active", GIMP_STOCK_LINKED,
-    N_("Edit Active Gradient"), NULL, NULL,
-    G_CALLBACK (data_editor_edit_active_cmd_callback),
+  { "gradient-editor-edit-active", GIMP_ICON_LINKED,
+    NC_("gradient-editor-action", "Edit Active Gradient"), NULL, { NULL }, NULL,
+    data_editor_edit_active_cmd_callback,
     FALSE,
     GIMP_HELP_GRADIENT_EDITOR_EDIT_ACTIVE }
 };
@@ -131,44 +131,44 @@ static const GimpToggleActionEntry gradient_editor_toggle_actions[] =
 
 #define LOAD_LEFT_FROM(num,magic) \
   { "gradient-editor-load-left-" num, NULL, \
-    num, NULL, NULL, \
+    num, NULL, { NULL }, NULL, \
     (magic), FALSE, \
     GIMP_HELP_GRADIENT_EDITOR_LEFT_LOAD }
 #define SAVE_LEFT_TO(num,magic) \
   { "gradient-editor-save-left-" num, NULL, \
-    num, NULL, NULL, \
+    num, NULL, { NULL }, NULL, \
     (magic), FALSE, \
     GIMP_HELP_GRADIENT_EDITOR_LEFT_SAVE }
 #define LOAD_RIGHT_FROM(num,magic) \
   { "gradient-editor-load-right-" num, NULL, \
-    num, NULL, NULL, \
+    num, NULL, { NULL }, NULL, \
     (magic), FALSE, \
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_LOAD }
 #define SAVE_RIGHT_TO(num,magic) \
   { "gradient-editor-save-right-" num, NULL, \
-    num, NULL, NULL, \
+    num, NULL, { NULL }, NULL, \
     (magic), FALSE, \
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_SAVE }
 
 static const GimpEnumActionEntry gradient_editor_load_left_actions[] =
 {
   { "gradient-editor-load-left-left-neighbor", NULL,
-    N_("_Left Neighbor's Right Endpoint"), NULL, NULL,
+    NC_("gradient-editor-action", "_Left Neighbor's Right Endpoint"), NULL, { NULL }, NULL,
     GRADIENT_EDITOR_COLOR_NEIGHBOR_ENDPOINT, FALSE,
     GIMP_HELP_GRADIENT_EDITOR_LEFT_LOAD },
 
   { "gradient-editor-load-left-right-endpoint", NULL,
-    N_("_Right Endpoint"), NULL, NULL,
+    NC_("gradient-editor-action", "_Right Endpoint"), NULL, { NULL }, NULL,
     GRADIENT_EDITOR_COLOR_OTHER_ENDPOINT, FALSE,
     GIMP_HELP_GRADIENT_EDITOR_LEFT_LOAD },
 
   { "gradient-editor-load-left-fg", NULL,
-    N_("_Foreground Color"), NULL, NULL,
+    NC_("gradient-editor-action", "_Foreground Color"), NULL, { NULL }, NULL,
     GRADIENT_EDITOR_COLOR_FOREGROUND, FALSE,
     GIMP_HELP_GRADIENT_EDITOR_LEFT_LOAD },
 
   { "gradient-editor-load-left-bg", NULL,
-    N_("_Background Color"), NULL, NULL,
+    NC_("gradient-editor-action", "_Background Color"), NULL, { NULL }, NULL,
     GRADIENT_EDITOR_COLOR_BACKGROUND, FALSE,
     GIMP_HELP_GRADIENT_EDITOR_LEFT_LOAD },
 
@@ -201,22 +201,22 @@ static const GimpEnumActionEntry gradient_editor_save_left_actions[] =
 static const GimpEnumActionEntry gradient_editor_load_right_actions[] =
 {
   { "gradient-editor-load-right-right-neighbor", NULL,
-    N_("_Right Neighbor's Left Endpoint"), NULL, NULL,
+    NC_("gradient-editor-action", "_Right Neighbor's Left Endpoint"), NULL, { NULL }, NULL,
     GRADIENT_EDITOR_COLOR_NEIGHBOR_ENDPOINT, FALSE,
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_LOAD },
 
   { "gradient-editor-load-right-left-endpoint", NULL,
-    N_("_Left Endpoint"), NULL, NULL,
+    NC_("gradient-editor-action", "_Left Endpoint"), NULL, { NULL }, NULL,
     GRADIENT_EDITOR_COLOR_OTHER_ENDPOINT, FALSE,
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_LOAD },
 
   { "gradient-editor-load-right-fg", NULL,
-    N_("_Foreground Color"), NULL, NULL,
+    NC_("gradient-editor-action", "_Foreground Color"), NULL, { NULL }, NULL,
     GRADIENT_EDITOR_COLOR_FOREGROUND, FALSE,
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_LOAD },
 
   { "gradient-editor-load-right-bg", NULL,
-    N_("_Background Color"), NULL, NULL,
+    NC_("gradient-editor-action", "_Background Color"), NULL, { NULL }, NULL,
     GRADIENT_EDITOR_COLOR_BACKGROUND, FALSE,
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_LOAD },
 
@@ -255,27 +255,27 @@ static const GimpEnumActionEntry gradient_editor_save_right_actions[] =
 static const GimpRadioActionEntry gradient_editor_left_color_type_actions[] =
 {
   { "gradient-editor-left-color-fixed", NULL,
-    N_("_Fixed"), NULL, NULL,
+    NC_("gradient-editor-color-type", "_Fixed"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_COLOR_FIXED,
     GIMP_HELP_GRADIENT_EDITOR_LEFT_COLOR },
 
   { "gradient-editor-left-color-foreground", NULL,
-    N_("F_oreground Color"), NULL, NULL,
+    NC_("gradient-editor-color-type", "F_oreground Color"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_COLOR_FOREGROUND,
     GIMP_HELP_GRADIENT_EDITOR_LEFT_COLOR },
 
   { "gradient-editor-left-color-foreground-transparent", NULL,
-    N_("Fo_reground Color (Transparent)"), NULL, NULL,
+    NC_("gradient-editor-color-type", "Fo_reground Color (Transparent)"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT,
     GIMP_HELP_GRADIENT_EDITOR_LEFT_COLOR },
 
   { "gradient-editor-left-color-background", NULL,
-    N_("_Background Color"), NULL, NULL,
+    NC_("gradient-editor-color-type", "_Background Color"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_COLOR_BACKGROUND,
     GIMP_HELP_GRADIENT_EDITOR_LEFT_COLOR },
 
   { "gradient-editor-left-color-background-transparent", NULL,
-    N_("B_ackground Color (Transparent)"), NULL, NULL,
+    NC_("gradient-editor-color-type", "B_ackground Color (Transparent)"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT,
     GIMP_HELP_GRADIENT_EDITOR_LEFT_COLOR }
 };
@@ -283,27 +283,27 @@ static const GimpRadioActionEntry gradient_editor_left_color_type_actions[] =
 static const GimpRadioActionEntry gradient_editor_right_color_type_actions[] =
 {
   { "gradient-editor-right-color-fixed", NULL,
-    N_("_Fixed"), NULL, NULL,
+    NC_("gradient-editor-color-type", "_Fixed"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_COLOR_FIXED,
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_COLOR },
 
   { "gradient-editor-right-color-foreground", NULL,
-    N_("F_oreground Color"), NULL, NULL,
+    NC_("gradient-editor-color-type", "F_oreground Color"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_COLOR_FOREGROUND,
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_COLOR },
 
   { "gradient-editor-right-color-foreground-transparent", NULL,
-    N_("Fo_reground Color (Transparent)"), NULL, NULL,
+    NC_("gradient-editor-color-type", "Fo_reground Color (Transparent)"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT,
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_COLOR },
 
   { "gradient-editor-right-color-background", NULL,
-    N_("_Background Color"), NULL, NULL,
+    NC_("gradient-editor-color-type", "_Background Color"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_COLOR_BACKGROUND,
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_COLOR },
 
   { "gradient-editor-right-color-background-transparent", NULL,
-    N_("B_ackground Color (Transparent)"), NULL, NULL,
+    NC_("gradient-editor-color-type", "B_ackground Color (Transparent)"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT,
     GIMP_HELP_GRADIENT_EDITOR_RIGHT_COLOR }
 };
@@ -311,32 +311,37 @@ static const GimpRadioActionEntry gradient_editor_right_color_type_actions[] =
 static const GimpRadioActionEntry gradient_editor_blending_actions[] =
 {
   { "gradient-editor-blending-linear", NULL,
-    N_("_Linear"), NULL, NULL,
+    NC_("gradient-editor-blending", "_Linear"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_SEGMENT_LINEAR,
     GIMP_HELP_GRADIENT_EDITOR_BLENDING },
 
   { "gradient-editor-blending-curved", NULL,
-    N_("_Curved"), NULL, NULL,
+    NC_("gradient-editor-blending", "_Curved"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_SEGMENT_CURVED,
     GIMP_HELP_GRADIENT_EDITOR_BLENDING },
 
   { "gradient-editor-blending-sine", NULL,
-    N_("_Sinusoidal"), NULL, NULL,
+    NC_("gradient-editor-blending", "_Sinusoidal"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_SEGMENT_SINE,
     GIMP_HELP_GRADIENT_EDITOR_BLENDING },
 
   { "gradient-editor-blending-sphere-increasing", NULL,
-    N_("Spherical (i_ncreasing)"), NULL, NULL,
+    NC_("gradient-editor-blending", "Spherical (i_ncreasing)"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_SEGMENT_SPHERE_INCREASING,
     GIMP_HELP_GRADIENT_EDITOR_BLENDING },
 
   { "gradient-editor-blending-sphere-decreasing", NULL,
-    N_("Spherical (_decreasing)"), NULL, NULL,
+    NC_("gradient-editor-blending", "Spherical (_decreasing)"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_SEGMENT_SPHERE_DECREASING,
     GIMP_HELP_GRADIENT_EDITOR_BLENDING },
 
+  { "gradient-editor-blending-step", NULL,
+    NC_("gradient-editor-blending", "S_tep"), NULL, { NULL }, NULL,
+    GIMP_GRADIENT_SEGMENT_STEP,
+    GIMP_HELP_GRADIENT_EDITOR_BLENDING },
+
   { "gradient-editor-blending-varies", NULL,
-    N_("(Varies)"), NULL, NULL,
+    NC_("gradient-editor-blending", "(Varies)"), NULL, { NULL }, NULL,
     -1,
     GIMP_HELP_GRADIENT_EDITOR_BLENDING }
 };
@@ -344,42 +349,42 @@ static const GimpRadioActionEntry gradient_editor_blending_actions[] =
 static const GimpRadioActionEntry gradient_editor_coloring_actions[] =
 {
   { "gradient-editor-coloring-rgb", NULL,
-    N_("_RGB"), NULL, NULL,
+    NC_("gradient-editor-coloring", "_RGB"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_SEGMENT_RGB,
     GIMP_HELP_GRADIENT_EDITOR_COLORING },
 
   { "gradient-editor-coloring-hsv-ccw", NULL,
-    N_("HSV (_counter-clockwise hue)"), NULL, NULL,
+    NC_("gradient-editor-coloring", "HSV (_counter-clockwise hue)"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_SEGMENT_HSV_CCW,
     GIMP_HELP_GRADIENT_EDITOR_COLORING },
 
   { "gradient-editor-coloring-hsv-cw", NULL,
-    N_("HSV (clockwise _hue)"), NULL, NULL,
+    NC_("gradient-editor-coloring", "HSV (clockwise _hue)"), NULL, { NULL }, NULL,
     GIMP_GRADIENT_SEGMENT_HSV_CW,
     GIMP_HELP_GRADIENT_EDITOR_COLORING },
 
   { "gradient-editor-coloring-varies", NULL,
-    N_("(Varies)"), NULL, NULL,
+    NC_("gradient-editor-coloring", "(Varies)"), NULL, { NULL }, NULL,
     -1,
     GIMP_HELP_GRADIENT_EDITOR_COLORING }
 };
 
 static const GimpEnumActionEntry gradient_editor_zoom_actions[] =
 {
-  { "gradient-editor-zoom-in", GTK_STOCK_ZOOM_IN,
-    N_("Zoom In"), NULL,
+  { "gradient-editor-zoom-in", GIMP_ICON_ZOOM_IN,
+    N_("Zoom In"), NULL, { NULL },
     N_("Zoom in"),
     GIMP_ZOOM_IN, FALSE,
     GIMP_HELP_GRADIENT_EDITOR_ZOOM_IN },
 
-  { "gradient-editor-zoom-out", GTK_STOCK_ZOOM_OUT,
-    N_("Zoom Out"), NULL,
+  { "gradient-editor-zoom-out", GIMP_ICON_ZOOM_OUT,
+    N_("Zoom Out"), NULL, { NULL },
     N_("Zoom out"),
     GIMP_ZOOM_OUT, FALSE,
     GIMP_HELP_GRADIENT_EDITOR_ZOOM_OUT },
 
-  { "gradient-editor-zoom-all", GTK_STOCK_ZOOM_FIT,
-    N_("Zoom All"), NULL,
+  { "gradient-editor-zoom-all", GIMP_ICON_ZOOM_FIT_BEST,
+    N_("Zoom All"), NULL, { NULL },
     N_("Zoom all"),
     GIMP_ZOOM_OUT_MAX, FALSE,
     GIMP_HELP_GRADIENT_EDITOR_ZOOM_ALL }
@@ -389,67 +394,67 @@ static const GimpEnumActionEntry gradient_editor_zoom_actions[] =
 void
 gradient_editor_actions_setup (GimpActionGroup *group)
 {
-  gimp_action_group_add_actions (group,
+  gimp_action_group_add_actions (group, "gradient-editor-action",
                                  gradient_editor_actions,
                                  G_N_ELEMENTS (gradient_editor_actions));
 
-  gimp_action_group_add_toggle_actions (group,
+  gimp_action_group_add_toggle_actions (group, "gradient-editor-action",
                                         gradient_editor_toggle_actions,
                                         G_N_ELEMENTS (gradient_editor_toggle_actions));
 
-  gimp_action_group_add_enum_actions (group,
+  gimp_action_group_add_enum_actions (group, "gradient-editor-action",
                                       gradient_editor_load_left_actions,
                                       G_N_ELEMENTS (gradient_editor_load_left_actions),
-                                      G_CALLBACK (gradient_editor_load_left_cmd_callback));
+                                      gradient_editor_load_left_cmd_callback);
 
-  gimp_action_group_add_enum_actions (group,
+  gimp_action_group_add_enum_actions (group, NULL,
                                       gradient_editor_save_left_actions,
                                       G_N_ELEMENTS (gradient_editor_save_left_actions),
-                                      G_CALLBACK (gradient_editor_save_left_cmd_callback));
+                                      gradient_editor_save_left_cmd_callback);
 
-  gimp_action_group_add_enum_actions (group,
+  gimp_action_group_add_enum_actions (group, "gradient-editor-action",
                                       gradient_editor_load_right_actions,
                                       G_N_ELEMENTS (gradient_editor_load_right_actions),
-                                      G_CALLBACK (gradient_editor_load_right_cmd_callback));
+                                      gradient_editor_load_right_cmd_callback);
 
 
-  gimp_action_group_add_enum_actions (group,
+  gimp_action_group_add_enum_actions (group, NULL,
                                       gradient_editor_save_right_actions,
                                       G_N_ELEMENTS (gradient_editor_save_right_actions),
-                                      G_CALLBACK (gradient_editor_save_right_cmd_callback));
+                                      gradient_editor_save_right_cmd_callback);
 
-  gimp_action_group_add_radio_actions (group,
+  gimp_action_group_add_radio_actions (group, "gradient-editor-color-type",
                                        gradient_editor_left_color_type_actions,
                                        G_N_ELEMENTS (gradient_editor_left_color_type_actions),
                                        NULL,
                                        0,
-                                       G_CALLBACK (gradient_editor_left_color_type_cmd_callback));
+                                       gradient_editor_left_color_type_cmd_callback);
 
-  gimp_action_group_add_radio_actions (group,
+  gimp_action_group_add_radio_actions (group, "gradient-editor-color-type",
                                        gradient_editor_right_color_type_actions,
                                        G_N_ELEMENTS (gradient_editor_right_color_type_actions),
                                        NULL,
                                        0,
-                                       G_CALLBACK (gradient_editor_right_color_type_cmd_callback));
+                                       gradient_editor_right_color_type_cmd_callback);
 
-  gimp_action_group_add_radio_actions (group,
+  gimp_action_group_add_radio_actions (group, "gradient-editor-blending",
                                        gradient_editor_blending_actions,
                                        G_N_ELEMENTS (gradient_editor_blending_actions),
                                        NULL,
                                        0,
-                                       G_CALLBACK (gradient_editor_blending_func_cmd_callback));
+                                       gradient_editor_blending_func_cmd_callback);
 
-  gimp_action_group_add_radio_actions (group,
+  gimp_action_group_add_radio_actions (group, "gradient-editor-coloring",
                                        gradient_editor_coloring_actions,
                                        G_N_ELEMENTS (gradient_editor_coloring_actions),
                                        NULL,
                                        0,
-                                       G_CALLBACK (gradient_editor_coloring_type_cmd_callback));
+                                       gradient_editor_coloring_type_cmd_callback);
 
-  gimp_action_group_add_enum_actions (group,
+  gimp_action_group_add_enum_actions (group, NULL,
                                       gradient_editor_zoom_actions,
                                       G_N_ELEMENTS (gradient_editor_zoom_actions),
-                                      G_CALLBACK (gradient_editor_zoom_cmd_callback));
+                                      gradient_editor_zoom_cmd_callback);
 }
 
 void
@@ -487,33 +492,35 @@ gradient_editor_actions_update (GimpActionGroup *group,
       if (data_editor->data_editable)
         editable = TRUE;
 
-      gimp_gradient_get_color_at (gradient, data_editor->context,
-                                  editor->control_sel_l,
-                                  editor->control_sel_l->left, FALSE,
-                                  &left_color);
+      gimp_gradient_segment_get_left_flat_color (gradient,
+                                                 data_editor->context,
+                                                 editor->control_sel_l,
+                                                 &left_color);
 
       if (editor->control_sel_l->prev)
         left_seg = editor->control_sel_l->prev;
       else
         left_seg = gimp_gradient_segment_get_last (editor->control_sel_l);
 
-      gimp_gradient_get_color_at (gradient, data_editor->context,
-                                  left_seg, left_seg->right, FALSE,
-                                  &left_seg_color);
+      gimp_gradient_segment_get_right_flat_color (gradient,
+                                                  data_editor->context,
+                                                  left_seg,
+                                                  &left_seg_color);
 
-      gimp_gradient_get_color_at (gradient, data_editor->context,
-                                  editor->control_sel_r,
-                                  editor->control_sel_r->right, FALSE,
-                                  &right_color);
+      gimp_gradient_segment_get_right_flat_color (gradient,
+                                                  data_editor->context,
+                                                  editor->control_sel_r,
+                                                  &right_color);
 
       if (editor->control_sel_r->next)
         right_seg = editor->control_sel_r->next;
       else
         right_seg = gimp_gradient_segment_get_first (editor->control_sel_r);
 
-      gimp_gradient_get_color_at (gradient, data_editor->context,
-                                  right_seg, right_seg->left, FALSE,
-                                  &right_seg_color);
+      gimp_gradient_segment_get_left_flat_color (gradient,
+                                                 data_editor->context,
+                                                 right_seg,
+                                                 &right_seg_color);
 
       left_editable  = (editor->control_sel_l->left_color_type ==
                         GIMP_GRADIENT_COLOR_FIXED);
@@ -549,7 +556,7 @@ gradient_editor_actions_update (GimpActionGroup *group,
    *  insensitive. prevents the gradient from being modified while a
    *  dialog is running. bug #161411 --mitch
    */
-  if (! GTK_WIDGET_SENSITIVE (editor))
+  if (! gtk_widget_is_sensitive (GTK_WIDGET (editor)))
     editable = FALSE;
 
   if (! editable)
@@ -566,8 +573,11 @@ gradient_editor_actions_update (GimpActionGroup *group,
         gimp_action_group_set_action_color (group, action, (color), (set_label))
 #define SET_LABEL(action,label) \
         gimp_action_group_set_action_label (group, action, (label))
+#define SET_GROUP_LABEL(action,label) \
+        gimp_radio_action_set_group_label (GIMP_RADIO_ACTION (gimp_action_group_get_action (group, action)), \
+                                           label)
 #define SET_SENSITIVE(action,condition) \
-        gimp_action_group_set_action_sensitive (group, action, (condition) != 0)
+        gimp_action_group_set_action_sensitive (group, action, (condition) != 0, NULL)
 #define SET_VISIBLE(action,condition) \
         gimp_action_group_set_action_visible (group, action, (condition) != 0)
 
@@ -769,10 +779,10 @@ gradient_editor_actions_update (GimpActionGroup *group,
 
   if (! selection)
     {
-      SET_LABEL ("gradient-editor-blending-func",
-                 _("_Blending Function for Segment"));
-      SET_LABEL ("gradient-editor-coloring-type",
-                 _("Coloring _Type for Segment"));
+      SET_GROUP_LABEL ("gradient-editor-blending-linear",
+                       _("_Blending Function for Segment"));
+      SET_GROUP_LABEL ("gradient-editor-coloring-rgb",
+                       _("Coloring _Type for Segment"));
 
       SET_LABEL ("gradient-editor-flip",
                  _("_Flip Segment"));
@@ -791,10 +801,10 @@ gradient_editor_actions_update (GimpActionGroup *group,
     }
   else
     {
-      SET_LABEL ("gradient-editor-blending-func",
-                 _("_Blending Function for Selection"));
-      SET_LABEL ("gradient-editor-coloring-type",
-                 _("Coloring _Type for Selection"));
+      SET_GROUP_LABEL ("gradient-editor-blending-linear",
+                       _("_Blending Function for Selection"));
+      SET_GROUP_LABEL ("gradient-editor-coloring-rgb",
+                       _("Coloring _Type for Selection"));
 
       SET_LABEL ("gradient-editor-flip",
                  _("_Flip Selection"));
@@ -820,6 +830,7 @@ gradient_editor_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("gradient-editor-blending-sine",              editable);
   SET_SENSITIVE ("gradient-editor-blending-sphere-increasing", editable);
   SET_SENSITIVE ("gradient-editor-blending-sphere-decreasing", editable);
+  SET_SENSITIVE ("gradient-editor-blending-step",              editable);
 
   if (blending_equal && gradient)
     {
@@ -840,6 +851,9 @@ gradient_editor_actions_update (GimpActionGroup *group,
         case GIMP_GRADIENT_SEGMENT_SPHERE_DECREASING:
           SET_ACTIVE ("gradient-editor-blending-sphere-decreasing", TRUE);
           break;
+        case GIMP_GRADIENT_SEGMENT_STEP:
+          SET_ACTIVE ("gradient-editor-blending-step", TRUE);
+          break;
         }
     }
   else
