@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -20,6 +20,7 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpconfig/gimpconfig.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "actions-types.h"
@@ -35,6 +36,9 @@
 #include "core/gimpimage-color-profile.h"
 #include "core/gimpitemstack.h"
 
+#include "display/gimpdisplay.h"
+#include "display/gimpdisplayshell.h"
+
 #include "widgets/gimpactiongroup.h"
 #include "widgets/gimphelp-ids.h"
 
@@ -47,159 +51,148 @@
 
 static const GimpActionEntry image_actions[] =
 {
-  { "image-menubar", NULL,
-    NC_("image-action", "Image Menu"), NULL, NULL, NULL,
-    GIMP_HELP_IMAGE_WINDOW },
-
-  { "image-popup", NULL,
-    NC_("image-action", "Image Menu"), NULL, NULL, NULL,
-    GIMP_HELP_IMAGE_WINDOW },
-
-  { "image-menu",                  NULL, NC_("image-action", "_Image")     },
-  { "image-mode-menu",             NULL, NC_("image-action", "_Mode")      },
-  { "image-precision-menu",        NULL, NC_("image-action", "Pr_ecision") },
-  { "image-color-management-menu", NULL, NC_("image-action",
-                                             "Color Ma_nagement")          },
-  { "image-transform-menu",        NULL, NC_("image-action", "_Transform") },
-  { "image-guides-menu",           NULL, NC_("image-action", "_Guides")    },
-
-  { "colors-menu",              NULL, NC_("image-action", "_Colors")       },
-  { "colors-info-menu",         NULL, NC_("image-action", "I_nfo")         },
-  { "colors-auto-menu",         NULL, NC_("image-action", "_Auto")         },
-  { "colors-map-menu",          NULL, NC_("image-action", "_Map")          },
-  { "colors-tone-mapping-menu", NULL, NC_("image-action", "_Tone Mapping") },
-  { "colors-components-menu",   NULL, NC_("image-action", "C_omponents")   },
-  { "colors-desaturate-menu",   NULL, NC_("image-action", "D_esaturate")   },
-
   { "image-new", GIMP_ICON_DOCUMENT_NEW,
-    NC_("image-action", "_New..."), "<primary>N",
+    NC_("image-action", "_New..."), NULL, { "<primary>N", NULL },
     NC_("image-action", "Create a new image"),
-    G_CALLBACK (image_new_cmd_callback),
+    image_new_cmd_callback,
     GIMP_HELP_FILE_NEW },
 
   { "image-duplicate", GIMP_ICON_OBJECT_DUPLICATE,
-    NC_("image-action", "_Duplicate"), "<primary>D",
+    NC_("image-action", "_Duplicate"), NULL, { "<primary>D", NULL },
     NC_("image-action", "Create a duplicate of this image"),
-    G_CALLBACK (image_duplicate_cmd_callback),
+    image_duplicate_cmd_callback,
     GIMP_HELP_IMAGE_DUPLICATE },
 
   { "image-color-profile-assign", NULL,
-    NC_("image-action", "_Assign Color Profile..."), NULL,
+    NC_("image-action", "_Assign Color Profile..."), NULL, { NULL },
     NC_("image-action", "Set a color profile on the image"),
-    G_CALLBACK (image_color_profile_assign_cmd_callback),
+    image_color_profile_assign_cmd_callback,
     GIMP_HELP_IMAGE_COLOR_PROFILE_ASSIGN },
 
   { "image-color-profile-convert", NULL,
-    NC_("image-action", "_Convert to Color Profile..."), NULL,
+    NC_("image-action", "_Convert to Color Profile..."), NULL, { NULL },
     NC_("image-action", "Apply a color profile to the image"),
-    G_CALLBACK (image_color_profile_convert_cmd_callback),
+    image_color_profile_convert_cmd_callback,
     GIMP_HELP_IMAGE_COLOR_PROFILE_CONVERT },
 
   { "image-color-profile-discard", NULL,
-    NC_("image-action", "_Discard Color Profile"), NULL,
+    NC_("image-action", "_Discard Color Profile"), NULL, { NULL },
     NC_("image-action", "Remove the image's color profile"),
-    G_CALLBACK (image_color_profile_discard_cmd_callback),
+    image_color_profile_discard_cmd_callback,
     GIMP_HELP_IMAGE_COLOR_PROFILE_DISCARD },
 
+  { "image-softproof-profile", NULL,
+    NC_("image-action", "_Soft-proof Profile..."), NULL, { NULL },
+    NC_("image-action", "Set the soft-proofing profile"),
+    image_softproof_profile_cmd_callback,
+    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+
   { "image-color-profile-save", NULL,
-    NC_("image-action", "_Save Color Profile to File..."), NULL,
+    NC_("image-action", "_Save Color Profile to File..."), NULL, { NULL },
     NC_("image-action", "Save the image's color profile to an ICC file"),
-    G_CALLBACK (image_color_profile_save_cmd_callback),
+    image_color_profile_save_cmd_callback,
     GIMP_HELP_IMAGE_COLOR_PROFILE_SAVE },
 
   { "image-resize", GIMP_ICON_OBJECT_RESIZE,
-    NC_("image-action", "Can_vas Size..."), NULL,
+    NC_("image-action", "Can_vas Size..."), NULL, { NULL },
     NC_("image-action", "Adjust the image dimensions"),
-    G_CALLBACK (image_resize_cmd_callback),
+    image_resize_cmd_callback,
     GIMP_HELP_IMAGE_RESIZE },
 
   { "image-resize-to-layers", NULL,
-    NC_("image-action", "Fit Canvas to L_ayers"), NULL,
+    NC_("image-action", "Fit Canvas to L_ayers"), NULL, { NULL },
     NC_("image-action", "Resize the image to enclose all layers"),
-    G_CALLBACK (image_resize_to_layers_cmd_callback),
+    image_resize_to_layers_cmd_callback,
     GIMP_HELP_IMAGE_RESIZE_TO_LAYERS },
 
   { "image-resize-to-selection", NULL,
-    NC_("image-action", "F_it Canvas to Selection"), NULL,
+    NC_("image-action", "F_it Canvas to Selection"), NULL, { NULL },
     NC_("image-action", "Resize the image to the extents of the selection"),
-    G_CALLBACK (image_resize_to_selection_cmd_callback),
+    image_resize_to_selection_cmd_callback,
     GIMP_HELP_IMAGE_RESIZE_TO_SELECTION },
 
   { "image-print-size", GIMP_ICON_DOCUMENT_PRINT_RESOLUTION,
-    NC_("image-action", "_Print Size..."), NULL,
+    NC_("image-action", "_Print Size..."), NULL, { NULL },
     NC_("image-action", "Adjust the print resolution"),
-    G_CALLBACK (image_print_size_cmd_callback),
+    image_print_size_cmd_callback,
     GIMP_HELP_IMAGE_PRINT_SIZE },
 
   { "image-scale", GIMP_ICON_OBJECT_SCALE,
-    NC_("image-action", "_Scale Image..."), NULL,
+    NC_("image-action", "_Scale Image..."), NULL, { NULL },
     NC_("image-action", "Change the size of the image content"),
-    G_CALLBACK (image_scale_cmd_callback),
+    image_scale_cmd_callback,
     GIMP_HELP_IMAGE_SCALE },
 
   { "image-crop-to-selection", GIMP_ICON_TOOL_CROP,
-    NC_("image-action", "_Crop to Selection"), NULL,
+    NC_("image-action", "_Crop to Selection"), NULL, { NULL },
     NC_("image-action", "Crop the image to the extents of the selection"),
-    G_CALLBACK (image_crop_to_selection_cmd_callback),
+    image_crop_to_selection_cmd_callback,
     GIMP_HELP_IMAGE_CROP },
 
   { "image-crop-to-content", GIMP_ICON_TOOL_CROP,
-    NC_("image-action", "Crop to C_ontent"), NULL,
+    NC_("image-action", "Crop to C_ontent"), NULL, { NULL },
     NC_("image-action", "Crop the image to the extents of its content (remove empty borders from the image)"),
-    G_CALLBACK (image_crop_to_content_cmd_callback),
+    image_crop_to_content_cmd_callback,
     GIMP_HELP_IMAGE_CROP },
 
   { "image-merge-layers", NULL,
-    NC_("image-action", "Merge Visible _Layers..."), "<primary>M",
+    NC_("image-action", "Merge Visible _Layers..."), NULL, { "<primary>M", NULL },
     NC_("image-action", "Merge all visible layers into one layer"),
-    G_CALLBACK (image_merge_layers_cmd_callback),
+    image_merge_layers_cmd_callback,
     GIMP_HELP_IMAGE_MERGE_LAYERS },
 
   { "image-flatten", NULL,
-    NC_("image-action", "_Flatten Image"), NULL,
+    NC_("image-action", "_Flatten Image"), NULL, { NULL },
     NC_("image-action", "Merge all layers into one and remove transparency"),
-    G_CALLBACK (image_flatten_image_cmd_callback),
+    image_flatten_image_cmd_callback,
     GIMP_HELP_IMAGE_FLATTEN },
 
   { "image-configure-grid", GIMP_ICON_GRID,
-    NC_("image-action", "Configure G_rid..."), NULL,
+    NC_("image-action", "Configure G_rid..."), NULL, { NULL },
     NC_("image-action", "Configure the grid for this image"),
-    G_CALLBACK (image_configure_grid_cmd_callback),
+    image_configure_grid_cmd_callback,
     GIMP_HELP_IMAGE_GRID },
 
   { "image-properties", "dialog-information",
-    NC_("image-action", "Image Pr_operties"), "<alt>Return",
+    NC_("image-action", "Image Pr_operties"), NULL, { "<alt>Return", NULL },
     NC_("image-action", "Display information about this image"),
-    G_CALLBACK (image_properties_cmd_callback),
+    image_properties_cmd_callback,
     GIMP_HELP_IMAGE_PROPERTIES }
 };
 
 static const GimpToggleActionEntry image_toggle_actions[] =
 {
-  { "image-color-management-enabled", NULL,
-    NC_("image-action", "_Enable Color Management"), NULL,
-    NC_("image-action", "Whether the image is color managed. Disabling "
-        "color management is equivalent to assigning a built-in sRGB "
-        "color profile. Better leave color management enabled."),
-    G_CALLBACK (image_color_management_enabled_cmd_callback),
+  { "image-color-profile-use-srgb", NULL,
+    NC_("image-action", "Use _sRGB Profile"), NULL, { NULL },
+    NC_("image-action", "Temporarily use an sRGB profile for the image. "
+        "This is the same as discarding the image's color profile, but "
+        "allows to easily restore the profile."),
+    image_color_profile_use_srgb_cmd_callback,
     TRUE,
-    GIMP_HELP_IMAGE_COLOR_MANAGEMENT_ENABLED }
+    GIMP_HELP_IMAGE_COLOR_PROFILE_USE_SRGB },
+
+  { "image-softproof-black-point-compensation", NULL,
+    NC_("image-action", "_Black Point Compensation"), NULL, { NULL },
+    NC_("image-action", "Use black point compensation for soft-proofing"),
+    image_softproof_bpc_cmd_callback,
+    TRUE,
+    GIMP_HELP_VIEW_COLOR_MANAGEMENT }
 };
 
 static const GimpRadioActionEntry image_convert_base_type_actions[] =
 {
   { "image-convert-rgb", GIMP_ICON_CONVERT_RGB,
-    NC_("image-convert-action", "_RGB"), NULL,
+    NC_("image-convert-action", "_RGB"), NULL, { NULL },
     NC_("image-convert-action", "Convert the image to the RGB colorspace"),
     GIMP_RGB, GIMP_HELP_IMAGE_CONVERT_RGB },
 
   { "image-convert-grayscale", GIMP_ICON_CONVERT_GRAYSCALE,
-    NC_("image-convert-action", "_Grayscale"), NULL,
+    NC_("image-convert-action", "_Grayscale"), NULL, { NULL },
     NC_("image-convert-action", "Convert the image to grayscale"),
     GIMP_GRAY, GIMP_HELP_IMAGE_CONVERT_GRAYSCALE },
 
   { "image-convert-indexed", GIMP_ICON_CONVERT_INDEXED,
-    NC_("image-convert-action", "_Indexed..."), NULL,
+    NC_("image-convert-action", "_Indexed..."), NULL, { NULL },
     NC_("image-convert-action", "Convert the image to indexed colors"),
     GIMP_INDEXED, GIMP_HELP_IMAGE_CONVERT_INDEXED }
 };
@@ -207,67 +200,73 @@ static const GimpRadioActionEntry image_convert_base_type_actions[] =
 static const GimpRadioActionEntry image_convert_precision_actions[] =
 {
   { "image-convert-u8", NULL,
-    NC_("image-convert-action", "8 bit integer"), NULL,
+    NC_("image-convert-action", "8-bit integer"), NULL, { NULL },
     NC_("image-convert-action",
-        "Convert the image to 8 bit integer"),
+        "Convert the image to 8-bit integer"),
     GIMP_COMPONENT_TYPE_U8, GIMP_HELP_IMAGE_CONVERT_U8 },
 
   { "image-convert-u16", NULL,
-    NC_("image-convert-action", "16 bit integer"), NULL,
+    NC_("image-convert-action", "16-bit integer"), NULL, { NULL },
     NC_("image-convert-action",
-        "Convert the image to 16 bit integer"),
+        "Convert the image to 16-bit integer"),
     GIMP_COMPONENT_TYPE_U16, GIMP_HELP_IMAGE_CONVERT_U16 },
 
   { "image-convert-u32", NULL,
-    NC_("image-convert-action", "32 bit integer"), NULL,
+    NC_("image-convert-action", "32-bit integer"), NULL, { NULL },
     NC_("image-convert-action",
-        "Convert the image to 32 bit integer"),
+        "Convert the image to 32-bit integer"),
     GIMP_COMPONENT_TYPE_U32, GIMP_HELP_IMAGE_CONVERT_U32 },
 
   { "image-convert-half", NULL,
-    NC_("image-convert-action", "16 bit floating point"), NULL,
+    NC_("image-convert-action", "16-bit floating point"), NULL, { NULL },
     NC_("image-convert-action",
-        "Convert the image to 16 bit floating point"),
+        "Convert the image to 16-bit floating point"),
     GIMP_COMPONENT_TYPE_HALF, GIMP_HELP_IMAGE_CONVERT_HALF },
 
   { "image-convert-float", NULL,
-    NC_("image-convert-action", "32 bit floating point"), NULL,
+    NC_("image-convert-action", "32-bit floating point"), NULL, { NULL },
     NC_("image-convert-action",
-        "Convert the image to 32 bit floating point"),
+        "Convert the image to 32-bit floating point"),
     GIMP_COMPONENT_TYPE_FLOAT, GIMP_HELP_IMAGE_CONVERT_FLOAT },
 
   { "image-convert-double", NULL,
-    NC_("image-convert-action", "64 bit floating point"), NULL,
+    NC_("image-convert-action", "64-bit floating point"), NULL, { NULL },
     NC_("image-convert-action",
-        "Convert the image to 64 bit floating point"),
+        "Convert the image to 64-bit floating point"),
     GIMP_COMPONENT_TYPE_DOUBLE, GIMP_HELP_IMAGE_CONVERT_DOUBLE }
 };
 
-static const GimpRadioActionEntry image_convert_gamma_actions[] =
+static const GimpRadioActionEntry image_convert_trc_actions[] =
 {
-  { "image-convert-gamma", NULL,
-    NC_("image-convert-action", "Perceptual gamma (sRGB)"), NULL,
-    NC_("image-convert-action",
-        "Convert the image to preceptual (sRGB) gamma"),
-    FALSE, GIMP_HELP_IMAGE_CONVERT_GAMMA },
-
   { "image-convert-linear", NULL,
-    NC_("image-convert-action", "Linear light"), NULL,
+    NC_("image-convert-action", "Linear light"), NULL, { NULL },
     NC_("image-convert-action",
         "Convert the image to linear light"),
-    TRUE, GIMP_HELP_IMAGE_CONVERT_GAMMA }
+    GIMP_TRC_LINEAR, GIMP_HELP_IMAGE_CONVERT_GAMMA },
+
+  { "image-convert-non-linear", NULL,
+    NC_("image-convert-action", "Non-Linear"), NULL, { NULL },
+    NC_("image-convert-action",
+        "Convert the image to non-linear gamma from the color profile"),
+    GIMP_TRC_NON_LINEAR, GIMP_HELP_IMAGE_CONVERT_GAMMA },
+
+  { "image-convert-perceptual", NULL,
+    NC_("image-convert-action", "Perceptual (sRGB)"), NULL, { NULL },
+    NC_("image-convert-action",
+        "Convert the image to perceptual (sRGB) gamma"),
+    GIMP_TRC_PERCEPTUAL, GIMP_HELP_IMAGE_CONVERT_GAMMA }
 };
 
 static const GimpEnumActionEntry image_flip_actions[] =
 {
   { "image-flip-horizontal", GIMP_ICON_OBJECT_FLIP_HORIZONTAL,
-    NC_("image-action", "Flip _Horizontally"), NULL,
+    NC_("image-action", "Flip _Horizontally"), NULL, { NULL },
     NC_("image-action", "Flip image horizontally"),
     GIMP_ORIENTATION_HORIZONTAL, FALSE,
     GIMP_HELP_IMAGE_FLIP_HORIZONTAL },
 
   { "image-flip-vertical", GIMP_ICON_OBJECT_FLIP_VERTICAL,
-    NC_("image-action", "Flip _Vertically"), NULL,
+    NC_("image-action", "Flip _Vertically"), NULL, { NULL },
     NC_("image-action", "Flip image vertically"),
     GIMP_ORIENTATION_VERTICAL, FALSE,
     GIMP_HELP_IMAGE_FLIP_VERTICAL }
@@ -276,24 +275,51 @@ static const GimpEnumActionEntry image_flip_actions[] =
 static const GimpEnumActionEntry image_rotate_actions[] =
 {
   { "image-rotate-90", GIMP_ICON_OBJECT_ROTATE_90,
-    NC_("image-action", "Rotate 90° _clockwise"), NULL,
+    NC_("image-action", "Rotate 90° _clockwise"), NULL, { NULL },
     NC_("image-action", "Rotate the image 90 degrees to the right"),
     GIMP_ROTATE_90, FALSE,
     GIMP_HELP_IMAGE_ROTATE_90 },
 
   { "image-rotate-180", GIMP_ICON_OBJECT_ROTATE_180,
-    NC_("image-action", "Rotate _180°"), NULL,
+    NC_("image-action", "Rotate _180°"), NULL, { NULL },
     NC_("image-action", "Turn the image upside-down"),
     GIMP_ROTATE_180, FALSE,
     GIMP_HELP_IMAGE_ROTATE_180 },
 
   { "image-rotate-270", GIMP_ICON_OBJECT_ROTATE_270,
-    NC_("image-action", "Rotate 90° counter-clock_wise"), NULL,
+    NC_("image-action", "Rotate 90° counter-clock_wise"), NULL, { NULL },
     NC_("image-action", "Rotate the image 90 degrees to the left"),
     GIMP_ROTATE_270, FALSE,
     GIMP_HELP_IMAGE_ROTATE_270 }
 };
 
+static const GimpRadioActionEntry image_softproof_intent_actions[] =
+{
+  { "image-softproof-intent-perceptual", NULL,
+    NC_("image-action", "_Perceptual"), NULL, { NULL },
+    NC_("image-action", "Soft-proofing rendering intent is perceptual"),
+    GIMP_COLOR_RENDERING_INTENT_PERCEPTUAL,
+    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+
+  { "image-softproof-intent-relative-colorimetric", NULL,
+    NC_("image-action", "_Relative Colorimetric"), NULL, { NULL },
+    NC_("image-action", "Soft-proofing rendering intent is relative colorimetric"),
+    GIMP_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC,
+    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+
+  { "image-softproof-intent-saturation", NULL,
+    NC_("image-action", "_Saturation"), NULL, { NULL },
+    NC_("image-action", "Soft-proofing rendering intent is saturation"),
+    GIMP_COLOR_RENDERING_INTENT_SATURATION,
+    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+
+  { "image-softproof-intent-absolute-colorimetric", NULL,
+    NC_("image-action", "_Absolute Colorimetric"), NULL, { NULL },
+    NC_("image-action", "Soft-proofing rendering intent is absolute colorimetric"),
+    GIMP_COLOR_RENDERING_INTENT_ABSOLUTE_COLORIMETRIC,
+    GIMP_HELP_VIEW_COLOR_MANAGEMENT }
+};
+
 
 void
 image_actions_setup (GimpActionGroup *group)
@@ -306,33 +332,40 @@ image_actions_setup (GimpActionGroup *group)
                                         image_toggle_actions,
                                         G_N_ELEMENTS (image_toggle_actions));
 
+  gimp_action_group_add_radio_actions (group, "image-action",
+                                       image_softproof_intent_actions,
+                                       G_N_ELEMENTS (image_softproof_intent_actions),
+                                       NULL,
+                                       GIMP_COLOR_MANAGEMENT_DISPLAY,
+                                       image_softproof_intent_cmd_callback);
+
   gimp_action_group_add_radio_actions (group, "image-convert-action",
                                        image_convert_base_type_actions,
                                        G_N_ELEMENTS (image_convert_base_type_actions),
                                        NULL, 0,
-                                       G_CALLBACK (image_convert_base_type_cmd_callback));
+                                       image_convert_base_type_cmd_callback);
 
   gimp_action_group_add_radio_actions (group, "image-convert-action",
                                        image_convert_precision_actions,
                                        G_N_ELEMENTS (image_convert_precision_actions),
                                        NULL, 0,
-                                       G_CALLBACK (image_convert_precision_cmd_callback));
+                                       image_convert_precision_cmd_callback);
 
   gimp_action_group_add_radio_actions (group, "image-convert-action",
-                                       image_convert_gamma_actions,
-                                       G_N_ELEMENTS (image_convert_gamma_actions),
+                                       image_convert_trc_actions,
+                                       G_N_ELEMENTS (image_convert_trc_actions),
                                        NULL, 0,
-                                       G_CALLBACK (image_convert_gamma_cmd_callback));
+                                       image_convert_trc_cmd_callback);
 
   gimp_action_group_add_enum_actions (group, "image-action",
                                       image_flip_actions,
                                       G_N_ELEMENTS (image_flip_actions),
-                                      G_CALLBACK (image_flip_cmd_callback));
+                                      image_flip_cmd_callback);
 
   gimp_action_group_add_enum_actions (group, "image-action",
                                       image_rotate_actions,
                                       G_N_ELEMENTS (image_rotate_actions),
-                                      G_CALLBACK (image_rotate_cmd_callback));
+                                      image_rotate_cmd_callback);
 
 #define SET_ALWAYS_SHOW_IMAGE(action,show) \
         gimp_action_group_set_action_always_show_image (group, action, show)
@@ -348,16 +381,27 @@ void
 image_actions_update (GimpActionGroup *group,
                       gpointer         data)
 {
-  GimpImage *image         = action_data_get_image (data);
-  gboolean   is_indexed    = FALSE;
-  gboolean   is_u8_gamma   = FALSE;
-  gboolean   is_double     = FALSE;
-  gboolean   aux           = FALSE;
-  gboolean   lp            = FALSE;
-  gboolean   sel           = FALSE;
-  gboolean   groups        = FALSE;
-  gboolean   color_managed = FALSE;
-  gboolean   profile       = FALSE;
+  GimpImage        *image          = action_data_get_image (data);
+  gboolean          is_indexed     = FALSE;
+  gboolean          is_u8_gamma    = FALSE;
+  gboolean          is_double      = FALSE;
+  gboolean          aux            = FALSE;
+  gboolean          lp             = FALSE;
+  gboolean          sel            = FALSE;
+  gboolean          groups         = FALSE;
+  gboolean          profile_srgb   = FALSE;
+  gboolean          profile_hidden = FALSE;
+  gboolean          profile        = FALSE;
+  gboolean          s_bpc          = FALSE;
+
+#define SET_LABEL(action,label) \
+        gimp_action_group_set_action_label (group, action, (label))
+#define SET_SENSITIVE(action,condition) \
+        gimp_action_group_set_action_sensitive (group, action, (condition) != 0, NULL)
+#define SET_ACTIVE(action,condition) \
+        gimp_action_group_set_action_active (group, action, (condition) != 0)
+#define SET_VISIBLE(action,condition) \
+        gimp_action_group_set_action_visible (group, action, (condition) != 0)
 
   if (image)
     {
@@ -366,10 +410,13 @@ image_actions_update (GimpActionGroup *group,
       GimpImageBaseType  base_type;
       GimpPrecision      precision;
       GimpComponentType  component_type;
+      GimpTRCType        trc;
 
       base_type      = gimp_image_get_base_type (image);
       precision      = gimp_image_get_precision (image);
       component_type = gimp_image_get_component_type (image);
+      trc            = gimp_babl_format_get_trc
+                         (gimp_image_get_layer_format (image, FALSE));
 
       switch (base_type)
         {
@@ -378,7 +425,7 @@ image_actions_update (GimpActionGroup *group,
         case GIMP_INDEXED: action = "image-convert-indexed";   break;
         }
 
-      gimp_action_group_set_action_active (group, action, TRUE);
+      SET_ACTIVE (action, TRUE);
 
       switch (component_type)
         {
@@ -390,24 +437,32 @@ image_actions_update (GimpActionGroup *group,
         case GIMP_COMPONENT_TYPE_DOUBLE: action = "image-convert-double"; break;
         }
 
-      gimp_action_group_set_action_active (group, action, TRUE);
+      SET_ACTIVE (action, TRUE);
 
-      if (gimp_babl_format_get_linear (gimp_image_get_layer_format (image,
-                                                                    FALSE)))
+      switch (trc)
         {
-          gimp_action_group_set_action_active (group, "image-convert-linear",
-                                               TRUE);
-        }
-      else
-        {
-          gimp_action_group_set_action_active (group, "image-convert-gamma",
-                                               TRUE);
+        case GIMP_TRC_LINEAR:
+          action = "image-convert-linear";
+          SET_VISIBLE ("image-convert-perceptual", FALSE);
+          break;
+
+        case GIMP_TRC_NON_LINEAR:
+          action = "image-convert-non-linear";
+          SET_VISIBLE ("image-convert-perceptual", FALSE);
+          break;
+
+        case GIMP_TRC_PERCEPTUAL:
+          action = "image-convert-perceptual";
+          SET_VISIBLE ("image-convert-perceptual", TRUE);
+          break;
         }
 
+      SET_ACTIVE (action, TRUE);
+
       is_indexed  = (base_type == GIMP_INDEXED);
-      is_u8_gamma = (precision == GIMP_PRECISION_U8_GAMMA);
+      is_u8_gamma = (precision == GIMP_PRECISION_U8_NON_LINEAR);
       is_double   = (component_type == GIMP_COMPONENT_TYPE_DOUBLE);
-      aux         = (gimp_image_get_active_channel (image) != NULL);
+      aux         = (gimp_image_get_selected_channels (image) != NULL);
       lp          = ! gimp_image_is_empty (image);
       sel         = ! gimp_channel_is_empty (gimp_image_get_mask (image));
 
@@ -415,18 +470,36 @@ image_actions_update (GimpActionGroup *group,
 
       groups = ! gimp_item_stack_is_flat (GIMP_ITEM_STACK (layers));
 
-      color_managed = gimp_image_get_is_color_managed (image);
-      profile       = (gimp_image_get_color_profile (image) != NULL);
-    }
+      profile_srgb = gimp_image_get_use_srgb_profile (image, &profile_hidden);
+      profile      = (gimp_image_get_color_profile (image) != NULL);
 
-#define SET_LABEL(action,label) \
-        gimp_action_group_set_action_label (group, action, (label))
-#define SET_SENSITIVE(action,condition) \
-        gimp_action_group_set_action_sensitive (group, action, (condition) != 0)
-#define SET_ACTIVE(action,condition) \
-        gimp_action_group_set_action_active (group, action, (condition) != 0)
-#define SET_VISIBLE(action,condition) \
-        gimp_action_group_set_action_visible (group, action, (condition) != 0)
+      switch (gimp_image_get_simulation_intent (image))
+        {
+        case GIMP_COLOR_RENDERING_INTENT_PERCEPTUAL:
+          action = "image-softproof-intent-perceptual";
+          break;
+
+        case GIMP_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC:
+          action = "image-softproof-intent-relative-colorimetric";
+          break;
+
+        case GIMP_COLOR_RENDERING_INTENT_SATURATION:
+          action = "image-softproof-intent-saturation";
+          break;
+
+        case GIMP_COLOR_RENDERING_INTENT_ABSOLUTE_COLORIMETRIC:
+          action = "image-softproof-intent-absolute-colorimetric";
+          break;
+        }
+
+      gimp_action_group_set_action_active (group, action, TRUE);
+
+      s_bpc  = gimp_image_get_simulation_bpc (image);
+    }
+  else
+    {
+      SET_VISIBLE ("image-convert-perceptual", FALSE);
+    }
 
   SET_SENSITIVE ("image-duplicate", image);
 
@@ -449,6 +522,14 @@ image_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("image-convert-grayscale", image);
   SET_SENSITIVE ("image-convert-indexed",   image && !groups && is_u8_gamma);
 
+  SET_SENSITIVE ("image-softproof-profile",                      image);
+  SET_SENSITIVE ("image-softproof-intent-perceptual",            image);
+  SET_SENSITIVE ("image-softproof-intent-relative-colorimetric", image);
+  SET_SENSITIVE ("image-softproof-intent-saturation",            image);
+  SET_SENSITIVE ("image-softproof-intent-absolute-colorimetric", image);
+  SET_SENSITIVE ("image-softproof-black-point-compensation",     image);
+  SET_ACTIVE    ("image-softproof-black-point-compensation",     s_bpc);
+
   SET_SENSITIVE ("image-convert-u8",     image);
   SET_SENSITIVE ("image-convert-u16",    image && !is_indexed);
   SET_SENSITIVE ("image-convert-u32",    image && !is_indexed);
@@ -457,11 +538,12 @@ image_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("image-convert-double", image && !is_indexed);
   SET_VISIBLE   ("image-convert-double", is_double);
 
-  SET_SENSITIVE ("image-convert-gamma",  image);
-  SET_SENSITIVE ("image-convert-linear", image && !is_indexed);
+  SET_SENSITIVE ("image-convert-linear",     image && !is_indexed);
+  SET_SENSITIVE ("image-convert-non-linear", image);
+  SET_SENSITIVE ("image-convert-perceptual", image && !is_indexed);
 
-  SET_SENSITIVE ("image-color-management-enabled", image);
-  SET_ACTIVE    ("image-color-management-enabled", image && color_managed);
+  SET_SENSITIVE ("image-color-profile-use-srgb", image && (profile || profile_hidden));
+  SET_ACTIVE    ("image-color-profile-use-srgb", image && profile_srgb);
 
   SET_SENSITIVE ("image-color-profile-assign",  image);
   SET_SENSITIVE ("image-color-profile-convert", image);
