@@ -53,7 +53,6 @@ struct _GimpGuiConfig
   gboolean             restore_session;
   gboolean             restore_monitor;
   gboolean             save_tool_options;
-  gboolean             can_change_accels;
   gboolean             save_accels;
   gboolean             restore_accels;
   gint                 last_opened_size;
@@ -69,6 +68,8 @@ struct _GimpGuiConfig
   gchar               *icon_theme_path;
   gchar               *icon_theme;
   gboolean             prefer_symbolic_icons;
+  gboolean             override_icon_size;
+  GimpIconSize         custom_icon_size;
   gboolean             use_help;
   gboolean             show_help_button;
   gchar               *help_locales;
@@ -78,6 +79,7 @@ struct _GimpGuiConfig
   gint                 action_history_size;
   GimpWindowHint       dock_window_hint;
   GimpHandedness       cursor_handedness;
+  gboolean             custom_title_bar;
 
   /* experimental playground */
   gboolean             playground_npd_tool;
@@ -90,6 +92,11 @@ struct _GimpGuiConfig
   gboolean             show_tabs;
   GimpPosition         tabs_position;
   gint                 last_tip_shown;
+
+  /* Currently ignored snce "gtk-can-change-accels" settings got deprecated in
+   * GTK+ 3.10.
+   */
+  gboolean             can_change_accels;
 };
 
 struct _GimpGuiConfigClass
