@@ -77,6 +77,9 @@ _("Context-dependent mouse pointers are helpful.  They are enabled by " \
 "Specify a default tool preset.  The tool preset is searched for in the " \
 "specified tool prests path."
 
+#define CUSTOM_TITLE_BAR_BLURB \
+_("Merge menu and title bar (client-side decoration)")
+
 #define DEFAULT_SHOW_ALL_BLURB \
 _("Show full image content by default.")
 
@@ -540,6 +543,12 @@ _("The name of the theme to use.")
 #define ICON_THEME_BLURB \
 "The name of the icon theme to use."
 
+#define OVERRIDE_THEME_ICON_SIZE_BLURB \
+_("Override theme-set icon sizes.")
+
+#define ICON_SIZE_BLURB \
+_("The size of the icons to use.")
+
 #define PREFER_SYMBOLIC_ICONS_BLURB \
 _("When enabled, symbolic icons will be preferred if available.")
 
@@ -644,7 +653,7 @@ _("Sets the default path name for the 'New Path' dialog.")
 _("Sets the default folder path for the 'Export Path' dialog.")
 
 #define VECTORS_EXPORT_ACTIVE_ONLY_BLURB \
-_("Sets the default 'Export the active path' state for the 'Export Path' dialog.")
+_("Sets the default 'Export the selected paths' state for the 'Export Path' dialog.")
 
 #define VECTORS_IMPORT_PATH_BLURB \
 _("Sets the default folder path for the 'Import Path' dialog.")
