@@ -112,6 +112,7 @@ typedef enum  /*< pdb-skip >*/
   GIMP_COLOR_PICK_MODE_PIXEL,       /*< desc="Pixel"        >*/
   GIMP_COLOR_PICK_MODE_RGB_PERCENT, /*< desc="RGB (%)"      >*/
   GIMP_COLOR_PICK_MODE_RGB_U8,      /*< desc="RGB (0..255)" >*/
+  GIMP_COLOR_PICK_MODE_GRAYSCALE,   /*< desc="Grayscale (%)">*/
   GIMP_COLOR_PICK_MODE_HSV,         /*< desc="HSV"          >*/
   GIMP_COLOR_PICK_MODE_LCH,         /*< desc="CIE LCh"      >*/
   GIMP_COLOR_PICK_MODE_LAB,         /*< desc="CIE LAB"      >*/
@@ -289,14 +290,25 @@ typedef enum  /*< pdb-skip >*/
 } GimpDynamicsOutputType;
 
 
+#define GIMP_TYPE_CUSTOM_STYLE (gimp_custom_style_get_type ())
+
+GType gimp_custom_style_get_type (void) G_GNUC_CONST;
+
+typedef enum  /*< pdb-skip >*/
+{
+  GIMP_CUSTOM_STYLE_SOLID_COLOR,  /*< desc="Solid color" >*/
+  GIMP_CUSTOM_STYLE_PATTERN       /*< desc="Pattern"          >*/
+} GimpCustomStyle;
+
 #define GIMP_TYPE_FILL_STYLE (gimp_fill_style_get_type ())
 
 GType gimp_fill_style_get_type (void) G_GNUC_CONST;
 
 typedef enum  /*< pdb-skip >*/
 {
-  GIMP_FILL_STYLE_SOLID,  /*< desc="Solid color" >*/
-  GIMP_FILL_STYLE_PATTERN /*< desc="Pattern"     >*/
+  GIMP_FILL_STYLE_FG_COLOR,  /*< desc="Foreground color" >*/
+  GIMP_FILL_STYLE_BG_COLOR,  /*< desc="Background color" >*/
+  GIMP_FILL_STYLE_PATTERN    /*< desc="Pattern"          >*/
 } GimpFillStyle;
 
 
