@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /* This file contains functions to help migrate the settings from a
@@ -69,6 +69,8 @@ struct _GimpUserInstall
   gint                    old_major;
   gint                    old_minor;
 
+  gint                    scale_factor;
+
   const gchar            *migrate;
 
   GimpUserInstallLogFunc  log;
@@ -88,8 +90,6 @@ static const struct
 }
 gimp_user_install_items[] =
 {
-  { "gtkrc",           USER_INSTALL_COPY  },
-  { "menurc",          USER_INSTALL_COPY  },
   { "brushes",         USER_INSTALL_MKDIR },
   { "dynamics",        USER_INSTALL_MKDIR },
   { "fonts",           USER_INSTALL_MKDIR },
@@ -137,8 +137,11 @@ static gboolean  user_install_file_copy          (GimpUserInstall    *install,
                                                   const gchar        *old_options_regexp,
                                                   GRegexEvalCallback  update_callback);
 static gboolean  user_install_dir_copy           (GimpUserInstall    *install,
+                                                  gint                level,
                                                   const gchar        *source,
-                                                  const gchar        *base);
+                                                  const gchar        *base,
+                                                  const gchar        *update_pattern,
+                                                  GRegexEvalCallback  update_callback);
 
 static gboolean  user_install_create_files       (GimpUserInstall    *install);
 static gboolean  user_install_migrate_files      (GimpUserInstall    *install);
@@ -200,12 +203,14 @@ gimp_user_install_new (GObject  *gimp,
 }
 
 gboolean
-gimp_user_install_run (GimpUserInstall *install)
+gimp_user_install_run (GimpUserInstall *install,
+                       gint             scale_factor)
 {
   gchar *dirname;
 
   g_return_val_if_fail (install != NULL, FALSE);
 
+  install->scale_factor = scale_factor;
   dirname = g_filename_display_name (gimp_directory ());
 
   if (install->migrate)
@@ -267,6 +272,7 @@ user_install_detect_old (GimpUserInstall *install,
   gboolean  migrate = FALSE;
 
   version = strstr (dir, GIMP_APP_VERSION);
+  g_snprintf (version, 5, "%d.XY", 2);
 
   if (version)
     {
@@ -275,7 +281,7 @@ user_install_detect_old (GimpUserInstall *install,
       for (i = (GIMP_MINOR_VERSION & ~1); i >= 0; i -= 2)
         {
           /*  we assume that GIMP_APP_VERSION is in the form '2.x'  */
-          g_snprintf (version + 2, 2, "%d", i);
+          g_snprintf (version + 2, 3, "%d", i);
 
           migrate = g_file_test (dir, G_FILE_TEST_IS_DIR);
 
@@ -418,7 +424,7 @@ user_install_file_copy (GimpUserInstall    *install,
                     gimp_filename_to_utf8 (dest),
                     gimp_filename_to_utf8 (source));
 
-  success = gimp_config_file_copy (source, dest, old_options_regexp, update_callback, &error);
+  success = gimp_config_file_copy (source, dest, old_options_regexp, update_callback, install, &error);
 
   user_install_log_error (install, &error);
 
@@ -482,13 +488,8 @@ user_install_mkdir_with_parents (GimpUserInstall *install,
  * the next release
  */
 #define MENURC_OVER20_UPDATE_PATTERN \
-  "\"<Actions>/file/file-export-to\""           "|" \
-  "\"<Actions>/file/file-export\""              "|" \
-  "\"<Actions>/edit/edit-paste-as-new\""        "|" \
-  "\"<Actions>/buffers/buffers-paste-as-new\""  "|" \
-  "\"<Actions>/tools/tools-value-[1-4]-.*\""    "|" \
-  "\"<Actions>/layers/layers-text-tool\""       "|" \
-  "\"<Actions>/vectors/vectors-path-tool\""
+  "(;)? *\\(gtk_accel_path \"<Actions>/[a-zA-Z0-9-]*/([a-zA-Z0-9-]*)\" *\"([a-zA-Z0-9<>_]*)\"\\)" "|" \
+  "(;.*)"
 
 /**
  * callback to use for updating a menurc from GIMP over 2.0.
@@ -500,69 +501,79 @@ user_update_menurc_over20 (const GMatchInfo *matched_value,
                            GString          *new_value,
                            gpointer          data)
 {
-  gchar *match = g_match_info_fetch (matched_value, 0);
+  GimpUserInstall *install         = (GimpUserInstall *) data;
+  gchar           *comment_match   = g_match_info_fetch (matched_value, 1);
+  gchar           *action_match    = g_match_info_fetch (matched_value, 2);
+  gchar           *accel_match     = g_match_info_fetch (matched_value, 3);
+  gchar           *ignore_match    = g_match_info_fetch (matched_value, 4);
+  gchar           *new_action_name = NULL;
 
-  /* file-export-* changes to follow file-save-* patterns.  Actions
-   * available since GIMP 2.8, changed for 2.10 in commit 4b14ed2.
-   */
-  if (g_strcmp0 (match, "\"<Actions>/file/file-export\"") == 0)
+  if (strlen (ignore_match) == 0)
     {
-      g_string_append (new_value, "\"<Actions>/file/file-export-as\"");
-    }
-  else if (g_strcmp0 (match, "\"<Actions>/file/file-export-to\"") == 0)
-    {
-      g_string_append (new_value, "\"<Actions>/file/file-export\"");
-    }
-  /* "*-paste-as-new" renamed to "*-paste-as-new-image"
-   */
-  else if (g_strcmp0 (match, "\"<Actions>/edit/edit-paste-as-new\"") == 0)
-    {
-      g_string_append (new_value, "\"<Actions>/edit/edit-paste-as-new-image\"");
-    }
-  else if (g_strcmp0 (match, "\"<Actions>/buffers/buffers-paste-as-new\"") == 0)
-    {
-      g_string_append (new_value, "\"<Actions>/buffers/buffers-paste-as-new-image\"");
-    }
-  /* Tools settings renamed more user-friendly.  Actions available
-   * since GIMP 2.4, changed for 2.10 in commit 0bdb747.
-   */
-  else if (g_str_has_prefix (match, "\"<Actions>/tools/tools-value-1-"))
-    {
-      g_string_append (new_value, "\"<Actions>/tools/tools-opacity-");
-      g_string_append (new_value, match + 31);
-    }
-  else if (g_str_has_prefix (match, "\"<Actions>/tools/tools-value-2-"))
-    {
-      g_string_append (new_value, "\"<Actions>/tools/tools-size-");
-      g_string_append (new_value, match + 31);
-    }
-  else if (g_str_has_prefix (match, "\"<Actions>/tools/tools-value-3-"))
-    {
-      g_string_append (new_value, "\"<Actions>/tools/tools-aspect-");
-      g_string_append (new_value, match + 31);
-    }
-  else if (g_str_has_prefix (match, "\"<Actions>/tools/tools-value-4-"))
-    {
-      g_string_append (new_value, "\"<Actions>/tools/tools-angle-");
-      g_string_append (new_value, match + 31);
-    }
-  else if (g_strcmp0 (match, "\"<Actions>/layers/layers-text-tool\"") == 0)
-    {
-      g_string_append (new_value, "\"<Actions>/layers/layers-edit\"");
-    }
-  else if (g_strcmp0 (match, "\"<Actions>/vectors/vectors-path-tool\"") == 0)
-    {
-      g_string_append (new_value, "\"<Actions>/vectors/vectors-edit\"");
-    }
-  /* Should not happen. Just in case we match something unexpected by
-   * mistake.
-   */
-  else
-    {
-      g_string_append (new_value, match);
+      /* "*-paste-as-new" renamed to "*-paste-as-new-image" */
+      if (g_strcmp0 (action_match, "buffers-paste-as-new") == 0)
+        new_action_name = g_strdup ("buffers-paste-as-new-image");
+      else if (g_strcmp0 (action_match, "edit-paste-as-new") == 0)
+        new_action_name = g_strdup ("edit-paste-as-new-image");
+      /* file-export-* changes to follow file-save-* patterns.  Actions
+       * available since GIMP 2.8, changed for 2.10 in commit 4b14ed2.
+       */
+      else if (g_strcmp0 (action_match, "file-export") == 0 &&
+               install->old_major == 2 && install->old_minor <= 8)
+        new_action_name = g_strdup ("file-export-as");
+      else if (g_strcmp0 (action_match, "file-export-to") == 0 &&
+               install->old_major == 2 && install->old_minor <= 8)
+        new_action_name = g_strdup ("file-export");
+      else if (g_strcmp0 (action_match, "layers-text-tool") == 0)
+        new_action_name = g_strdup ("layers-edit");
+      /* plug-in-gauss doesn't exist anymore since commit ff59aebbe88.
+       * The expected replacement would be filters-gaussian-blur which is
+       * gegl:gaussian-blur operation. See also bug 775931.
+       */
+      else if (g_strcmp0 (action_match, "plug-in-gauss") == 0)
+        new_action_name = g_strdup ("filters-gaussian-blur");
+      /* Tools settings renamed more user-friendly.  Actions available
+       * since GIMP 2.4, changed for 2.10 in commit 0bdb747.
+       */
+      else if (g_str_has_prefix (action_match, "tools-value-1-"))
+        new_action_name = g_strdup ("tools-opacity-");
+      else if (g_str_has_prefix (action_match, "tools-value-2-"))
+        new_action_name = g_strdup_printf ("tools-size-%s", action_match + 14);
+      else if (g_str_has_prefix (action_match, "tools-value-3-"))
+        new_action_name = g_strdup_printf ("tools-aspect-%s", action_match + 14);
+      else if (g_str_has_prefix (action_match, "tools-value-4-"))
+        new_action_name = g_strdup_printf ("tools-angle-%s", action_match + 14);
+      else if (g_strcmp0 (action_match, "vectors-path-tool") == 0)
+        new_action_name = g_strdup ("vectors-edit");
+      else if (g_strcmp0 (action_match, "tools-blend") == 0)
+        new_action_name = g_strdup ("tools-gradient");
+      /* view-rotate-reset became view-reset and new view-rotate-reset and
+       * view-flip-reset actions were created.  See commit 15fb4a7be0.
+       */
+      else if (g_strcmp0 (action_match, "view-rotate-reset") == 0 &&
+               install->old_major == 2)
+        new_action_name = g_strdup ("view-reset");
+
+      if (new_action_name == NULL)
+        new_action_name = g_strdup (action_match);
+
+      if (g_strcmp0 (comment_match, ";") == 0)
+        g_string_append (new_value, "# ");
+
+      if (strlen (accel_match) > 0)
+        g_string_append_printf (new_value, "(action \"%s\" \"%s\")",
+                                new_action_name, accel_match);
+      else
+        g_string_append_printf (new_value, "(action \"%s\")",
+                                new_action_name);
     }
 
-  g_free (match);
+  g_free (comment_match);
+  g_free (action_match);
+  g_free (accel_match);
+  g_free (ignore_match);
+  g_free (new_action_name);
+
   return FALSE;
 }
 
@@ -595,38 +606,261 @@ user_update_controllerrc (const GMatchInfo *matched_value,
   return FALSE;
 }
 
+#define SESSIONRC_UPDATE_PATTERN \
+  "\\(position [0-9]* [0-9]*\\)"        "|"  \
+  "\\(size [0-9]* [0-9]*\\)"            "|" \
+  "\\(left-docks-width \"?[0-9]*\"?\\)" "|" \
+  "\\(right-docks-width \"?[0-9]*\"?\\)"
+
+static gboolean
+user_update_sessionrc (const GMatchInfo *matched_value,
+                       GString          *new_value,
+                       gpointer          data)
+{
+  GimpUserInstall *install = (GimpUserInstall *) data;
+  gchar           *original;
+
+  original = g_match_info_fetch (matched_value, 0);
+
+  if (install->scale_factor != 1 && install->scale_factor > 0)
+    {
+      /* GTK < 3.0 didn't have scale factor support. It means that any
+       * size and position back then would be in real pixel size. Now
+       * with GTK3 and over, we need to think of position and size in
+       * virtual/application pixels.
+       * In particular it means that if we were to just copy the numbers
+       * from GTK2 to GTK3 on a display with scale factor of 2, every
+       * width, height and position would be 2 times too big (a full
+       * screen window would end up off-screen).
+       */
+      GRegex     *regexp;
+      GMatchInfo *match_info;
+      gchar      *match;
+
+      /* First copy the pattern title. */
+      regexp = g_regex_new ("\\([a-z-]* ", 0, 0, NULL);
+      g_regex_match (regexp, original, 0, &match_info);
+      match = g_match_info_fetch (match_info, 0);
+      g_string_append (new_value, match);
+
+      g_match_info_free (match_info);
+      g_regex_unref (regexp);
+      g_free (match);
+
+      /* Now copy the numbers. */
+      regexp = g_regex_new ("[0-9]+|\"", 0, 0, NULL);
+      g_regex_match (regexp, original, 0, &match_info);
+
+      while (g_match_info_matches (match_info))
+        {
+          match = g_match_info_fetch (match_info, 0);
+          if (g_strcmp0 (match, "\"") == 0)
+            {
+              g_string_append (new_value, match);
+            }
+          else
+            {
+              gint num;
+
+              num = g_ascii_strtoll (match, NULL, 10);
+              num /= install->scale_factor;
+
+              g_string_append_printf (new_value, " %d", num);
+            }
+
+          g_free (match);
+          g_match_info_next (match_info, NULL);
+        }
+
+      g_match_info_free (match_info);
+      g_regex_unref (regexp);
+
+      g_string_append (new_value, ")");
+    }
+  else
+    {
+      /* Just copy as-is. */
+      g_string_append (new_value, original);
+    }
+
+  g_free (original);
+
+  return FALSE;
+}
+
 #define GIMPRC_UPDATE_PATTERN \
-  "\\(theme [^)]*\\)"    "|" \
-  "\\(.*-path [^)]*\\)"
+  "\\(theme [^)]*\\)"          "|" \
+  "^ *\\(.*-path \".*\"\\) *$" "|" \
+  "\\(style solid\\)"
 
 static gboolean
 user_update_gimprc (const GMatchInfo *matched_value,
                     GString          *new_value,
                     gpointer          data)
 {
-  /* Do not migrate paths and themes from GIMP < 2.10. */
+  gchar *match = g_match_info_fetch (matched_value, 0);
+
+  if (g_strcmp0 (match, "(style solid)") == 0)
+    {
+      /* See MR !706: GIMP_FILL_STYLE_SOLID was split in
+       * GIMP_FILL_STYLE_FG_COLOR and GIMP_FILL_STYLE_BG_COLOR.
+       */
+      g_string_append (new_value, "(style fg-color)");
+    }
+  else
+    {
+      /* Do not migrate paths and themes from GIMP < 3.0. */
+    }
+
+  g_free (match);
+  return FALSE;
+}
+
+#define GIMPRESSIONIST_UPDATE_PATTERN \
+  "selectedbrush=Brushes/paintbrush.pgm"
+
+static gboolean
+user_update_gimpressionist (const GMatchInfo *matched_value,
+                            GString          *new_value,
+                            gpointer          data)
+{
+  gchar *match = g_match_info_fetch (matched_value, 0);
+
+  /* See bug 791934: both brushes are identical. */
+  if (g_strcmp0 (match, "selectedbrush=Brushes/paintbrush.pgm") == 0)
+    {
+      g_string_append (new_value, "selectedbrush=Brushes/paintbrush01.pgm");
+    }
+  else
+    {
+      g_message ("(WARNING) %s: invalid match \"%s\"", G_STRFUNC, match);
+      g_string_append (new_value, match);
+    }
+
+  g_free (match);
+  return FALSE;
+}
+
+#define TOOL_PRESETS_UPDATE_PATTERN \
+  "GimpImageMapOptions"       "|" \
+  "GimpBlendOptions"          "|" \
+  "gimp-blend-tool"           "|" \
+  "gimp-tool-blend"           "|" \
+  "dynamics \"Dynamics Off\"" "|" \
+  "\\(dynamics-expanded yes\\)"
+
+static gboolean
+user_update_tool_presets (const GMatchInfo *matched_value,
+                          GString          *new_value,
+                          gpointer          data)
+{
+  gchar *match = g_match_info_fetch (matched_value, 0);
+
+  if (g_strcmp0 (match, "GimpImageMapOptions") == 0)
+    {
+      g_string_append (new_value, "GimpFilterOptions");
+    }
+  else if (g_strcmp0 (match, "GimpBlendOptions") == 0)
+    {
+      g_string_append (new_value, "GimpGradientOptions");
+    }
+  else if (g_strcmp0 (match, "gimp-blend-tool") == 0)
+    {
+      g_string_append (new_value, "gimp-gradient-tool");
+    }
+  else if (g_strcmp0 (match, "gimp-tool-blend") == 0)
+    {
+      g_string_append (new_value, "gimp-tool-gradient");
+    }
+  else if (g_strcmp0 (match, "dynamics \"Dynamics Off\"") == 0)
+    {
+      g_string_append (new_value, "dynamics-enabled no");
+    }
+  else if (g_strcmp0 (match, "(dynamics-expanded yes)") == 0)
+    {
+      /* This option just doesn't exist anymore. */
+    }
+  else
+    {
+      g_message ("(WARNING) %s: invalid match \"%s\"", G_STRFUNC, match);
+      g_string_append (new_value, match);
+    }
+
+  g_free (match);
+  return FALSE;
+}
+
+/* Actually not only for contextrc, but all other files where
+ * gimp-blend-tool may appear. Apparently that is also "devicerc", as
+ * well as "toolrc" (but this one is skipped anyway).
+ */
+#define CONTEXTRC_UPDATE_PATTERN \
+  "gimp-blend-tool"           "|" \
+  "dynamics \"Dynamics Off\"" "|" \
+  "\\(dynamics-expanded yes\\)"
+
+static gboolean
+user_update_contextrc_over20 (const GMatchInfo *matched_value,
+                              GString          *new_value,
+                              gpointer          data)
+{
+  gchar *match = g_match_info_fetch (matched_value, 0);
+
+  if (g_strcmp0 (match, "gimp-blend-tool") == 0)
+    {
+      g_string_append (new_value, "gimp-gradient-tool");
+    }
+  else if (g_strcmp0 (match, "dynamics \"Dynamics Off\"") == 0)
+    {
+      g_string_append (new_value, "dynamics-enabled no");
+    }
+  else if (g_strcmp0 (match, "(dynamics-expanded yes)") == 0)
+    {
+      /* This option just doesn't exist anymore. */
+    }
+  else
+    {
+      g_message ("(WARNING) %s: invalid match \"%s\"", G_STRFUNC, match);
+      g_string_append (new_value, match);
+    }
+
+  g_free (match);
   return FALSE;
 }
 
 static gboolean
-user_install_dir_copy (GimpUserInstall *install,
-                       const gchar     *source,
-                       const gchar     *base)
+user_install_dir_copy (GimpUserInstall    *install,
+                       gint                level,
+                       const gchar        *source,
+                       const gchar        *base,
+                       const gchar        *update_pattern,
+                       GRegexEvalCallback  update_callback)
 {
   GDir        *source_dir = NULL;
   GDir        *dest_dir   = NULL;
   gchar        dest[1024];
   const gchar *basename;
-  gchar       *dirname;
-  gboolean     success;
-  GError      *error = NULL;
+  gchar       *dirname = NULL;
+  gchar       *name;
+  GError      *error   = NULL;
+  gboolean     success = FALSE;
 
-  {
-    gchar *basename = g_path_get_basename (source);
+  if (level >= 5)
+    {
+      /* Config migration is recursive, but we can't go on forever,
+       * since we may fall into recursive symlinks in particular (which
+       * is a security risk to fill a disk, and would also block GIMP
+       * forever at migration stage).
+       * Let's just break the recursivity at 5 levels, which is just an
+       * arbitrary value (but I don't think there should be any data
+       * deeper than this).
+       */
+      goto error;
+    }
 
-    dirname = g_build_filename (base, basename, NULL);
-    g_free (basename);
-  }
+  name = g_path_get_basename (source);
+  dirname = g_build_filename (base, name, NULL);
+  g_free (name);
 
   success = user_install_mkdir (install, dirname);
   if (! success)
@@ -642,19 +876,27 @@ user_install_dir_copy (GimpUserInstall *install,
 
   while ((basename = g_dir_read_name (source_dir)) != NULL)
     {
-      gchar *name = g_build_filename (source, basename, NULL);
+      name = g_build_filename (source, basename, NULL);
 
       if (g_file_test (name, G_FILE_TEST_IS_REGULAR))
         {
           g_snprintf (dest, sizeof (dest), "%s%c%s",
                       dirname, G_DIR_SEPARATOR, basename);
 
-          if (! user_install_file_copy (install, name, dest, NULL, NULL))
+          success = user_install_file_copy (install, name, dest,
+                                            update_pattern,
+                                            update_callback);
+          if (! success)
             {
               g_free (name);
               goto error;
             }
         }
+      else
+        {
+          user_install_dir_copy (install, level + 1, name, dirname,
+                                 update_pattern, update_callback);
+        }
 
       g_free (name);
     }
@@ -668,7 +910,8 @@ user_install_dir_copy (GimpUserInstall *install,
   if (dest_dir)
     g_dir_close (dest_dir);
 
-  g_free (dirname);
+  if (dirname)
+    g_free (dirname);
 
   return success;
 }
@@ -743,7 +986,8 @@ user_install_migrate_files (GimpUserInstall *install)
 
   while ((basename = g_dir_read_name (dir)) != NULL)
     {
-      gchar *source = g_build_filename (install->old_dir, basename, NULL);
+      gchar       *source   = g_build_filename (install->old_dir, basename, NULL);
+      const gchar *new_dest = NULL;
 
       if (g_file_test (source, G_FILE_TEST_IS_REGULAR))
         {
@@ -755,10 +999,20 @@ user_install_migrate_files (GimpUserInstall *install)
               g_str_has_prefix (basename, "gimpswap.") ||
               strcmp (basename, "pluginrc") == 0       ||
               strcmp (basename, "themerc") == 0        ||
-              strcmp (basename, "toolrc") == 0)
+              strcmp (basename, "toolrc") == 0         ||
+              strcmp (basename, "gtkrc") == 0)
             {
               goto next_file;
             }
+          else if (install->old_major < 3 &&
+                   strcmp (basename, "sessionrc") == 0)
+            {
+              /* We need to update size and positions because of scale
+               * factor support.
+               */
+              update_pattern  = SESSIONRC_UPDATE_PATTERN;
+              update_callback = user_update_sessionrc;
+            }
           else if (strcmp (basename, "menurc") == 0)
             {
               switch (install->old_minor)
@@ -770,6 +1024,8 @@ user_install_migrate_files (GimpUserInstall *install)
                 default:
                   update_pattern  = MENURC_OVER20_UPDATE_PATTERN;
                   update_callback = user_update_menurc_over20;
+                  /* menurc becomes shortcutsrc in 3.0. */
+                  new_dest        = "shortcutsrc";
                   break;
                 }
             }
@@ -783,15 +1039,25 @@ user_install_migrate_files (GimpUserInstall *install)
               update_pattern  = GIMPRC_UPDATE_PATTERN;
               update_callback = user_update_gimprc;
             }
+          else if (strcmp (basename, "contextrc") == 0      ||
+                   strcmp (basename, "devicerc") == 0)
+            {
+              update_pattern  = CONTEXTRC_UPDATE_PATTERN;
+              update_callback = user_update_contextrc_over20;
+            }
 
           g_snprintf (dest, sizeof (dest), "%s%c%s",
-                      gimp_directory (), G_DIR_SEPARATOR, basename);
+                      gimp_directory (), G_DIR_SEPARATOR,
+                      new_dest ? new_dest : basename);
 
           user_install_file_copy (install, source, dest,
                                   update_pattern, update_callback);
         }
       else if (g_file_test (source, G_FILE_TEST_IS_DIR))
         {
+          const gchar        *update_pattern = NULL;
+          GRegexEvalCallback  update_callback = NULL;
+
           /*  skip these directories for all old versions  */
           if (strcmp (basename, "tmp") == 0          ||
               strcmp (basename, "tool-options") == 0 ||
@@ -799,8 +1065,26 @@ user_install_migrate_files (GimpUserInstall *install)
             {
               goto next_file;
             }
+          else if (install->old_major < 3 &&
+                   (strcmp (basename, "plug-ins") == 0 ||
+                    strcmp (basename, "scripts") == 0))
+            {
+              /* Major API update. */
+              goto next_file;
+            }
 
-          user_install_dir_copy (install, source, gimp_directory ());
+          if (strcmp (basename, "gimpressionist") == 0)
+            {
+              update_pattern  = GIMPRESSIONIST_UPDATE_PATTERN;
+              update_callback = user_update_gimpressionist;
+            }
+          else if (strcmp (basename, "tool-presets") == 0)
+            {
+              update_pattern  = TOOL_PRESETS_UPDATE_PATTERN;
+              update_callback = user_update_tool_presets;
+            }
+          user_install_dir_copy (install, 0, source, gimp_directory (),
+                                 update_pattern, update_callback);
         }
 
     next_file:
