@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -20,20 +20,20 @@
 #include <string.h>
 
 #include <cairo.h>
-#include <glib-object.h>
+#include <gegl.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "libgimpcolor/gimpcolor.h"
 #include "libgimpmath/gimpmath.h"
 
 #include "core-types.h"
 
-#include "base/temp-buf.h"
-
 #include "gimpcontext.h"
 #include "gimpgradient.h"
 #include "gimpgradient-load.h"
 #include "gimpgradient-save.h"
 #include "gimptagged.h"
+#include "gimptempbuf.h"
 
 
 #define EPSILON 1e-10
@@ -57,19 +57,27 @@ static gboolean      gimp_gradient_get_popup_size    (GimpViewable        *viewa
                                                       gboolean             dot_for_dot,
                                                       gint                *popup_width,
                                                       gint                *popup_height);
-static TempBuf     * gimp_gradient_get_new_preview   (GimpViewable        *viewable,
+static GimpTempBuf * gimp_gradient_get_new_preview   (GimpViewable        *viewable,
                                                       GimpContext         *context,
                                                       gint                 width,
                                                       gint                 height);
+
 static const gchar * gimp_gradient_get_extension     (GimpData            *data);
-static GimpData    * gimp_gradient_duplicate         (GimpData            *data);
+static void          gimp_gradient_copy              (GimpData            *data,
+                                                      GimpData            *src_data);
+static gint          gimp_gradient_compare           (GimpData            *data1,
+                                                      GimpData            *data2);
 
 static gchar       * gimp_gradient_get_checksum      (GimpTagged          *tagged);
 
-static GimpGradientSegment *
+static inline GimpGradientSegment *
               gimp_gradient_get_segment_at_internal  (GimpGradient        *gradient,
                                                       GimpGradientSegment *seg,
                                                       gdouble              pos);
+static void          gimp_gradient_get_flat_color    (GimpContext         *context,
+                                                      const GimpRGB       *color,
+                                                      GimpGradientColor    color_type,
+                                                      GimpRGB             *flat_color);
 
 
 static inline gdouble  gimp_gradient_calc_linear_factor            (gdouble  middle,
@@ -82,6 +90,8 @@ static inline gdouble  gimp_gradient_calc_sphere_increasing_factor (gdouble  mid
                                                                     gdouble  pos);
 static inline gdouble  gimp_gradient_calc_sphere_decreasing_factor (gdouble  middle,
                                                                     gdouble  pos);
+static inline gdouble  gimp_gradient_calc_step_factor              (gdouble  middle,
+                                                                    gdouble  pos);
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpGradient, gimp_gradient, GIMP_TYPE_DATA,
@@ -90,6 +100,11 @@ G_DEFINE_TYPE_WITH_CODE (GimpGradient, gimp_gradient, GIMP_TYPE_DATA,
 
 #define parent_class gimp_gradient_parent_class
 
+static const Babl *fish_srgb_to_linear_rgb = NULL;
+static const Babl *fish_linear_rgb_to_srgb = NULL;
+static const Babl *fish_srgb_to_cie_lab    = NULL;
+static const Babl *fish_cie_lab_to_srgb    = NULL;
+
 
 static void
 gimp_gradient_class_init (GimpGradientClass *klass)
@@ -99,18 +114,28 @@ gimp_gradient_class_init (GimpGradientClass *klass)
   GimpViewableClass *viewable_class    = GIMP_VIEWABLE_CLASS (klass);
   GimpDataClass     *data_class        = GIMP_DATA_CLASS (klass);
 
-  object_class->finalize           = gimp_gradient_finalize;
+  object_class->finalize            = gimp_gradient_finalize;
 
-  gimp_object_class->get_memsize   = gimp_gradient_get_memsize;
+  gimp_object_class->get_memsize    = gimp_gradient_get_memsize;
 
-  viewable_class->default_stock_id = "gimp-gradient";
-  viewable_class->get_preview_size = gimp_gradient_get_preview_size;
-  viewable_class->get_popup_size   = gimp_gradient_get_popup_size;
-  viewable_class->get_new_preview  = gimp_gradient_get_new_preview;
+  viewable_class->default_icon_name = "gimp-tool-gradient";
+  viewable_class->get_preview_size  = gimp_gradient_get_preview_size;
+  viewable_class->get_popup_size    = gimp_gradient_get_popup_size;
+  viewable_class->get_new_preview   = gimp_gradient_get_new_preview;
 
-  data_class->save                 = gimp_gradient_save;
-  data_class->get_extension        = gimp_gradient_get_extension;
-  data_class->duplicate            = gimp_gradient_duplicate;
+  data_class->save                  = gimp_gradient_save;
+  data_class->get_extension         = gimp_gradient_get_extension;
+  data_class->copy                  = gimp_gradient_copy;
+  data_class->compare               = gimp_gradient_compare;
+
+  fish_srgb_to_linear_rgb = babl_fish (babl_format ("R'G'B' double"),
+                                       babl_format ("RGB double"));
+  fish_linear_rgb_to_srgb = babl_fish (babl_format ("RGB double"),
+                                       babl_format ("R'G'B' double"));
+  fish_srgb_to_cie_lab    = babl_fish (babl_format ("R'G'B' double"),
+                                       babl_format ("CIE Lab double"));
+  fish_cie_lab_to_srgb    = babl_fish (babl_format ("CIE Lab double"),
+                                       babl_format ("R'G'B' double"));
 }
 
 static void
@@ -185,7 +210,7 @@ gimp_gradient_get_popup_size (GimpViewable *viewable,
   return FALSE;
 }
 
-static TempBuf *
+static GimpTempBuf *
 gimp_gradient_get_new_preview (GimpViewable *viewable,
                                GimpContext  *context,
                                gint          width,
@@ -193,7 +218,7 @@ gimp_gradient_get_new_preview (GimpViewable *viewable,
 {
   GimpGradient        *gradient = GIMP_GRADIENT (viewable);
   GimpGradientSegment *seg      = NULL;
-  TempBuf             *temp_buf;
+  GimpTempBuf         *temp_buf;
   guchar              *buf;
   guchar              *p;
   guchar              *row;
@@ -210,7 +235,9 @@ gimp_gradient_get_new_preview (GimpViewable *viewable,
   for (x = 0; x < width; x++)
     {
       seg = gimp_gradient_get_color_at (gradient, context, seg, cur_x,
-                                        FALSE, &color);
+                                        FALSE,
+                                        GIMP_GRADIENT_BLEND_RGB_PERCEPTUAL,
+                                        &color);
 
       *p++ = ROUND (color.r * 255.0);
       *p++ = ROUND (color.g * 255.0);
@@ -220,9 +247,9 @@ gimp_gradient_get_new_preview (GimpViewable *viewable,
       cur_x += dx;
     }
 
-  temp_buf = temp_buf_new (width, height, 4, 0, 0, NULL);
+  temp_buf = gimp_temp_buf_new (width, height, babl_format ("R'G'B'A u8"));
 
-  buf = temp_buf_get_data (temp_buf);
+  buf = gimp_temp_buf_get_data (temp_buf);
 
   for (y = 0; y < height; y++)
     memcpy (buf + (width * y * 4), row, width * 4);
@@ -232,16 +259,22 @@ gimp_gradient_get_new_preview (GimpViewable *viewable,
   return temp_buf;
 }
 
-static GimpData *
-gimp_gradient_duplicate (GimpData *data)
+static void
+gimp_gradient_copy (GimpData *data,
+                    GimpData *src_data)
 {
-  GimpGradient        *gradient;
+  GimpGradient        *gradient     = GIMP_GRADIENT (data);
+  GimpGradient        *src_gradient = GIMP_GRADIENT (src_data);
   GimpGradientSegment *head, *prev, *cur, *orig;
 
-  gradient = g_object_new (GIMP_TYPE_GRADIENT, NULL);
+  if (gradient->segments)
+    {
+      gimp_gradient_segments_free (gradient->segments);
+      gradient->segments = NULL;
+    }
 
   prev = NULL;
-  orig = GIMP_GRADIENT (data)->segments;
+  orig = src_gradient->segments;
   head = NULL;
 
   while (orig)
@@ -264,7 +297,38 @@ gimp_gradient_duplicate (GimpData *data)
 
   gradient->segments = head;
 
-  return GIMP_DATA (gradient);
+  gimp_data_dirty (GIMP_DATA (gradient));
+}
+
+static gint
+gimp_gradient_compare (GimpData *data1,
+                       GimpData *data2)
+{
+  gboolean is_custom1;
+  gboolean is_custom2;
+
+  /* check whether data1 and data2 are the custom gradient, which is the only
+   * writable internal gradient.
+   */
+  is_custom1 = gimp_data_is_internal (data1) && gimp_data_is_writable (data1);
+  is_custom2 = gimp_data_is_internal (data2) && gimp_data_is_writable (data2);
+
+  /* order the custom gradient before all the other gradients; use the default
+   * ordering for the rest.
+   */
+  if (is_custom1)
+    {
+      if (is_custom2)
+        return 0;
+      else
+        return -1;
+    }
+  else if (is_custom2)
+    {
+      return +1;
+    }
+  else
+    return GIMP_DATA_CLASS (parent_class)->compare (data1, data2);
 }
 
 static gchar *
@@ -347,13 +411,11 @@ gimp_gradient_get_standard (GimpContext *context)
 
   if (! standard_gradient)
     {
-      standard_gradient = gimp_gradient_new (context, "Standard");
+      g_set_weak_pointer (&standard_gradient,
+                          gimp_gradient_new (context, "Standard"));
 
       gimp_data_clean (standard_gradient);
       gimp_data_make_internal (standard_gradient, "gimp-gradient-standard");
-
-      g_object_add_weak_pointer (G_OBJECT (standard_gradient),
-                                 (gpointer *) &standard_gradient);
     }
 
   return standard_gradient;
@@ -367,25 +429,27 @@ gimp_gradient_get_extension (GimpData *data)
 
 /**
  * gimp_gradient_get_color_at:
- * @gradient: a gradient
- * @context:  a context
- * @seg:      a segment to seed the search with (or %NULL)
- * @pos:      position in the gradient (between 0.0 and 1.0)
- * @reverse:  when %TRUE, use the reversed gradient
- * @color:    returns the color
+ * @gradient:          a gradient
+ * @context:           a context
+ * @seg: (nullable):   a segment to seed the search with (or %NULL)
+ * @pos:               position in the gradient (between 0.0 and 1.0)
+ * @reverse:           when %TRUE, use the reversed gradient
+ * @blend_color_space: color space to use for blending RGB segments
+ * @color:             returns the color
  *
  * If you are iterating over an gradient, you should pass the the
  * return value from the last call for @seg.
  *
- * Return value: the gradient segment the color is from
+ * Returns: the gradient segment the color is from
  **/
 GimpGradientSegment *
-gimp_gradient_get_color_at (GimpGradient        *gradient,
-                            GimpContext         *context,
-                            GimpGradientSegment *seg,
-                            gdouble              pos,
-                            gboolean             reverse,
-                            GimpRGB             *color)
+gimp_gradient_get_color_at (GimpGradient                *gradient,
+                            GimpContext                 *context,
+                            GimpGradientSegment         *seg,
+                            gdouble                      pos,
+                            gboolean                     reverse,
+                            GimpGradientBlendColorSpace  blend_color_space,
+                            GimpRGB                     *color)
 {
   gdouble  factor = 0.0;
   gdouble  seg_len;
@@ -394,8 +458,8 @@ gimp_gradient_get_color_at (GimpGradient        *gradient,
   GimpRGB  right_color;
   GimpRGB  rgb;
 
-  g_return_val_if_fail (GIMP_IS_GRADIENT (gradient), NULL);
-  g_return_val_if_fail (GIMP_IS_CONTEXT (context), NULL);
+  /* type-check disabled to improve speed */
+  /* g_return_val_if_fail (GIMP_IS_GRADIENT (gradient), NULL); */
   g_return_val_if_fail (color != NULL, NULL);
 
   pos = CLAMP (pos, 0.0, 1.0);
@@ -440,6 +504,10 @@ gimp_gradient_get_color_at (GimpGradient        *gradient,
       factor = gimp_gradient_calc_sphere_decreasing_factor (middle, pos);
       break;
 
+    case GIMP_GRADIENT_SEGMENT_STEP:
+      factor = gimp_gradient_calc_step_factor (middle, pos);
+      break;
+
     default:
       g_warning ("%s: Unknown gradient type %d.", G_STRFUNC, seg->type);
       break;
@@ -447,59 +515,61 @@ gimp_gradient_get_color_at (GimpGradient        *gradient,
 
   /* Get left/right colors */
 
-  switch (seg->left_color_type)
+  if (context)
     {
-    case GIMP_GRADIENT_COLOR_FIXED:
-      left_color = seg->left_color;
-      break;
+      gimp_gradient_segment_get_left_flat_color (gradient,
+                                                 context, seg, &left_color);
 
-    case GIMP_GRADIENT_COLOR_FOREGROUND:
-    case GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT:
-      gimp_context_get_foreground (context, &left_color);
-
-      if (seg->left_color_type == GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT)
-        gimp_rgb_set_alpha (&left_color, 0.0);
-      break;
-
-    case GIMP_GRADIENT_COLOR_BACKGROUND:
-    case GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT:
-      gimp_context_get_background (context, &left_color);
-
-      if (seg->left_color_type == GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT)
-        gimp_rgb_set_alpha (&left_color, 0.0);
-      break;
+      gimp_gradient_segment_get_right_flat_color (gradient,
+                                                  context, seg, &right_color);
     }
-
-  switch (seg->right_color_type)
+  else
     {
-    case GIMP_GRADIENT_COLOR_FIXED:
+      left_color  = seg->left_color;
       right_color = seg->right_color;
-      break;
-
-    case GIMP_GRADIENT_COLOR_FOREGROUND:
-    case GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT:
-      gimp_context_get_foreground (context, &right_color);
-
-      if (seg->right_color_type == GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT)
-        gimp_rgb_set_alpha (&right_color, 0.0);
-      break;
-
-    case GIMP_GRADIENT_COLOR_BACKGROUND:
-    case GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT:
-      gimp_context_get_background (context, &right_color);
-
-      if (seg->right_color_type == GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT)
-        gimp_rgb_set_alpha (&right_color, 0.0);
-      break;
     }
 
   /* Calculate color components */
 
   if (seg->color == GIMP_GRADIENT_SEGMENT_RGB)
     {
+      switch (blend_color_space)
+        {
+        case GIMP_GRADIENT_BLEND_CIE_LAB:
+          babl_process (fish_srgb_to_cie_lab,
+                        &left_color, &left_color, 1);
+          babl_process (fish_srgb_to_cie_lab,
+                        &right_color, &right_color, 1);
+          break;
+
+        case GIMP_GRADIENT_BLEND_RGB_LINEAR:
+          babl_process (fish_srgb_to_linear_rgb,
+                        &left_color, &left_color, 1);
+          babl_process (fish_srgb_to_linear_rgb,
+                        &right_color, &right_color, 1);
+
+        case GIMP_GRADIENT_BLEND_RGB_PERCEPTUAL:
+          break;
+        }
+
       rgb.r = left_color.r + (right_color.r - left_color.r) * factor;
       rgb.g = left_color.g + (right_color.g - left_color.g) * factor;
       rgb.b = left_color.b + (right_color.b - left_color.b) * factor;
+
+      switch (blend_color_space)
+        {
+        case GIMP_GRADIENT_BLEND_CIE_LAB:
+          babl_process (fish_cie_lab_to_srgb,
+                        &rgb, &rgb, 1);
+          break;
+
+        case GIMP_GRADIENT_BLEND_RGB_LINEAR:
+          babl_process (fish_linear_rgb_to_srgb,
+                        &rgb, &rgb, 1);
+
+        case GIMP_GRADIENT_BLEND_RGB_PERCEPTUAL:
+          break;
+        }
     }
   else
     {
@@ -584,6 +654,74 @@ gimp_gradient_has_fg_bg_segments (GimpGradient *gradient)
   return FALSE;
 }
 
+void
+gimp_gradient_split_at (GimpGradient                 *gradient,
+                        GimpContext                  *context,
+                        GimpGradientSegment          *seg,
+                        gdouble                       pos,
+                        GimpGradientBlendColorSpace   blend_color_space,
+                        GimpGradientSegment         **newl,
+                        GimpGradientSegment         **newr)
+{
+  GimpRGB              color;
+  GimpGradientSegment *newseg;
+
+  g_return_if_fail (GIMP_IS_GRADIENT (gradient));
+  g_return_if_fail (GIMP_IS_CONTEXT (context));
+
+  gimp_data_freeze (GIMP_DATA (gradient));
+
+  pos = CLAMP (pos, 0.0, 1.0);
+  seg = gimp_gradient_get_segment_at_internal (gradient, seg, pos);
+
+  /* Get color at pos */
+  gimp_gradient_get_color_at (gradient, context, seg, pos,
+                              FALSE, blend_color_space, &color);
+
+  /* Create a new segment and insert it in the list */
+
+  newseg = gimp_gradient_segment_new ();
+
+  newseg->prev = seg;
+  newseg->next = seg->next;
+
+  seg->next = newseg;
+
+  if (newseg->next)
+    newseg->next->prev = newseg;
+
+  /* Set coordinates of new segment */
+
+  newseg->left   = pos;
+  newseg->right  = seg->right;
+  newseg->middle = (newseg->left + newseg->right) / 2.0;
+
+  /* Set coordinates of original segment */
+
+  seg->right  = newseg->left;
+  seg->middle = (seg->left + seg->right) / 2.0;
+
+  /* Set colors of both segments */
+
+  newseg->right_color_type = seg->right_color_type;
+  newseg->right_color      = seg->right_color;
+
+  seg->right_color_type = newseg->left_color_type = GIMP_GRADIENT_COLOR_FIXED;
+  seg->right_color      = newseg->left_color      = color;
+
+  /* Set parameters of new segment */
+
+  newseg->type  = seg->type;
+  newseg->color = seg->color;
+
+  /* Done */
+
+  if (newl) *newl = seg;
+  if (newr) *newr = newseg;
+
+  gimp_data_thaw (GIMP_DATA (gradient));
+}
+
 GimpGradient *
 gimp_gradient_flatten (GimpGradient *gradient,
                        GimpContext  *context)
@@ -598,51 +736,15 @@ gimp_gradient_flatten (GimpGradient *gradient,
 
   for (seg = flat->segments; seg; seg = seg->next)
     {
-      switch (seg->left_color_type)
-        {
-        case GIMP_GRADIENT_COLOR_FIXED:
-          break;
-
-        case GIMP_GRADIENT_COLOR_FOREGROUND:
-        case GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT:
-          gimp_context_get_foreground (context, &seg->left_color);
-
-          if (seg->left_color_type == GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT)
-            gimp_rgb_set_alpha (&seg->left_color, 0.0);
-          break;
-
-        case GIMP_GRADIENT_COLOR_BACKGROUND:
-        case GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT:
-          gimp_context_get_background (context, &seg->left_color);
-
-          if (seg->left_color_type == GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT)
-            gimp_rgb_set_alpha (&seg->left_color, 0.0);
-          break;
-        }
+      gimp_gradient_segment_get_left_flat_color (gradient,
+                                                 context, seg,
+                                                 &seg->left_color);
 
       seg->left_color_type = GIMP_GRADIENT_COLOR_FIXED;
 
-      switch (seg->right_color_type)
-        {
-        case GIMP_GRADIENT_COLOR_FIXED:
-          break;
-
-        case GIMP_GRADIENT_COLOR_FOREGROUND:
-        case GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT:
-          gimp_context_get_foreground (context, &seg->right_color);
-
-          if (seg->right_color_type == GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT)
-            gimp_rgb_set_alpha (&seg->right_color, 0.0);
-          break;
-
-        case GIMP_GRADIENT_COLOR_BACKGROUND:
-        case GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT:
-          gimp_context_get_background (context, &seg->right_color);
-
-          if (seg->right_color_type == GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT)
-            gimp_rgb_set_alpha (&seg->right_color, 0.0);
-          break;
-        }
+      gimp_gradient_segment_get_right_flat_color (gradient,
+                                                  context, seg,
+                                                  &seg->right_color);
 
       seg->right_color_type = GIMP_GRADIENT_COLOR_FIXED;
     }
@@ -658,7 +760,7 @@ gimp_gradient_segment_new (void)
 {
   GimpGradientSegment *seg;
 
-  seg = g_slice_new (GimpGradientSegment);
+  seg = g_slice_new0 (GimpGradientSegment);
 
   seg->left   = 0.0;
   seg->middle = 0.5;
@@ -743,78 +845,31 @@ gimp_gradient_segment_get_nth (GimpGradientSegment *seg,
 }
 
 void
-gimp_gradient_segment_split_midpoint (GimpGradient         *gradient,
-                                      GimpContext          *context,
-                                      GimpGradientSegment  *lseg,
-                                      GimpGradientSegment **newl,
-                                      GimpGradientSegment **newr)
+gimp_gradient_segment_split_midpoint (GimpGradient                 *gradient,
+                                      GimpContext                  *context,
+                                      GimpGradientSegment          *lseg,
+                                      GimpGradientBlendColorSpace   blend_color_space,
+                                      GimpGradientSegment         **newl,
+                                      GimpGradientSegment         **newr)
 {
-  GimpRGB              color;
-  GimpGradientSegment *newseg;
-
   g_return_if_fail (GIMP_IS_GRADIENT (gradient));
   g_return_if_fail (GIMP_IS_CONTEXT (context));
   g_return_if_fail (lseg != NULL);
   g_return_if_fail (newl != NULL);
   g_return_if_fail (newr != NULL);
 
-  gimp_data_freeze (GIMP_DATA (gradient));
-
-  /* Get color at original segment's midpoint */
-  gimp_gradient_get_color_at (gradient, context, lseg, lseg->middle,
-                              FALSE, &color);
-
-  /* Create a new segment and insert it in the list */
-
-  newseg = gimp_gradient_segment_new ();
-
-  newseg->prev = lseg;
-  newseg->next = lseg->next;
-
-  lseg->next = newseg;
-
-  if (newseg->next)
-    newseg->next->prev = newseg;
-
-  /* Set coordinates of new segment */
-
-  newseg->left   = lseg->middle;
-  newseg->right  = lseg->right;
-  newseg->middle = (newseg->left + newseg->right) / 2.0;
-
-  /* Set coordinates of original segment */
-
-  lseg->right  = newseg->left;
-  lseg->middle = (lseg->left + lseg->right) / 2.0;
-
-  /* Set colors of both segments */
-
-  newseg->right_color_type = lseg->right_color_type;
-  newseg->right_color      = lseg->right_color;
-
-  lseg->right_color_type = newseg->left_color_type = GIMP_GRADIENT_COLOR_FIXED;
-  lseg->right_color      = newseg->left_color      = color;
-
-  /* Set parameters of new segment */
-
-  newseg->type  = lseg->type;
-  newseg->color = lseg->color;
-
-  /* Done */
-
-  *newl = lseg;
-  *newr = newseg;
-
-  gimp_data_thaw (GIMP_DATA (gradient));
+  gimp_gradient_split_at (gradient, context, lseg, lseg->middle,
+                          blend_color_space, newl, newr);
 }
 
 void
-gimp_gradient_segment_split_uniform (GimpGradient         *gradient,
-                                     GimpContext          *context,
-                                     GimpGradientSegment  *lseg,
-                                     gint                  parts,
-                                     GimpGradientSegment **newl,
-                                     GimpGradientSegment **newr)
+gimp_gradient_segment_split_uniform (GimpGradient                 *gradient,
+                                     GimpContext                  *context,
+                                     GimpGradientSegment          *lseg,
+                                     gint                          parts,
+                                     GimpGradientBlendColorSpace   blend_color_space,
+                                     GimpGradientSegment         **newl,
+                                     GimpGradientSegment         **newr)
 {
   GimpGradientSegment *seg, *prev, *tmp;
   gdouble              seg_len;
@@ -849,9 +904,11 @@ gimp_gradient_segment_split_uniform (GimpGradient         *gradient,
       seg->right_color_type = GIMP_GRADIENT_COLOR_FIXED;
 
       gimp_gradient_get_color_at (gradient, context, lseg,
-                                  seg->left,  FALSE, &seg->left_color);
+                                  seg->left,  FALSE, blend_color_space,
+                                  &seg->left_color);
       gimp_gradient_get_color_at (gradient, context, lseg,
-                                  seg->right, FALSE, &seg->right_color);
+                                  seg->right, FALSE, blend_color_space,
+                                  &seg->right_color);
 
       seg->type  = lseg->type;
       seg->color = lseg->color;
@@ -1009,6 +1066,36 @@ gimp_gradient_segment_set_right_color_type (GimpGradient        *gradient,
   gimp_data_thaw (GIMP_DATA (gradient));
 }
 
+void
+gimp_gradient_segment_get_left_flat_color (GimpGradient        *gradient,
+                                           GimpContext         *context,
+                                           GimpGradientSegment *seg,
+                                           GimpRGB             *color)
+{
+  g_return_if_fail (GIMP_IS_GRADIENT (gradient));
+  g_return_if_fail (seg != NULL);
+  g_return_if_fail (color != NULL);
+
+  gimp_gradient_get_flat_color (context,
+                                &seg->left_color, seg->left_color_type,
+                                color);
+}
+
+void
+gimp_gradient_segment_get_right_flat_color (GimpGradient        *gradient,
+                                            GimpContext         *context,
+                                            GimpGradientSegment *seg,
+                                            GimpRGB             *color)
+{
+  g_return_if_fail (GIMP_IS_GRADIENT (gradient));
+  g_return_if_fail (seg != NULL);
+  g_return_if_fail (color != NULL);
+
+  gimp_gradient_get_flat_color (context,
+                                &seg->right_color, seg->right_color_type,
+                                color);
+}
+
 gdouble
 gimp_gradient_segment_get_left_pos (GimpGradient        *gradient,
                                     GimpGradientSegment *seg)
@@ -1137,6 +1224,25 @@ gimp_gradient_segment_get_coloring_type (GimpGradient        *gradient,
   return seg->color;
 }
 
+gint
+gimp_gradient_segment_range_get_n_segments (GimpGradient        *gradient,
+                                            GimpGradientSegment *range_l,
+                                            GimpGradientSegment *range_r)
+{
+  gint n_segments = 0;
+
+  g_return_val_if_fail (GIMP_IS_GRADIENT (gradient), 0);
+  g_return_val_if_fail (range_l != NULL, 0);
+
+  for (; range_l != range_r; range_l = range_l->next)
+    n_segments++;
+
+  if (range_r != NULL)
+    n_segments++;
+
+  return n_segments;
+}
+
 void
 gimp_gradient_segment_range_compress (GimpGradient        *gradient,
                                       GimpGradientSegment *range_l,
@@ -1145,7 +1251,6 @@ gimp_gradient_segment_range_compress (GimpGradient        *gradient,
                                       gdouble              new_r)
 {
   gdouble              orig_l, orig_r;
-  gdouble              scale;
   GimpGradientSegment *seg, *aseg;
 
   g_return_if_fail (GIMP_IS_GRADIENT (gradient));
@@ -1159,24 +1264,54 @@ gimp_gradient_segment_range_compress (GimpGradient        *gradient,
   orig_l = range_l->left;
   orig_r = range_r->right;
 
-  scale = (new_r - new_l) / (orig_r - orig_l);
-
-  seg = range_l;
-
-  do
+  if (orig_r - orig_l > EPSILON)
     {
-      if (seg->prev)
-        seg->left   = new_l + (seg->left - orig_l) * scale;
-      seg->middle = new_l + (seg->middle - orig_l) * scale;
-      if (seg->next)
-        seg->right  = new_l + (seg->right - orig_l) * scale;
+      gdouble scale;
 
-      /* Next */
+      scale = (new_r - new_l) / (orig_r - orig_l);
 
-      aseg = seg;
-      seg  = seg->next;
+      seg = range_l;
+
+      do
+        {
+          if (seg->prev)
+            seg->left  = new_l + (seg->left - orig_l) * scale;
+          seg->middle  = new_l + (seg->middle - orig_l) * scale;
+          if (seg->next)
+            seg->right = new_l + (seg->right - orig_l) * scale;
+
+          /* Next */
+
+          aseg = seg;
+          seg  = seg->next;
+        }
+      while (aseg != range_r);
     }
-  while (aseg != range_r);
+  else
+    {
+      gint n;
+      gint i;
+
+      n = gimp_gradient_segment_range_get_n_segments (gradient,
+                                                      range_l, range_r);
+
+      for (i = 0, seg = range_l; i < n; i++, seg = seg->next)
+        {
+          if (seg->prev)
+            seg->left  = new_l + (new_r - new_l) * (i + 0.0) / n;
+          seg->middle  = new_l + (new_r - new_l) * (i + 0.5) / n;;
+          if (seg->next)
+            seg->right = new_l + (new_r - new_l) * (i + 1.0) / n;
+        }
+    }
+
+  /* Make sure that the left and right endpoints of the range are *exactly*
+   * equal to new_l and new_r; the above computations can introduce
+   * numerical inaccuracies.
+   */
+
+  range_l->left  = new_l;
+  range_l->right = new_r;
 
   gimp_data_thaw (GIMP_DATA (gradient));
 }
@@ -1562,12 +1697,13 @@ gimp_gradient_segment_range_replicate (GimpGradient         *gradient,
 }
 
 void
-gimp_gradient_segment_range_split_midpoint (GimpGradient         *gradient,
-                                            GimpContext          *context,
-                                            GimpGradientSegment  *start_seg,
-                                            GimpGradientSegment  *end_seg,
-                                            GimpGradientSegment **final_start_seg,
-                                            GimpGradientSegment **final_end_seg)
+gimp_gradient_segment_range_split_midpoint (GimpGradient                 *gradient,
+                                            GimpContext                  *context,
+                                            GimpGradientSegment          *start_seg,
+                                            GimpGradientSegment          *end_seg,
+                                            GimpGradientBlendColorSpace   blend_color_space,
+                                            GimpGradientSegment         **final_start_seg,
+                                            GimpGradientSegment         **final_end_seg)
 {
   GimpGradientSegment *seg, *lseg, *rseg;
 
@@ -1584,7 +1720,8 @@ gimp_gradient_segment_range_split_midpoint (GimpGradient         *gradient,
   do
     {
       gimp_gradient_segment_split_midpoint (gradient, context,
-                                            seg, &lseg, &rseg);
+                                            seg, blend_color_space,
+                                            &lseg, &rseg);
       seg = rseg->next;
     }
   while (lseg != end_seg);
@@ -1599,13 +1736,14 @@ gimp_gradient_segment_range_split_midpoint (GimpGradient         *gradient,
 }
 
 void
-gimp_gradient_segment_range_split_uniform (GimpGradient         *gradient,
-                                           GimpContext          *context,
-                                           GimpGradientSegment  *start_seg,
-                                           GimpGradientSegment  *end_seg,
-                                           gint                  parts,
-                                           GimpGradientSegment **final_start_seg,
-                                           GimpGradientSegment **final_end_seg)
+gimp_gradient_segment_range_split_uniform (GimpGradient                 *gradient,
+                                           GimpContext                  *context,
+                                           GimpGradientSegment          *start_seg,
+                                           GimpGradientSegment          *end_seg,
+                                           gint                          parts,
+                                           GimpGradientBlendColorSpace   blend_color_space,
+                                           GimpGradientSegment         **final_start_seg,
+                                           GimpGradientSegment         **final_end_seg)
 {
   GimpGradientSegment *seg, *aseg, *lseg, *rseg, *lsel;
 
@@ -1632,7 +1770,7 @@ gimp_gradient_segment_range_split_uniform (GimpGradient         *gradient,
       aseg = seg;
 
       gimp_gradient_segment_split_uniform (gradient, context, seg,
-                                           parts,
+                                           parts, blend_color_space,
                                            &lseg, &rseg);
 
       if (seg == start_seg)
@@ -1754,6 +1892,70 @@ gimp_gradient_segment_range_delete (GimpGradient         *gradient,
     *final_end_seg = end_seg;
 }
 
+void
+gimp_gradient_segment_range_merge (GimpGradient         *gradient,
+                                   GimpGradientSegment  *start_seg,
+                                   GimpGradientSegment  *end_seg,
+                                   GimpGradientSegment **final_start_seg,
+                                   GimpGradientSegment **final_end_seg)
+{
+  GimpGradientSegment *seg;
+
+  g_return_if_fail (GIMP_IS_GRADIENT (gradient));
+
+  if (! end_seg)
+    end_seg = gimp_gradient_segment_get_last (start_seg);
+
+  gimp_data_freeze (GIMP_DATA (gradient));
+
+  /* Copy the end segment's right position and color to the start segment */
+
+  start_seg->right            = end_seg->right;
+  start_seg->right_color_type = end_seg->right_color_type;
+  start_seg->right_color      = end_seg->right_color;
+
+  /* Center the start segment's midpoint */
+
+  start_seg->middle = (start_seg->left + start_seg->right) / 2.0;
+
+  /* Remove range segments past the start segment from the segment list */
+
+  start_seg->next = end_seg->next;
+
+  if (start_seg->next)
+    start_seg->next->prev = start_seg;
+
+  /* Merge the range's blend function and coloring type, and free the rest of
+   * the segments.
+   */
+
+  seg = end_seg;
+
+  while (seg != start_seg)
+    {
+      GimpGradientSegment *prev = seg->prev;
+
+      /* If the blend function and/or coloring type aren't uniform, reset them. */
+
+      if (seg->type != start_seg->type)
+        start_seg->type = GIMP_GRADIENT_SEGMENT_LINEAR;
+
+      if (seg->color != start_seg->color)
+        start_seg->color = GIMP_GRADIENT_SEGMENT_RGB;
+
+      gimp_gradient_segment_free (seg);
+
+      seg = prev;
+    }
+
+  if (final_start_seg)
+    *final_start_seg = start_seg;
+  if (final_end_seg)
+    *final_end_seg = start_seg;
+
+  gimp_data_thaw (GIMP_DATA (gradient));
+}
+
 void
 gimp_gradient_segment_range_recenter_handles (GimpGradient        *gradient,
                                               GimpGradientSegment *start_seg,
@@ -1854,7 +2056,7 @@ gimp_gradient_segment_range_move (GimpGradient        *gradient,
 
   gimp_data_freeze (GIMP_DATA (gradient));
 
-  if (! range_l)
+  if (! range_r)
     range_r = gimp_gradient_segment_get_last (range_l);
 
   /* First or last segments in gradient? */
@@ -1965,7 +2167,7 @@ gimp_gradient_segment_range_move (GimpGradient        *gradient,
 
 /*  private functions  */
 
-static GimpGradientSegment *
+static inline GimpGradientSegment *
 gimp_gradient_get_segment_at_internal (GimpGradient        *gradient,
                                        GimpGradientSegment *seg,
                                        gdouble              pos)
@@ -1976,29 +2178,49 @@ gimp_gradient_get_segment_at_internal (GimpGradient        *gradient,
   if (! seg)
     seg = gradient->segments;
 
-  while (seg)
+  if (pos >= seg->left)
     {
-      if (pos >= seg->left)
-        {
-          if (pos <= seg->right)
-            {
-              return seg;
-            }
-          else
-            {
-              seg = seg->next;
-            }
-        }
-      else
-        {
-          seg = seg->prev;
-        }
+      while (seg->next && pos >= seg->right)
+        seg = seg->next;
+    }
+  else
+    {
+      do
+        seg = seg->prev;
+      while (pos < seg->left);
     }
 
-  /* Oops: we should have found a segment, but we didn't */
-  g_warning ("%s: no matching segment for position %0.15f", G_STRFUNC, pos);
+  return seg;
+}
 
-  return NULL;
+static void
+gimp_gradient_get_flat_color (GimpContext       *context,
+                              const GimpRGB     *color,
+                              GimpGradientColor  color_type,
+                              GimpRGB           *flat_color)
+{
+  switch (color_type)
+    {
+    case GIMP_GRADIENT_COLOR_FIXED:
+      *flat_color = *color;
+      break;
+
+    case GIMP_GRADIENT_COLOR_FOREGROUND:
+    case GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT:
+      gimp_context_get_foreground (context, flat_color);
+
+      if (color_type == GIMP_GRADIENT_COLOR_FOREGROUND_TRANSPARENT)
+        gimp_rgb_set_alpha (flat_color, 0.0);
+      break;
+
+    case GIMP_GRADIENT_COLOR_BACKGROUND:
+    case GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT:
+      gimp_context_get_background (context, flat_color);
+
+      if (color_type == GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT)
+        gimp_rgb_set_alpha (flat_color, 0.0);
+      break;
+    }
 }
 
 static inline gdouble
@@ -2029,9 +2251,11 @@ gimp_gradient_calc_curved_factor (gdouble middle,
                                   gdouble pos)
 {
   if (middle < EPSILON)
-    middle = EPSILON;
+    return 1.0;
+  else if (1.0 - middle < EPSILON)
+    return 0.0;
 
-  return pow (pos, log (0.5) / log (middle));
+  return exp (-G_LN2 * log (pos) / log (middle));
 }
 
 static inline gdouble
@@ -2062,3 +2286,10 @@ gimp_gradient_calc_sphere_decreasing_factor (gdouble middle,
   /* Works for convex decreasing and concave increasing */
   return 1.0 - sqrt(1.0 - pos * pos);
 }
+
+static inline gdouble
+gimp_gradient_calc_step_factor (gdouble middle,
+                                gdouble pos)
+{
+  return pos >= middle;
+}
