@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -40,8 +40,9 @@
 
 #define RESPONSE_RESET 1
 
+typedef struct _ScaleDialog ScaleDialog;
 
-typedef struct
+struct _ScaleDialog
 {
   GimpViewable          *viewable;
   GimpUnit               unit;
@@ -50,16 +51,20 @@ typedef struct
   GtkWidget             *combo;
   GimpScaleCallback      callback;
   gpointer               user_data;
-} ScaleDialog;
+};
 
 
+/*  local function prototypes  */
+
+static void   scale_dialog_free     (ScaleDialog *private);
 static void   scale_dialog_response (GtkWidget   *dialog,
                                      gint         response_id,
                                      ScaleDialog *private);
 static void   scale_dialog_reset    (ScaleDialog *private);
-static void   scale_dialog_free     (ScaleDialog *private);
 
 
+/*  public function  */
+
 GtkWidget *
 scale_dialog_new (GimpViewable          *viewable,
                   GimpContext           *context,
@@ -113,30 +118,8 @@ scale_dialog_new (GimpViewable          *viewable,
       g_return_val_if_reached (NULL);
     }
 
-  dialog = gimp_viewable_dialog_new (viewable, context,
-                                     title, role, GIMP_STOCK_SCALE, title,
-                                     parent,
-                                     help_func, help_id,
-
-                                     GIMP_STOCK_RESET, RESPONSE_RESET,
-                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                     GIMP_STOCK_SCALE, GTK_RESPONSE_OK,
-
-                                     NULL);
-
-  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                           RESPONSE_RESET,
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
-                                           -1);
-
-  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-
   private = g_slice_new0 (ScaleDialog);
 
-  g_object_weak_ref (G_OBJECT (dialog),
-                     (GWeakNotify) scale_dialog_free, private);
-
   private->viewable      = viewable;
   private->interpolation = interpolation;
   private->unit          = unit;
@@ -145,6 +128,32 @@ scale_dialog_new (GimpViewable          *viewable,
 
   gimp_image_get_resolution (image, &xres, &yres);
 
+  dialog = gimp_viewable_dialog_new (g_list_prepend (NULL, viewable), context,
+                                     title, role, GIMP_ICON_OBJECT_SCALE, title,
+                                     parent,
+                                     help_func, help_id,
+
+                                     _("_Reset"),  RESPONSE_RESET,
+                                     _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                     _("_Scale"),  GTK_RESPONSE_OK,
+
+                                     NULL);
+
+  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                           RESPONSE_RESET,
+                                           GTK_RESPONSE_OK,
+                                           GTK_RESPONSE_CANCEL,
+                                           -1);
+
+  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+  g_object_weak_ref (G_OBJECT (dialog),
+                     (GWeakNotify) scale_dialog_free, private);
+
+  g_signal_connect (dialog, "response",
+                    G_CALLBACK (scale_dialog_response),
+                    private);
+
   private->box = g_object_new (GIMP_TYPE_SIZE_BOX,
                                "width",           width,
                                "height",          height,
@@ -156,10 +165,6 @@ scale_dialog_new (GimpViewable          *viewable,
                                "edit-resolution", GIMP_IS_IMAGE (viewable),
                                NULL);
 
-  g_signal_connect (dialog, "response",
-                    G_CALLBACK (scale_dialog_response),
-                    private);
-
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
@@ -199,24 +204,18 @@ scale_dialog_new (GimpViewable          *viewable,
   gimp_int_combo_box_set_active (GIMP_INT_COMBO_BOX (private->combo),
                                  private->interpolation);
 
-  if (gimp_image_base_type (image) == GIMP_INDEXED)
-    {
-      GtkWidget *box = gimp_message_box_new (GIMP_STOCK_INFO);
-
-      gimp_message_box_set_text (GIMP_MESSAGE_BOX (box),
-                                 _("Indexed color layers are always scaled "
-                                   "without interpolation. The chosen "
-                                   "interpolation type will affect channels "
-                                   "and layer masks only."));
-
-      gtk_container_set_border_width (GTK_CONTAINER (box), 0);
-      gtk_box_pack_start (GTK_BOX (vbox), box, FALSE, FALSE, 0);
-      gtk_widget_show (box);
-    }
-
   return dialog;
 }
 
+
+/*  private functions  */
+
+static void
+scale_dialog_free (ScaleDialog *private)
+{
+  g_slice_free (ScaleDialog, private);
+}
+
 static void
 scale_dialog_response (GtkWidget   *dialog,
                        gint         response_id,
@@ -310,9 +309,3 @@ scale_dialog_reset (ScaleDialog *private)
   gimp_int_combo_box_set_active (GIMP_INT_COMBO_BOX (private->combo),
                                  private->interpolation);
 }
-
-static void
-scale_dialog_free (ScaleDialog *private)
-{
-  g_slice_free (ScaleDialog, private);
-}
