@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -55,14 +55,17 @@
 enum
 {
   PROP_0,
+  PROP_GIMP,
   PROP_SAMPLE_MERGED
 };
 
 
-struct _GimpCursorViewPriv
+struct _GimpCursorViewPrivate
 {
   GimpEditor        parent_instance;
 
+  Gimp             *gimp;
+
   GtkWidget        *coord_hbox;
   GtkWidget        *selection_hbox;
   GtkWidget        *color_hbox;
@@ -83,58 +86,63 @@ struct _GimpCursorViewPriv
   GimpContext      *context;
   GimpDisplayShell *shell;
   GimpImage        *image;
+
   GimpUnit          unit;
+  guint             cursor_idle_id;
+  GimpImage        *cursor_image;
+  GimpUnit          cursor_unit;
+  gdouble           cursor_x;
+  gdouble           cursor_y;
 };
 
 
-static void   gimp_cursor_view_docked_iface_init     (GimpDockedInterface *iface);
+static void       gimp_cursor_view_docked_iface_init     (GimpDockedInterface *iface);
 
-static void   gimp_cursor_view_dispose               (GObject             *object);
-static void   gimp_cursor_view_set_property          (GObject             *object,
-                                                      guint                property_id,
-                                                      const GValue        *value,
-                                                      GParamSpec          *pspec);
-static void   gimp_cursor_view_get_property          (GObject             *object,
-                                                      guint                property_id,
-                                                      GValue              *value,
-                                                      GParamSpec          *pspec);
-
-static void   gimp_cursor_view_style_set             (GtkWidget           *widget,
-                                                      GtkStyle            *prev_style);
-
-static void   gimp_cursor_view_set_aux_info          (GimpDocked          *docked,
-                                                      GList               *aux_info);
-static GList *gimp_cursor_view_get_aux_info          (GimpDocked          *docked);
-
-static void   gimp_cursor_view_set_context           (GimpDocked          *docked,
-                                                      GimpContext         *context);
-static void   gimp_cursor_view_image_changed         (GimpCursorView      *view,
-                                                      GimpImage           *image,
-                                                      GimpContext         *context);
-static void   gimp_cursor_view_mask_changed          (GimpCursorView      *view,
-                                                      GimpImage           *image);
-static void   gimp_cursor_view_diplay_changed        (GimpCursorView      *view,
-                                                      GimpDisplay         *display,
-                                                      GimpContext         *context);
-static void   gimp_cursor_view_shell_unit_changed    (GimpCursorView      *view,
-                                                      GParamSpec          *pspec,
-                                                      GimpDisplayShell    *shell);
-static void   gimp_cursor_view_format_as_unit        (GimpUnit             unit,
-                                                      gchar               *output_buf,
-                                                      gint                 output_buf_size,
-                                                      gdouble              pixel_value,
-                                                      gdouble              image_res);
-static void   gimp_cursor_view_set_label_italic      (GtkWidget           *label,
-                                                      gboolean             italic);
-static void   gimp_cursor_view_update_selection_info (GimpCursorView      *view,
-                                                      GimpImage           *image,
-                                                      GimpUnit             unit);
+static void       gimp_cursor_view_dispose               (GObject             *object);
+static void       gimp_cursor_view_constructed           (GObject             *object);
+static void       gimp_cursor_view_set_property          (GObject             *object,
+                                                          guint                property_id,
+                                                          const GValue        *value,
+                                                          GParamSpec          *pspec);
+static void       gimp_cursor_view_get_property          (GObject             *object,
+                                                          guint                property_id,
+                                                          GValue              *value,
+                                                          GParamSpec          *pspec);
 
+static void       gimp_cursor_view_style_updated         (GtkWidget           *widget);
 
+static void       gimp_cursor_view_set_aux_info          (GimpDocked          *docked,
+                                                          GList               *aux_info);
+static GList    * gimp_cursor_view_get_aux_info          (GimpDocked          *docked);
 
+static void       gimp_cursor_view_set_context           (GimpDocked          *docked,
+                                                          GimpContext         *context);
+static void       gimp_cursor_view_image_changed         (GimpCursorView      *view,
+                                                          GimpImage           *image,
+                                                          GimpContext         *context);
+static void       gimp_cursor_view_mask_changed          (GimpCursorView      *view,
+                                                          GimpImage           *image);
+static void       gimp_cursor_view_diplay_changed        (GimpCursorView      *view,
+                                                          GimpDisplay         *display,
+                                                          GimpContext         *context);
+static void       gimp_cursor_view_shell_unit_changed    (GimpCursorView      *view,
+                                                          GParamSpec          *pspec,
+                                                          GimpDisplayShell    *shell);
+static void       gimp_cursor_view_format_as_unit        (GimpUnit             unit,
+                                                          gchar               *output_buf,
+                                                          gint                 output_buf_size,
+                                                          gdouble              pixel_value,
+                                                          gdouble              image_res);
+static void       gimp_cursor_view_set_label_italic      (GtkWidget           *label,
+                                                          gboolean             italic);
+static void       gimp_cursor_view_update_selection_info (GimpCursorView      *view,
+                                                          GimpImage           *image,
+                                                          GimpUnit             unit);
+static gboolean   gimp_cursor_view_cursor_idle           (GimpCursorView      *view);
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpCursorView, gimp_cursor_view, GIMP_TYPE_EDITOR,
+                         G_ADD_PRIVATE (GimpCursorView)
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_DOCKED,
                                                 gimp_cursor_view_docked_iface_init))
 
@@ -149,11 +157,19 @@ gimp_cursor_view_class_init (GimpCursorViewClass* klass)
   GObjectClass   *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-  object_class->dispose      = gimp_cursor_view_dispose;
-  object_class->get_property = gimp_cursor_view_get_property;
-  object_class->set_property = gimp_cursor_view_set_property;
+  object_class->constructed   = gimp_cursor_view_constructed;
+  object_class->dispose       = gimp_cursor_view_dispose;
+  object_class->get_property  = gimp_cursor_view_get_property;
+  object_class->set_property  = gimp_cursor_view_set_property;
 
-  widget_class->style_set    = gimp_cursor_view_style_set;
+  widget_class->style_updated = gimp_cursor_view_style_updated;
+
+  g_object_class_install_property (object_class, PROP_GIMP,
+                                   g_param_spec_object ("gimp",
+                                                        NULL, NULL,
+                                                        GIMP_TYPE_GIMP,
+                                                        GIMP_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT_ONLY));
 
   g_object_class_install_property (object_class, PROP_SAMPLE_MERGED,
                                    g_param_spec_boolean ("sample-merged",
@@ -161,27 +177,24 @@ gimp_cursor_view_class_init (GimpCursorViewClass* klass)
                                                          TRUE,
                                                          GIMP_PARAM_READWRITE |
                                                          G_PARAM_CONSTRUCT));
-
-  g_type_class_add_private (klass, sizeof (GimpCursorViewPriv));
 }
 
 static void
 gimp_cursor_view_init (GimpCursorView *view)
 {
   GtkWidget *frame;
-  GtkWidget *table;
+  GtkWidget *grid;
   GtkWidget *toggle;
   gint       content_spacing;
 
-  view->priv = G_TYPE_INSTANCE_GET_PRIVATE (view,
-                                            GIMP_TYPE_CURSOR_VIEW,
-                                            GimpCursorViewPriv);
+  view->priv = gimp_cursor_view_get_instance_private (view);
 
-  view->priv->sample_merged = TRUE;
-  view->priv->context       = NULL;
-  view->priv->shell         = NULL;
-  view->priv->image         = NULL;
-  view->priv->unit          = GIMP_UNIT_PIXEL;
+  view->priv->sample_merged  = TRUE;
+  view->priv->context        = NULL;
+  view->priv->shell          = NULL;
+  view->priv->image          = NULL;
+  view->priv->unit           = GIMP_UNIT_PIXEL;
+  view->priv->cursor_idle_id = 0;
 
   gtk_widget_style_get (GTK_WIDGET (view),
                         "content-spacing", &content_spacing,
@@ -211,23 +224,23 @@ gimp_cursor_view_init (GimpCursorView *view)
   gtk_box_pack_start (GTK_BOX (view->priv->coord_hbox), frame, TRUE, TRUE, 0);
   gtk_widget_show (frame);
 
-  table = gtk_table_new (2, 2, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_container_add (GTK_CONTAINER (frame), table);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
+  gtk_container_add (GTK_CONTAINER (frame), grid);
+  gtk_widget_show (grid);
 
   view->priv->pixel_x_label = gtk_label_new (_("n/a"));
   gtk_label_set_xalign (GTK_LABEL (view->priv->pixel_x_label), 1.0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-                             _("X"), 0.5, 0.5,
-                             view->priv->pixel_x_label, 1, FALSE);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 0,
+                            _("X"), 0.5, 0.5,
+                            view->priv->pixel_x_label, 1);
 
   view->priv->pixel_y_label = gtk_label_new (_("n/a"));
   gtk_label_set_xalign (GTK_LABEL (view->priv->pixel_y_label), 1.0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-                             _("Y"), 0.5, 0.5,
-                             view->priv->pixel_y_label, 1, FALSE);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 1,
+                            _("Y"), 0.5, 0.5,
+                            view->priv->pixel_y_label, 1);
 
 
   /* Units */
@@ -236,102 +249,80 @@ gimp_cursor_view_init (GimpCursorView *view)
   gtk_box_pack_start (GTK_BOX (view->priv->coord_hbox), frame, TRUE, TRUE, 0);
   gtk_widget_show (frame);
 
-  table = gtk_table_new (2, 2, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_container_add (GTK_CONTAINER (frame), table);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 4);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
+  gtk_container_add (GTK_CONTAINER (frame), grid);
+  gtk_widget_show (grid);
 
   view->priv->unit_x_label = gtk_label_new (_("n/a"));
   gtk_label_set_xalign (GTK_LABEL (view->priv->unit_x_label), 1.0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-                             _("X"), 0.5, 0.5,
-                             view->priv->unit_x_label, 1, FALSE);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 0,
+                            _("X"), 0.5, 0.5,
+                            view->priv->unit_x_label, 1);
 
   view->priv->unit_y_label = gtk_label_new (_("n/a"));
   gtk_label_set_xalign (GTK_LABEL (view->priv->unit_y_label), 1.0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-                             _("Y"), 0.5, 0.5,
-                             view->priv->unit_y_label, 1, FALSE);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 1,
+                            _("Y"), 0.5, 0.5,
+                            view->priv->unit_y_label, 1);
 
 
   /* Selection Bounding Box */
 
-  frame = gimp_frame_new (_("Selection Bounding Box"));
+  frame = gimp_frame_new (_("Selection"));
   gtk_box_pack_start (GTK_BOX (view->priv->selection_hbox), frame, TRUE, TRUE, 0);
   gtk_widget_show (frame);
 
-  table = gtk_table_new (2, 2, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_container_add (GTK_CONTAINER (frame), table);
-  gtk_widget_show (table);
+  gimp_help_set_help_data (frame, _("The selection's bounding box"), NULL);
+
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
+  gtk_container_add (GTK_CONTAINER (frame), grid);
+  gtk_widget_show (grid);
 
   view->priv->selection_x_label = gtk_label_new (_("n/a"));
   gtk_label_set_xalign (GTK_LABEL (view->priv->selection_x_label), 1.0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-                             _("X"), 0.5, 0.5,
-                             view->priv->selection_x_label, 1, FALSE);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 0,
+                            _("X"), 0.5, 0.5,
+                            view->priv->selection_x_label, 1);
 
   view->priv->selection_y_label = gtk_label_new (_("n/a"));
   gtk_label_set_xalign (GTK_LABEL (view->priv->selection_y_label), 1.0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-                             _("Y"), 0.5, 0.5,
-                             view->priv->selection_y_label, 1, FALSE);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 1,
+                            _("Y"), 0.5, 0.5,
+                            view->priv->selection_y_label, 1);
 
   frame = gimp_frame_new ("");
   gtk_box_pack_start (GTK_BOX (view->priv->selection_hbox), frame, TRUE, TRUE, 0);
   gtk_widget_show (frame);
 
-  table = gtk_table_new (2, 2, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_container_add (GTK_CONTAINER (frame), table);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 4);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
+  gtk_container_add (GTK_CONTAINER (frame), grid);
+  gtk_widget_show (grid);
 
   view->priv->selection_width_label = gtk_label_new (_("n/a"));
   gtk_label_set_xalign (GTK_LABEL (view->priv->selection_width_label), 1.0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-                             /* Width */
-                             _("W"), 0.5, 0.5,
-                             view->priv->selection_width_label, 1, FALSE);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 0,
+                            /* Width */
+                            _("W"), 0.5, 0.5,
+                            view->priv->selection_width_label, 1);
 
   view->priv->selection_height_label = gtk_label_new (_("n/a"));
   gtk_label_set_xalign (GTK_LABEL (view->priv->selection_height_label), 1.0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-                             /* Height */
-                             _("H"), 0.5, 0.5,
-                             view->priv->selection_height_label, 1, FALSE);
-
-
-  /* color information */
-
-  view->priv->color_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL,
-                                        content_spacing);
-  gtk_box_set_homogeneous (GTK_BOX (view->priv->color_hbox), TRUE);
-  gtk_box_pack_start (GTK_BOX (view), view->priv->color_hbox, FALSE, FALSE, 0);
-  gtk_widget_show (view->priv->color_hbox);
-
-  view->priv->color_frame_1 = gimp_color_frame_new ();
-  gimp_color_frame_set_mode (GIMP_COLOR_FRAME (view->priv->color_frame_1),
-                             GIMP_COLOR_FRAME_MODE_PIXEL);
-  gtk_box_pack_start (GTK_BOX (view->priv->color_hbox), view->priv->color_frame_1,
-                      TRUE, TRUE, 0);
-  gtk_widget_show (view->priv->color_frame_1);
-
-  view->priv->color_frame_2 = gimp_color_frame_new ();
-  gimp_color_frame_set_mode (GIMP_COLOR_FRAME (view->priv->color_frame_2),
-                             GIMP_COLOR_FRAME_MODE_RGB_PERCENT);
-  gtk_box_pack_start (GTK_BOX (view->priv->color_hbox), view->priv->color_frame_2,
-                      TRUE, TRUE, 0);
-  gtk_widget_show (view->priv->color_frame_2);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 1,
+                            /* Height */
+                            _("H"), 0.5, 0.5,
+                            view->priv->selection_height_label, 1);
 
   /* sample merged toggle */
 
   toggle = gimp_prop_check_button_new (G_OBJECT (view), "sample-merged",
                                        _("_Sample Merged"));
   gtk_box_pack_start (GTK_BOX (view), toggle, FALSE, FALSE, 0);
-  gtk_widget_show (toggle);
 }
 
 static void
@@ -347,6 +338,41 @@ gimp_cursor_view_docked_iface_init (GimpDockedInterface *iface)
   iface->set_context  = gimp_cursor_view_set_context;
 }
 
+static void
+gimp_cursor_view_constructed (GObject             *object)
+{
+  GimpCursorView *view = GIMP_CURSOR_VIEW (object);
+  gint            content_spacing;
+
+  gtk_widget_style_get (GTK_WIDGET (view),
+                        "content-spacing", &content_spacing,
+                        NULL);
+
+  /* color information */
+  view->priv->color_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL,
+                                        content_spacing);
+  gtk_box_set_homogeneous (GTK_BOX (view->priv->color_hbox), TRUE);
+  gtk_box_pack_start (GTK_BOX (view), view->priv->color_hbox, FALSE, FALSE, 0);
+  gtk_widget_show (view->priv->color_hbox);
+
+  view->priv->color_frame_1 = gimp_color_frame_new (view->priv->gimp);
+  gimp_color_frame_set_mode (GIMP_COLOR_FRAME (view->priv->color_frame_1),
+                             GIMP_COLOR_PICK_MODE_PIXEL);
+  gimp_color_frame_set_ellipsize (GIMP_COLOR_FRAME (view->priv->color_frame_1),
+                                  PANGO_ELLIPSIZE_END);
+  gtk_box_pack_start (GTK_BOX (view->priv->color_hbox), view->priv->color_frame_1,
+                      TRUE, TRUE, 0);
+  gtk_widget_show (view->priv->color_frame_1);
+
+  view->priv->color_frame_2 = gimp_color_frame_new (view->priv->gimp);
+  gimp_color_frame_set_mode (GIMP_COLOR_FRAME (view->priv->color_frame_2),
+                             GIMP_COLOR_PICK_MODE_RGB_PERCENT);
+  gtk_box_pack_start (GTK_BOX (view->priv->color_hbox), view->priv->color_frame_2,
+                      TRUE, TRUE, 0);
+  gtk_widget_show (view->priv->color_frame_2);
+
+}
+
 static void
 gimp_cursor_view_dispose (GObject *object)
 {
@@ -355,6 +381,17 @@ gimp_cursor_view_dispose (GObject *object)
   if (view->priv->context)
     gimp_docked_set_context (GIMP_DOCKED (view), NULL);
 
+  if (view->priv->cursor_idle_id)
+    {
+      g_source_remove (view->priv->cursor_idle_id);
+      view->priv->cursor_idle_id = 0;
+    }
+
+  view->priv->color_frame_1 = NULL;
+  view->priv->color_frame_2 = NULL;
+
+  g_clear_object (&view->priv->cursor_image);
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -368,6 +405,10 @@ gimp_cursor_view_set_property (GObject      *object,
 
   switch (property_id)
     {
+    case PROP_GIMP:
+      view->priv->gimp = g_value_get_object (value);
+      break;
+
     case PROP_SAMPLE_MERGED:
       view->priv->sample_merged = g_value_get_boolean (value);
       break;
@@ -388,6 +429,10 @@ gimp_cursor_view_get_property (GObject    *object,
 
   switch (property_id)
     {
+    case PROP_GIMP:
+      g_value_set_object (value, view->priv->gimp);
+      break;
+
     case PROP_SAMPLE_MERGED:
       g_value_set_boolean (value, view->priv->sample_merged);
       break;
@@ -425,7 +470,7 @@ gimp_cursor_view_set_aux_info (GimpDocked *docked,
           GEnumClass *enum_class;
           GEnumValue *enum_value;
 
-          enum_class = g_type_class_peek (GIMP_TYPE_COLOR_FRAME_MODE);
+          enum_class = g_type_class_peek (GIMP_TYPE_COLOR_PICK_MODE);
           enum_value = g_enum_get_value_by_nick (enum_class, aux->value);
 
           if (enum_value)
@@ -445,16 +490,16 @@ gimp_cursor_view_get_aux_info (GimpDocked *docked)
 
   aux_info = parent_docked_iface->get_aux_info (docked);
 
-  if (gimp_enum_get_value (GIMP_TYPE_COLOR_FRAME_MODE,
-                           GIMP_COLOR_FRAME (view->priv->color_frame_1)->frame_mode,
+  if (gimp_enum_get_value (GIMP_TYPE_COLOR_PICK_MODE,
+                           GIMP_COLOR_FRAME (view->priv->color_frame_1)->pick_mode,
                            NULL, &nick, NULL, NULL))
     {
       aux = gimp_session_info_aux_new (AUX_INFO_FRAME_1_MODE, nick);
       aux_info = g_list_append (aux_info, aux);
     }
 
-  if (gimp_enum_get_value (GIMP_TYPE_COLOR_FRAME_MODE,
-                           GIMP_COLOR_FRAME (view->priv->color_frame_2)->frame_mode,
+  if (gimp_enum_get_value (GIMP_TYPE_COLOR_PICK_MODE,
+                           GIMP_COLOR_FRAME (view->priv->color_frame_2)->pick_mode,
                            NULL, &nick, NULL, NULL))
     {
       aux = gimp_session_info_aux_new (AUX_INFO_FRAME_2_MODE, nick);
@@ -502,13 +547,12 @@ gimp_cursor_view_set_label_italic (GtkWidget *label,
 }
 
 static void
-gimp_cursor_view_style_set (GtkWidget *widget,
-                            GtkStyle  *prev_style)
+gimp_cursor_view_style_updated (GtkWidget *widget)
 {
   GimpCursorView *view = GIMP_CURSOR_VIEW (widget);
   gint            content_spacing;
 
-  GTK_WIDGET_CLASS (parent_class)->style_set (widget, prev_style);
+  GTK_WIDGET_CLASS (parent_class)->style_updated (widget);
 
   gtk_widget_style_get (GTK_WIDGET (view),
                         "content-spacing", &content_spacing,
@@ -700,15 +744,112 @@ gimp_cursor_view_update_selection_info (GimpCursorView *view,
     }
 }
 
+static gboolean
+gimp_cursor_view_cursor_idle (GimpCursorView *view)
+{
+
+  if (view->priv->cursor_image)
+    {
+      GimpImage  *image = view->priv->cursor_image;
+      GimpUnit    unit  = view->priv->cursor_unit;
+      gdouble     x     = view->priv->cursor_x;
+      gdouble     y     = view->priv->cursor_y;
+      gboolean    in_image;
+      gchar       buf[32];
+      const Babl *sample_format;
+      gdouble     pixel[4];
+      GimpRGB     color;
+      gdouble     xres;
+      gdouble     yres;
+      gint        int_x;
+      gint        int_y;
+
+      if (unit == GIMP_UNIT_PIXEL)
+        unit = gimp_image_get_unit (image);
+
+      gimp_image_get_resolution (image, &xres, &yres);
+
+      in_image = (x >= 0.0 && x < gimp_image_get_width  (image) &&
+                  y >= 0.0 && y < gimp_image_get_height (image));
+
+      g_snprintf (buf, sizeof (buf), "%d", (gint) floor (x));
+      gtk_label_set_text (GTK_LABEL (view->priv->pixel_x_label), buf);
+      gimp_cursor_view_set_label_italic (view->priv->pixel_x_label, ! in_image);
+
+      g_snprintf (buf, sizeof (buf), "%d", (gint) floor (y));
+      gtk_label_set_text (GTK_LABEL (view->priv->pixel_y_label), buf);
+      gimp_cursor_view_set_label_italic (view->priv->pixel_y_label, ! in_image);
+
+      gimp_cursor_view_format_as_unit (unit, buf, sizeof (buf), x, xres);
+      gtk_label_set_text (GTK_LABEL (view->priv->unit_x_label), buf);
+      gimp_cursor_view_set_label_italic (view->priv->unit_x_label, ! in_image);
+
+      gimp_cursor_view_format_as_unit (unit, buf, sizeof (buf), y, yres);
+      gtk_label_set_text (GTK_LABEL (view->priv->unit_y_label), buf);
+      gimp_cursor_view_set_label_italic (view->priv->unit_y_label, ! in_image);
+
+      int_x = (gint) floor (x);
+      int_y = (gint) floor (y);
+
+      if (gimp_image_pick_color (image, NULL,
+                                 int_x, int_y,
+                                 view->priv->shell->show_all,
+                                 view->priv->sample_merged,
+                                 FALSE, 0.0,
+                                 &sample_format, pixel, &color))
+        {
+          gimp_color_frame_set_color (GIMP_COLOR_FRAME (view->priv->color_frame_1),
+                                      FALSE, sample_format, pixel, &color,
+                                      int_x, int_y);
+          gimp_color_frame_set_color (GIMP_COLOR_FRAME (view->priv->color_frame_2),
+                                      FALSE, sample_format, pixel, &color,
+                                      int_x, int_y);
+        }
+      else
+        {
+          gimp_color_frame_set_invalid (GIMP_COLOR_FRAME (view->priv->color_frame_1));
+          gimp_color_frame_set_invalid (GIMP_COLOR_FRAME (view->priv->color_frame_2));
+        }
+
+      /* Show the selection info from the image under the cursor if any */
+      gimp_cursor_view_update_selection_info (view,
+                                              image,
+                                              view->priv->cursor_unit);
+
+      g_clear_object (&view->priv->cursor_image);
+    }
+  else
+    {
+      gtk_label_set_text (GTK_LABEL (view->priv->pixel_x_label), _("n/a"));
+      gtk_label_set_text (GTK_LABEL (view->priv->pixel_y_label), _("n/a"));
+      gtk_label_set_text (GTK_LABEL (view->priv->unit_x_label),  _("n/a"));
+      gtk_label_set_text (GTK_LABEL (view->priv->unit_y_label),  _("n/a"));
+
+      gimp_color_frame_set_invalid (GIMP_COLOR_FRAME (view->priv->color_frame_1));
+      gimp_color_frame_set_invalid (GIMP_COLOR_FRAME (view->priv->color_frame_2));
+
+      /* Start showing selection info from the active image again */
+      gimp_cursor_view_update_selection_info (view,
+                                              view->priv->image,
+                                              view->priv->unit);
+    }
+
+  view->priv->cursor_idle_id = 0;
+
+  return G_SOURCE_REMOVE;
+}
+
 
 /*  public functions  */
 
 GtkWidget *
-gimp_cursor_view_new (GimpMenuFactory *menu_factory)
+gimp_cursor_view_new (Gimp            *gimp,
+                      GimpMenuFactory *menu_factory)
 {
   g_return_val_if_fail (GIMP_IS_MENU_FACTORY (menu_factory), NULL);
 
   return g_object_new (GIMP_TYPE_CURSOR_VIEW,
+                       "gimp",            gimp,
                        "menu-factory",    menu_factory,
                        "menu-identifier", "<CursorInfo>",
                        "ui-path",         "/cursor-info-popup",
@@ -746,68 +887,21 @@ gimp_cursor_view_update_cursor (GimpCursorView   *view,
                                 gdouble           x,
                                 gdouble           y)
 {
-  GimpUnit      unit = shell_unit;
-  gboolean      in_image;
-  gchar         buf[32];
-  const Babl   *sample_format;
-  guchar        pixel[32];
-  GimpRGB       color;
-  gdouble       xres;
-  gdouble       yres;
-  gint          int_x;
-  gint          int_y;
-
   g_return_if_fail (GIMP_IS_CURSOR_VIEW (view));
   g_return_if_fail (GIMP_IS_IMAGE (image));
 
-  if (unit == GIMP_UNIT_PIXEL)
-    unit = gimp_image_get_unit (image);
+  g_clear_object (&view->priv->cursor_image);
 
-  gimp_image_get_resolution (image, &xres, &yres);
+  view->priv->cursor_image = g_object_ref (image);
+  view->priv->cursor_unit  = shell_unit;
+  view->priv->cursor_x     = x;
+  view->priv->cursor_y     = y;
 
-  in_image = (x >= 0.0 && x < gimp_image_get_width  (image) &&
-              y >= 0.0 && y < gimp_image_get_height (image));
-
-  g_snprintf (buf, sizeof (buf), "%d", (gint) floor (x));
-  gtk_label_set_text (GTK_LABEL (view->priv->pixel_x_label), buf);
-  gimp_cursor_view_set_label_italic (view->priv->pixel_x_label, ! in_image);
-
-  g_snprintf (buf, sizeof (buf), "%d", (gint) floor (y));
-  gtk_label_set_text (GTK_LABEL (view->priv->pixel_y_label), buf);
-  gimp_cursor_view_set_label_italic (view->priv->pixel_y_label, ! in_image);
-
-  gimp_cursor_view_format_as_unit (unit, buf, sizeof (buf), x, xres);
-  gtk_label_set_text (GTK_LABEL (view->priv->unit_x_label), buf);
-  gimp_cursor_view_set_label_italic (view->priv->unit_x_label, ! in_image);
-
-  gimp_cursor_view_format_as_unit (unit, buf, sizeof (buf), y, yres);
-  gtk_label_set_text (GTK_LABEL (view->priv->unit_y_label), buf);
-  gimp_cursor_view_set_label_italic (view->priv->unit_y_label, ! in_image);
-
-  int_x = (gint) floor (x);
-  int_y = (gint) floor (y);
-
-  if (gimp_image_pick_color (image, NULL,
-                             int_x, int_y,
-                             view->priv->sample_merged,
-                             FALSE, 0.0,
-                             &sample_format, pixel, &color))
+  if (view->priv->cursor_idle_id == 0)
     {
-      gimp_color_frame_set_color (GIMP_COLOR_FRAME (view->priv->color_frame_1),
-                                  FALSE, sample_format, pixel, &color,
-                                  int_x, int_y);
-      gimp_color_frame_set_color (GIMP_COLOR_FRAME (view->priv->color_frame_2),
-                                  FALSE, sample_format, pixel, &color,
-                                  int_x, int_y);
+      view->priv->cursor_idle_id =
+        g_idle_add ((GSourceFunc) gimp_cursor_view_cursor_idle, view);
     }
-  else
-    {
-      gimp_color_frame_set_invalid (GIMP_COLOR_FRAME (view->priv->color_frame_1));
-      gimp_color_frame_set_invalid (GIMP_COLOR_FRAME (view->priv->color_frame_2));
-    }
-
-  /* Show the selection info from the image under the cursor if any */
-  gimp_cursor_view_update_selection_info (view, image, shell_unit);
 }
 
 void
@@ -815,16 +909,11 @@ gimp_cursor_view_clear_cursor (GimpCursorView *view)
 {
   g_return_if_fail (GIMP_IS_CURSOR_VIEW (view));
 
-  gtk_label_set_text (GTK_LABEL (view->priv->pixel_x_label), _("n/a"));
-  gtk_label_set_text (GTK_LABEL (view->priv->pixel_y_label), _("n/a"));
-  gtk_label_set_text (GTK_LABEL (view->priv->unit_x_label),  _("n/a"));
-  gtk_label_set_text (GTK_LABEL (view->priv->unit_y_label),  _("n/a"));
+  g_clear_object (&view->priv->cursor_image);
 
-  gimp_color_frame_set_invalid (GIMP_COLOR_FRAME (view->priv->color_frame_1));
-  gimp_color_frame_set_invalid (GIMP_COLOR_FRAME (view->priv->color_frame_2));
-
-  /* Start showing selection info from the active image again */
-  gimp_cursor_view_update_selection_info (view,
-                                          view->priv->image,
-                                          view->priv->unit);
+  if (view->priv->cursor_idle_id == 0)
+    {
+      view->priv->cursor_idle_id =
+        g_idle_add ((GSourceFunc) gimp_cursor_view_cursor_idle, view);
+    }
 }
