@@ -50,6 +50,7 @@
 #include "widgets/gimpdockcolumns.h"
 #include "widgets/gimpdockcontainer.h"
 #include "widgets/gimphelp-ids.h"
+#include "widgets/gimpmenubar.h"
 #include "widgets/gimpmenufactory.h"
 #include "widgets/gimpsessioninfo.h"
 #include "widgets/gimpsessioninfo-aux.h"
@@ -61,6 +62,8 @@
 #include "widgets/gimpviewrenderer.h"
 #include "widgets/gimpwidgets-utils.h"
 
+#include "menus/menus.h"
+
 #include "gimpdisplay.h"
 #include "gimpdisplay-foreach.h"
 #include "gimpdisplayshell.h"
@@ -109,12 +112,13 @@ typedef struct _GimpImageWindowPrivate GimpImageWindowPrivate;
 struct _GimpImageWindowPrivate
 {
   Gimp              *gimp;
-  GimpUIManager     *menubar_manager;
   GimpDialogFactory *dialog_factory;
 
   GList             *shells;
   GimpDisplayShell  *active_shell;
 
+  GimpMenuModel     *menubar_model;
+
   GtkWidget         *main_vbox;
   GtkWidget         *menubar;
   GtkWidget         *hbox;
@@ -345,20 +349,16 @@ gimp_image_window_constructed (GObject *object)
 {
   GimpImageWindow        *window  = GIMP_IMAGE_WINDOW (object);
   GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
-  GimpMenuFactory        *menu_factory;
+  GimpUIManager          *menubar_manager;
   GimpGuiConfig          *config;
+  gboolean                use_gtk_menubar = TRUE;
+  gboolean                use_app_menu    = TRUE;
 
   G_OBJECT_CLASS (parent_class)->constructed (object);
 
   gimp_assert (GIMP_IS_GIMP (private->gimp));
   gimp_assert (GIMP_IS_DIALOG_FACTORY (private->dialog_factory));
 
-  menu_factory = gimp_dialog_factory_get_menu_factory (private->dialog_factory);
-
-  private->menubar_manager = gimp_menu_factory_manager_new (menu_factory,
-                                                            "<Image>",
-                                                            window);
-
   g_signal_connect_object (private->dialog_factory, "dock-window-added",
                            G_CALLBACK (gimp_image_window_update_ui_manager),
                            window, G_CONNECT_SWAPPED);
@@ -366,13 +366,11 @@ gimp_image_window_constructed (GObject *object)
                            G_CALLBACK (gimp_image_window_update_ui_manager),
                            window, G_CONNECT_SWAPPED);
 
-  gtk_window_add_accel_group (GTK_WINDOW (window),
-                              gimp_ui_manager_get_accel_group (private->menubar_manager));
-
-  g_signal_connect (private->menubar_manager, "show-tooltip",
+  menubar_manager = menus_get_image_manager_singleton (private->gimp);
+  g_signal_connect (menubar_manager, "show-tooltip",
                     G_CALLBACK (gimp_image_window_show_tooltip),
                     window);
-  g_signal_connect (private->menubar_manager, "hide-tooltip",
+  g_signal_connect (menubar_manager, "hide-tooltip",
                     G_CALLBACK (gimp_image_window_hide_tooltip),
                     window);
 
@@ -384,14 +382,28 @@ gimp_image_window_constructed (GObject *object)
   gtk_widget_show (private->main_vbox);
 
   /* Create the menubar */
+  private->menubar_model = gimp_ui_manager_get_model (menubar_manager,
+                                                      "/image-menubar");
+
 #ifndef GDK_WINDOWING_QUARTZ
-  private->menubar = gimp_ui_manager_get_widget (private->menubar_manager,
-                                                 "/image-menubar");
+  /* macOS has its native menubar system, which is implemented by
+   * gtk_application_set_menubar(). Unfortunately the GTK menubar feature
+   * doesn't support tooltips (though for macOS, having "native" application
+   * menus was deemed worth the feature loss).
+   * As a special-case, we allow testing the GTK menubar through setting the
+   * environment variable GIMP_GTK_MENUBAR on non-macOS platforms.
+   */
+  use_gtk_menubar = (g_getenv ("GIMP_GTK_MENUBAR") != NULL);
 #endif /* !GDK_WINDOWING_QUARTZ */
-  if (private->menubar)
+
+  if (use_gtk_menubar)
     {
-      gtk_box_pack_start (GTK_BOX (private->main_vbox),
-                          private->menubar, FALSE, FALSE, 0);
+      gtk_application_set_menubar (GTK_APPLICATION (private->gimp->app),
+                                   G_MENU_MODEL (private->menubar_model));
+    }
+  else
+    {
+      private->menubar = gimp_menu_bar_new (private->menubar_model, menubar_manager);
 
       /*  make sure we can activate accels even if the menubar is invisible
        *  (see https://bugzilla.gnome.org/show_bug.cgi?id=137151)
@@ -410,6 +422,54 @@ gimp_image_window_constructed (GObject *object)
       g_signal_connect (private->menubar, "key-press-event",
                         G_CALLBACK (gimp_image_window_shell_events),
                         window);
+
+      if (config->custom_title_bar)
+        {
+          GtkWidget *headerbar;
+
+          headerbar = gtk_header_bar_new ();
+          gtk_window_set_titlebar (GTK_WINDOW (window), headerbar);
+
+          /* This is important to turn off space reservation for the subtitle
+           * (resulting in too high header bar).
+           */
+          gtk_header_bar_set_has_subtitle (GTK_HEADER_BAR (headerbar), FALSE);
+
+          gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (headerbar), TRUE);
+          gtk_header_bar_pack_start (GTK_HEADER_BAR (headerbar), private->menubar);
+          gtk_widget_show (headerbar);
+
+          /* XXX There are competing propositions on how the title should be
+           * aligned. GTK is trying to center it (relatively to the window) as
+           * much as it can. But some people seem to thing it should be centered
+           * relatively to the remaining empty space after the menu.
+           * I am personally unsure so leaving GTK's defaults for now.
+           */
+        }
+      else
+        {
+          gtk_box_pack_start (GTK_BOX (private->main_vbox),
+                              private->menubar, FALSE, FALSE, 0);
+        }
+    }
+
+#ifndef GDK_WINDOWING_QUARTZ
+  /* Docs says that macOS always returns FALSE but we actually want to create
+   * our custom macOS menu.
+   */
+  use_app_menu = gtk_application_prefers_app_menu (GTK_APPLICATION (private->gimp->app));
+#endif /* !GDK_WINDOWING_QUARTZ */
+
+  if (use_app_menu)
+    {
+      GimpUIManager *app_menu_manager;
+      GimpMenuModel *app_menu_model;
+
+      app_menu_manager = gimp_menu_factory_get_manager (menus_get_global_menu_factory (private->gimp),
+                                                        "<AppMenu>", private->gimp);
+      app_menu_model   = gimp_ui_manager_get_model (app_menu_manager, "/app-menu");
+      gtk_application_set_app_menu (GTK_APPLICATION (private->gimp->app),
+                                    G_MENU_MODEL (app_menu_model));
     }
 
   /* Create the hbox that contains docks and images */
@@ -429,7 +489,7 @@ gimp_image_window_constructed (GObject *object)
   private->left_docks =
     gimp_dock_columns_new (gimp_get_user_context (private->gimp),
                            private->dialog_factory,
-                           private->menubar_manager);
+                           menubar_manager);
   gtk_paned_pack1 (GTK_PANED (private->left_hpane), private->left_docks,
                    FALSE, FALSE);
   gtk_widget_set_visible (private->left_docks, config->single_window_mode);
@@ -465,7 +525,7 @@ gimp_image_window_constructed (GObject *object)
   private->right_docks =
     gimp_dock_columns_new (gimp_get_user_context (private->gimp),
                            private->dialog_factory,
-                           private->menubar_manager);
+                           menubar_manager);
   gtk_paned_pack2 (GTK_PANED (private->right_hpane), private->right_docks,
                    FALSE, FALSE);
   gtk_widget_set_visible (private->right_docks, config->single_window_mode);
@@ -502,14 +562,14 @@ gimp_image_window_dispose (GObject *object)
       private->dialog_factory = NULL;
     }
 
-  g_clear_object (&private->menubar_manager);
-
   if (private->update_ui_manager_idle_id)
     {
       g_source_remove (private->update_ui_manager_idle_id);
       private->update_ui_manager_idle_id = 0;
     }
 
+  g_clear_object (&private->menubar_model);
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -591,7 +651,7 @@ gimp_image_window_delete_event (GtkWidget   *widget,
   GimpGuiConfig          *config  = GIMP_GUI_CONFIG (private->gimp->config);
 
   if (config->single_window_mode)
-    gimp_ui_manager_activate_action (gimp_image_window_get_ui_manager (window),
+    gimp_ui_manager_activate_action (menus_get_image_manager_singleton (private->gimp),
                                      "file", "file-quit");
   else if (shell)
     gimp_display_shell_close (shell, FALSE);
@@ -657,10 +717,42 @@ static void
 gimp_image_window_update_csd_on_fullscreen (GtkWidget *child,
                                             gpointer   user_data)
 {
-  gboolean fullscreen = GPOINTER_TO_INT (user_data);
+  GimpImageWindow        *window  = GIMP_IMAGE_WINDOW (user_data);
+  GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
+  gboolean                fullscreen;
+
+  fullscreen = gimp_image_window_get_fullscreen (window);
 
   if (GTK_IS_HEADER_BAR (child))
-    gtk_widget_set_visible (child, !fullscreen);
+    {
+      gtk_widget_set_visible (child, !fullscreen);
+
+      /* When we manage the menubar ourselves (i.e. not when using the GTK
+       * menubar, e.g. on macOS), if the menubar was set to stay visible in
+       * fullscreen mode, we need to move it out of the header bar.
+       * Note that here we only move the menubar from one parent to another.
+       * This is not where we handle whether we make it visible or not.
+       */
+      if (private->menubar)
+        {
+          GtkWidget *parent = gtk_widget_get_parent (private->menubar);
+
+          g_object_ref (private->menubar);
+          gtk_container_remove (GTK_CONTAINER (parent), private->menubar);
+          if (fullscreen)
+            {
+              gtk_box_pack_start (GTK_BOX (private->main_vbox),
+                                  private->menubar, FALSE, FALSE, 0);
+              gtk_box_reorder_child (GTK_BOX (private->main_vbox),
+                                     private->menubar, 0);
+            }
+          else
+            {
+              gtk_header_bar_pack_start (GTK_HEADER_BAR (child), private->menubar);
+            }
+          g_object_unref (private->menubar);
+        }
+    }
 }
 
 static gboolean
@@ -698,7 +790,7 @@ gimp_image_window_window_state_event (GtkWidget           *widget,
        * an internal child, so we use this workaround instead */
       gtk_container_forall (GTK_CONTAINER (window),
                             gimp_image_window_update_csd_on_fullscreen,
-                            GINT_TO_POINTER (fullscreen));
+                            window);
     }
 
   if (event->changed_mask & GDK_WINDOW_STATE_ICONIFIED)
@@ -848,7 +940,7 @@ gimp_image_window_dock_container_get_ui_manager (GimpDockContainer *dock_contain
 {
   GimpImageWindow *window = GIMP_IMAGE_WINDOW (dock_container);
 
-  return gimp_image_window_get_ui_manager (window);
+  return menus_get_image_manager_singleton (GIMP_IMAGE_WINDOW_GET_PRIVATE (window)->gimp);
 }
 
 void
@@ -990,9 +1082,9 @@ gimp_image_window_set_aux_info (GimpSessionManaged *session_managed,
   gboolean                wait_with_right_docks = FALSE;
   gboolean                maximized             = FALSE;
 #ifdef G_OS_WIN32
-  STARTUPINFO             StartupInfo;
+  STARTUPINFOW            StartupInfo;
 
-  GetStartupInfo (&StartupInfo);
+  GetStartupInfoW (&StartupInfo);
 #endif
 
   g_return_if_fail (GIMP_IS_IMAGE_WINDOW (session_managed));
@@ -1167,18 +1259,6 @@ gimp_image_window_destroy (GimpImageWindow *window)
   gtk_widget_destroy (GTK_WIDGET (window));
 }
 
-GimpUIManager *
-gimp_image_window_get_ui_manager (GimpImageWindow *window)
-{
-  GimpImageWindowPrivate *private;
-
-  g_return_val_if_fail (GIMP_IS_IMAGE_WINDOW (window), FALSE);
-
-  private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
-
-  return private->menubar_manager;
-}
-
 GimpDockColumns  *
 gimp_image_window_get_left_docks (GimpImageWindow  *window)
 {
@@ -1308,6 +1388,18 @@ gimp_image_window_get_active_shell (GimpImageWindow *window)
   return private->active_shell;
 }
 
+GimpMenuModel *
+gimp_image_window_get_menubar_model (GimpImageWindow *window)
+{
+  GimpImageWindowPrivate *private;
+
+  g_return_val_if_fail (GIMP_IS_IMAGE_WINDOW (window), NULL);
+
+  private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
+
+  return private->menubar_model;
+}
+
 void
 gimp_image_window_set_fullscreen (GimpImageWindow *window,
                                   gboolean         fullscreen)
@@ -1878,7 +1970,7 @@ gimp_image_window_update_ui_manager_idle (GimpImageWindow *window)
 
   gimp_assert (private->active_shell != NULL);
 
-  gimp_ui_manager_update (private->menubar_manager,
+  gimp_ui_manager_update (menus_get_image_manager_singleton (private->active_shell->display->gimp),
                           private->active_shell->display);
 
   private->update_ui_manager_idle_id = 0;
@@ -2082,8 +2174,7 @@ gimp_image_window_disconnect_from_active_shell (GimpImageWindow *window)
                                         gimp_image_window_shell_title_notify,
                                         window);
 
-  if (private->menubar_manager)
-    gimp_image_window_hide_tooltip (private->menubar_manager, window);
+  gimp_image_window_hide_tooltip (menus_get_image_manager_singleton (private->gimp), window);
 
   if (private->update_ui_manager_idle_id)
     {
