@@ -108,6 +108,8 @@
 #include "icon-themes.h"
 #include "themes.h"
 
+#include "gimp-intl.h"
+
 
 /*  local function prototypes  */
 
@@ -186,6 +188,10 @@ static GimpMetadataRotationPolicy
                                                   GimpContext         *context,
                                                   gboolean            *dont_ask);
 
+static void           gui_inhibit                (Gimp                *gimp);
+static void           gui_image_disconnect       (GimpImage           *image,
+                                                  Gimp                *gimp);
+
 
 /*  public functions  */
 
@@ -314,7 +320,7 @@ gui_get_display_name (Gimp     *gimp,
 
   *monitor_number = get_monitor_number (GDK_MONITOR (*monitor));
 
-  return gdk_screen_make_display_name (gdk_display_get_default_screen (gdk_display));
+  return g_strdup (gdk_display_get_name (gdk_display));
 }
 
 static guint32
@@ -424,6 +430,25 @@ gui_display_create (Gimp      *gimp,
       gimp_context_set_display (context, display);
     }
 
+  if (image)
+    {
+      g_signal_handlers_disconnect_by_func (image,
+                                            G_CALLBACK (gui_inhibit),
+                                            gimp);
+      g_signal_handlers_disconnect_by_func (image,
+                                            G_CALLBACK (gui_image_disconnect),
+                                            gimp);
+      g_signal_connect_swapped (image, "dirty",
+                                G_CALLBACK (gui_inhibit),
+                                gimp);
+      g_signal_connect_swapped (image, "clean",
+                                G_CALLBACK (gui_inhibit),
+                                gimp);
+      g_signal_connect_after (image, "disconnect",
+                              G_CALLBACK (gui_image_disconnect),
+                              gimp);
+    }
+
   return display;
 }
 
@@ -658,7 +683,7 @@ gui_pdb_dialog_new (Gimp          *gimp,
                                           "select-type",    gimp_container_get_children_type (container),
                                           "initial-object", object,
                                           "callback-name",  callback_name,
-                                          "menu-factory",   global_menu_factory,
+                                          "menu-factory",   menus_get_global_menu_factory (gimp),
                                           NULL);
 
           names = gimp_properties_append_valist (dialog_type,
@@ -923,3 +948,119 @@ gui_query_rotation_policy (Gimp        *gimp,
   return metadata_rotation_import_dialog_run (image, context,
                                               NULL, dont_ask);
 }
+
+static void
+gui_inhibit (Gimp *gimp)
+{
+  static gboolean  in_test      = TRUE;
+  static gint      cookie       = 0;
+  static gint      n_images     = 0;
+  static gint64    last_failure = 0;
+  GtkApplication  *app;
+  GimpContainer   *images  = NULL;
+
+  if (in_test)
+    {
+      /* Do not call inhibit code while unit-testing the UI code. */
+      in_test = (g_getenv ("GIMP_TESTING_ABS_TOP_SRCDIR") != NULL);
+      if (in_test)
+        return;
+    }
+
+  app = GTK_APPLICATION (g_application_get_default ());
+
+  if (gimp_displays_dirty (gimp))
+    {
+      gint n_dirty_images;
+
+      images         = gimp_displays_get_dirty_images (gimp);
+      n_dirty_images = gimp_container_get_n_children (images);
+
+      if (cookie && n_images == n_dirty_images)
+        {
+          g_object_unref (images);
+          return;
+        }
+
+      n_images = n_dirty_images;
+    }
+
+  if (gtk_application_is_inhibited (app, GTK_APPLICATION_INHIBIT_LOGOUT))
+    {
+      gtk_application_uninhibit (app, cookie);
+      cookie = 0;
+    }
+
+  if (last_failure != 0 && g_get_monotonic_time () - last_failure < G_TIME_SPAN_MINUTE * 10)
+    {
+      /* Don't repeatedly try to inhibit when we are constantly failing.
+       * Especially as in some case, it may lock the thread.
+       */
+      g_clear_object (&images);
+      return;
+    }
+  last_failure = 0;
+
+  if (gimp_displays_dirty (gimp))
+    {
+      GimpImage *image;
+      GList     *list;
+      GtkWindow *window = NULL;
+      gchar     *reason;
+
+      image = (GimpImage *) gimp_container_get_first_child (images);
+      g_object_unref (images);
+
+      for (list = gimp_get_display_iter (gimp); list; list = list->next)
+        {
+          GimpDisplay *display = list->data;
+
+          if (gimp_display_get_image (display) == image)
+            {
+              GimpDisplayShell *shell;
+              GtkWidget        *toplevel;
+
+              shell    = gimp_display_get_shell (display);
+              toplevel = gtk_widget_get_toplevel (GTK_WIDGET (shell));
+
+              if (GTK_IS_WINDOW (toplevel))
+                {
+                  window = GTK_WINDOW (toplevel);
+                  break;
+                }
+            }
+        }
+
+      /* TRANSLATORS: unless your language msgstr[0] applies to 1 only (as in
+       * English), replace "one" with %d.
+       */
+      reason = g_strdup_printf (ngettext ("There is one image with unsaved changes!",
+                                          "There are %d images with unsaved changes!",
+                                          n_images),
+                                n_images);
+      cookie = gtk_application_inhibit (app, window,
+                                        GTK_APPLICATION_INHIBIT_LOGOUT,
+                                        reason);
+      g_free (reason);
+
+      if (cookie == 0)
+        last_failure = g_get_monotonic_time ();
+    }
+
+  if (cookie == 0)
+    n_images = 0;
+}
+
+static void
+gui_image_disconnect (GimpImage *image,
+                      Gimp      *gimp)
+{
+  gui_inhibit (gimp);
+
+  g_signal_handlers_disconnect_by_func (image,
+                                        G_CALLBACK (gui_inhibit),
+                                        gimp);
+  g_signal_handlers_disconnect_by_func (image,
+                                        G_CALLBACK (gui_image_disconnect),
+                                        gimp);
+}
