@@ -87,7 +87,6 @@
 
 #ifdef GDK_WINDOWING_QUARTZ
 #import <AppKit/AppKit.h>
-#include <gtkosxapplication.h>
 
 /* Forward declare since we are building against old SDKs. */
 #if !defined(MAC_OS_X_VERSION_10_12) || \
@@ -147,23 +146,12 @@ static void       gui_display_changed           (GimpContext        *context,
                                                  GimpDisplay        *display,
                                                  Gimp               *gimp);
 
-static void       gui_compare_accelerator       (gpointer            data,
-                                                 const gchar        *accel_path,
-                                                 guint               accel_key,
-                                                 GdkModifierType     accel_mods,
-                                                 gboolean            changed);
-static void       gui_check_unique_accelerator  (gpointer            data,
-                                                 const gchar        *accel_path,
-                                                 guint               accel_key,
-                                                 GdkModifierType     accel_mods,
-                                                 gboolean            changed);
-static gboolean   gui_check_action_exists       (const gchar        *accel_path);
+static void       gui_check_unique_accelerators (Gimp               *gimp);
 
 
 /*  private variables  */
 
 static Gimp             *the_gui_gimp     = NULL;
-static GimpUIManager    *image_ui_manager = NULL;
 static GimpUIConfigurer *ui_configurer    = NULL;
 static GdkMonitor       *initial_monitor  = NULL;
 
@@ -378,8 +366,8 @@ static gchar *
 gui_sanity_check (void)
 {
 #define GTK_REQUIRED_MAJOR 3
-#define GTK_REQUIRED_MINOR 22
-#define GTK_REQUIRED_MICRO 29
+#define GTK_REQUIRED_MINOR 24
+#define GTK_REQUIRED_MICRO 0
 
   const gchar *mismatch = gtk_check_version (GTK_REQUIRED_MAJOR,
                                              GTK_REQUIRED_MINOR,
@@ -517,10 +505,10 @@ gui_restore_callback (Gimp               *gimp,
     }
 
   actions_init (gimp);
-  menus_init (gimp, global_action_factory);
+  menus_init (gimp);
   gimp_render_init (gimp);
 
-  dialogs_init (gimp, global_menu_factory);
+  dialogs_init (gimp);
 
   gimp_clipboard_init (gimp);
   if (gimp_get_clipboard_image (gimp))
@@ -543,36 +531,12 @@ gui_restore_callback (Gimp               *gimp,
   gimp_tools_restore (gimp);
 }
 
-#ifdef GDK_WINDOWING_QUARTZ
-static void
-gui_add_to_app_menu (GimpUIManager     *ui_manager,
-                     GtkosxApplication *osx_app,
-                     const gchar       *action_path,
-                     gint               index)
-{
-  GtkWidget *item;
-
-  item = gimp_ui_manager_get_widget (ui_manager, action_path);
-
-  if (GTK_IS_MENU_ITEM (item))
-    gtkosx_application_insert_app_menu_item (osx_app, GTK_WIDGET (item), index);
-}
-
-static gboolean
-gui_quartz_quit_callback (GtkosxApplication *osx_app,
-                          GimpUIManager     *ui_manager)
-{
-  gimp_ui_manager_activate_action (ui_manager, "file", "file-quit");
-
-  return TRUE;
-}
-#endif
-
 static void
 gui_restore_after_callback (Gimp               *gimp,
                             GimpInitStatusFunc  status_callback)
 {
   GimpGuiConfig *gui_config = GIMP_GUI_CONFIG (gimp->config);
+  GimpUIManager *image_ui_manager;
   GimpDisplay   *display;
 
   if (gimp->be_verbose)
@@ -597,21 +561,18 @@ gui_restore_after_callback (Gimp               *gimp,
                     NULL);
     }
 
-  if (gui_config->restore_accels)
-    menus_restore (gimp);
-
   ui_configurer = g_object_new (GIMP_TYPE_UI_CONFIGURER,
                                 "gimp", gimp,
                                 NULL);
 
-  image_ui_manager = gimp_menu_factory_manager_new (global_menu_factory,
-                                                    "<Image>",
-                                                    gimp);
+  image_ui_manager = menus_get_image_manager_singleton (gimp);
   gimp_ui_manager_update (image_ui_manager, gimp);
 
+  if (gui_config->restore_accels)
+    menus_restore (gimp);
+
   /* Check that every accelerator is unique. */
-  gtk_accel_map_foreach_unfiltered (NULL,
-                                    gui_check_unique_accelerator);
+  gui_check_unique_accelerators (gimp);
 
   gimp_action_history_init (gimp);
 
@@ -649,66 +610,6 @@ gui_restore_after_callback (Gimp               *gimp,
 
       toplevel = gtk_widget_get_toplevel (GTK_WIDGET (shell));
 
-#ifdef GDK_WINDOWING_QUARTZ
-      {
-        GtkosxApplication *osx_app;
-        GtkWidget         *menu;
-        GtkWidget         *item;
-
-        [[NSUserDefaults standardUserDefaults] setObject:@"NO"
-                                                  forKey:@"NSTreatUnknownArgumentsAsOpen"];
-
-        osx_app = gtkosx_application_get ();
-
-        menu = gimp_ui_manager_get_widget (image_ui_manager,
-                                           "/image-menubar");
-        /* menu should have window parent for accelerator support */
-        gtk_widget_set_parent(menu, toplevel);
-
-        if (GTK_IS_MENU_ITEM (menu))
-          menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
-
-        /* do not activate OSX menu if tests are running */
-        if (! g_getenv ("GIMP_TESTING_ABS_TOP_SRCDIR"))
-          gtkosx_application_set_menu_bar (osx_app, GTK_MENU_SHELL (menu));
-
-        gtkosx_application_set_use_quartz_accelerators (osx_app, FALSE);
-
-        gui_add_to_app_menu (image_ui_manager, osx_app,
-                             "/image-menubar/Help/dialogs-about", 0);
-        gui_add_to_app_menu (image_ui_manager, osx_app,
-                             "/image-menubar/Help/dialogs-search-action", 1);
-
-#define PREFERENCES "/image-menubar/Edit/Preferences/"
-
-        gui_add_to_app_menu (image_ui_manager, osx_app,
-                             PREFERENCES "dialogs-preferences", 3);
-        gui_add_to_app_menu (image_ui_manager, osx_app,
-                             PREFERENCES "dialogs-input-devices", 4);
-        gui_add_to_app_menu (image_ui_manager, osx_app,
-                             PREFERENCES "dialogs-keyboard-shortcuts", 5);
-        gui_add_to_app_menu (image_ui_manager, osx_app,
-                             PREFERENCES "dialogs-module-dialog", 6);
-        gui_add_to_app_menu (image_ui_manager, osx_app,
-                             PREFERENCES "plug-in-unit-editor", 7);
-
-#undef PREFERENCES
-
-        item = gtk_separator_menu_item_new ();
-        gtkosx_application_insert_app_menu_item (osx_app, item, 8);
-
-        item = gimp_ui_manager_get_widget (image_ui_manager,
-                                           "/image-menubar/File/file-quit");
-        gtk_widget_hide (item);
-
-        g_signal_connect (osx_app, "NSApplicationBlockTermination",
-                          G_CALLBACK (gui_quartz_quit_callback),
-                          image_ui_manager);
-
-        gtkosx_application_ready (osx_app);
-      }
-#endif /* GDK_WINDOWING_QUARTZ */
-
       /*  move keyboard focus to the display  */
       gtk_window_present (GTK_WINDOW (toplevel));
     }
@@ -807,9 +708,6 @@ gui_exit_after_callback (Gimp     *gimp,
 
   gimp_action_history_exit (gimp);
 
-  g_object_unref (image_ui_manager);
-  image_ui_manager = NULL;
-
   g_object_unref (ui_configurer);
   ui_configurer = NULL;
 
@@ -941,7 +839,8 @@ gui_display_changed (GimpContext *context,
         }
     }
 
-  gimp_ui_manager_update (image_ui_manager, display);
+  gimp_ui_manager_update (menus_get_image_manager_singleton (gimp),
+                          display);
 }
 
 typedef struct
@@ -953,81 +852,84 @@ typedef struct
 accelData;
 
 static void
-gui_compare_accelerator (gpointer         data,
-                         const gchar     *accel_path,
-                         guint            accel_key,
-                         GdkModifierType  accel_mods,
-                         gboolean         changed)
+gui_check_unique_accelerators (Gimp *gimp)
 {
-  accelData *accel = data;
+  gchar **actions;
 
-  if (accel->key == accel_key && accel->mods == accel_mods &&
-      g_strcmp0 (accel->path, accel_path))
+  actions = g_action_group_list_actions (G_ACTION_GROUP (gimp->app));
+
+  for (gint i = 0; actions[i] != NULL; i++)
     {
-      g_printerr ("Actions \"%s\" and \"%s\" use the same accelerator.\n"
-                  "  Disabling the accelerator on \"%s\".\n",
-                  accel->path, accel_path, accel_path);
-      gtk_accel_map_change_entry (accel_path, 0, 0, FALSE);
-    }
-}
+      gchar **accels;
+      gchar  *detailed_name;
 
-static void
-gui_check_unique_accelerator (gpointer         data,
-                              const gchar     *accel_path,
-                              guint            accel_key,
-                              GdkModifierType  accel_mods,
-                              gboolean         changed)
-{
-  if (gtk_accelerator_valid (accel_key, accel_mods) &&
-      gui_check_action_exists (accel_path))
-    {
-      accelData accel;
+      detailed_name = g_strdup_printf ("app.%s", actions[i]);
+      accels = gtk_application_get_accels_for_action (GTK_APPLICATION (gimp->app),
+                                                      detailed_name);
+      g_free (detailed_name);
 
-      accel.path = accel_path;
-      accel.key  = accel_key;
-      accel.mods = accel_mods;
-
-      gtk_accel_map_foreach_unfiltered (&accel,
-                                        gui_compare_accelerator);
-    }
-}
-
-static gboolean
-gui_check_action_exists (const gchar *accel_path)
-{
-  GimpUIManager *manager;
-  gboolean       action_exists = FALSE;
-  GList         *list;
-
-  manager = gimp_ui_managers_from_name ("<Image>")->data;
-
-  for (list = gimp_ui_manager_get_action_groups (manager);
-       list;
-       list = g_list_next (list))
-    {
-      GimpActionGroup *group   = list->data;
-      GList           *actions = NULL;
-      GList           *list2;
-
-      actions = gimp_action_group_list_actions (group);
-
-      for (list2 = actions; list2; list2 = g_list_next (list2))
+      for (gint j = 0; accels[j] != NULL; j++)
         {
-          GimpAction  *action = list2->data;
-          const gchar *path   = gimp_action_get_accel_path (action);
-
-          if (g_strcmp0 (path, accel_path) == 0)
+          for (gint k = i + 1; actions[k] != NULL; k++)
             {
-              action_exists = TRUE;
-              break;
+              gchar **accels2;
+              gchar  *detailed_name2;
+
+              detailed_name2 = g_strdup_printf ("app.%s", actions[k]);
+              accels2 = gtk_application_get_accels_for_action (GTK_APPLICATION (gimp->app),
+                                                               detailed_name2);
+              g_free (detailed_name2);
+
+              for (gint l = 0; accels2[l] != NULL; l++)
+                {
+                  if (g_strcmp0 (accels[j], accels2[l]) == 0)
+                    {
+                      GAction  *action;
+                      gchar    *disabled_action;
+                      gchar   **disabled_accels;
+                      gint      len;
+                      gint      remove;
+
+                      /* Just keep the first one (no reason other than we have
+                       * to choose), unless it's a secondary shortcut, and the
+                       * second is a primary shortcut.
+                       */
+                      if (l == 0 && j != 0)
+                        {
+                          disabled_action = actions[i];
+                          disabled_accels = accels;
+                          remove = j;
+                        }
+                      else
+                        {
+                          disabled_action = actions[k];
+                          disabled_accels = accels2;
+                          remove = l;
+                        }
+                      /* Remove only the duplicate shortcut but keep others. */
+                      len = g_strv_length (disabled_accels);
+                      g_free (disabled_accels[remove]);
+                      memmove (&disabled_accels[remove],
+                               &disabled_accels[remove + 1],
+                               sizeof (char *) * (len - remove));
+
+                      g_printerr ("Actions \"%s\" and \"%s\" use the same accelerator.\n"
+                                  "  Disabling the accelerator on \"%s\".\n",
+                                  actions[i], actions[k], disabled_action);
+
+                      action = g_action_map_lookup_action (G_ACTION_MAP (gimp->app),
+                                                           disabled_action);
+                      gimp_action_set_accels (GIMP_ACTION (action),
+                                              (const gchar **) disabled_accels);
+                    }
+                }
+
+              g_strfreev (accels2);
             }
         }
 
-      g_list_free (actions);
-
-      if (action_exists)
-        break;
+      g_strfreev (accels);
     }
 
-  return action_exists;
+  g_strfreev (actions);
 }
