@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -31,25 +31,36 @@
 
 #include <locale.h>
 
-#include <glib-object.h>
+#include <gio/gio.h>
+#include <glib/gstdio.h>
 
 #ifdef G_OS_WIN32
 #include <io.h> /* get_osfhandle */
-#endif
+
+#endif /* G_OS_WIN32 */
+
+#if defined(ENABLE_RELOCATABLE_RESOURCES) && defined(__APPLE__)
+#include <libgen.h> /* dirname */
+#include <sys/stat.h>
+#endif /* __APPLE__ */
 
 #ifndef GIMP_CONSOLE_COMPILATION
-#include <gdk/gdk.h>
+#include <gtk/gtk.h>
+#else
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #endif
 
+#include <babl/babl.h>
+
 #include "libgimpbase/gimpbase.h"
 
 #include "pdb/pdb-types.h"
 
-#include "base/tile.h"
-
+#include "config/gimpearlyrc.h"
 #include "config/gimpconfig-dump.h"
 
 #include "core/gimp.h"
+#include "core/gimpbacktrace.h"
 
 #include "pdb/gimppdb.h"
 #include "pdb/gimpprocedure.h"
@@ -57,16 +68,18 @@
 
 #include "about.h"
 #include "app.h"
+#include "language.h"
 #include "sanity.h"
 #include "signals.h"
 #include "unique.h"
-#include "units.h"
-#include "version.h"
 
 #ifdef G_OS_WIN32
 /* To get PROCESS_DEP_* defined we need _WIN32_WINNT at 0x0601. We still
  * use the API optionally only if present, though.
  */
+#ifdef _WIN32_WINNT
+#undef _WIN32_WINNT
+#endif
 #define _WIN32_WINNT 0x0601
 #include <windows.h>
 #include <conio.h>
@@ -74,6 +87,7 @@
 
 #include "gimp-log.h"
 #include "gimp-intl.h"
+#include "gimp-version.h"
 
 
 static gboolean  gimp_option_fatal_warnings   (const gchar  *option_name,
@@ -116,6 +130,7 @@ static const gchar        *session_name      = NULL;
 static const gchar        *batch_interpreter = NULL;
 static const gchar       **batch_commands    = NULL;
 static const gchar       **filenames         = NULL;
+static gboolean            quit              = FALSE;
 static gboolean            as_new            = FALSE;
 static gboolean            no_interface      = FALSE;
 static gboolean            no_data           = FALSE;
@@ -133,9 +148,13 @@ static gboolean            console_messages  = FALSE;
 static gboolean            use_debug_handler = FALSE;
 
 #ifdef GIMP_UNSTABLE
+static gboolean            show_playground   = TRUE;
+static gboolean            show_debug_menu   = TRUE;
 static GimpStackTraceMode  stack_trace_mode  = GIMP_STACK_TRACE_QUERY;
 static GimpPDBCompatMode   pdb_compat_mode   = GIMP_PDB_COMPAT_WARN;
 #else
+static gboolean            show_playground   = FALSE;
+static gboolean            show_debug_menu   = FALSE;
 static GimpStackTraceMode  stack_trace_mode  = GIMP_STACK_TRACE_NEVER;
 static GimpPDBCompatMode   pdb_compat_mode   = GIMP_PDB_COMPAT_ON;
 #endif
@@ -190,7 +209,7 @@ static const GOptionEntry main_entries[] =
   {
     "no-shm", 0, G_OPTION_FLAG_REVERSE,
     G_OPTION_ARG_NONE, &use_shm,
-    N_("Do not use shared memory between GIMP and plugins"), NULL
+    N_("Do not use shared memory between GIMP and plug-ins"), NULL
   },
   {
     "no-cpu-accel", 0, G_OPTION_FLAG_REVERSE,
@@ -222,6 +241,11 @@ static const GOptionEntry main_entries[] =
     G_OPTION_ARG_STRING, &batch_interpreter,
     N_("The procedure to process batch commands with"), "<proc>"
   },
+  {
+    "quit", 0, 0,
+    G_OPTION_ARG_NONE, &quit,
+    N_("Quit immediately after performing requested actions"), NULL
+  },
   {
     "console-messages", 'c', 0,
     G_OPTION_ARG_NONE, &console_messages,
@@ -270,6 +294,16 @@ static const GOptionEntry main_entries[] =
     G_OPTION_ARG_CALLBACK, gimp_option_dump_pdb_procedures_deprecated,
     N_("Output a sorted list of deprecated procedures in the PDB"), NULL
   },
+  {
+    "show-playground", 0, 0,
+    G_OPTION_ARG_NONE, &show_playground,
+    N_("Show a preferences page with experimental features"), NULL
+  },
+  {
+    "show-debug-menu", 0, G_OPTION_FLAG_HIDDEN,
+    G_OPTION_ARG_NONE, &show_debug_menu,
+    N_("Show an image submenu with debug actions"), NULL
+  },
   {
     G_OPTION_REMAINING, 0, 0,
     G_OPTION_ARG_FILENAME_ARRAY, &filenames,
@@ -278,6 +312,243 @@ static const GOptionEntry main_entries[] =
   { NULL }
 };
 
+#if defined(ENABLE_RELOCATABLE_RESOURCES) && defined(__APPLE__)
+static void
+gimp_macos_setenv (const char * progname)
+{
+  /* helper to set environment variables for GIMP to be relocatable.
+   * Due to the latest changes it is not recommended to set it in the shell
+   * wrapper anymore.
+   */
+  gchar  *resolved_path;
+  /* on some OSX installations open file limit is 256 and GIMP needs more */
+  struct  rlimit limit;
+
+  limit.rlim_cur = 10000;
+  limit.rlim_max = 10000;
+  setrlimit (RLIMIT_NOFILE, &limit);
+  resolved_path = g_canonicalize_filename (progname, NULL);
+  if (resolved_path && ! g_getenv ("GIMP_NO_WRAPPER"))
+    {
+      /* set path to the app folder to make sure that our python is called
+       * instead of system one
+       */
+      static gboolean            show_playground   = TRUE;
+
+      gchar   *path;
+      gchar   *tmp;
+      gchar   *app_dir;
+      gchar   *res_dir;
+      size_t   path_len;
+      struct   stat sb;
+      gboolean need_pythonhome = TRUE;
+
+      app_dir = g_path_get_dirname (resolved_path);
+      tmp = g_strdup_printf ("%s/../Resources", app_dir);
+      res_dir = g_canonicalize_filename (tmp, NULL);
+      g_free (tmp);
+      if (res_dir && !stat (res_dir, &sb) && S_ISDIR (sb.st_mode))
+        {
+          g_print ("GIMP is started as MacOS application\n");
+        }
+      else
+        {
+          tmp = g_strdup_printf ("%s/../share", app_dir);
+          res_dir = g_canonicalize_filename (tmp, NULL);
+          g_free (tmp);
+          if (res_dir && !stat (res_dir, &sb) && S_ISDIR (sb.st_mode))
+            {
+              g_free (res_dir);
+
+              g_print ("GIMP is started in the build directory\n");
+
+              tmp = g_strdup_printf ("%s/..", app_dir); /* running in build dir */
+              res_dir = g_canonicalize_filename (tmp, NULL);
+              g_free (tmp);
+            }
+          else
+            {
+              g_free (res_dir);
+              return;
+            }
+        }
+
+      /* Detect we were built in homebrew for MacOS */
+      tmp = g_strdup_printf ("%s/Frameworks/Python.framework", res_dir);
+      if (tmp && !stat (tmp, &sb) && S_ISDIR (sb.st_mode))
+        {
+          g_print ("GIMP was built with homebrew\n");
+          need_pythonhome = FALSE;
+        }
+      g_free (tmp);
+      /* Detect we were built in MacPorts for MacOS */
+      tmp = g_strdup_printf ("%s/Library/Frameworks/Python.framework", res_dir);
+      if (tmp && !stat (tmp, &sb) && S_ISDIR (sb.st_mode))
+        {
+          g_print ("GIMP was built with MacPorts\n");
+          need_pythonhome = FALSE;
+        }
+      g_free (tmp);
+
+      path_len = strlen (g_getenv ("PATH") ? g_getenv ("PATH") : "") + strlen (app_dir) + 2;
+      path = g_try_malloc (path_len);
+      if (path == NULL)
+        {
+          g_warning ("Failed to allocate memory");
+          app_exit (EXIT_FAILURE);
+        }
+      if (g_getenv ("PATH"))
+        g_snprintf (path, path_len, "%s:%s", app_dir, g_getenv ("PATH"));
+      else
+        g_snprintf (path, path_len, "%s", app_dir);
+      g_free (app_dir);
+      g_setenv ("PATH", path, TRUE);
+      g_free (path);
+      tmp = g_strdup_printf ("%s/lib/gtk-3.0/3.0.0", res_dir);
+      g_setenv ("GTK_PATH", tmp, TRUE);
+      g_free (tmp);
+      tmp = g_strdup_printf ("%s/lib/gegl-0.4", res_dir);
+      g_setenv ("GEGL_PATH", tmp, TRUE);
+      g_free (tmp);
+      tmp = g_strdup_printf ("%s/lib/babl-0.1", res_dir);
+      g_setenv ("BABL_PATH", tmp, TRUE);
+      g_free (tmp);
+      tmp = g_strdup_printf ("%s/lib/gdk-pixbuf-2.0/2.10.0/loaders.cache", res_dir);
+      g_setenv ("GDK_PIXBUF_MODULE_FILE", tmp, TRUE);
+      g_free (tmp);
+      tmp = g_strdup_printf ("%s/etc/fonts", res_dir);
+      g_setenv ("FONTCONFIG_PATH", tmp, TRUE);
+      g_free (tmp);
+      if (need_pythonhome)
+        {
+          tmp = g_strdup_printf ("%s", res_dir);
+          g_setenv ("PYTHONHOME", tmp, TRUE);
+          g_free (tmp);
+        }
+      tmp = g_strdup_printf ("%s/lib/python3.9", res_dir);
+      g_setenv ("PYTHONPATH", tmp, TRUE);
+      g_free (tmp);
+      tmp = g_strdup_printf ("%s/lib/gio/modules", res_dir);
+      g_setenv ("GIO_MODULE_DIR", tmp, TRUE);
+      g_free (tmp);
+      tmp = g_strdup_printf ("%s/share/libwmf/fonts", res_dir);
+      g_setenv ("WMF_FONTDIR", tmp, TRUE);
+      g_free (tmp);
+      if (g_getenv ("DYLD_LIBRARY_PATH"))
+        tmp = g_strdup_printf ("%s/lib:%s", res_dir, g_getenv ("DYLD_LIBRARY_PATH"));
+      else
+        tmp = g_strdup_printf ("%s/lib", res_dir);
+      g_setenv ("DYLD_LIBRARY_PATH", tmp, TRUE);
+      g_free (tmp);
+      if (g_getenv ("XDG_DATA_DIRS"))
+        tmp = g_strdup_printf ("%s/share:%s", res_dir, g_getenv ("XDG_DATA_DIRS"));
+      else
+        tmp = g_strdup_printf ("%s/share", res_dir);
+      g_setenv ("XDG_DATA_DIRS", tmp, TRUE);
+      g_free (tmp);
+      tmp = g_strdup_printf ("%s/lib/girepository-1.0", res_dir);
+      g_setenv ("GI_TYPELIB_PATH", tmp, TRUE);
+      g_free (tmp);
+      if (g_getenv ("HOME") != NULL)
+        {
+          tmp = g_strdup_printf ("%s/Library/Application Support/GIMP/3.00/cache",
+                                 g_getenv ("HOME"));
+          g_setenv ("XDG_CACHE_HOME", tmp, TRUE);
+          g_free (tmp);
+        }
+      g_free (res_dir);
+    }
+  g_free (resolved_path);
+}
+#endif
+
+/* gimp_early_configuration () is executed as soon as we can read
+ * the "gimprc" files, but before any library initialization takes
+ * place
+ */
+static void
+gimp_early_configuration (void)
+{
+  GFile       *system_gimprc_file = NULL;
+  GFile       *user_gimprc_file   = NULL;
+  GimpEarlyRc *earlyrc;
+  gchar       *language;
+
+  if (system_gimprc)
+    system_gimprc_file = g_file_new_for_commandline_arg (system_gimprc);
+
+  if (user_gimprc)
+    user_gimprc_file = g_file_new_for_commandline_arg (user_gimprc);
+
+  /* GimpEarlyRc is reponsible for reading "gimprc" files for the
+   * sole purpose of getting some configuration data that is needed
+   * in the early initialization phase
+   */
+  earlyrc = gimp_early_rc_new (system_gimprc_file,
+                               user_gimprc_file,
+                               be_verbose);
+
+  /* Language needs to be determined first, before any GimpContext is
+   * instantiated (which happens when the Gimp object is created)
+   * because its properties need to be properly localized in the
+   * settings language (if different from system language). Otherwise we
+   * end up with pieces of GUI always using the system language (cf. bug
+   * 787457)
+   */
+  language = gimp_early_rc_get_language (earlyrc);
+
+  /*  change the locale if a language if specified  */
+  language_init (language);
+  if (language)
+    g_free (language);
+
+#if defined (G_OS_WIN32) && !defined (GIMP_CONSOLE_COMPILATION)
+
+#if GTK_MAJOR_VERSION > 3
+#warning For GTK4 and above use the proper backend-specific API instead of the GDK_WIN32_TABLET_INPUT_API environment variable
+#endif
+
+  /* Set a GdkWin32-specific environment variable to specify
+   * the desired pen / touch input API to use on Windows
+  */
+  if (gtk_get_major_version () == 3 &&
+      (gtk_get_minor_version () > 24 ||
+       (gtk_get_minor_version () == 24 &&
+        gtk_get_micro_version () >= 30)))
+    {
+      GimpWin32PointerInputAPI api = gimp_early_rc_get_win32_pointer_input_api (earlyrc);
+
+      switch (api)
+        {
+        case GIMP_WIN32_POINTER_INPUT_API_WINTAB:
+          g_setenv ("GDK_WIN32_TABLET_INPUT_API", "wintab", TRUE);
+          break;
+        case GIMP_WIN32_POINTER_INPUT_API_WINDOWS_INK:
+          g_setenv ("GDK_WIN32_TABLET_INPUT_API", "winpointer", TRUE);
+          break;
+        }
+    }
+
+#endif
+
+  g_object_unref (earlyrc);
+
+  g_clear_object (&system_gimprc_file);
+  g_clear_object (&user_gimprc_file);
+}
+
+static gboolean
+gimp_options_group_parse_hook (GOptionContext   *context,
+                               GOptionGroup     *group,
+                               gpointer          data,
+                               GError          **error)
+{
+  /*  early initialization from data stored in "gimprc" files  */
+  gimp_early_configuration ();
+
+  return TRUE;
+}
+
 int
 main (int    argc,
       char **argv)
@@ -286,8 +557,36 @@ main (int    argc,
   GError         *error = NULL;
   const gchar    *abort_message;
   gchar          *basename;
+  GFile          *system_gimprc_file = NULL;
+  GFile          *user_gimprc_file   = NULL;
+  GOptionGroup   *gimp_group         = NULL;
+  gchar          *backtrace_file     = NULL;
+  gint            retval;
   gint            i;
 
+#ifdef ENABLE_WIN32_DEBUG_CONSOLE
+  gimp_open_console_window ();
+#endif
+#if defined(ENABLE_RELOCATABLE_RESOURCES) && defined(__APPLE__)
+  /* remove MacOS session identifier from the command line args */
+  gint newargc = 0;
+  for (gint i = 0; i < argc; i++)
+    {
+      if (!g_str_has_prefix (argv[i], "-psn_"))
+        {
+          argv[newargc] = argv[i];
+          newargc++;
+        }
+    }
+  if (argc > newargc)
+    {
+      argv[newargc] = NULL; /* glib expects NULL terminated array */
+      argc = newargc;
+    }
+
+  gimp_macos_setenv (argv[0]);
+#endif
+
 #if defined (__GNUC__) && defined (_WIN64)
   /* mingw-w64, at least the unstable build from late July 2008,
    * starts subsystem:windows programs in main(), but passes them
@@ -298,34 +597,65 @@ main (int    argc,
   argv = __argv;
 #endif
 
+  /* Initialize GimpBacktrace early on.  In particular, we want the
+   * Windows backend to catch the SET_THREAD_NAME exceptions of newly
+   * created threads.
+   */
+  gimp_backtrace_init ();
+
+  /* Start signal handlers early. */
+  gimp_init_signal_handlers (&backtrace_file);
+
 #ifdef G_OS_WIN32
   /* Reduce risks */
-  {
-    typedef BOOL (WINAPI *t_SetDllDirectoryA) (LPCSTR lpPathName);
-    t_SetDllDirectoryA p_SetDllDirectoryA;
+  SetDllDirectoryW (L"");
 
-    p_SetDllDirectoryA = GetProcAddress (GetModuleHandle ("kernel32.dll"),
-					 "SetDllDirectoryA");
-    if (p_SetDllDirectoryA)
-      (*p_SetDllDirectoryA) ("");
+  /* On Windows, set DLL search path to $INSTALLDIR/bin so that .exe
+     plug-ins in the plug-ins directory can find libgimp and file
+     library DLLs without needing to set external PATH. */
+  {
+    const gchar *install_dir;
+    gchar       *bin_dir;
+    LPWSTR       w_bin_dir;
+
+    w_bin_dir = NULL;
+    install_dir = gimp_installation_directory ();
+    bin_dir = g_build_filename (install_dir, "bin", NULL);
+
+    w_bin_dir = g_utf8_to_utf16 (bin_dir, -1, NULL, NULL, NULL);
+    if (w_bin_dir)
+      {
+        SetDllDirectoryW (w_bin_dir);
+        g_free (w_bin_dir);
+      }
+
+    g_free (bin_dir);
   }
+
 #ifndef _WIN64
   {
     typedef BOOL (WINAPI *t_SetProcessDEPPolicy) (DWORD dwFlags);
     t_SetProcessDEPPolicy p_SetProcessDEPPolicy;
 
-    p_SetProcessDEPPolicy = GetProcAddress (GetModuleHandle ("kernel32.dll"),
-					    "SetProcessDEPPolicy");
+    p_SetProcessDEPPolicy =
+      (t_SetProcessDEPPolicy) GetProcAddress (GetModuleHandleW (L"kernel32.dll"),
+                                              "SetProcessDEPPolicy");
     if (p_SetProcessDEPPolicy)
       (*p_SetProcessDEPPolicy) (PROCESS_DEP_ENABLE|PROCESS_DEP_DISABLE_ATL_THUNK_EMULATION);
   }
-#endif
 #endif
 
-  g_thread_init (NULL);
+  /* Group all our windows together on the taskbar */
+  {
+    typedef HRESULT (WINAPI *t_SetCurrentProcessExplicitAppUserModelID) (PCWSTR lpPathName);
+    t_SetCurrentProcessExplicitAppUserModelID p_SetCurrentProcessExplicitAppUserModelID;
 
-#ifdef GIMP_UNSTABLE
-  gimp_open_console_window ();
+    p_SetCurrentProcessExplicitAppUserModelID =
+      (t_SetCurrentProcessExplicitAppUserModelID) GetProcAddress (GetModuleHandleW (L"shell32.dll"),
+                                                                  "SetCurrentProcessExplicitAppUserModelID");
+    if (p_SetCurrentProcessExplicitAppUserModelID)
+      (*p_SetCurrentProcessExplicitAppUserModelID) (L"gimp.GimpApplication");
+  }
 #endif
 
   gimp_init_malloc ();
@@ -338,6 +668,12 @@ main (int    argc,
 
   g_set_application_name (GIMP_NAME);
 
+#ifdef G_OS_WIN32
+  argv = g_win32_get_command_line ();
+#else
+  argv = g_strdupv (argv);
+#endif
+
   basename = g_path_get_basename (argv[0]);
   g_set_prgname (basename);
   g_free (basename);
@@ -391,9 +727,19 @@ main (int    argc,
 
   g_option_context_add_main_entries (context, main_entries, GETTEXT_PACKAGE);
 
+  /* The GIMP option group is just an empty option group, created for the sole
+   * purpose of running a post-parse hook before any other of dependant libraries
+   * are run. This makes it possible to apply options from configuration data
+   * obtained from "gimprc" files, before other libraries have a chance to run
+   * some of their intialization code.
+   */
+  gimp_group = g_option_group_new ("gimp", "", "", NULL, NULL);
+  g_option_group_set_parse_hooks (gimp_group, NULL, gimp_options_group_parse_hook);
+  g_option_context_add_group (context, gimp_group);
+
   app_libs_init (context, no_interface);
 
-  if (! g_option_context_parse (context, &argc, &argv, &error))
+  if (! g_option_context_parse_strv (context, &argv, &error))
     {
       if (error)
         {
@@ -421,45 +767,66 @@ main (int    argc,
 #ifndef GIMP_CONSOLE_COMPILATION
   if (! new_instance && gimp_unique_open (filenames, as_new))
     {
+      int success = EXIT_SUCCESS;
+
       if (be_verbose)
-	g_print ("%s\n",
-		 _("Another GIMP instance is already running."));
+        g_print ("%s\n",
+                 _("Another GIMP instance is already running."));
+
+      if (batch_commands &&
+          ! gimp_unique_batch_run (batch_interpreter, batch_commands))
+        success = EXIT_FAILURE;
 
       gdk_notify_startup_complete ();
 
-      return EXIT_SUCCESS;
+      return success;
     }
 #endif
 
-  abort_message = sanity_check ();
+  abort_message = sanity_check_early ();
   if (abort_message)
     app_abort (no_interface, abort_message);
 
-  gimp_init_signal_handlers (stack_trace_mode);
+  if (system_gimprc)
+    system_gimprc_file = g_file_new_for_commandline_arg (system_gimprc);
 
-  app_run (argv[0],
-           filenames,
-           system_gimprc,
-           user_gimprc,
-           session_name,
-           batch_interpreter,
-           batch_commands,
-           as_new,
-           no_interface,
-           no_data,
-           no_fonts,
-           no_splash,
-           be_verbose,
-           use_shm,
-           use_cpu_accel,
-           console_messages,
-           use_debug_handler,
-           stack_trace_mode,
-           pdb_compat_mode);
+  if (user_gimprc)
+    user_gimprc_file = g_file_new_for_commandline_arg (user_gimprc);
+
+  retval = app_run (argv[0],
+                    filenames,
+                    system_gimprc_file,
+                    user_gimprc_file,
+                    session_name,
+                    batch_interpreter,
+                    batch_commands,
+                    quit,
+                    as_new,
+                    no_interface,
+                    no_data,
+                    no_fonts,
+                    no_splash,
+                    be_verbose,
+                    use_shm,
+                    use_cpu_accel,
+                    console_messages,
+                    use_debug_handler,
+                    show_playground,
+                    show_debug_menu,
+                    stack_trace_mode,
+                    pdb_compat_mode,
+                    backtrace_file);
+
+  g_free (backtrace_file);
+
+  g_clear_object (&system_gimprc_file);
+  g_clear_object (&user_gimprc_file);
+
+  g_strfreev (argv);
 
   g_option_context_free (context);
 
-  return EXIT_SUCCESS;
+  return retval;
 }
 
 
@@ -491,7 +858,7 @@ WinMain (struct HINSTANCE__ *hInstance,
 static void
 wait_console_window (void)
 {
-  FILE *console = fopen ("CONOUT$", "w");
+  FILE *console = g_fopen ("CONOUT$", "w");
 
   SetConsoleTitleW (g_utf8_to_utf16 (_("GIMP output. Type any character to close this window."), -1, NULL, NULL, NULL));
   fprintf (console, _("(Type any character to close this window)\n"));
@@ -595,11 +962,11 @@ gimp_option_dump_gimprc (const gchar  *option_name,
       Gimp     *gimp;
       gboolean  success;
 
+      babl_init ();
       gimp = g_object_new (GIMP_TYPE_GIMP, NULL);
+      gimp_load_config (gimp, NULL, NULL);
 
-      units_init (gimp);
-
-      success = gimp_config_dump (format);
+      success = gimp_config_dump (G_OBJECT (gimp), format);
 
       g_object_unref (gimp);
 
@@ -619,7 +986,9 @@ gimp_option_dump_pdb_procedures_deprecated (const gchar  *option_name,
   GList *deprecated_procs;
   GList *iter;
 
+  babl_init ();
   gimp = g_object_new (GIMP_TYPE_GIMP, NULL);
+  gimp_load_config (gimp, NULL, NULL);
 
   /* Make sure to turn on compatibility mode so deprecated procedures
    * are included
@@ -636,7 +1005,7 @@ gimp_option_dump_pdb_procedures_deprecated (const gchar  *option_name,
     {
       GimpProcedure *procedure = GIMP_PROCEDURE (iter->data);
 
-      g_print ("%s\n", procedure->original_name);
+      g_print ("%s\n", gimp_object_get_name (procedure));
     }
 
   g_list_free (deprecated_procs);
@@ -690,8 +1059,10 @@ gimp_init_malloc (void)
    * An alternative to tuning this parameter would be to use
    * malloc_trim(), for example after releasing a large tile-manager.
    */
+#if 0
   mallopt (M_MMAP_THRESHOLD, TILE_WIDTH * TILE_HEIGHT);
 #endif
+#endif
 }
 
 static void
@@ -705,12 +1076,12 @@ gimp_init_i18n (void)
 
   setlocale (LC_ALL, "");
 
-  bindtextdomain (GETTEXT_PACKAGE"-libgimp", gimp_locale_directory ());
+  gimp_bind_text_domain (GETTEXT_PACKAGE"-libgimp", gimp_locale_directory ());
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
   bind_textdomain_codeset (GETTEXT_PACKAGE"-libgimp", "UTF-8");
 #endif
 
-  bindtextdomain (GETTEXT_PACKAGE, gimp_locale_directory ());
+  gimp_bind_text_domain (GETTEXT_PACKAGE, gimp_locale_directory ());
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif
