@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -27,6 +27,7 @@
 #include "paint-types.h"
 
 #include "core/gimp.h"
+#include "core/gimpbrush-header.h"
 #include "core/gimpbrushgenerated.h"
 #include "core/gimpimage.h"
 #include "core/gimpdynamics.h"
@@ -34,28 +35,34 @@
 #include "core/gimpgradient.h"
 #include "core/gimppaintinfo.h"
 
+#include "gimpbrushcore.h"
 #include "gimppaintoptions.h"
 
 #include "gimp-intl.h"
 
 
 #define DEFAULT_BRUSH_SIZE              20.0
-#define DEFAULT_BRUSH_ZOOM              FALSE
-
 #define DEFAULT_BRUSH_ASPECT_RATIO      0.0
 #define DEFAULT_BRUSH_ANGLE             0.0
 #define DEFAULT_BRUSH_SPACING           0.1
-
 #define DEFAULT_BRUSH_HARDNESS          1.0 /* Generated brushes have their own */
 #define DEFAULT_BRUSH_FORCE             0.5
 
+#define DEFAULT_BRUSH_LINK_SIZE         TRUE
+#define DEFAULT_BRUSH_LINK_ASPECT_RATIO TRUE
+#define DEFAULT_BRUSH_LINK_ANGLE        TRUE
+#define DEFAULT_BRUSH_LINK_SPACING      TRUE
+#define DEFAULT_BRUSH_LINK_HARDNESS     TRUE
+
+#define DEFAULT_BRUSH_LOCK_TO_VIEW      FALSE
+
 #define DEFAULT_APPLICATION_MODE        GIMP_PAINT_CONSTANT
 #define DEFAULT_HARD                    FALSE
 
 #define DEFAULT_USE_JITTER              FALSE
 #define DEFAULT_JITTER_AMOUNT           0.2
 
-#define DEFAULT_DYNAMICS_EXPANDED       FALSE
+#define DEFAULT_DYNAMICS_ENABLED        TRUE
 
 #define DEFAULT_FADE_LENGTH             100.0
 #define DEFAULT_FADE_REVERSE            FALSE
@@ -63,9 +70,8 @@
 #define DEFAULT_FADE_UNIT               GIMP_UNIT_PIXEL
 
 #define DEFAULT_GRADIENT_REVERSE        FALSE
-#define DEFAULT_GRADIENT_REPEAT         GIMP_REPEAT_TRIANGULAR
-#define DEFAULT_GRADIENT_LENGTH         100.0
-#define DEFAULT_GRADIENT_UNIT           GIMP_UNIT_PIXEL
+#define DEFAULT_GRADIENT_BLEND_SPACE    GIMP_GRADIENT_BLEND_RGB_PERCEPTUAL
+#define DEFAULT_GRADIENT_REPEAT         GIMP_REPEAT_NONE
 
 #define DYNAMIC_MAX_VALUE               1.0
 #define DYNAMIC_MIN_VALUE               0.0
@@ -73,12 +79,6 @@
 #define DEFAULT_SMOOTHING_QUALITY       20
 #define DEFAULT_SMOOTHING_FACTOR        50
 
-#define DEFAULT_BRUSH_LINK_SIZE         TRUE
-#define DEFAULT_BRUSH_LINK_ASPECT_RATIO TRUE
-#define DEFAULT_BRUSH_LINK_ANGLE        TRUE
-#define DEFAULT_BRUSH_LINK_SPACING      TRUE
-#define DEFAULT_BRUSH_LINK_HARDNESS     TRUE
-
 enum
 {
   PROP_0,
@@ -88,20 +88,27 @@ enum
   PROP_USE_APPLICATOR, /* temp debug */
 
   PROP_BRUSH_SIZE,
-  PROP_BRUSH_ZOOM,
   PROP_BRUSH_ASPECT_RATIO,
   PROP_BRUSH_ANGLE,
   PROP_BRUSH_SPACING,
   PROP_BRUSH_HARDNESS,
   PROP_BRUSH_FORCE,
 
+  PROP_BRUSH_LINK_SIZE,
+  PROP_BRUSH_LINK_ASPECT_RATIO,
+  PROP_BRUSH_LINK_ANGLE,
+  PROP_BRUSH_LINK_SPACING,
+  PROP_BRUSH_LINK_HARDNESS,
+
+  PROP_BRUSH_LOCK_TO_VIEW,
+
   PROP_APPLICATION_MODE,
   PROP_HARD,
 
   PROP_USE_JITTER,
   PROP_JITTER_AMOUNT,
 
-  PROP_DYNAMICS_EXPANDED,
+  PROP_DYNAMICS_ENABLED,
 
   PROP_FADE_LENGTH,
   PROP_FADE_REVERSE,
@@ -109,6 +116,8 @@ enum
   PROP_FADE_UNIT,
 
   PROP_GRADIENT_REVERSE,
+  PROP_GRADIENT_BLEND_COLOR_SPACE,
+  PROP_GRADIENT_REPEAT,
 
   PROP_BRUSH_VIEW_TYPE,
   PROP_BRUSH_VIEW_SIZE,
@@ -121,34 +130,34 @@ enum
 
   PROP_USE_SMOOTHING,
   PROP_SMOOTHING_QUALITY,
-  PROP_SMOOTHING_FACTOR,
-
-  PROP_BRUSH_LINK_SIZE,
-  PROP_BRUSH_LINK_ASPECT_RATIO,
-  PROP_BRUSH_LINK_ANGLE,
-  PROP_BRUSH_LINK_SPACING,
-  PROP_BRUSH_LINK_HARDNESS
+  PROP_SMOOTHING_FACTOR
 };
 
 
 static void         gimp_paint_options_config_iface_init (GimpConfigInterface *config_iface);
 
-static void         gimp_paint_options_dispose           (GObject      *object);
-static void         gimp_paint_options_finalize          (GObject      *object);
-static void         gimp_paint_options_set_property      (GObject      *object,
-                                                          guint         property_id,
-                                                          const GValue *value,
-                                                          GParamSpec   *pspec);
-static void         gimp_paint_options_get_property      (GObject      *object,
-                                                          guint         property_id,
-                                                          GValue       *value,
-                                                          GParamSpec   *pspec);
+static void         gimp_paint_options_dispose           (GObject          *object);
+static void         gimp_paint_options_finalize          (GObject          *object);
+static void         gimp_paint_options_set_property      (GObject          *object,
+                                                          guint             property_id,
+                                                          const GValue     *value,
+                                                          GParamSpec       *pspec);
+static void         gimp_paint_options_get_property      (GObject          *object,
+                                                          guint             property_id,
+                                                          GValue           *value,
+                                                          GParamSpec       *pspec);
 
-static GimpConfig * gimp_paint_options_duplicate         (GimpConfig   *config);
-static gboolean     gimp_paint_options_copy              (GimpConfig   *src,
-                                                          GimpConfig   *dest,
-                                                          GParamFlags   flags);
-static void         gimp_paint_options_reset             (GimpConfig   *config);
+static void         gimp_paint_options_brush_changed     (GimpContext      *context,
+                                                          GimpBrush        *brush);
+static void         gimp_paint_options_brush_notify      (GimpBrush        *brush,
+                                                          const GParamSpec *pspec,
+                                                          GimpPaintOptions *options);
+
+static GimpConfig * gimp_paint_options_duplicate         (GimpConfig       *config);
+static gboolean     gimp_paint_options_copy              (GimpConfig       *src,
+                                                          GimpConfig       *dest,
+                                                          GParamFlags       flags);
+static void         gimp_paint_options_reset             (GimpConfig       *config);
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpPaintOptions, gimp_paint_options,
@@ -164,12 +173,15 @@ static GimpConfigInterface *parent_config_iface = NULL;
 static void
 gimp_paint_options_class_init (GimpPaintOptionsClass *klass)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GObjectClass     *object_class  = G_OBJECT_CLASS (klass);
+  GimpContextClass *context_class = GIMP_CONTEXT_CLASS (klass);
 
-  object_class->dispose        = gimp_paint_options_dispose;
-  object_class->finalize       = gimp_paint_options_finalize;
-  object_class->set_property   = gimp_paint_options_set_property;
-  object_class->get_property   = gimp_paint_options_get_property;
+  object_class->dispose         = gimp_paint_options_dispose;
+  object_class->finalize        = gimp_paint_options_finalize;
+  object_class->set_property    = gimp_paint_options_set_property;
+  object_class->get_property    = gimp_paint_options_get_property;
+
+  context_class->brush_changed  = gimp_paint_options_brush_changed;
 
   g_object_class_install_property (object_class, PROP_PAINT_INFO,
                                    g_param_spec_object ("paint-info",
@@ -193,13 +205,6 @@ gimp_paint_options_class_init (GimpPaintOptionsClass *klass)
                            1.0, GIMP_BRUSH_MAX_SIZE, DEFAULT_BRUSH_SIZE,
                            GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_BRUSH_ZOOM,
-                            "brush-zoom",
-                            _("Brush Zoom"),
-                            _("Link brush size with canvas zoom"),
-                            DEFAULT_BRUSH_ZOOM,
-                            GIMP_PARAM_STATIC_STRINGS);
-
   GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_BRUSH_ASPECT_RATIO,
                            "brush-aspect-ratio",
                            _("Aspect Ratio"),
@@ -270,6 +275,13 @@ gimp_paint_options_class_init (GimpPaintOptionsClass *klass)
                             DEFAULT_BRUSH_LINK_HARDNESS,
                             GIMP_PARAM_STATIC_STRINGS);
 
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_BRUSH_LOCK_TO_VIEW,
+                            "brush-lock-to-view",
+                            _("Lock brush to view"),
+                            _("Keep brush appearance fixed relative to the view"),
+                            DEFAULT_BRUSH_LOCK_TO_VIEW,
+                            GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_PROP_ENUM (object_class, PROP_APPLICATION_MODE,
                          "application-mode",
                          _("Incremental"),
@@ -291,6 +303,14 @@ gimp_paint_options_class_init (GimpPaintOptionsClass *klass)
                             _("Scatter brush as you paint"),
                             DEFAULT_USE_JITTER,
                             GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_DYNAMICS_ENABLED,
+                            "dynamics-enabled",
+                            _("Enable dynamics"),
+                            _("Apply dynamics curves to paint settings"),
+                            DEFAULT_DYNAMICS_ENABLED,
+                            GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_JITTER_AMOUNT,
                            "jitter-amount",
                            _("Amount"),
@@ -298,13 +318,6 @@ gimp_paint_options_class_init (GimpPaintOptionsClass *klass)
                            0.0, 50.0, DEFAULT_JITTER_AMOUNT,
                            GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_DYNAMICS_EXPANDED,
-                            "dynamics-expanded",
-                            _("Dynamics Options"),
-                            NULL,
-                            DEFAULT_DYNAMICS_EXPANDED,
-                            GIMP_PARAM_STATIC_STRINGS);
-
   GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_FADE_LENGTH,
                            "fade-length",
                            _("Fade length"),
@@ -335,6 +348,20 @@ gimp_paint_options_class_init (GimpPaintOptionsClass *klass)
                             NULL, NULL,
                             DEFAULT_GRADIENT_REVERSE,
                             GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_ENUM (object_class, PROP_GRADIENT_BLEND_COLOR_SPACE,
+                         "gradient-blend-color-space",
+                         _("Blend Color Space"),
+                         _("Which color space to use when blending RGB gradient segments"),
+                         GIMP_TYPE_GRADIENT_BLEND_COLOR_SPACE,
+                         DEFAULT_GRADIENT_BLEND_SPACE,
+                         GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_ENUM (object_class, PROP_GRADIENT_REPEAT,
+                         "gradient-repeat",
+                         _("Repeat"),
+                         NULL,
+                         GIMP_TYPE_REPEAT_MODE,
+                         DEFAULT_GRADIENT_REPEAT,
+                         GIMP_PARAM_STATIC_STRINGS);
 
   GIMP_CONFIG_PROP_ENUM (object_class, PROP_BRUSH_VIEW_TYPE,
                          "brush-view-type",
@@ -436,7 +463,7 @@ gimp_paint_options_init (GimpPaintOptions *options)
 
   options->jitter_options    = g_slice_new0 (GimpJitterOptions);
   options->fade_options      = g_slice_new0 (GimpFadeOptions);
-  options->gradient_options  = g_slice_new0 (GimpGradientOptions);
+  options->gradient_options  = g_slice_new0 (GimpGradientPaintOptions);
   options->smoothing_options = g_slice_new0 (GimpSmoothingOptions);
 }
 
@@ -445,10 +472,14 @@ gimp_paint_options_dispose (GObject *object)
 {
   GimpPaintOptions *options = GIMP_PAINT_OPTIONS (object);
 
-  if (options->paint_info)
+  g_clear_object (&options->paint_info);
+
+  if (options->brush)
     {
-      g_object_unref (options->paint_info);
-      options->paint_info = NULL;
+      g_signal_handlers_disconnect_by_func (options->brush,
+                                            gimp_paint_options_brush_notify,
+                                            options);
+      g_clear_weak_pointer (&options->brush);
     }
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -459,10 +490,10 @@ gimp_paint_options_finalize (GObject *object)
 {
   GimpPaintOptions *options = GIMP_PAINT_OPTIONS (object);
 
-  g_slice_free (GimpJitterOptions,    options->jitter_options);
-  g_slice_free (GimpFadeOptions,      options->fade_options);
-  g_slice_free (GimpGradientOptions,  options->gradient_options);
-  g_slice_free (GimpSmoothingOptions, options->smoothing_options);
+  g_slice_free (GimpJitterOptions,        options->jitter_options);
+  g_slice_free (GimpFadeOptions,          options->fade_options);
+  g_slice_free (GimpGradientPaintOptions, options->gradient_options);
+  g_slice_free (GimpSmoothingOptions,     options->smoothing_options);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -473,11 +504,11 @@ gimp_paint_options_set_property (GObject      *object,
                                  const GValue *value,
                                  GParamSpec   *pspec)
 {
-  GimpPaintOptions     *options           = GIMP_PAINT_OPTIONS (object);
-  GimpFadeOptions      *fade_options      = options->fade_options;
-  GimpJitterOptions    *jitter_options    = options->jitter_options;
-  GimpGradientOptions  *gradient_options  = options->gradient_options;
-  GimpSmoothingOptions *smoothing_options = options->smoothing_options;
+  GimpPaintOptions         *options           = GIMP_PAINT_OPTIONS (object);
+  GimpFadeOptions          *fade_options      = options->fade_options;
+  GimpJitterOptions        *jitter_options    = options->jitter_options;
+  GimpGradientPaintOptions *gradient_options  = options->gradient_options;
+  GimpSmoothingOptions     *smoothing_options = options->smoothing_options;
 
   switch (property_id)
     {
@@ -492,27 +523,18 @@ gimp_paint_options_set_property (GObject      *object,
     case PROP_BRUSH_SIZE:
       options->brush_size = g_value_get_double (value);
       break;
-
-    case PROP_BRUSH_ZOOM:
-      options->brush_zoom = g_value_get_boolean (value);
-      break;
-
     case PROP_BRUSH_ASPECT_RATIO:
       options->brush_aspect_ratio = g_value_get_double (value);
       break;
-
     case PROP_BRUSH_ANGLE:
       options->brush_angle = - 1.0 * g_value_get_double (value) / 360.0; /* let's make the angle mathematically correct */
       break;
-
     case PROP_BRUSH_SPACING:
       options->brush_spacing = g_value_get_double (value);
       break;
-
     case PROP_BRUSH_HARDNESS:
       options->brush_hardness = g_value_get_double (value);
       break;
-
     case PROP_BRUSH_FORCE:
       options->brush_force = g_value_get_double (value);
       break;
@@ -520,27 +542,26 @@ gimp_paint_options_set_property (GObject      *object,
     case PROP_BRUSH_LINK_SIZE:
       options->brush_link_size = g_value_get_boolean (value);
       break;
-
     case PROP_BRUSH_LINK_ASPECT_RATIO:
       options->brush_link_aspect_ratio = g_value_get_boolean (value);
       break;
-
     case PROP_BRUSH_LINK_ANGLE:
       options->brush_link_angle = g_value_get_boolean (value);
       break;
-
     case PROP_BRUSH_LINK_SPACING:
       options->brush_link_spacing = g_value_get_boolean (value);
       break;
-
     case PROP_BRUSH_LINK_HARDNESS:
       options->brush_link_hardness = g_value_get_boolean (value);
       break;
 
+    case PROP_BRUSH_LOCK_TO_VIEW:
+      options->brush_lock_to_view = g_value_get_boolean (value);
+      break;
+
     case PROP_APPLICATION_MODE:
       options->application_mode = g_value_get_enum (value);
       break;
-
     case PROP_HARD:
       options->hard = g_value_get_boolean (value);
       break;
@@ -548,27 +569,23 @@ gimp_paint_options_set_property (GObject      *object,
     case PROP_USE_JITTER:
       jitter_options->use_jitter = g_value_get_boolean (value);
       break;
-
     case PROP_JITTER_AMOUNT:
       jitter_options->jitter_amount = g_value_get_double (value);
       break;
 
-    case PROP_DYNAMICS_EXPANDED:
-      options->dynamics_expanded = g_value_get_boolean (value);
+    case PROP_DYNAMICS_ENABLED:
+      options->dynamics_enabled = g_value_get_boolean (value);
       break;
 
     case PROP_FADE_LENGTH:
       fade_options->fade_length = g_value_get_double (value);
       break;
-
     case PROP_FADE_REVERSE:
       fade_options->fade_reverse = g_value_get_boolean (value);
       break;
-
     case PROP_FADE_REPEAT:
       fade_options->fade_repeat = g_value_get_enum (value);
       break;
-
     case PROP_FADE_UNIT:
       fade_options->fade_unit = g_value_get_int (value);
       break;
@@ -576,11 +593,16 @@ gimp_paint_options_set_property (GObject      *object,
     case PROP_GRADIENT_REVERSE:
       gradient_options->gradient_reverse = g_value_get_boolean (value);
       break;
+    case PROP_GRADIENT_BLEND_COLOR_SPACE:
+      gradient_options->gradient_blend_color_space = g_value_get_enum (value);
+      break;
+    case PROP_GRADIENT_REPEAT:
+      gradient_options->gradient_repeat = g_value_get_enum (value);
+      break;
 
     case PROP_BRUSH_VIEW_TYPE:
       options->brush_view_type = g_value_get_enum (value);
       break;
-
     case PROP_BRUSH_VIEW_SIZE:
       options->brush_view_size = g_value_get_int (value);
       break;
@@ -588,7 +610,6 @@ gimp_paint_options_set_property (GObject      *object,
     case PROP_DYNAMICS_VIEW_TYPE:
       options->dynamics_view_type = g_value_get_enum (value);
       break;
-
     case PROP_DYNAMICS_VIEW_SIZE:
       options->dynamics_view_size = g_value_get_int (value);
       break;
@@ -596,7 +617,6 @@ gimp_paint_options_set_property (GObject      *object,
     case PROP_PATTERN_VIEW_TYPE:
       options->pattern_view_type = g_value_get_enum (value);
       break;
-
     case PROP_PATTERN_VIEW_SIZE:
       options->pattern_view_size = g_value_get_int (value);
       break;
@@ -604,7 +624,6 @@ gimp_paint_options_set_property (GObject      *object,
     case PROP_GRADIENT_VIEW_TYPE:
       options->gradient_view_type = g_value_get_enum (value);
       break;
-
     case PROP_GRADIENT_VIEW_SIZE:
       options->gradient_view_size = g_value_get_int (value);
       break;
@@ -612,11 +631,9 @@ gimp_paint_options_set_property (GObject      *object,
     case PROP_USE_SMOOTHING:
       smoothing_options->use_smoothing = g_value_get_boolean (value);
       break;
-
     case PROP_SMOOTHING_QUALITY:
       smoothing_options->smoothing_quality = g_value_get_int (value);
       break;
-
     case PROP_SMOOTHING_FACTOR:
       smoothing_options->smoothing_factor = g_value_get_double (value);
       break;
@@ -633,11 +650,11 @@ gimp_paint_options_get_property (GObject    *object,
                                  GValue     *value,
                                  GParamSpec *pspec)
 {
-  GimpPaintOptions     *options           = GIMP_PAINT_OPTIONS (object);
-  GimpFadeOptions      *fade_options      = options->fade_options;
-  GimpJitterOptions    *jitter_options    = options->jitter_options;
-  GimpGradientOptions  *gradient_options  = options->gradient_options;
-  GimpSmoothingOptions *smoothing_options = options->smoothing_options;
+  GimpPaintOptions         *options           = GIMP_PAINT_OPTIONS (object);
+  GimpFadeOptions          *fade_options      = options->fade_options;
+  GimpJitterOptions        *jitter_options    = options->jitter_options;
+  GimpGradientPaintOptions *gradient_options  = options->gradient_options;
+  GimpSmoothingOptions     *smoothing_options = options->smoothing_options;
 
   switch (property_id)
     {
@@ -652,27 +669,18 @@ gimp_paint_options_get_property (GObject    *object,
     case PROP_BRUSH_SIZE:
       g_value_set_double (value, options->brush_size);
       break;
-
-    case PROP_BRUSH_ZOOM:
-      g_value_set_boolean (value, options->brush_zoom);
-      break;
-
     case PROP_BRUSH_ASPECT_RATIO:
       g_value_set_double (value, options->brush_aspect_ratio);
       break;
-
     case PROP_BRUSH_ANGLE:
       g_value_set_double (value, - 1.0 * options->brush_angle * 360.0); /* mathematically correct -> intuitively correct */
       break;
-
     case PROP_BRUSH_SPACING:
       g_value_set_double (value, options->brush_spacing);
       break;
-
     case PROP_BRUSH_HARDNESS:
       g_value_set_double (value, options->brush_hardness);
       break;
-
     case PROP_BRUSH_FORCE:
       g_value_set_double (value, options->brush_force);
       break;
@@ -680,27 +688,26 @@ gimp_paint_options_get_property (GObject    *object,
     case PROP_BRUSH_LINK_SIZE:
       g_value_set_boolean (value, options->brush_link_size);
       break;
-
     case PROP_BRUSH_LINK_ASPECT_RATIO:
       g_value_set_boolean (value, options->brush_link_aspect_ratio);
       break;
-
     case PROP_BRUSH_LINK_ANGLE:
       g_value_set_boolean (value, options->brush_link_angle);
       break;
-
     case PROP_BRUSH_LINK_SPACING:
       g_value_set_boolean (value, options->brush_link_spacing);
       break;
-
     case PROP_BRUSH_LINK_HARDNESS:
       g_value_set_boolean (value, options->brush_link_hardness);
       break;
 
+    case PROP_BRUSH_LOCK_TO_VIEW:
+      g_value_set_boolean (value, options->brush_lock_to_view);
+      break;
+
     case PROP_APPLICATION_MODE:
       g_value_set_enum (value, options->application_mode);
       break;
-
     case PROP_HARD:
       g_value_set_boolean (value, options->hard);
       break;
@@ -708,27 +715,23 @@ gimp_paint_options_get_property (GObject    *object,
     case PROP_USE_JITTER:
       g_value_set_boolean (value, jitter_options->use_jitter);
       break;
-
     case PROP_JITTER_AMOUNT:
       g_value_set_double (value, jitter_options->jitter_amount);
       break;
 
-    case PROP_DYNAMICS_EXPANDED:
-      g_value_set_boolean (value, options->dynamics_expanded);
+    case PROP_DYNAMICS_ENABLED:
+      g_value_set_boolean (value, options->dynamics_enabled);
       break;
 
     case PROP_FADE_LENGTH:
       g_value_set_double (value, fade_options->fade_length);
       break;
-
     case PROP_FADE_REVERSE:
       g_value_set_boolean (value, fade_options->fade_reverse);
       break;
-
     case PROP_FADE_REPEAT:
       g_value_set_enum (value, fade_options->fade_repeat);
       break;
-
     case PROP_FADE_UNIT:
       g_value_set_int (value, fade_options->fade_unit);
       break;
@@ -736,11 +739,16 @@ gimp_paint_options_get_property (GObject    *object,
     case PROP_GRADIENT_REVERSE:
       g_value_set_boolean (value, gradient_options->gradient_reverse);
       break;
+    case PROP_GRADIENT_BLEND_COLOR_SPACE:
+      g_value_set_enum (value, gradient_options->gradient_blend_color_space);
+      break;
+    case PROP_GRADIENT_REPEAT:
+      g_value_set_enum (value, gradient_options->gradient_repeat);
+      break;
 
     case PROP_BRUSH_VIEW_TYPE:
       g_value_set_enum (value, options->brush_view_type);
       break;
-
     case PROP_BRUSH_VIEW_SIZE:
       g_value_set_int (value, options->brush_view_size);
       break;
@@ -748,7 +756,6 @@ gimp_paint_options_get_property (GObject    *object,
     case PROP_DYNAMICS_VIEW_TYPE:
       g_value_set_enum (value, options->dynamics_view_type);
       break;
-
     case PROP_DYNAMICS_VIEW_SIZE:
       g_value_set_int (value, options->dynamics_view_size);
       break;
@@ -756,7 +763,6 @@ gimp_paint_options_get_property (GObject    *object,
     case PROP_PATTERN_VIEW_TYPE:
       g_value_set_enum (value, options->pattern_view_type);
       break;
-
     case PROP_PATTERN_VIEW_SIZE:
       g_value_set_int (value, options->pattern_view_size);
       break;
@@ -764,7 +770,6 @@ gimp_paint_options_get_property (GObject    *object,
     case PROP_GRADIENT_VIEW_TYPE:
       g_value_set_enum (value, options->gradient_view_type);
       break;
-
     case PROP_GRADIENT_VIEW_SIZE:
       g_value_set_int (value, options->gradient_view_size);
       break;
@@ -772,11 +777,9 @@ gimp_paint_options_get_property (GObject    *object,
     case PROP_USE_SMOOTHING:
       g_value_set_boolean (value, smoothing_options->use_smoothing);
       break;
-
     case PROP_SMOOTHING_QUALITY:
       g_value_set_int (value, smoothing_options->smoothing_quality);
       break;
-
     case PROP_SMOOTHING_FACTOR:
       g_value_set_double (value, smoothing_options->smoothing_factor);
       break;
@@ -787,6 +790,64 @@ gimp_paint_options_get_property (GObject    *object,
     }
 }
 
+static void
+gimp_paint_options_brush_changed (GimpContext *context,
+                                  GimpBrush   *brush)
+{
+  GimpPaintOptions *options = GIMP_PAINT_OPTIONS (context);
+
+  if (options->paint_info &&
+      g_type_is_a (options->paint_info->paint_type,
+                   GIMP_TYPE_BRUSH_CORE))
+    {
+      if (options->brush)
+        {
+          g_signal_handlers_disconnect_by_func (options->brush,
+                                                gimp_paint_options_brush_notify,
+                                                options);
+        }
+
+      g_set_weak_pointer (&options->brush, brush);
+
+      if (options->brush)
+        {
+          g_signal_connect_object (options->brush, "notify",
+                                   G_CALLBACK (gimp_paint_options_brush_notify),
+                                   options, 0);
+
+          gimp_paint_options_brush_notify (options->brush, NULL, options);
+        }
+    }
+}
+
+static void
+gimp_paint_options_brush_notify (GimpBrush        *brush,
+                                 const GParamSpec *pspec,
+                                 GimpPaintOptions *options)
+{
+  if (gimp_tool_options_get_gui_mode (GIMP_TOOL_OPTIONS (options)))
+    {
+#define IS_PSPEC(p,n) (p == NULL || ! strcmp (n, p->name))
+
+      if (options->brush_link_size && IS_PSPEC (pspec, "radius"))
+        gimp_paint_options_set_default_brush_size (options, brush);
+
+      if (options->brush_link_aspect_ratio && IS_PSPEC (pspec, "aspect-ratio"))
+        gimp_paint_options_set_default_brush_aspect_ratio (options, brush);
+
+      if (options->brush_link_angle && IS_PSPEC (pspec, "angle"))
+        gimp_paint_options_set_default_brush_angle (options, brush);
+
+      if (options->brush_link_spacing && IS_PSPEC (pspec, "spacing"))
+        gimp_paint_options_set_default_brush_spacing (options, brush);
+
+      if (options->brush_link_hardness && IS_PSPEC (pspec, "hardness"))
+        gimp_paint_options_set_default_brush_hardness (options, brush);
+
+#undef IS_SPEC
+    }
+}
+
 static GimpConfig *
 gimp_paint_options_duplicate (GimpConfig *config)
 {
@@ -943,14 +1004,15 @@ gimp_paint_options_get_gradient_color (GimpPaintOptions *paint_options,
 
   if (gimp_dynamics_is_output_enabled (dynamics, GIMP_DYNAMICS_OUTPUT_COLOR))
     {
-      GimpGradientOptions *gradient_options = paint_options->gradient_options;
-      GimpGradient        *gradient;
+      GimpGradientPaintOptions *gradient_options = paint_options->gradient_options;
+      GimpGradient             *gradient;
 
       gradient = gimp_context_get_gradient (GIMP_CONTEXT (paint_options));
 
       gimp_gradient_get_color_at (gradient, GIMP_CONTEXT (paint_options),
                                   NULL, grad_point,
                                   gradient_options->gradient_reverse,
+                                  gradient_options->gradient_blend_color_space,
                                   color);
 
       return TRUE;
@@ -996,7 +1058,7 @@ gimp_paint_options_set_default_brush_size (GimpPaintOptions *paint_options,
       gint height;
       gint width;
 
-      gimp_brush_transform_size (brush, 1.0, 0.0, 0.0, &height, &width);
+      gimp_brush_transform_size (brush, 1.0, 0.0, 0.0, FALSE, &width, &height);
 
       g_object_set (paint_options,
                     "brush-size", (gdouble) MAX (height, width),
@@ -1105,102 +1167,129 @@ gimp_paint_options_set_default_brush_hardness (GimpPaintOptions *paint_options,
     }
 }
 
-void
-gimp_paint_options_copy_brush_props (GimpPaintOptions *src,
-                                     GimpPaintOptions *dest)
+gboolean
+gimp_paint_options_are_dynamics_enabled (GimpPaintOptions *paint_options)
 {
-  gdouble  brush_size;
-  gboolean brush_zoom;
-  gdouble  brush_angle;
-  gdouble  brush_aspect_ratio;
-  gdouble  brush_spacing;
-  gdouble  brush_hardness;
-  gdouble  brush_force;
+  return paint_options->dynamics_enabled;
+}
 
-  gboolean brush_link_size;
-  gboolean brush_link_angle;
-  gboolean brush_link_aspect_ratio;
-  gboolean brush_link_spacing;
-  gboolean brush_link_hardness;
+void
+gimp_paint_options_enable_dynamics (GimpPaintOptions *paint_options,
+                                    gboolean          enable)
+{
+  if (paint_options->dynamics_enabled != enable)
+    {
+      g_object_set (paint_options,
+                    "dynamics-enabled", enable,
+                    NULL);
+    }
+}
+
+static const gchar *brush_props[] =
+{
+  "brush-size",
+  "brush-angle",
+  "brush-aspect-ratio",
+  "brush-spacing",
+  "brush-hardness",
+  "brush-force",
+  "brush-link-size",
+  "brush-link-angle",
+  "brush-link-aspect-ratio",
+  "brush-link-spacing",
+  "brush-link-hardness",
+  "brush-lock-to-view"
+};
+
+static const gchar *dynamics_props[] =
+{
+  "dynamics-enabled",
+  "fade-reverse",
+  "fade-length",
+  "fade-unit",
+  "fade-repeat"
+};
+
+static const gchar *gradient_props[] =
+{
+  "gradient-reverse",
+  "gradient-blend-color-space",
+  "gradient-repeat"
+};
+
+static const gint max_n_props = (G_N_ELEMENTS (brush_props) +
+                                 G_N_ELEMENTS (dynamics_props) +
+                                 G_N_ELEMENTS (gradient_props));
+
+gboolean
+gimp_paint_options_is_prop (const gchar         *prop_name,
+                            GimpContextPropMask  prop_mask)
+{
+  gint i;
+
+  g_return_val_if_fail (prop_name != NULL, FALSE);
+
+  if (prop_mask & GIMP_CONTEXT_PROP_MASK_BRUSH)
+    {
+      for (i = 0; i < G_N_ELEMENTS (brush_props); i++)
+        if (! strcmp (prop_name, brush_props[i]))
+          return TRUE;
+    }
+
+  if (prop_mask & GIMP_CONTEXT_PROP_MASK_DYNAMICS)
+    {
+      for (i = 0; i < G_N_ELEMENTS (dynamics_props); i++)
+        if (! strcmp (prop_name, dynamics_props[i]))
+          return TRUE;
+    }
+
+  if (prop_mask & GIMP_CONTEXT_PROP_MASK_GRADIENT)
+    {
+      for (i = 0; i < G_N_ELEMENTS (gradient_props); i++)
+        if (! strcmp (prop_name, gradient_props[i]))
+          return TRUE;
+    }
+
+  return FALSE;
+}
+
+void
+gimp_paint_options_copy_props (GimpPaintOptions    *src,
+                               GimpPaintOptions    *dest,
+                               GimpContextPropMask  prop_mask)
+{
+  const gchar *names[max_n_props];
+  GValue       values[max_n_props];
+  gint         n_props = 0;
+  gint         i;
 
   g_return_if_fail (GIMP_IS_PAINT_OPTIONS (src));
   g_return_if_fail (GIMP_IS_PAINT_OPTIONS (dest));
 
-  g_object_get (src,
-                "brush-size",              &brush_size,
-                "brush-zoom",              &brush_zoom,
-                "brush-angle",             &brush_angle,
-                "brush-aspect-ratio",      &brush_aspect_ratio,
-                "brush-spacing",           &brush_spacing,
-                "brush-hardness",          &brush_hardness,
-                "brush-force",             &brush_force,
-                "brush-link-size",         &brush_link_size,
-                "brush-link-angle",        &brush_link_angle,
-                "brush-link-aspect-ratio", &brush_link_aspect_ratio,
-                "brush-link-spacing",      &brush_link_spacing,
-                "brush-link-hardness",     &brush_link_hardness,
-                NULL);
+  if (prop_mask & GIMP_CONTEXT_PROP_MASK_BRUSH)
+    {
+      for (i = 0; i < G_N_ELEMENTS (brush_props); i++)
+        names[n_props++] = brush_props[i];
+    }
 
-  g_object_set (dest,
-                "brush-size",              brush_size,
-                "brush-zoom",              brush_zoom,
-                "brush-angle",             brush_angle,
-                "brush-aspect-ratio",      brush_aspect_ratio,
-                "brush-spacing",           brush_spacing,
-                "brush-hardness",          brush_hardness,
-                "brush-force",             brush_force,
-                "brush-link-size",         brush_link_size,
-                "brush-link-angle",        brush_link_angle,
-                "brush-link-aspect-ratio", brush_link_aspect_ratio,
-                "brush-link-spacing",      brush_link_spacing,
-                "brush-link-hardness",     brush_link_hardness,
-                NULL);
-}
-
-void
-gimp_paint_options_copy_dynamics_props (GimpPaintOptions *src,
-                                        GimpPaintOptions *dest)
-{
-  gboolean       dynamics_expanded;
-  gboolean       fade_reverse;
-  gdouble        fade_length;
-  GimpUnit       fade_unit;
-  GimpRepeatMode fade_repeat;
-
-  g_return_if_fail (GIMP_IS_PAINT_OPTIONS (src));
-  g_return_if_fail (GIMP_IS_PAINT_OPTIONS (dest));
-
-  g_object_get (src,
-                "dynamics-expanded", &dynamics_expanded,
-                "fade-reverse",      &fade_reverse,
-                "fade-length",       &fade_length,
-                "fade-unit",         &fade_unit,
-                "fade-repeat",       &fade_repeat,
-                NULL);
-
-  g_object_set (dest,
-                "dynamics-expanded", dynamics_expanded,
-                "fade-reverse",      fade_reverse,
-                "fade-length",       fade_length,
-                "fade-unit",         fade_unit,
-                "fade-repeat",       fade_repeat,
-                NULL);
-}
-
-void
-gimp_paint_options_copy_gradient_props (GimpPaintOptions *src,
-                                        GimpPaintOptions *dest)
-{
-  gboolean gradient_reverse;
-
-  g_return_if_fail (GIMP_IS_PAINT_OPTIONS (src));
-  g_return_if_fail (GIMP_IS_PAINT_OPTIONS (dest));
-
-  g_object_get (src,
-                "gradient-reverse", &gradient_reverse,
-                NULL);
-
-  g_object_set (dest,
-                "gradient-reverse", gradient_reverse,
-                NULL);
+  if (prop_mask & GIMP_CONTEXT_PROP_MASK_DYNAMICS)
+    {
+      for (i = 0; i < G_N_ELEMENTS (dynamics_props); i++)
+        names[n_props++] = dynamics_props[i];
+    }
+
+  if (prop_mask & GIMP_CONTEXT_PROP_MASK_GRADIENT)
+    {
+      for (i = 0; i < G_N_ELEMENTS (gradient_props); i++)
+        names[n_props++] = gradient_props[i];
+    }
+
+  if (n_props > 0)
+    {
+      g_object_getv (G_OBJECT (src), n_props, names, values);
+      g_object_setv (G_OBJECT (dest), n_props, names, values);
+
+      while (n_props--)
+        g_value_unset (&values[n_props]);
+    }
 }
