@@ -34,6 +34,8 @@
 #include "display/gimpdisplayshell-transform.h"
 #include "display/gimpimagewindow.h"
 
+#include "menus/menus.h"
+
 #include "widgets/gimpdialogfactory.h"
 #include "widgets/gimpdock.h"
 #include "widgets/gimpdockable.h"
@@ -59,9 +61,10 @@
 #include "core/gimptoolinfo.h"
 #include "core/gimptooloptions.h"
 
-#include "tests.h"
+#include "gimpcoreapp.h"
 
 #include "gimp-app-test-utils.h"
+#include "tests.h"
 
 
 #define GIMP_UI_WINDOW_POSITION_EPSILON 30
@@ -111,8 +114,7 @@ tool_options_editor_updates (gconstpointer data)
   GimpDisplay           *display      = GIMP_DISPLAY (gimp_get_empty_display (gimp));
   GimpDisplayShell      *shell        = gimp_display_get_shell (display);
   GtkWidget             *toplevel     = gtk_widget_get_toplevel (GTK_WIDGET (shell));
-  GimpImageWindow       *image_window = GIMP_IMAGE_WINDOW (toplevel);
-  GimpUIManager         *ui_manager   = gimp_image_window_get_ui_manager (image_window);
+  GimpUIManager         *ui_manager   = menus_get_image_manager_singleton (gimp);
   GtkWidget             *dockable     = gimp_dialog_factory_dialog_new (gimp_dialog_factory_get_singleton (),
                                                                         gimp_widget_get_monitor (toplevel),
                                                                         NULL /*ui_manager*/,
@@ -362,10 +364,6 @@ restore_recently_closed_multi_column_dock (gconstpointer data)
                    >,
                    n_session_infos_after_close);
 
-#ifdef __GNUC__
-#warning FIXME test disabled until we depend on GTK+ >= 2.24.11
-#endif
-#if 0
   /* Restore the (only available) closed dock and make sure the session
    * infos in the global dock factory are increased again
    */
@@ -379,7 +377,6 @@ restore_recently_closed_multi_column_dock (gconstpointer data)
   g_assert_cmpint (n_session_infos_after_close,
                    <,
                    n_session_infos_after_restore);
-#endif
 }
 
 /**
@@ -864,8 +861,6 @@ int main(int argc, char **argv)
   gimp_test_bail_if_no_display ();
   gtk_test_init (&argc, &argv, NULL);
 
-  gimp_test_utils_set_gimp3_directory ("GIMP_TESTING_ABS_TOP_SRCDIR",
-                                       "app/tests/gimpdir");
   gimp_test_utils_setup_menus_path ();
 
   /* Start up GIMP */
@@ -884,12 +879,12 @@ int main(int argc, char **argv)
   ADD_TEST (restore_recently_closed_multi_column_dock);
   ADD_TEST (tab_toggle_dont_change_dock_window_position);
   ADD_TEST (switch_to_single_window_mode);
-#warning FIXME: hide/show docks doesn't work when running make check
+#warning "FIXME: hide/show docks doesn't work when running make check"
 #if 0
   ADD_TEST (hide_docks_in_single_window_mode);
   ADD_TEST (show_docks_in_single_window_mode);
 #endif
-#warning FIXME: maximize_state_in_aux_data doesn't work without WM
+#warning "FIXME: maximize_state_in_aux_data doesn't work without WM"
 #if 0
   ADD_TEST (maximize_state_in_aux_data);
 #endif
@@ -899,14 +894,11 @@ int main(int argc, char **argv)
   ADD_TEST (window_roles);
 
   /* Run the tests and return status */
-  result = g_test_run ();
+  g_application_run (gimp->app, 0, NULL);
+  result = gimp_core_app_get_exit_status (GIMP_CORE_APP (gimp->app));
 
-  /* Don't write files to the source dir */
-  gimp_test_utils_set_gimp3_directory ("GIMP_TESTING_ABS_TOP_BUILDDIR",
-                                       "app/tests/gimpdir-output");
-
-  /* Exit properly so we don't break script-fu plug-in wire */
-  gimp_exit (gimp, TRUE);
+  g_application_quit (G_APPLICATION (gimp->app));
+  g_clear_object (&gimp->app);
 
   return result;
 }
