@@ -5,9 +5,9 @@
  * Copyright (C) 2003 Michael Natterer <mitch@gimp.org>
  *                    Sven Neumann <sven@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,24 +16,31 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
-#include <glib-object.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gegl.h>
 
-#define PANGO_ENABLE_BACKEND 1   /* Argh */
-#include <pango/pangoft2.h>
+#include <pango/pangocairo.h>
+
+#include <hb.h>
+#include <hb-ot.h>
+#include <hb-ft.h>
 
 #define PANGO_ENABLE_ENGINE  1   /* Argh */
 #include <pango/pango-ot.h>
-#include <freetype/tttables.h>
+
+#include <ft2build.h>
+#include FT_TRUETYPE_TABLES_H
+
+#include "libgimpbase/gimpbase.h"
 
 #include "text-types.h"
 
-#include "base/temp-buf.h"
+#include "core/gimptempbuf.h"
 
 #include "gimpfont.h"
 
@@ -57,7 +64,7 @@ enum
 
 struct _GimpFont
 {
-  GimpViewable  parent_instance;
+  GimpData      parent_instance;
 
   PangoContext *pango_context;
 
@@ -68,38 +75,39 @@ struct _GimpFont
 
 struct _GimpFontClass
 {
-  GimpViewableClass   parent_class;
+  GimpDataClass   parent_class;
 };
 
 
-static void      gimp_font_finalize         (GObject       *object);
-static void      gimp_font_set_property     (GObject       *object,
-                                             guint          property_id,
-                                             const GValue  *value,
-                                             GParamSpec    *pspec);
+static void          gimp_font_constructed      (GObject       *object);
+static void          gimp_font_finalize         (GObject       *object);
+static void          gimp_font_set_property     (GObject       *object,
+                                                 guint          property_id,
+                                                 const GValue  *value,
+                                                 GParamSpec    *pspec);
 
-static void      gimp_font_get_preview_size (GimpViewable  *viewable,
-                                             gint           size,
-                                             gboolean       popup,
-                                             gboolean       dot_for_dot,
-                                             gint          *width,
-                                             gint          *height);
-static gboolean  gimp_font_get_popup_size   (GimpViewable  *viewable,
-                                             gint           width,
-                                             gint           height,
-                                             gboolean       dot_for_dot,
-                                             gint          *popup_width,
-                                             gint          *popup_height);
-static TempBuf * gimp_font_get_new_preview  (GimpViewable  *viewable,
-                                             GimpContext   *context,
-                                             gint           width,
-                                             gint           height);
+static void          gimp_font_get_preview_size (GimpViewable  *viewable,
+                                                 gint           size,
+                                                 gboolean       popup,
+                                                 gboolean       dot_for_dot,
+                                                 gint          *width,
+                                                 gint          *height);
+static gboolean      gimp_font_get_popup_size   (GimpViewable  *viewable,
+                                                 gint           width,
+                                                 gint           height,
+                                                 gboolean       dot_for_dot,
+                                                 gint          *popup_width,
+                                                 gint          *popup_height);
+static GimpTempBuf * gimp_font_get_new_preview  (GimpViewable  *viewable,
+                                                 GimpContext   *context,
+                                                 gint           width,
+                                                 gint           height);
 
 static const gchar * gimp_font_get_sample_string (PangoContext         *context,
                                                   PangoFontDescription *font_desc);
 
 
-G_DEFINE_TYPE (GimpFont, gimp_font, GIMP_TYPE_VIEWABLE)
+G_DEFINE_TYPE (GimpFont, gimp_font, GIMP_TYPE_DATA)
 
 #define parent_class gimp_font_parent_class
 
@@ -110,14 +118,15 @@ gimp_font_class_init (GimpFontClass *klass)
   GObjectClass      *object_class   = G_OBJECT_CLASS (klass);
   GimpViewableClass *viewable_class = GIMP_VIEWABLE_CLASS (klass);
 
-  object_class->finalize     = gimp_font_finalize;
-  object_class->set_property = gimp_font_set_property;
+  object_class->constructed         = gimp_font_constructed;
+  object_class->finalize            = gimp_font_finalize;
+  object_class->set_property        = gimp_font_set_property;
 
-  viewable_class->get_preview_size = gimp_font_get_preview_size;
-  viewable_class->get_popup_size   = gimp_font_get_popup_size;
-  viewable_class->get_new_preview  = gimp_font_get_new_preview;
+  viewable_class->get_preview_size  = gimp_font_get_preview_size;
+  viewable_class->get_popup_size    = gimp_font_get_popup_size;
+  viewable_class->get_new_preview   = gimp_font_get_new_preview;
 
-  viewable_class->default_stock_id = "gtk-select-font";
+  viewable_class->default_icon_name = "gtk-select-font";
 
   g_object_class_install_property (object_class, PROP_PANGO_CONTEXT,
                                    g_param_spec_object ("pango-context",
@@ -129,7 +138,15 @@ gimp_font_class_init (GimpFontClass *klass)
 static void
 gimp_font_init (GimpFont *font)
 {
-  font->pango_context = NULL;
+}
+
+static void
+gimp_font_constructed (GObject *object)
+{
+  G_OBJECT_CLASS (parent_class)->constructed (object);
+
+  gimp_data_make_internal (GIMP_DATA (object),
+                           gimp_object_get_name (object));
 }
 
 static void
@@ -137,17 +154,8 @@ gimp_font_finalize (GObject *object)
 {
   GimpFont *font = GIMP_FONT (object);
 
-  if (font->pango_context)
-    {
-      g_object_unref (font->pango_context);
-      font->pango_context = NULL;
-    }
-
-  if (font->popup_layout)
-    {
-      g_object_unref (font->popup_layout);
-      font->popup_layout = NULL;
-    }
+  g_clear_object (&font->pango_context);
+  g_clear_object (&font->popup_layout);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -203,7 +211,7 @@ gimp_font_get_popup_size (GimpViewable *viewable,
   if (! font->pango_context)
     return FALSE;
 
-  name = gimp_object_get_name (GIMP_OBJECT (font));
+  name = gimp_object_get_name (font);
 
   font_desc = pango_font_description_from_string (name);
   g_return_val_if_fail (font_desc != NULL, FALSE);
@@ -223,30 +231,31 @@ gimp_font_get_popup_size (GimpViewable *viewable,
   *popup_width  = MAX (ink.width,  logical.width)  + 6;
   *popup_height = MAX (ink.height, logical.height) + 6;
 
+  *popup_width = cairo_format_stride_for_width (CAIRO_FORMAT_A8, *popup_width);
+
   font->popup_width  = *popup_width;
   font->popup_height = *popup_height;
 
   return TRUE;
 }
 
-static TempBuf *
+static GimpTempBuf *
 gimp_font_get_new_preview (GimpViewable *viewable,
                            GimpContext  *context,
                            gint          width,
                            gint          height)
 {
-  GimpFont       *font = GIMP_FONT (viewable);
-  PangoLayout    *layout;
-  PangoRectangle  ink;
-  PangoRectangle  logical;
-  gint            layout_width;
-  gint            layout_height;
-  gint            layout_x;
-  gint            layout_y;
-  TempBuf        *temp_buf;
-  FT_Bitmap       bitmap;
-  guchar         *p;
-  guchar          black = 0;
+  GimpFont        *font = GIMP_FONT (viewable);
+  PangoLayout     *layout;
+  PangoRectangle   ink;
+  PangoRectangle   logical;
+  gint             layout_width;
+  gint             layout_height;
+  gint             layout_x;
+  gint             layout_y;
+  GimpTempBuf     *temp_buf;
+  cairo_t         *cr;
+  cairo_surface_t *surface;
 
   if (! font->pango_context)
     return NULL;
@@ -257,7 +266,7 @@ gimp_font_get_new_preview (GimpViewable *viewable,
       PangoFontDescription *font_desc;
       const gchar          *name;
 
-      name = gimp_object_get_name (GIMP_OBJECT (font));
+      name = gimp_object_get_name (font);
 
       DEBUGPRINT (("%s: ", name));
 
@@ -282,20 +291,22 @@ gimp_font_get_new_preview (GimpViewable *viewable,
       layout = g_object_ref (font->popup_layout);
     }
 
-  temp_buf = temp_buf_new (width, height, 1, 0, 0, &black);
+  width = cairo_format_stride_for_width (CAIRO_FORMAT_A8, width);
 
-  bitmap.width  = temp_buf->width;
-  bitmap.rows   = temp_buf->height;
-  bitmap.pitch  = temp_buf->width;
-  bitmap.buffer = temp_buf_data (temp_buf);
+  temp_buf = gimp_temp_buf_new (width, height, babl_format ("Y' u8"));
+  memset (gimp_temp_buf_get_data (temp_buf), 255, width * height);
+
+  surface = cairo_image_surface_create_for_data (gimp_temp_buf_get_data (temp_buf),
+                                                 CAIRO_FORMAT_A8,
+                                                 width, height, width);
 
   pango_layout_get_pixel_extents (layout, &ink, &logical);
 
   layout_width  = MAX (ink.width,  logical.width);
   layout_height = MAX (ink.height, logical.height);
 
-  layout_x = (bitmap.width - layout_width)  / 2;
-  layout_y = (bitmap.rows  - layout_height) / 2;
+  layout_x = (width - layout_width)  / 2;
+  layout_y = (height - layout_height) / 2;
 
   if (ink.x < logical.x)
     layout_x += logical.x - ink.x;
@@ -303,55 +314,80 @@ gimp_font_get_new_preview (GimpViewable *viewable,
   if (ink.y < logical.y)
     layout_y += logical.y - ink.y;
 
-  pango_ft2_render_layout (&bitmap, layout, layout_x, layout_y);
+  cr = cairo_create (surface);
+
+  cairo_translate (cr, layout_x, layout_y);
+  cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
+  pango_cairo_show_layout (cr, layout);
+
+  cairo_destroy (cr);
+  cairo_surface_destroy (surface);
 
   g_object_unref (layout);
 
-  p = temp_buf_data (temp_buf);
-
-  for (height = temp_buf->height; height; height--)
-    for (width = temp_buf->width; width; width--, p++)
-      *p = 255 - *p;
-
   return temp_buf;
 }
 
-GimpFont *
+GimpData *
 gimp_font_get_standard (void)
 {
-  static GimpFont *standard_font = NULL;
+  static GimpData *standard_font = NULL;
 
   if (! standard_font)
-    standard_font = g_object_new (GIMP_TYPE_FONT,
-                                  "name", "Sans",
-                                  NULL);
+    {
+      g_set_weak_pointer (&standard_font,
+                          g_object_new (GIMP_TYPE_FONT,
+                                        "name", "Standard",
+                                        NULL));;
+
+      gimp_data_clean (standard_font);
+      gimp_data_make_internal (standard_font, "gimp-font-standard");
+    }
 
   return standard_font;
 }
 
 
 static inline gboolean
-gimp_font_covers_string (PangoFcFont *font,
+gimp_font_covers_string (PangoFont   *font,
                          const gchar *sample)
 {
   const gchar *p;
 
   for (p = sample; *p; p = g_utf8_next_char (p))
     {
-      if (! pango_fc_font_has_char (font, g_utf8_get_char (p)))
+      if (! pango_font_has_char (font, g_utf8_get_char (p)))
         return FALSE;
     }
 
   return TRUE;
 }
 
+/* This function was picked up from Pango's pango-ot-info.c. Until there
+ * is a better way to get the tag, we use this.
+ */
+static hb_tag_t
+get_hb_table_type (PangoOTTableType table_type)
+{
+  switch (table_type)
+    {
+    case PANGO_OT_TABLE_GSUB:
+      return HB_OT_TAG_GSUB;
+    case PANGO_OT_TABLE_GPOS:
+      return HB_OT_TAG_GPOS;
+    default:
+      return HB_TAG_NONE;
+    }
+}
+
 /* Guess a suitable short sample string for the font. */
 static const gchar *
 gimp_font_get_sample_string (PangoContext         *context,
                              PangoFontDescription *font_desc)
 {
   PangoFont        *font;
-  PangoOTInfo      *ot_info;
+  hb_face_t        *hb_face;
+  hb_font_t        *hb_font;
   FT_Face           face;
   TT_OS2           *os2;
   PangoOTTableType  tt;
@@ -360,10 +396,10 @@ gimp_font_get_sample_string (PangoContext         *context,
   /* This is a table of scripts and corresponding short sample strings
    * to be used instead of the Latin sample string Aa. The script
    * codes are as in ISO15924 (see
-   * http://www.unicode.org/iso15924/iso15924-codes.html), but in
+   * https://www.unicode.org/iso15924/iso15924-codes.html), but in
    * lower case. The Unicode subrange bit numbers, as used in TrueType
    * so-called OS/2 tables, are from
-   * http://www.microsoft.com/typography/otspec/os2.htm#ur .
+   * https://www.microsoft.com/typography/otspec/os2.htm#ur .
    *
    * The table is mostly ordered by Unicode order. But as there are
    * fonts that support several of these scripts, the ordering is
@@ -379,7 +415,7 @@ gimp_font_get_sample_string (PangoContext         *context,
    * This table is used to determine the primary script a font has
    * been designed for.
    *
-   * Very useful link: http://www.travelphrases.info/fonts.html
+   * Very useful link: https://www.wazu.jp/index.html
    */
   static const struct
   {
@@ -387,12 +423,40 @@ gimp_font_get_sample_string (PangoContext         *context,
     gint         bit;
     const gchar *sample;
   } scripts[] = {
-    /* Han is first because fonts that support it presumably are primarily
-     * designed for it.
+    /* CJK are first because fonts that support these characters
+     * presumably are primarily designed for it.
+     *
+     * Though not a universal rules, most fonts targeting Korean will
+     * also have the ideographs as they were used historically (and
+     * still in some cases). As for Japanese, ideographs are still
+     * definitely used daily. After all the block is called "*CJK*
+     * (Chinese-Japanese-Korean) Unified Ideographs". So let's give the
+     * Chinese representation less priority towards finding
+     * Korean/Japanese targeting.
+     * Then we prioritize Korean because many fonts for Korean also
+     * design Hiragana/Katakana blocks. On the other hand, I could see
+     * very few (none so far) Japanese fonts designing also Hangul
+     * characters. So if we see both Hiragana and Hangul, we can assume
+     * this is a font for Korean market, whereas Hiragana only is for
+     * Japanese.
+     * XXX: of course we could probably come with a better algorithm to
+     * determine the target language. Probably looking at more than the
+     * existence of a single block would help (since some languages are
+     * spread on several blocks).
      */
     {
-      "hani",                   /* Han Ideographic */
-      59,
+      "hang",                   /* Korean alphabet (Hangul)   */
+      56,                       /* Hangul Syllables block     */
+      "\355\225\234"            /* U+D55C HANGUL SYLLABLE HAN */
+    },
+    {
+      "japa",                   /* Japanese                   */
+      49,                       /* Hiragana block             */
+      "\343\201\202"            /* U+3042 HIRAGANA LETTER A   */
+    },
+    {
+      "hani",                   /* Han Ideographic            */
+      59,                       /* CJK Unified Ideographs     */
       "\346\260\270"            /* U+6C38 "forever". Ed Trager says
                                  * this is a "pan-stroke" often used
                                  * in teaching Chinese calligraphy. It
@@ -539,13 +603,6 @@ gimp_font_get_sample_string (PangoContext         *context,
                                   * U+10D0 GEORGIAN LETTER AN
                                   */
     },
-    {
-      "hang",                   /* Hangul */
-      28,
-      "\341\204\200\341\204\201"/* U+1100 HANGUL CHOSEONG KIYEOK
-                                 * U+1101 HANGUL CHOSEONG SSANGKIYEOK
-                                 */
-    },
     {
       "ethi",                   /* Ethiopic */
       75,
@@ -635,8 +692,29 @@ gimp_font_get_sample_string (PangoContext         *context,
 
   g_return_val_if_fail (PANGO_IS_FC_FONT (font), "Aa");
 
-  face = pango_fc_font_lock_face (PANGO_FC_FONT (font));
-  ot_info = pango_ot_info_get (face);
+  hb_font = pango_font_get_hb_font (font);
+  g_return_val_if_fail (hb_font != NULL, "Aa");
+
+  /* These are needed to set hb_font to the right internal format, which
+   * can only work if the font is not immutable. Hence we make a copy.
+   */
+  hb_font = hb_font_create_sub_font (hb_font);
+  hb_ft_font_set_funcs (hb_font);
+  /* TODO: use hb_ft_font_lock_face/hb_ft_font_unlock_face() when we
+   * bump to harfbuzz >= 2.6.5.
+   */
+  face = hb_ft_font_get_face (hb_font);
+
+  /* Are there actual cases where this function could return NULL while
+   * it's not a bug in the code?
+   * For instance if the font file is broken, we don't want to return a
+   * CRITICAL, but properly address the issue (removing the font with a
+   * warning or whatnot).
+   * See #5922.
+   */
+  g_return_val_if_fail (face != NULL, "Aa");
+
+  hb_face = hb_ft_face_create (face, NULL);
 
   /* First check what script(s), if any, the font has GSUB or GPOS
    * OpenType layout tables for.
@@ -645,42 +723,50 @@ gimp_font_get_sample_string (PangoContext         *context,
        n_ot_alts < G_N_ELEMENTS (ot_alts) && tt <= PANGO_OT_TABLE_GPOS;
        tt++)
     {
-      PangoOTTag *slist = pango_ot_info_list_scripts (ot_info, tt);
+      hb_tag_t tag;
+      unsigned int count;
+      PangoOTTag *slist;
 
-      if (slist)
+      tag = get_hb_table_type (tt);
+      count = hb_ot_layout_table_get_script_tags (hb_face, tag, 0, NULL, NULL);
+      slist = g_new (PangoOTTag, count + 1);
+      hb_ot_layout_table_get_script_tags (hb_face, tag, 0, &count, slist);
+      slist[count] = 0;
+
+      for (i = 0;
+           n_ot_alts < G_N_ELEMENTS (ot_alts) && i < G_N_ELEMENTS (scripts);
+           i++)
         {
-          for (i = 0;
-               n_ot_alts < G_N_ELEMENTS (ot_alts) && i < G_N_ELEMENTS (scripts);
-               i++)
+          gint j, k;
+
+          for (k = 0; k < n_ot_alts; k++)
+            if (ot_alts[k] == i)
+              break;
+
+          if (k == n_ot_alts)
             {
-              gint j, k;
-
-              for (k = 0; k < n_ot_alts; k++)
-                if (ot_alts[k] == i)
-                  break;
-
-              if (k == n_ot_alts)
-                for (j = 0;
-                     n_ot_alts < G_N_ELEMENTS (ot_alts) && slist[j];
-                     j++)
-                  {
+              for (j = 0;
+                   n_ot_alts < G_N_ELEMENTS (ot_alts) && slist[j];
+                   j++)
+                {
 #define TAG(s) FT_MAKE_TAG (s[0], s[1], s[2], s[3])
 
-                    if (slist[j] == TAG (scripts[i].script) &&
-                        gimp_font_covers_string (PANGO_FC_FONT (font),
-                                                 scripts[i].sample))
-                      {
-                        ot_alts[n_ot_alts++] = i;
-                        DEBUGPRINT (("%.4s ", scripts[i].script));
-                      }
+                  if (slist[j] == TAG (scripts[i].script) &&
+                      gimp_font_covers_string (font, scripts[i].sample))
+                    {
+                      ot_alts[n_ot_alts++] = i;
+                      DEBUGPRINT (("%.4s ", scripts[i].script));
+                    }
 #undef TAG
-                  }
+                }
             }
-
-          g_free (slist);
         }
+
+      g_free (slist);
     }
 
+  hb_face_destroy (hb_face);
+
   DEBUGPRINT (("; OS/2: "));
 
   /* Next check the OS/2 table for Unicode ranges the font claims
@@ -697,8 +783,7 @@ gimp_font_get_sample_string (PangoContext         *context,
         {
           if (scripts[i].bit >= 0 &&
               (&os2->ulUnicodeRange1)[scripts[i].bit/32] & (1 << (scripts[i].bit % 32)) &&
-              gimp_font_covers_string (PANGO_FC_FONT (font),
-                                       scripts[i].sample))
+              gimp_font_covers_string (font, scripts[i].sample))
             {
               sr_alts[n_sr_alts++] = i;
               DEBUGPRINT (("%.4s ", scripts[i].script));
@@ -706,8 +791,7 @@ gimp_font_get_sample_string (PangoContext         *context,
         }
     }
 
-  pango_fc_font_unlock_face (PANGO_FC_FONT (font));
-
+  hb_font_destroy (hb_font);
   g_object_unref (font);
 
   if (n_ot_alts > 2)
