@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -30,10 +30,13 @@
 
 #include "tools-types.h"
 
+#include "config/gimpguiconfig.h"
+
 #include "gegl/gimp-gegl-utils.h"
 
 #include "operations/gimpcageconfig.h"
 
+#include "core/gimp.h"
 #include "core/gimpdrawablefilter.h"
 #include "core/gimperror.h"
 #include "core/gimpimage.h"
@@ -44,11 +47,13 @@
 #include "widgets/gimphelp-ids.h"
 #include "widgets/gimpwidgets-utils.h"
 
+#include "display/gimpcanvasitem.h"
 #include "display/gimpdisplay.h"
 
 #include "gimpcagetool.h"
 #include "gimpcageoptions.h"
 #include "gimptoolcontrol.h"
+#include "gimptools-utils.h"
 
 #include "gimp-intl.h"
 
@@ -194,6 +199,8 @@ gimp_cage_tool_init (GimpCageTool *self)
                                      GIMP_DIRTY_SELECTION       |
                                      GIMP_DIRTY_ACTIVE_DRAWABLE);
   gimp_tool_control_set_wants_click (tool->control, TRUE);
+  gimp_tool_control_set_precision   (tool->control,
+                                     GIMP_CURSOR_PRECISION_SUBPIXEL);
   gimp_tool_control_set_tool_cursor (tool->control,
                                      GIMP_TOOL_CURSOR_PERSPECTIVE);
 
@@ -207,11 +214,26 @@ gimp_cage_tool_initialize (GimpTool     *tool,
                            GimpDisplay  *display,
                            GError      **error)
 {
-  GimpImage    *image    = gimp_display_get_image (display);
-  GimpDrawable *drawable = gimp_image_get_active_drawable (image);
+  GimpGuiConfig *config      = GIMP_GUI_CONFIG (display->gimp->config);
+  GimpImage     *image       = gimp_display_get_image (display);
+  GimpItem      *locked_item = NULL;
+  GList         *drawables   = gimp_image_get_selected_drawables (image);
+  GimpDrawable  *drawable;
 
-  if (! drawable)
-    return FALSE;
+  if (g_list_length (drawables) != 1)
+    {
+      if (g_list_length (drawables) > 1)
+        g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
+                                   _("Cannot modify multiple layers. Select only one layer."));
+      else
+        g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED, _("No selected drawables."));
+
+      g_list_free (drawables);
+      return FALSE;
+    }
+
+  drawable = drawables->data;
+  g_list_free (drawables);
 
   if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
     {
@@ -220,17 +242,20 @@ gimp_cage_tool_initialize (GimpTool     *tool,
       return FALSE;
     }
 
-  if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+  if (gimp_item_is_content_locked (GIMP_ITEM (drawable), &locked_item))
     {
       g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
-                           _("The active layer's pixels are locked."));
+                           _("The selected item's pixels are locked."));
+      if (error)
+        gimp_tools_blink_lock_box (display->gimp, locked_item);
       return FALSE;
     }
 
-  if (! gimp_item_is_visible (GIMP_ITEM (drawable)))
+  if (! gimp_item_is_visible (GIMP_ITEM (drawable)) &&
+      ! config->edit_non_visible)
     {
       g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
-                           _("The active layer is not visible."));
+                           _("The active item is not visible."));
       return FALSE;
     }
 
@@ -299,7 +324,7 @@ gimp_cage_tool_button_press (GimpTool            *tool,
   switch (ct->tool_state)
     {
     case CAGE_STATE_INIT:
-      /* No handle yet, we add the first one and swith the tool to
+      /* No handle yet, we add the first one and switch the tool to
        * moving handle state.
        */
       gimp_cage_config_add_cage_point (ct->config,
@@ -605,7 +630,6 @@ gimp_cage_tool_key_press (GimpTool    *tool,
       else if (ct->tool_state == DEFORM_STATE_WAIT)
         {
           gimp_tool_control (tool, GIMP_TOOL_ACTION_COMMIT, display);
-          gimp_tool_control (tool, GIMP_TOOL_ACTION_HALT, display);
         }
       return TRUE;
 
@@ -661,6 +685,7 @@ gimp_cage_tool_cursor_update (GimpTool         *tool,
 {
   GimpCageTool       *ct       = GIMP_CAGE_TOOL (tool);
   GimpCageOptions    *options  = GIMP_CAGE_TOOL_GET_OPTIONS (ct);
+  GimpGuiConfig      *config   = GIMP_GUI_CONFIG (display->gimp->config);
   GimpCursorModifier  modifier = GIMP_CURSOR_MODIFIER_PLUS;
 
   if (tool->display)
@@ -682,12 +707,19 @@ gimp_cage_tool_cursor_update (GimpTool         *tool,
     }
   else
     {
-      GimpImage    *image    = gimp_display_get_image (display);
-      GimpDrawable *drawable = gimp_image_get_active_drawable (image);
+      GimpImage    *image     = gimp_display_get_image (display);
+      GList        *drawables = gimp_image_get_selected_drawables (image);
+      GimpDrawable *drawable  = NULL;
 
-      if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)) ||
-          gimp_item_is_content_locked (GIMP_ITEM (drawable))    ||
-          ! gimp_item_is_visible (GIMP_ITEM (drawable)))
+      if (g_list_length (drawables) == 1)
+        drawable = drawables->data;
+      g_list_free (drawables);
+
+      if (! drawable                                               ||
+          gimp_viewable_get_children (GIMP_VIEWABLE (drawable))    ||
+          gimp_item_is_content_locked (GIMP_ITEM (drawable), NULL) ||
+          ! (gimp_item_is_visible (GIMP_ITEM (drawable)) ||
+             config->edit_non_visible))
         {
           modifier = GIMP_CURSOR_MODIFIER_BAD;
         }
@@ -823,7 +855,8 @@ gimp_cage_tool_draw (GimpDrawTool *draw_tool)
   /* Draw the cage with handles. */
   for (i = 0; i < n_vertices; i++)
     {
-      GimpVector2 point1, point2;
+      GimpCanvasItem *item;
+      GimpVector2     point1, point2;
 
       point1 = gimp_cage_config_get_point_coordinate (ct->config,
                                                       options->cage_mode,
@@ -846,40 +879,54 @@ gimp_cage_tool_draw (GimpDrawTool *draw_tool)
           point2.x += ct->offset_x;
           point2.y += ct->offset_y;
 
-          gimp_draw_tool_push_group (draw_tool, stroke_group);
+          if (i != ct->hovering_edge ||
+              gimp_cage_tool_is_complete (ct))
+            {
+              gimp_draw_tool_push_group (draw_tool, stroke_group);
+            }
 
-          gimp_draw_tool_add_line (draw_tool,
-                                   point1.x,
-                                   point1.y,
-                                   point2.x,
-                                   point2.y);
+          item = gimp_draw_tool_add_line (draw_tool,
+                                          point1.x,
+                                          point1.y,
+                                          point2.x,
+                                          point2.y);
 
-          gimp_draw_tool_pop_group (draw_tool);
+          if (i == ct->hovering_edge &&
+              ! gimp_cage_tool_is_complete (ct))
+            {
+              gimp_canvas_item_set_highlight (item, TRUE);
+            }
+          else
+            {
+              gimp_draw_tool_pop_group (draw_tool);
+            }
         }
 
-      if (i == ct->hovering_handle)
-        handle = GIMP_HANDLE_FILLED_CIRCLE;
-      else
-        handle = GIMP_HANDLE_CIRCLE;
-
-      gimp_draw_tool_add_handle (draw_tool,
-                                 handle,
-                                 point1.x,
-                                 point1.y,
-                                 GIMP_TOOL_HANDLE_SIZE_CIRCLE,
-                                 GIMP_TOOL_HANDLE_SIZE_CIRCLE,
-                                 GIMP_HANDLE_ANCHOR_CENTER);
-
       if (gimp_cage_config_point_is_selected (ct->config, i))
         {
-          gimp_draw_tool_add_handle (draw_tool,
-                                     GIMP_HANDLE_SQUARE,
-                                     point1.x,
-                                     point1.y,
-                                     GIMP_TOOL_HANDLE_SIZE_CIRCLE,
-                                     GIMP_TOOL_HANDLE_SIZE_CIRCLE,
-                                     GIMP_HANDLE_ANCHOR_CENTER);
+          if (i == ct->hovering_handle)
+            handle = GIMP_HANDLE_FILLED_SQUARE;
+          else
+            handle = GIMP_HANDLE_SQUARE;
         }
+      else
+        {
+          if (i == ct->hovering_handle)
+            handle = GIMP_HANDLE_FILLED_CIRCLE;
+          else
+            handle = GIMP_HANDLE_CIRCLE;
+        }
+
+      item = gimp_draw_tool_add_handle (draw_tool,
+                                        handle,
+                                        point1.x,
+                                        point1.y,
+                                        GIMP_TOOL_HANDLE_SIZE_CIRCLE,
+                                        GIMP_TOOL_HANDLE_SIZE_CIRCLE,
+                                        GIMP_HANDLE_ANCHOR_CENTER);
+
+      if (i == ct->hovering_handle)
+        gimp_canvas_item_set_highlight (item, TRUE);
     }
 
   if (ct->tool_state == DEFORM_STATE_SELECTING ||
@@ -900,37 +947,30 @@ gimp_cage_tool_start (GimpCageTool *ct,
 {
   GimpTool     *tool     = GIMP_TOOL (ct);
   GimpImage    *image    = gimp_display_get_image (display);
-  GimpDrawable *drawable = gimp_image_get_active_drawable (image);
+  GList        *drawables = gimp_image_get_selected_drawables (image);
 
-  tool->display  = display;
-  tool->drawable = drawable;
+  g_return_if_fail (g_list_length (drawables) == 1);
 
-  if (ct->config)
-    {
-      g_object_unref (ct->config);
-      ct->config = NULL;
-    }
+  tool->display   = display;
+  g_list_free (tool->drawables);
+  tool->drawables = drawables;
 
-  if (ct->coef)
-    {
-      g_object_unref (ct->coef);
-      ct->dirty_coef = TRUE;
-      ct->coef = NULL;
-    }
+  g_clear_object (&ct->config);
+
+  g_clear_object (&ct->coef);
+  ct->dirty_coef = TRUE;
 
   if (ct->filter)
     {
       gimp_drawable_filter_abort (ct->filter);
-      g_object_unref (ct->filter);
-      ct->filter = NULL;
+      g_clear_object (&ct->filter);
     }
 
   if (ct->render_node)
     {
-      g_object_unref (ct->render_node);
-      ct->render_node = NULL;
-      ct->coef_node   = NULL;
-      ct->cage_node   = NULL;
+      g_clear_object (&ct->render_node);
+      ct->coef_node = NULL;
+      ct->cage_node = NULL;
     }
 
   ct->config          = g_object_new (GIMP_TYPE_CAGE_CONFIG, NULL);
@@ -941,7 +981,7 @@ gimp_cage_tool_start (GimpCageTool *ct,
   /* Setting up cage offset to convert the cage point coords to
    * drawable coords
    */
-  gimp_item_get_offset (GIMP_ITEM (tool->drawable),
+  gimp_item_get_offset (GIMP_ITEM (tool->drawables->data),
                         &ct->offset_x, &ct->offset_y);
 
   gimp_draw_tool_start (GIMP_DRAW_TOOL (ct), display);
@@ -952,42 +992,28 @@ gimp_cage_tool_halt (GimpCageTool *ct)
 {
   GimpTool *tool = GIMP_TOOL (ct);
 
-  if (ct->config)
-    {
-      g_object_unref (ct->config);
-      ct->config = NULL;
-    }
-
-  if (ct->coef)
-    {
-      g_object_unref (ct->coef);
-      ct->coef = NULL;
-    }
-
-  if (ct->render_node)
-    {
-      g_object_unref (ct->render_node);
-      ct->render_node = NULL;
-      ct->coef_node   = NULL;
-      ct->cage_node   = NULL;
-    }
+  g_clear_object (&ct->config);
+  g_clear_object (&ct->coef);
+  g_clear_object (&ct->render_node);
+  ct->coef_node = NULL;
+  ct->cage_node = NULL;
 
   if (ct->filter)
     {
       gimp_tool_control_push_preserve (tool->control, TRUE);
 
       gimp_drawable_filter_abort (ct->filter);
-      g_object_unref (ct->filter);
-      ct->filter = NULL;
+      g_clear_object (&ct->filter);
 
       gimp_tool_control_pop_preserve (tool->control);
 
       gimp_image_flush (gimp_display_get_image (tool->display));
     }
 
-  tool->display  = NULL;
-  tool->drawable = NULL;
-  ct->tool_state = CAGE_STATE_INIT;
+  tool->display   = NULL;
+  g_list_free (tool->drawables);
+  tool->drawables = NULL;
+  ct->tool_state  = CAGE_STATE_INIT;
 
   g_object_set (gimp_tool_get_options (tool),
                 "cage-mode", GIMP_CAGE_MODE_CAGE_CHANGE,
@@ -1004,8 +1030,7 @@ gimp_cage_tool_commit (GimpCageTool *ct)
       gimp_tool_control_push_preserve (tool->control, TRUE);
 
       gimp_drawable_filter_commit (ct->filter, GIMP_PROGRESS (tool), FALSE);
-      g_object_unref (ct->filter);
-      ct->filter = NULL;
+      g_clear_object (&ct->filter);
 
       gimp_tool_control_pop_preserve (tool->control);
 
@@ -1143,11 +1168,7 @@ gimp_cage_tool_compute_coef (GimpCageTool *ct)
   progress = gimp_progress_start (GIMP_PROGRESS (ct), FALSE,
                                   _("Computing Cage Coefficients"));
 
-  if (ct->coef)
-    {
-      g_object_unref (ct->coef);
-      ct->coef = NULL;
-    }
+  g_clear_object (&ct->coef);
 
   format = babl_format_n (babl_type ("float"),
                           gimp_cage_config_get_n_points (config) * 2);
@@ -1166,8 +1187,7 @@ gimp_cage_tool_compute_coef (GimpCageTool *ct)
                                 "format",    format,
                                 NULL);
 
-  gegl_node_connect_to (input, "output",
-                        output, "input");
+  gegl_node_link (input, output);
 
   processor = gegl_node_new_processor (output, NULL);
 
@@ -1219,20 +1239,17 @@ gimp_cage_tool_create_render_node (GimpCageTool *ct)
                                 "operation", "gegl:map-absolute",
                                 NULL);
 
-  gegl_node_connect_to (input,         "output",
-                        ct->cage_node, "input");
+  gegl_node_link (input, ct->cage_node);
 
-  gegl_node_connect_to (ct->coef_node, "output",
-                        ct->cage_node, "aux");
+  gegl_node_connect (ct->coef_node, "output",
+                     ct->cage_node, "aux");
 
-  gegl_node_connect_to (input,  "output",
-                        render, "input");
+  gegl_node_link (input, render);
 
-  gegl_node_connect_to (ct->cage_node, "output",
-                        render,        "aux");
+  gegl_node_connect (ct->cage_node, "output",
+                     render,        "aux");
 
-  gegl_node_connect_to (render, "output",
-                        output, "input");
+  gegl_node_link (render, output);
 
   gimp_gegl_progress_connect (ct->cage_node, GIMP_PROGRESS (ct),
                               _("Cage Transform"));
@@ -1277,10 +1294,11 @@ gimp_cage_tool_create_filter (GimpCageTool *ct)
   if (! ct->render_node)
     gimp_cage_tool_create_render_node (ct);
 
-  ct->filter = gimp_drawable_filter_new (GIMP_TOOL (ct)->drawable,
+  ct->filter = gimp_drawable_filter_new (GIMP_TOOL (ct)->drawables->data,
                                          _("Cage transform"),
                                          ct->render_node,
                                          GIMP_ICON_TOOL_CAGE);
+  gimp_drawable_filter_set_region (ct->filter, GIMP_FILTER_REGION_DRAWABLE);
 
   g_signal_connect (ct->filter, "flush",
                     G_CALLBACK (gimp_cage_tool_filter_flush),
