@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -41,6 +41,7 @@
 #include "widgets/gimpdockable.h"
 #include "widgets/gimpdockcontainer.h"
 #include "widgets/gimppaletteeditor.h"
+#include "widgets/gimpwidgets-utils.h"
 #include "widgets/gimpwindowstrategy.h"
 
 #include "display/gimpcanvasitem.h"
@@ -97,19 +98,19 @@ static void   gimp_color_tool_cursor_update  (GimpTool              *tool,
 static void   gimp_color_tool_draw           (GimpDrawTool          *draw_tool);
 
 static gboolean
-               gimp_color_tool_real_can_pick (GimpColorTool         *color_tool,
+              gimp_color_tool_real_can_pick  (GimpColorTool         *color_tool,
                                               const GimpCoords      *coords,
                                               GimpDisplay           *display);
 static gboolean   gimp_color_tool_real_pick  (GimpColorTool         *color_tool,
-                                              gint                   x,
-                                              gint                   y,
+                                              const GimpCoords      *coords,
+                                              GimpDisplay           *display,
                                               const Babl           **sample_format,
                                               gpointer               pixel,
                                               GimpRGB               *color);
 static void   gimp_color_tool_real_picked    (GimpColorTool         *color_tool,
+                                              const GimpCoords      *coords,
+                                              GimpDisplay           *display,
                                               GimpColorPickState     pick_state,
-                                              gdouble                x,
-                                              gdouble                y,
                                               const Babl            *sample_format,
                                               gpointer               pixel,
                                               const GimpRGB         *color);
@@ -118,12 +119,12 @@ static gboolean   gimp_color_tool_can_pick   (GimpColorTool         *tool,
                                               const GimpCoords      *coords,
                                               GimpDisplay           *display);
 static void   gimp_color_tool_pick           (GimpColorTool         *tool,
-                                              GimpColorPickState     pick_state,
-                                              gint                   x,
-                                              gint                   y);
+                                              const GimpCoords      *coords,
+                                              GimpDisplay           *display,
+                                              GimpColorPickState     pick_state);
 
 
-G_DEFINE_TYPE (GimpColorTool, gimp_color_tool, GIMP_TYPE_DRAW_TOOL);
+G_DEFINE_TYPE (GimpColorTool, gimp_color_tool, GIMP_TYPE_DRAW_TOOL)
 
 #define parent_class gimp_color_tool_parent_class
 
@@ -143,11 +144,11 @@ gimp_color_tool_class_init (GimpColorToolClass *klass)
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GimpColorToolClass, picked),
                   NULL, NULL,
-                  gimp_marshal_VOID__ENUM_DOUBLE_DOUBLE_POINTER_POINTER_BOXED,
+                  gimp_marshal_VOID__POINTER_OBJECT_ENUM_POINTER_POINTER_BOXED,
                   G_TYPE_NONE, 6,
+                  G_TYPE_POINTER,
+                  GIMP_TYPE_DISPLAY,
                   GIMP_TYPE_COLOR_PICK_STATE,
-                  G_TYPE_DOUBLE,
-                  G_TYPE_DOUBLE,
                   G_TYPE_POINTER,
                   G_TYPE_POINTER,
                   GIMP_TYPE_RGB | G_SIGNAL_TYPE_STATIC_SCOPE);
@@ -173,9 +174,7 @@ gimp_color_tool_init (GimpColorTool *color_tool)
   GimpTool *tool = GIMP_TOOL (color_tool);
 
   gimp_tool_control_set_action_size (tool->control,
-                                     "tools/tools-color-average-radius-set");
-
-  color_tool->pick_mode = GIMP_COLOR_PICK_MODE_NONE;
+                                     "tools-color-average-radius-set");
 }
 
 static void
@@ -183,11 +182,7 @@ gimp_color_tool_finalize (GObject *object)
 {
   GimpColorTool *color_tool = GIMP_COLOR_TOOL (object);
 
-  if (color_tool->options)
-    {
-      g_object_unref (color_tool->options);
-      color_tool->options = NULL;
-    }
+  g_clear_object (&color_tool->options);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -202,24 +197,25 @@ gimp_color_tool_button_press (GimpTool            *tool,
 {
   GimpColorTool *color_tool = GIMP_COLOR_TOOL (tool);
 
-  /*  Chain up to activate the tool  */
-  GIMP_TOOL_CLASS (parent_class)->button_press (tool, coords, time, state,
-                                                press_type, display);
-
-  if (! color_tool->enabled)
-    return;
-
-  if (color_tool->sample_point)
+  if (color_tool->enabled)
     {
-      gimp_tool_control_halt (tool->control);
+      if (color_tool->sample_point)
+        {
+          gimp_sample_point_tool_start_edit (tool, display,
+                                             color_tool->sample_point);
+        }
+      else if (gimp_color_tool_can_pick (color_tool, coords, display))
+        {
+          gimp_color_tool_pick (color_tool, coords, display,
+                                GIMP_COLOR_PICK_STATE_START);
 
-      gimp_sample_point_tool_start_edit (tool, display,
-                                         color_tool->sample_point);
+          gimp_tool_control_activate (tool->control);
+        }
     }
-  else if (gimp_color_tool_can_pick (color_tool, coords, display))
+  else
     {
-      gimp_color_tool_pick (color_tool, GIMP_COLOR_PICK_STATE_START,
-                            coords->x, coords->y);
+      GIMP_TOOL_CLASS (parent_class)->button_press (tool, coords, time, state,
+                                                    press_type, display);
     }
 }
 
@@ -233,18 +229,21 @@ gimp_color_tool_button_release (GimpTool              *tool,
 {
   GimpColorTool *color_tool = GIMP_COLOR_TOOL (tool);
 
-  /*  Chain up to halt the tool  */
-  GIMP_TOOL_CLASS (parent_class)->button_release (tool, coords, time, state,
-                                                  release_type, display);
-
-  if (! color_tool->enabled)
-    return;
-
-  if (! color_tool->sample_point &&
-      gimp_color_tool_can_pick (color_tool, coords, display))
+  if (color_tool->enabled)
     {
-      gimp_color_tool_pick (color_tool, GIMP_COLOR_PICK_STATE_END,
-                            coords->x, coords->y);
+      gimp_tool_control_halt (tool->control);
+
+      if (! color_tool->sample_point &&
+          gimp_color_tool_can_pick (color_tool, coords, display))
+        {
+          gimp_color_tool_pick (color_tool, coords, display,
+                                GIMP_COLOR_PICK_STATE_END);
+        }
+    }
+  else
+    {
+      GIMP_TOOL_CLASS (parent_class)->button_release (tool, coords, time, state,
+                                                      release_type, display);
     }
 }
 
@@ -257,25 +256,30 @@ gimp_color_tool_motion (GimpTool         *tool,
 {
   GimpColorTool *color_tool = GIMP_COLOR_TOOL (tool);
 
-  if (! color_tool->enabled)
-    return;
-
-  if (! color_tool->sample_point)
+  if (color_tool->enabled)
     {
-      gimp_draw_tool_pause (GIMP_DRAW_TOOL (tool));
-
-      color_tool->can_pick = gimp_color_tool_can_pick (color_tool,
-                                                       coords, display);
-      color_tool->center_x = coords->x;
-      color_tool->center_y = coords->y;
-
-      if (color_tool->can_pick)
+      if (! color_tool->sample_point)
         {
-          gimp_color_tool_pick (color_tool, GIMP_COLOR_PICK_STATE_UPDATE,
-                                coords->x, coords->y);
-        }
+          gimp_draw_tool_pause (GIMP_DRAW_TOOL (tool));
 
-      gimp_draw_tool_resume (GIMP_DRAW_TOOL (tool));
+          color_tool->can_pick = gimp_color_tool_can_pick (color_tool,
+                                                           coords, display);
+          color_tool->center_x = coords->x;
+          color_tool->center_y = coords->y;
+
+          if (color_tool->can_pick)
+            {
+              gimp_color_tool_pick (color_tool, coords, display,
+                                    GIMP_COLOR_PICK_STATE_UPDATE);
+            }
+
+          gimp_draw_tool_resume (GIMP_DRAW_TOOL (tool));
+        }
+    }
+  else
+    {
+      GIMP_TOOL_CLASS (parent_class)->motion (tool, coords, time, state,
+                                              display);
     }
 }
 
@@ -296,7 +300,9 @@ gimp_color_tool_oper_update (GimpTool         *tool,
 
       gimp_draw_tool_pause (draw_tool);
 
-      if (gimp_draw_tool_is_active (draw_tool) &&
+      if (! draw_tool->widget &&
+
+          gimp_draw_tool_is_active (draw_tool) &&
           (draw_tool->display != display ||
            ! proximity))
         {
@@ -323,7 +329,9 @@ gimp_color_tool_oper_update (GimpTool         *tool,
       color_tool->center_x = coords->x;
       color_tool->center_y = coords->y;
 
-      if (! gimp_draw_tool_is_active (draw_tool) &&
+      if (! draw_tool->widget &&
+
+          ! gimp_draw_tool_is_active (draw_tool) &&
           proximity)
         {
           gimp_draw_tool_start (draw_tool, display);
@@ -331,6 +339,11 @@ gimp_color_tool_oper_update (GimpTool         *tool,
 
       gimp_draw_tool_resume (draw_tool);
     }
+  else
+    {
+      GIMP_TOOL_CLASS (parent_class)->oper_update (tool, coords, state,
+                                                   proximity, display);
+    }
 }
 
 static void
@@ -356,18 +369,18 @@ gimp_color_tool_cursor_update (GimpTool         *tool,
 
           if (gimp_color_tool_can_pick (color_tool, coords, display))
             {
-              switch (color_tool->pick_mode)
+              switch (color_tool->pick_target)
                 {
-                case GIMP_COLOR_PICK_MODE_NONE:
+                case GIMP_COLOR_PICK_TARGET_NONE:
                   modifier = GIMP_CURSOR_MODIFIER_NONE;
                   break;
-                case GIMP_COLOR_PICK_MODE_FOREGROUND:
+                case GIMP_COLOR_PICK_TARGET_FOREGROUND:
                   modifier = GIMP_CURSOR_MODIFIER_FOREGROUND;
                   break;
-                case GIMP_COLOR_PICK_MODE_BACKGROUND:
+                case GIMP_COLOR_PICK_TARGET_BACKGROUND:
                   modifier = GIMP_CURSOR_MODIFIER_BACKGROUND;
                   break;
-                case GIMP_COLOR_PICK_MODE_PALETTE:
+                case GIMP_COLOR_PICK_TARGET_PALETTE:
                   modifier = GIMP_CURSOR_MODIFIER_PLUS;
                   break;
                 }
@@ -378,11 +391,12 @@ gimp_color_tool_cursor_update (GimpTool         *tool,
                                 GIMP_TOOL_CURSOR_COLOR_PICKER,
                                 modifier);
         }
-
-      return;  /*  don't chain up  */
     }
-
-  GIMP_TOOL_CLASS (parent_class)->cursor_update (tool, coords, state, display);
+  else
+    {
+      GIMP_TOOL_CLASS (parent_class)->cursor_update (tool, coords, state,
+                                                     display);
+    }
 }
 
 static void
@@ -390,6 +404,8 @@ gimp_color_tool_draw (GimpDrawTool *draw_tool)
 {
   GimpColorTool *color_tool = GIMP_COLOR_TOOL (draw_tool);
 
+  GIMP_DRAW_TOOL_CLASS (parent_class)->draw (draw_tool);
+
   if (color_tool->enabled)
     {
       if (color_tool->sample_point)
@@ -427,29 +443,32 @@ gimp_color_tool_real_can_pick (GimpColorTool    *color_tool,
                                const GimpCoords *coords,
                                GimpDisplay      *display)
 {
-  GimpImage *image = gimp_display_get_image (display);
+  GimpDisplayShell *shell = gimp_display_get_shell (display);
+  GimpImage        *image = gimp_display_get_image (display);
 
-  return
-    gimp_image_coords_in_active_pickable (image, coords,
-                                          color_tool->options->sample_merged,
-                                          FALSE);
+  return gimp_image_coords_in_active_pickable (image, coords,
+                                               shell->show_all,
+                                               color_tool->options->sample_merged,
+                                               FALSE);
 }
 
 static gboolean
-gimp_color_tool_real_pick (GimpColorTool  *color_tool,
-                           gint            x,
-                           gint            y,
-                           const Babl    **sample_format,
-                           gpointer        pixel,
-                           GimpRGB        *color)
+gimp_color_tool_real_pick (GimpColorTool     *color_tool,
+                           const GimpCoords  *coords,
+                           GimpDisplay       *display,
+                           const Babl       **sample_format,
+                           gpointer           pixel,
+                           GimpRGB           *color)
 {
-  GimpTool  *tool  = GIMP_TOOL (color_tool);
-  GimpImage *image = gimp_display_get_image (tool->display);
+  GimpDisplayShell *shell     = gimp_display_get_shell (display);
+  GimpImage        *image     = gimp_display_get_image (display);
+  GList            *drawables = gimp_image_get_selected_drawables (image);
 
-  g_return_val_if_fail (tool->display != NULL, FALSE);
-  g_return_val_if_fail (tool->drawable != NULL, FALSE);
+  g_return_val_if_fail (drawables != NULL, FALSE);
 
-  return gimp_image_pick_color (image, tool->drawable, x, y,
+  return gimp_image_pick_color (image, drawables,
+                                coords->x, coords->y,
+                                shell->show_all,
                                 color_tool->options->sample_merged,
                                 color_tool->options->sample_average,
                                 color_tool->options->average_radius,
@@ -460,15 +479,15 @@ gimp_color_tool_real_pick (GimpColorTool  *color_tool,
 
 static void
 gimp_color_tool_real_picked (GimpColorTool      *color_tool,
+                             const GimpCoords   *coords,
+                             GimpDisplay        *display,
                              GimpColorPickState  pick_state,
-                             gdouble             x,
-                             gdouble             y,
                              const Babl         *sample_format,
                              gpointer            pixel,
                              const GimpRGB      *color)
 {
   GimpTool          *tool  = GIMP_TOOL (color_tool);
-  GimpDisplayShell  *shell = gimp_display_get_shell (tool->display);
+  GimpDisplayShell  *shell = gimp_display_get_shell (display);
   GimpImageWindow   *image_window;
   GimpDialogFactory *dialog_factory;
   GimpContext       *context;
@@ -479,23 +498,44 @@ gimp_color_tool_real_picked (GimpColorTool      *color_tool,
   /*  use this tool's own options here (NOT color_tool->options)  */
   context = GIMP_CONTEXT (gimp_tool_get_options (tool));
 
-  if (color_tool->pick_mode == GIMP_COLOR_PICK_MODE_FOREGROUND ||
-      color_tool->pick_mode == GIMP_COLOR_PICK_MODE_BACKGROUND)
+  if (color_tool->pick_target == GIMP_COLOR_PICK_TARGET_FOREGROUND ||
+      color_tool->pick_target == GIMP_COLOR_PICK_TARGET_BACKGROUND)
     {
       GtkWidget *widget;
 
-      if (babl_format_is_palette (sample_format))
+      widget = gimp_dialog_factory_find_widget (dialog_factory,
+                                                "gimp-indexed-palette");
+      if (widget)
         {
-          widget = gimp_dialog_factory_find_widget (dialog_factory,
-                                                    "gimp-indexed-palette");
-          if (widget)
+          GtkWidget *editor = gtk_bin_get_child (GTK_BIN (widget));
+          GimpImage *image  = gimp_display_get_image (display);
+
+          if (babl_format_is_palette (sample_format))
             {
-              GtkWidget *editor = gtk_bin_get_child (GTK_BIN (widget));
-              guchar    *index  = pixel;
+              guchar *index  = pixel;
 
               gimp_colormap_editor_set_index (GIMP_COLORMAP_EDITOR (editor),
                                               *index, NULL);
             }
+          else if (gimp_image_get_base_type (image) == GIMP_INDEXED)
+            {
+              /* When Sample merged is set, we don't have the index
+               * information and it is possible to pick colors out of
+               * the colormap (with compositing). In such a case, the
+               * sample format will not be a palette format even though
+               * the image is indexed. Still search if the color exists
+               * in the colormap.
+               * Note that even if it does, we might still pick the
+               * wrong color, since several indexes may contain the same
+               * color and we can't know for sure which is the right
+               * one.
+               */
+              gint index = gimp_colormap_editor_get_index (GIMP_COLORMAP_EDITOR (editor),
+                                                           color);
+              if (index > -1)
+                gimp_colormap_editor_set_index (GIMP_COLORMAP_EDITOR (editor),
+                                                index, NULL);
+            }
         }
 
       widget = gimp_dialog_factory_find_widget (dialog_factory,
@@ -513,30 +553,28 @@ gimp_color_tool_real_picked (GimpColorTool      *color_tool,
         }
     }
 
-  switch (color_tool->pick_mode)
+  switch (color_tool->pick_target)
     {
-    case GIMP_COLOR_PICK_MODE_NONE:
+    case GIMP_COLOR_PICK_TARGET_NONE:
       break;
 
-    case GIMP_COLOR_PICK_MODE_FOREGROUND:
+    case GIMP_COLOR_PICK_TARGET_FOREGROUND:
       gimp_context_set_foreground (context, color);
       break;
 
-    case GIMP_COLOR_PICK_MODE_BACKGROUND:
+    case GIMP_COLOR_PICK_TARGET_BACKGROUND:
       gimp_context_set_background (context, color);
       break;
 
-    case GIMP_COLOR_PICK_MODE_PALETTE:
+    case GIMP_COLOR_PICK_TARGET_PALETTE:
       {
-        GdkScreen *screen  = gtk_widget_get_screen (GTK_WIDGET (shell));
-        gint       monitor = gimp_widget_get_monitor (GTK_WIDGET (shell));
-        GtkWidget *dockable;
+        GdkMonitor *monitor = gimp_widget_get_monitor (GTK_WIDGET (shell));
+        GtkWidget  *dockable;
 
         dockable =
-          gimp_window_strategy_show_dockable_dialog (GIMP_WINDOW_STRATEGY (gimp_get_window_strategy (tool->display->gimp)),
-                                                     tool->display->gimp,
+          gimp_window_strategy_show_dockable_dialog (GIMP_WINDOW_STRATEGY (gimp_get_window_strategy (display->gimp)),
+                                                     display->gimp,
                                                      dialog_factory,
-                                                     screen,
                                                      monitor,
                                                      "gimp-palette-editor");
 
@@ -547,7 +585,7 @@ gimp_color_tool_real_picked (GimpColorTool      *color_tool,
 
             /* don't blink like mad when updating */
             if (pick_state != GIMP_COLOR_PICK_STATE_START)
-              gimp_dockable_blink_cancel (GIMP_DOCKABLE (dockable));
+              gimp_widget_blink_cancel (dockable);
 
             palette_editor = gtk_bin_get_child (GTK_BIN (dockable));
 
@@ -583,23 +621,22 @@ gimp_color_tool_can_pick (GimpColorTool    *tool,
 
 static void
 gimp_color_tool_pick (GimpColorTool      *tool,
-                      GimpColorPickState  pick_state,
-                      gint                x,
-                      gint                y)
+                      const GimpCoords   *coords,
+                      GimpDisplay        *display,
+                      GimpColorPickState  pick_state)
 {
   GimpColorToolClass *klass;
   const Babl         *sample_format;
-  guchar              pixel[32];
+  gdouble             pixel[4];
   GimpRGB             color;
 
   klass = GIMP_COLOR_TOOL_GET_CLASS (tool);
 
   if (klass->pick &&
-      klass->pick (tool, x, y, &sample_format, pixel, &color))
+      klass->pick (tool, coords, display, &sample_format, pixel, &color))
     {
       g_signal_emit (tool, gimp_color_tool_signals[PICKED], 0,
-                     pick_state,
-                     (gdouble) x, (gdouble) y,
+                     coords, display, pick_state,
                      sample_format, pixel, &color);
     }
 }
@@ -624,10 +661,7 @@ gimp_color_tool_enable (GimpColorTool    *color_tool,
       return;
     }
 
-  if (color_tool->options)
-    g_object_unref (color_tool->options);
-
-  color_tool->options = g_object_ref (options);
+  g_set_object (&color_tool->options, options);
 
   /*  color picking doesn't snap, see bug #768058  */
   color_tool->saved_snap_to = gimp_tool_control_get_snap_to (tool->control);
@@ -651,11 +685,7 @@ gimp_color_tool_disable (GimpColorTool *color_tool)
       return;
     }
 
-  if (color_tool->options)
-    {
-      g_object_unref (color_tool->options);
-      color_tool->options = NULL;
-    }
+  g_clear_object (&color_tool->options);
 
   gimp_tool_control_set_snap_to (tool->control, color_tool->saved_snap_to);
   color_tool->saved_snap_to = FALSE;
