@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -32,11 +32,17 @@
 #include "operations/gimplevelsconfig.h"
 #include "operations/gimpoperationlevels.h"
 
+#include "core/gimp-gui.h"
+#include "core/gimpasync.h"
+#include "core/gimpcancelable.h"
 #include "core/gimpdrawable.h"
 #include "core/gimpdrawable-histogram.h"
 #include "core/gimperror.h"
 #include "core/gimphistogram.h"
 #include "core/gimpimage.h"
+#include "core/gimptoolinfo.h"
+#include "core/gimptriviallycancelablewaitable.h"
+#include "core/gimpwaitable.h"
 
 #include "widgets/gimpcolorbar.h"
 #include "widgets/gimphandlebar.h"
@@ -65,7 +71,6 @@
 
 /*  local function prototypes  */
 
-static void       gimp_levels_tool_constructed    (GObject          *object);
 static void       gimp_levels_tool_finalize       (GObject          *object);
 
 static gboolean   gimp_levels_tool_initialize     (GimpTool         *tool,
@@ -73,16 +78,12 @@ static gboolean   gimp_levels_tool_initialize     (GimpTool         *tool,
                                                    GError          **error);
 
 static gchar    * gimp_levels_tool_get_operation  (GimpFilterTool   *filter_tool,
-                                                   gchar           **title,
-                                                   gchar           **description,
-                                                   gchar           **undo_desc,
-                                                   gchar           **icon_name,
-                                                   gchar           **help_id,
-                                                   gboolean         *has_settings,
-                                                   gchar           **import_dialog_title,
-                                                   gchar           **export_dialog_title);
+                                                   gchar           **description);
 static void       gimp_levels_tool_dialog         (GimpFilterTool   *filter_tool);
 static void       gimp_levels_tool_reset          (GimpFilterTool   *filter_tool);
+static void       gimp_levels_tool_config_notify  (GimpFilterTool   *filter_tool,
+                                                   GimpConfig       *config,
+                                                   const GParamSpec *pspec);
 static gboolean   gimp_levels_tool_settings_import(GimpFilterTool   *filter_tool,
                                                    GInputStream     *input,
                                                    GError          **error);
@@ -100,9 +101,6 @@ static void       gimp_levels_tool_export_setup   (GimpSettingsBox  *settings_bo
                                                    GtkFileChooserDialog *dialog,
                                                    gboolean          export,
                                                    GimpLevelsTool   *tool);
-static void       gimp_levels_tool_config_notify  (GObject          *object,
-                                                   GParamSpec       *pspec,
-                                                   GimpLevelsTool   *tool);
 
 static void       levels_update_input_bar         (GimpLevelsTool   *tool);
 
@@ -138,7 +136,7 @@ gimp_levels_tool_register (GimpToolRegisterCallback  callback,
                 0,
                 "gimp-levels-tool",
                 _("Levels"),
-                _("Levels Tool: Adjust color levels"),
+                _("Adjust color levels"),
                 N_("_Levels..."), NULL,
                 NULL, GIMP_HELP_TOOL_LEVELS,
                 GIMP_ICON_TOOL_LEVELS,
@@ -152,7 +150,6 @@ gimp_levels_tool_class_init (GimpLevelsToolClass *klass)
   GimpToolClass       *tool_class        = GIMP_TOOL_CLASS (klass);
   GimpFilterToolClass *filter_tool_class = GIMP_FILTER_TOOL_CLASS (klass);
 
-  object_class->constructed          = gimp_levels_tool_constructed;
   object_class->finalize             = gimp_levels_tool_finalize;
 
   tool_class->initialize             = gimp_levels_tool_initialize;
@@ -160,6 +157,7 @@ gimp_levels_tool_class_init (GimpLevelsToolClass *klass)
   filter_tool_class->get_operation   = gimp_levels_tool_get_operation;
   filter_tool_class->dialog          = gimp_levels_tool_dialog;
   filter_tool_class->reset           = gimp_levels_tool_reset;
+  filter_tool_class->config_notify   = gimp_levels_tool_config_notify;
   filter_tool_class->settings_import = gimp_levels_tool_settings_import;
   filter_tool_class->settings_export = gimp_levels_tool_settings_export;
   filter_tool_class->color_picked    = gimp_levels_tool_color_picked;
@@ -168,17 +166,6 @@ gimp_levels_tool_class_init (GimpLevelsToolClass *klass)
 static void
 gimp_levels_tool_init (GimpLevelsTool *tool)
 {
-  tool->histogram = gimp_histogram_new (FALSE);
-}
-
-static void
-gimp_levels_tool_constructed (GObject *object)
-{
-  G_OBJECT_CLASS (parent_class)->constructed (object);
-
-  g_signal_connect_object (GIMP_FILTER_TOOL (object)->config, "notify",
-                           G_CALLBACK (gimp_levels_tool_config_notify),
-                           object, 0);
 }
 
 static void
@@ -186,11 +173,8 @@ gimp_levels_tool_finalize (GObject *object)
 {
   GimpLevelsTool *tool = GIMP_LEVELS_TOOL (object);
 
-  if (tool->histogram)
-    {
-      g_object_unref (tool->histogram);
-      tool->histogram = NULL;
-    }
+  g_clear_object (&tool->histogram);
+  g_clear_object (&tool->histogram_async);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -200,9 +184,12 @@ gimp_levels_tool_initialize (GimpTool     *tool,
                              GimpDisplay  *display,
                              GError      **error)
 {
-  GimpLevelsTool   *l_tool   = GIMP_LEVELS_TOOL (tool);
-  GimpImage        *image    = gimp_display_get_image (display);
-  GimpDrawable     *drawable = gimp_image_get_active_drawable (image);
+  GimpFilterTool   *filter_tool = GIMP_FILTER_TOOL (tool);
+  GimpLevelsTool   *l_tool      = GIMP_LEVELS_TOOL (tool);
+  GimpImage        *image       = gimp_display_get_image (display);
+  GList            *drawables;
+  GimpDrawable     *drawable;
+  GimpLevelsConfig *config;
   gdouble           scale_factor;
   gdouble           step_increment;
   gdouble           page_increment;
@@ -213,7 +200,30 @@ gimp_levels_tool_initialize (GimpTool     *tool,
       return FALSE;
     }
 
-  gimp_drawable_calculate_histogram (drawable, l_tool->histogram);
+  drawables = gimp_image_get_selected_drawables (image);
+  if (g_list_length (drawables) != 1)
+    {
+      if (g_list_length (drawables) > 1)
+        gimp_tool_message_literal (tool, display,
+                                   _("Cannot modify multiple drawables. Select only one."));
+      else
+        gimp_tool_message_literal (tool, display, _("No selected drawables."));
+
+      g_list_free (drawables);
+      return FALSE;
+    }
+
+  drawable = drawables->data;
+  g_list_free (drawables);
+
+  config = GIMP_LEVELS_CONFIG (filter_tool->config);
+
+  g_clear_object (&l_tool->histogram);
+  g_clear_object (&l_tool->histogram_async);
+  l_tool->histogram = gimp_histogram_new (config->trc);
+
+  l_tool->histogram_async = gimp_drawable_calculate_histogram_async
+    (drawable, l_tool->histogram, FALSE);
   gimp_histogram_view_set_histogram (GIMP_HISTOGRAM_VIEW (l_tool->histogram_view),
                                      l_tool->histogram);
 
@@ -254,19 +264,9 @@ gimp_levels_tool_initialize (GimpTool     *tool,
 
 static gchar *
 gimp_levels_tool_get_operation (GimpFilterTool  *filter_tool,
-                                gchar          **title,
-                                gchar          **description,
-                                gchar          **undo_desc,
-                                gchar          **icon_name,
-                                gchar          **help_id,
-                                gboolean        *has_settings,
-                                gchar          **import_dialog_title,
-                                gchar          **export_dialog_title)
+                                gchar          **description)
 {
-  *description         = g_strdup (_("Adjust Color Levels"));
-  *has_settings        = TRUE;
-  *import_dialog_title = g_strdup (_("Import Levels"));
-  *export_dialog_title = g_strdup (_("Export Levels"));
+  *description = g_strdup (_("Adjust Color Levels"));
 
   return g_strdup ("gimp:levels");
 }
@@ -321,7 +321,8 @@ gimp_levels_tool_color_picker_new (GimpLevelsTool *tool,
                                             GUINT_TO_POINTER (value),
                                             icon_name,
                                             help,
-                                            /* pick_abyss = */ FALSE);
+                                            /* pick_abyss = */ FALSE,
+                                            NULL, NULL);
 }
 
 static void
@@ -373,8 +374,8 @@ gimp_levels_tool_dialog (GimpFilterTool *filter_tool)
   store = gimp_enum_store_new_with_range (GIMP_TYPE_HISTOGRAM_CHANNEL,
                                           GIMP_HISTOGRAM_VALUE,
                                           GIMP_HISTOGRAM_ALPHA);
-  tool->channel_menu =
-    gimp_enum_combo_box_new_with_model (GIMP_ENUM_STORE (store));
+  g_set_weak_pointer (&tool->channel_menu,
+                      gimp_enum_combo_box_new_with_model (GIMP_ENUM_STORE (store)));
   g_object_unref (store);
 
   gimp_enum_combo_box_set_icon_prefix (GIMP_ENUM_COMBO_BOX (tool->channel_menu),
@@ -398,11 +399,17 @@ gimp_levels_tool_dialog (GimpFilterTool *filter_tool)
                     G_CALLBACK (levels_channel_reset_callback),
                     tool);
 
+  /*  The histogram scale radio buttons  */
   hbox2 = gimp_prop_enum_icon_box_new (G_OBJECT (tool_options),
                                        "histogram-scale", "gimp-histogram",
                                        0, 0);
   gtk_box_pack_end (GTK_BOX (hbox), hbox2, FALSE, FALSE, 0);
-  gtk_widget_show (hbox2);
+
+  /*  The linear/perceptual radio buttons  */
+  hbox2 = gimp_prop_enum_icon_box_new (G_OBJECT (config), "trc",
+                                       "gimp-color-space",
+                                       -1, -1);
+  gtk_box_pack_end (GTK_BOX (hbox), hbox2, FALSE, FALSE, 0);
 
   frame_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 4);
   gtk_container_add (GTK_CONTAINER (main_frame), frame_vbox);
@@ -426,7 +433,9 @@ gimp_levels_tool_dialog (GimpFilterTool *filter_tool)
   gtk_container_add (GTK_CONTAINER (frame), vbox2);
   gtk_widget_show (vbox2);
 
-  tool->histogram_view = gimp_histogram_view_new (FALSE);
+  g_set_weak_pointer (&tool->histogram_view,
+                      gimp_histogram_view_new (FALSE));
+
   gtk_box_pack_start (GTK_BOX (vbox2), tool->histogram_view, TRUE, TRUE, 0);
   gtk_widget_show (GTK_WIDGET (tool->histogram_view));
 
@@ -480,23 +489,29 @@ gimp_levels_tool_dialog (GimpFilterTool *filter_tool)
     gimp_prop_spin_button_new (filter_tool->config, "low-input",
                                0.01, 0.1, 1);
   gtk_box_pack_start (GTK_BOX (hbox2), spinbutton, FALSE, FALSE, 0);
-  gtk_widget_show (spinbutton);
 
   tool->low_input = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (spinbutton));
   gimp_handle_bar_set_adjustment (GIMP_HANDLE_BAR (handle_bar), 0,
                                   tool->low_input);
 
+  /*  clamp input toggle  */
+  hbox2 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+  gtk_box_pack_start (GTK_BOX (hbox), hbox2, TRUE, FALSE, 0);
+  gtk_widget_show (hbox2);
+
+  button = gimp_prop_check_button_new (filter_tool->config, "clamp-input",
+                                       _("Clamp _input"));
+  gtk_box_pack_start (GTK_BOX (hbox2), button, FALSE, FALSE, 0);
+
   /*  input gamma spin  */
   spinbutton = gimp_prop_spin_button_new (filter_tool->config, "gamma",
                                           0.01, 0.1, 2);
-  gtk_box_pack_start (GTK_BOX (hbox), spinbutton, TRUE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox2), spinbutton, FALSE, FALSE, 0);
   gimp_help_set_help_data (spinbutton, _("Gamma"), NULL);
-  gtk_widget_show (spinbutton);
 
   tool->gamma = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (spinbutton));
 
-  tool->gamma_linear = GTK_ADJUSTMENT (gtk_adjustment_new (127, 0, 255,
-                                                           0.1, 1.0, 0.0));
+  tool->gamma_linear = gtk_adjustment_new (127, 0, 255, 0.1, 1.0, 0.0);
   g_signal_connect (tool->gamma_linear, "value-changed",
                     G_CALLBACK (levels_linear_gamma_changed),
                     tool);
@@ -517,7 +532,6 @@ gimp_levels_tool_dialog (GimpFilterTool *filter_tool)
   spinbutton = gimp_prop_spin_button_new (filter_tool->config, "high-input",
                                           0.01, 0.1, 1);
   gtk_box_pack_start (GTK_BOX (hbox2), spinbutton, FALSE, FALSE, 0);
-  gtk_widget_show (spinbutton);
   tool->high_input_spinbutton = spinbutton;
 
   tool->high_input = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (spinbutton));
@@ -566,22 +580,24 @@ gimp_levels_tool_dialog (GimpFilterTool *filter_tool)
     gimp_prop_spin_button_new (filter_tool->config, "low-output",
                                0.01, 0.1, 1);
   gtk_box_pack_start (GTK_BOX (hbox), spinbutton, FALSE, FALSE, 0);
-  gtk_widget_show (spinbutton);
 
   adjustment = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (spinbutton));
   gimp_handle_bar_set_adjustment (GIMP_HANDLE_BAR (handle_bar), 0, adjustment);
 
+  /*  clamp output toggle  */
+  button = gimp_prop_check_button_new (filter_tool->config, "clamp-output",
+                                       _("Clamp outpu_t"));
+  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, FALSE, 0);
+
   /*  high output spin  */
   tool->high_output_spinbutton = spinbutton =
     gimp_prop_spin_button_new (filter_tool->config, "high-output",
                                0.01, 0.1, 1);
   gtk_box_pack_end (GTK_BOX (hbox), spinbutton, FALSE, FALSE, 0);
-  gtk_widget_show (spinbutton);
 
   adjustment = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (spinbutton));
   gimp_handle_bar_set_adjustment (GIMP_HANDLE_BAR (handle_bar), 2, adjustment);
 
-
   /*  all channels frame  */
   main_frame = gimp_frame_new (_("All Channels"));
   gtk_box_pack_start (GTK_BOX (main_vbox), main_frame, FALSE, FALSE, 0);
@@ -653,6 +669,66 @@ gimp_levels_tool_reset (GimpFilterTool *filter_tool)
                 NULL);
 }
 
+static void
+gimp_levels_tool_config_notify (GimpFilterTool   *filter_tool,
+                                GimpConfig       *config,
+                                const GParamSpec *pspec)
+{
+  GimpLevelsTool   *levels_tool   = GIMP_LEVELS_TOOL (filter_tool);
+  GimpLevelsConfig *levels_config = GIMP_LEVELS_CONFIG (config);
+
+  GIMP_FILTER_TOOL_CLASS (parent_class)->config_notify (filter_tool,
+                                                        config, pspec);
+
+  if (! levels_tool->channel_menu ||
+      ! levels_tool->histogram_view)
+    return;
+
+  if (! strcmp (pspec->name, "trc"))
+    {
+      g_clear_object (&levels_tool->histogram);
+      g_clear_object (&levels_tool->histogram_async);
+      levels_tool->histogram = gimp_histogram_new (levels_config->trc);
+
+      levels_tool->histogram_async = gimp_drawable_calculate_histogram_async
+        (GIMP_TOOL (filter_tool)->drawables->data, levels_tool->histogram, FALSE);
+      gimp_histogram_view_set_histogram (GIMP_HISTOGRAM_VIEW (levels_tool->histogram_view),
+                                         levels_tool->histogram);
+    }
+  else if (! strcmp (pspec->name, "channel"))
+    {
+      gimp_histogram_view_set_channel (GIMP_HISTOGRAM_VIEW (levels_tool->histogram_view),
+                                       levels_config->channel);
+      gimp_color_bar_set_channel (GIMP_COLOR_BAR (levels_tool->output_bar),
+                                  levels_config->channel);
+      gimp_int_combo_box_set_active (GIMP_INT_COMBO_BOX (levels_tool->channel_menu),
+                                     levels_config->channel);
+    }
+  else if (! strcmp (pspec->name, "gamma")     ||
+           ! strcmp (pspec->name, "low-input") ||
+           ! strcmp (pspec->name, "high-input"))
+    {
+      gdouble low  = gtk_adjustment_get_value (levels_tool->low_input);
+      gdouble high = gtk_adjustment_get_value (levels_tool->high_input);
+      gdouble delta, mid, tmp, value;
+
+      gtk_adjustment_set_lower (levels_tool->high_input,   low);
+      gtk_adjustment_set_lower (levels_tool->gamma_linear, low);
+
+      gtk_adjustment_set_upper (levels_tool->low_input,    high);
+      gtk_adjustment_set_upper (levels_tool->gamma_linear, high);
+
+      levels_update_input_bar (levels_tool);
+
+      delta = (high - low) / 2.0;
+      mid   = low + delta;
+      tmp   = log10 (1.0 / levels_config->gamma[levels_config->channel]);
+      value = mid + delta * tmp;
+
+      gtk_adjustment_set_value (levels_tool->gamma_linear, value);
+    }
+}
+
 static gboolean
 gimp_levels_tool_settings_import (GimpFilterTool  *filter_tool,
                                   GInputStream    *input,
@@ -728,8 +804,14 @@ gimp_levels_tool_color_picked (GimpFilterTool *color_tool,
 {
   GimpFilterTool   *filter_tool = GIMP_FILTER_TOOL (color_tool);
   GimpLevelsConfig *config      = GIMP_LEVELS_CONFIG (filter_tool->config);
+  GimpRGB           rgb         = *color;
   guint             value       = GPOINTER_TO_UINT (identifier);
 
+  if (config->trc == GIMP_TRC_LINEAR)
+    babl_process (babl_fish (babl_format ("R'G'B'A double"),
+                             babl_format ("RGBA double")),
+                  &rgb, &rgb, 1);
+
   if (value & PICK_ALL_CHANNELS &&
       gimp_babl_format_get_base_type (sample_format) == GIMP_RGB)
     {
@@ -756,12 +838,12 @@ gimp_levels_tool_color_picked (GimpFilterTool *color_tool,
            channel <= GIMP_HISTOGRAM_BLUE;
            channel++)
         {
-          levels_input_adjust_by_color (config, value, channel, color);
+          levels_input_adjust_by_color (config, value, channel, &rgb);
         }
     }
   else
     {
-      levels_input_adjust_by_color (config, value, config->channel, color);
+      levels_input_adjust_by_color (config, value, config->channel, &rgb);
     }
 }
 
@@ -787,50 +869,6 @@ gimp_levels_tool_export_setup (GimpSettingsBox      *settings_box,
                     &tool->export_old_format);
 }
 
-static void
-gimp_levels_tool_config_notify (GObject        *object,
-                                GParamSpec     *pspec,
-                                GimpLevelsTool *tool)
-{
-  GimpLevelsConfig *config = GIMP_LEVELS_CONFIG (object);
-
-  if (! tool->low_input)
-    return;
-
-  if (! strcmp (pspec->name, "channel"))
-    {
-      gimp_histogram_view_set_channel (GIMP_HISTOGRAM_VIEW (tool->histogram_view),
-                                       config->channel);
-      gimp_color_bar_set_channel (GIMP_COLOR_BAR (tool->output_bar),
-                                  config->channel);
-      gimp_int_combo_box_set_active (GIMP_INT_COMBO_BOX (tool->channel_menu),
-                                     config->channel);
-    }
-  else if (! strcmp (pspec->name, "gamma")     ||
-           ! strcmp (pspec->name, "low-input") ||
-           ! strcmp (pspec->name, "high-input"))
-    {
-      gdouble low  = gtk_adjustment_get_value (tool->low_input);
-      gdouble high = gtk_adjustment_get_value (tool->high_input);
-      gdouble delta, mid, tmp, value;
-
-      gtk_adjustment_set_lower (tool->high_input,   low);
-      gtk_adjustment_set_lower (tool->gamma_linear, low);
-
-      gtk_adjustment_set_upper (tool->low_input,    high);
-      gtk_adjustment_set_upper (tool->gamma_linear, high);
-
-      levels_update_input_bar (tool);
-
-      delta = (high - low) / 2.0;
-      mid   = low + delta;
-      tmp   = log10 (1.0 / config->gamma[config->channel]);
-      value = mid + delta * tmp;
-
-      gtk_adjustment_set_value (tool->gamma_linear, value);
-    }
-}
-
 static void
 levels_update_input_bar (GimpLevelsTool *tool)
 {
@@ -923,12 +961,14 @@ static gboolean
 levels_menu_sensitivity (gint      value,
                          gpointer  data)
 {
-  GimpDrawable         *drawable = GIMP_FILTER_TOOL (data)->drawable;
+  GimpDrawable         *drawable;
   GimpHistogramChannel  channel  = value;
 
-  if (!drawable)
+  if (! GIMP_TOOL (data)->drawables)
     return FALSE;
 
+  drawable = GIMP_TOOL (data)->drawables->data;
+
   switch (channel)
     {
     case GIMP_HISTOGRAM_VALUE:
@@ -954,13 +994,26 @@ levels_menu_sensitivity (gint      value,
 
 static void
 levels_stretch_callback (GtkWidget      *widget,
-                         GimpLevelsTool *tool)
+                         GimpLevelsTool *levels_tool)
 {
-  GimpFilterTool *filter_tool = GIMP_FILTER_TOOL (tool);
+  GimpTool       *tool        = GIMP_TOOL (levels_tool);
+  GimpFilterTool *filter_tool = GIMP_FILTER_TOOL (levels_tool);
+  GimpWaitable   *waitable;
 
-  gimp_levels_config_stretch (GIMP_LEVELS_CONFIG (filter_tool->config),
-                              tool->histogram,
-                              gimp_drawable_is_rgb (filter_tool->drawable));
+  waitable = gimp_trivially_cancelable_waitable_new (
+    GIMP_WAITABLE (levels_tool->histogram_async));
+
+  gimp_wait (tool->tool_info->gimp, waitable, _("Calculating histogram..."));
+
+  g_object_unref (waitable);
+
+  if (gimp_async_is_synced   (levels_tool->histogram_async) &&
+      gimp_async_is_finished (levels_tool->histogram_async))
+    {
+      gimp_levels_config_stretch (GIMP_LEVELS_CONFIG (filter_tool->config),
+                                  levels_tool->histogram,
+                                  gimp_drawable_is_rgb (tool->drawables->data));
+    }
 }
 
 static void
