@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -21,23 +21,30 @@
 #include <gtk/gtk.h>
 
 #include "libgimpbase/gimpbase.h"
+#include "libgimpcolor/gimpcolor.h"
 #include "libgimpconfig/gimpconfig.h"
 #include "libgimpwidgets/gimpwidgets.h"
+#include "libgimpwidgets/gimpwidgets-private.h"
 
 #include "tools-types.h"
 
 #include "config/gimpconfig-utils.h"
 
 #include "core/gimp.h"
+#include "core/gimpcontainer.h"
+#include "core/gimpdashpattern.h"
+#include "core/gimpdatafactory.h"
+#include "core/gimppattern.h"
+#include "core/gimpstrokeoptions.h"
+#include "core/gimptoolinfo.h"
 #include "core/gimpviewable.h"
 
 #include "text/gimptext.h"
 
 #include "widgets/gimpcolorpanel.h"
-#include "widgets/gimpdialogfactory.h"
-#include "widgets/gimpdock.h"
 #include "widgets/gimpmenufactory.h"
 #include "widgets/gimppropwidgets.h"
+#include "widgets/gimpstrokeeditor.h"
 #include "widgets/gimptextbuffer.h"
 #include "widgets/gimptexteditor.h"
 #include "widgets/gimpviewablebox.h"
@@ -45,14 +52,13 @@
 
 #include "gimptextoptions.h"
 #include "gimptooloptions-gui.h"
-#include "gimprectangleoptions.h"
 
 #include "gimp-intl.h"
 
 
 enum
 {
-  PROP_0 = GIMP_RECTANGLE_OPTIONS_PROP_LAST + 1,
+  PROP_0,
   PROP_FONT_SIZE,
   PROP_UNIT,
   PROP_ANTIALIAS,
@@ -65,47 +71,75 @@ enum
   PROP_LETTER_SPACING,
   PROP_BOX_MODE,
 
+  PROP_OUTLINE,
+  PROP_OUTLINE_STYLE,       /* fill-options */
+  PROP_OUTLINE_FOREGROUND,  /* context */
+  PROP_OUTLINE_PATTERN,     /* context */
+  PROP_OUTLINE_WIDTH,       /* stroke-options */
+  PROP_OUTLINE_UNIT,
+  PROP_OUTLINE_CAP_STYLE,
+  PROP_OUTLINE_JOIN_STYLE,
+  PROP_OUTLINE_MITER_LIMIT,
+  PROP_OUTLINE_ANTIALIAS,   /* fill-options */
+  PROP_OUTLINE_DASH_OFFSET,
+  PROP_OUTLINE_DASH_INFO,
   PROP_USE_EDITOR,
+  PROP_SHOW_ON_CANVAS_EDITOR,
 
   PROP_FONT_VIEW_TYPE,
   PROP_FONT_VIEW_SIZE
 };
 
 
-static void  gimp_text_options_config_iface_init (GimpConfigInterface *config_iface);
+static void
+             gimp_text_options_config_iface_init    (GimpConfigInterface *config_iface);
+static gboolean
+             gimp_text_options_serialize_property   (GimpConfig          *config,
+                                                     guint                property_id,
+                                                     const GValue        *value,
+                                                     GParamSpec          *pspec,
+                                                     GimpConfigWriter    *writer);
+static gboolean
+             gimp_text_options_deserialize_property (GimpConfig          *config,
+                                                     guint                property_id,
+                                                     GValue              *value,
+                                                     GParamSpec          *pspec,
+                                                     GScanner            *scanner,
+                                                     GTokenType          *expected);
 
-static void  gimp_text_options_finalize           (GObject         *object);
-static void  gimp_text_options_set_property       (GObject         *object,
-                                                   guint            property_id,
-                                                   const GValue    *value,
-                                                   GParamSpec      *pspec);
-static void  gimp_text_options_get_property       (GObject         *object,
-                                                   guint            property_id,
-                                                   GValue          *value,
-                                                   GParamSpec      *pspec);
+static void  gimp_text_options_finalize             (GObject             *object);
+static void  gimp_text_options_set_property         (GObject             *object,
+                                                     guint                property_id,
+                                                     const GValue        *value,
+                                                     GParamSpec          *pspec);
+static void  gimp_text_options_get_property         (GObject             *object,
+                                                     guint                property_id,
+                                                     GValue              *value,
+                                                     GParamSpec          *pspec);
 
-static void  gimp_text_options_reset              (GimpConfig      *config);
+static void  gimp_text_options_reset                (GimpConfig          *config);
+
+static void  gimp_text_options_notify_font          (GimpContext         *context,
+                                                     GParamSpec          *pspec,
+                                                     GimpText            *text);
+static void  gimp_text_options_notify_text_font     (GimpText            *text,
+                                                     GParamSpec          *pspec,
+                                                     GimpContext         *context);
+static void  gimp_text_options_notify_color         (GimpContext         *context,
+                                                     GParamSpec          *pspec,
+                                                     GimpText            *text);
+static void  gimp_text_options_notify_text_color    (GimpText            *text,
+                                                     GParamSpec          *pspec,
+                                                     GimpContext         *context);
+static void  gimp_text_options_outline_changed      (GtkWidget           *combo,
+                                                     GtkWidget           *vbox);
 
-static void  gimp_text_options_notify_font        (GimpContext     *context,
-                                                   GParamSpec      *pspec,
-                                                   GimpText        *text);
-static void  gimp_text_options_notify_text_font   (GimpText        *text,
-                                                   GParamSpec      *pspec,
-                                                   GimpContext     *context);
-static void  gimp_text_options_notify_color       (GimpContext     *context,
-                                                   GParamSpec      *pspec,
-                                                   GimpText        *text);
-static void  gimp_text_options_notify_text_color  (GimpText        *text,
-                                                   GParamSpec      *pspec,
-                                                   GimpContext     *context);
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpTextOptions, gimp_text_options,
                          GIMP_TYPE_TOOL_OPTIONS,
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_CONFIG,
-                                                gimp_text_options_config_iface_init)
-                         G_IMPLEMENT_INTERFACE (GIMP_TYPE_RECTANGLE_OPTIONS,
-                                                NULL))
+                                                gimp_text_options_config_iface_init))
 
 #define parent_class gimp_text_options_parent_class
 
@@ -116,22 +150,14 @@ static void
 gimp_text_options_class_init (GimpTextOptionsClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GimpRGB       gray;
+  GParamSpec   *array_spec;
 
+  gimp_rgba_set (&gray, 0.75, 0.75, 0.75, GIMP_OPACITY_OPAQUE);
   object_class->finalize     = gimp_text_options_finalize;
   object_class->set_property = gimp_text_options_set_property;
   object_class->get_property = gimp_text_options_get_property;
 
-  /* The 'highlight' property is defined here because we want different
-   * default values for the Crop, Text and the Rectangle Select tools.
-   */
-  GIMP_CONFIG_PROP_BOOLEAN (object_class,
-                            GIMP_RECTANGLE_OPTIONS_PROP_HIGHLIGHT,
-                            "highlight",
-                            _("Highlight"),
-                            NULL,
-                            FALSE,
-                            GIMP_PARAM_STATIC_STRINGS);
-
   GIMP_CONFIG_PROP_UNIT (object_class, PROP_UNIT,
                          "font-size-unit",
                          _("Unit"),
@@ -220,11 +246,18 @@ gimp_text_options_class_init (GimpTextOptionsClass *klass)
 
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_USE_EDITOR,
                             "use-editor",
-                            _("Use editor"),
+                            _("Use editor window"),
                             _("Use an external editor window for text entry"),
                             FALSE,
                             GIMP_PARAM_STATIC_STRINGS);
 
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SHOW_ON_CANVAS_EDITOR,
+                            "show-on-canvas",
+                            _("Show on-canvas editor"),
+                            _("Show on-canvas text editor"),
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_PROP_ENUM (object_class, PROP_FONT_VIEW_TYPE,
                          "font-view-type",
                          NULL, NULL,
@@ -238,8 +271,82 @@ gimp_text_options_class_init (GimpTextOptionsClass *klass)
                         GIMP_VIEWABLE_MAX_BUTTON_SIZE,
                         GIMP_VIEW_SIZE_SMALL,
                         GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_ENUM (object_class, PROP_OUTLINE,
+                         "outline",
+                         NULL, NULL,
+                         GIMP_TYPE_TEXT_OUTLINE,
+                         GIMP_TEXT_OUTLINE_NONE,
+                         GIMP_PARAM_STATIC_STRINGS |
+                         GIMP_CONFIG_PARAM_DEFAULTS);
+   GIMP_CONFIG_PROP_ENUM (object_class, PROP_OUTLINE_STYLE,
+                          "outline-custom-style",
+                          NULL, NULL,
+                          GIMP_TYPE_CUSTOM_STYLE,
+                          GIMP_CUSTOM_STYLE_SOLID_COLOR,
+                          GIMP_PARAM_STATIC_STRINGS);
+   GIMP_CONFIG_PROP_RGB (object_class, PROP_OUTLINE_FOREGROUND,
+                         "outline-foreground",
+                         NULL, NULL,
+                         FALSE, &gray,
+                         GIMP_PARAM_STATIC_STRINGS);
+   GIMP_CONFIG_PROP_OBJECT (object_class, PROP_OUTLINE_PATTERN,
+                            "outline-pattern",
+                            NULL, NULL,
+                            GIMP_TYPE_PATTERN,
+                            GIMP_PARAM_STATIC_STRINGS);
+   GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_OUTLINE_WIDTH,
+                            "outline-width",
+                            _("Outline width"),
+                            _("Adjust outline width"),
+                            0, 8192.0, 2.0,
+                            GIMP_PARAM_STATIC_STRINGS |
+                            GIMP_CONFIG_PARAM_DEFAULTS);
+   GIMP_CONFIG_PROP_UNIT (object_class, PROP_OUTLINE_UNIT,
+                          "outline-unit",
+                          _("Unit"),
+                          _("Outline width unit"),
+                          TRUE, FALSE, GIMP_UNIT_PIXEL,
+                          GIMP_PARAM_STATIC_STRINGS);
+   GIMP_CONFIG_PROP_ENUM (object_class, PROP_OUTLINE_CAP_STYLE,
+                          "outline-cap-style",
+                          NULL, NULL,
+                          GIMP_TYPE_CAP_STYLE, GIMP_CAP_BUTT,
+                          GIMP_PARAM_STATIC_STRINGS);
+   GIMP_CONFIG_PROP_ENUM (object_class, PROP_OUTLINE_JOIN_STYLE,
+                          "outline-join-style",
+                          NULL, NULL,
+                          GIMP_TYPE_JOIN_STYLE, GIMP_JOIN_MITER,
+                          GIMP_PARAM_STATIC_STRINGS);
+   GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_OUTLINE_MITER_LIMIT,
+                            "outline-miter-limit",
+                            _("Outline miter limit"),
+                            _("Convert a mitered join to a bevelled "
+                              "join if the miter would extend to a "
+                              "distance of more than miter-limit * "
+                              "line-width from the actual join point."),
+                            0.0, 100.0, 10.0,
+                            GIMP_PARAM_STATIC_STRINGS);
+   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_OUTLINE_ANTIALIAS,
+                             "outline-antialias",
+                             NULL, NULL,
+                             TRUE,
+                             GIMP_PARAM_STATIC_STRINGS);
+   GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_OUTLINE_DASH_OFFSET,
+                            "outline-dash-offset",
+                            NULL, NULL,
+                            0.0, 2000.0, 0.0,
+                            GIMP_PARAM_STATIC_STRINGS);
 
-  gimp_rectangle_options_install_properties (object_class);
+   array_spec = g_param_spec_double ("outline-dash-length",
+                                     NULL, NULL,
+                                     0.0, 2000.0, 1.0,
+                                     GIMP_PARAM_READWRITE);
+   g_object_class_install_property (object_class, PROP_OUTLINE_DASH_INFO,
+                                    gimp_param_spec_value_array ("outline-dash-info",
+                                                                 NULL, NULL,
+                                                                 array_spec,
+                                                                 GIMP_PARAM_STATIC_STRINGS |
+                                                                 GIMP_CONFIG_PARAM_FLAGS));
 }
 
 static void
@@ -248,6 +355,9 @@ gimp_text_options_config_iface_init (GimpConfigInterface *config_iface)
   parent_config_iface = g_type_interface_peek_parent (config_iface);
 
   config_iface->reset = gimp_text_options_reset;
+
+  config_iface->serialize_property   = gimp_text_options_serialize_property;
+  config_iface->deserialize_property = gimp_text_options_deserialize_property;
 }
 
 static void
@@ -261,11 +371,7 @@ gimp_text_options_finalize (GObject *object)
 {
   GimpTextOptions *options = GIMP_TEXT_OPTIONS (object);
 
-  if (options->language)
-    {
-      g_free (options->language);
-      options->language = NULL;
-    }
+  g_clear_pointer (&options->language, g_free);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -314,9 +420,54 @@ gimp_text_options_get_property (GObject    *object,
       g_value_set_enum (value, options->box_mode);
       break;
 
+    case PROP_OUTLINE:
+      g_value_set_enum (value, options->outline);
+      break;
+    case PROP_OUTLINE_STYLE:
+      g_value_set_enum (value, options->outline_style);
+      break;
+    case PROP_OUTLINE_FOREGROUND:
+      g_value_set_boxed (value, &options->outline_foreground);
+      break;
+    case PROP_OUTLINE_PATTERN:
+      g_value_set_object (value, options->outline_pattern);
+      break;
+    case PROP_OUTLINE_WIDTH:
+      g_value_set_double (value, options->outline_width);
+      break;
+    case PROP_OUTLINE_UNIT:
+      g_value_set_int (value, options->outline_unit);
+      break;
+    case PROP_OUTLINE_CAP_STYLE:
+      g_value_set_enum (value, options->outline_cap_style);
+      break;
+    case PROP_OUTLINE_JOIN_STYLE:
+      g_value_set_enum (value, options->outline_join_style);
+      break;
+    case PROP_OUTLINE_MITER_LIMIT:
+      g_value_set_double (value, options->outline_miter_limit);
+      break;
+    case PROP_OUTLINE_ANTIALIAS:
+      g_value_set_boolean (value, options->outline_antialias);
+      break;
+    case PROP_OUTLINE_DASH_OFFSET:
+      g_value_set_double (value, options->outline_dash_offset);
+      break;
+    case PROP_OUTLINE_DASH_INFO:
+      {
+        GimpValueArray *value_array;
+
+        value_array = gimp_dash_pattern_to_value_array (options->outline_dash_info);
+        g_value_take_boxed (value, value_array);
+      }
+      break;
+
     case PROP_USE_EDITOR:
       g_value_set_boolean (value, options->use_editor);
       break;
+    case PROP_SHOW_ON_CANVAS_EDITOR:
+      g_value_set_boolean (value, options->show_on_canvas);
+      break;
 
     case PROP_FONT_VIEW_TYPE:
       g_value_set_enum (value, options->font_view_type);
@@ -326,7 +477,7 @@ gimp_text_options_get_property (GObject    *object,
       break;
 
     default:
-      gimp_rectangle_options_get_property (object, property_id, value, pspec);
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
     }
 }
@@ -338,6 +489,7 @@ gimp_text_options_set_property (GObject      *object,
                                 GParamSpec   *pspec)
 {
   GimpTextOptions *options = GIMP_TEXT_OPTIONS (object);
+  GimpRGB         *color;
 
   switch (property_id)
     {
@@ -376,9 +528,63 @@ gimp_text_options_set_property (GObject      *object,
       options->box_mode = g_value_get_enum (value);
       break;
 
+    case PROP_OUTLINE:
+      options->outline = g_value_get_enum (value);
+      break;
+    case PROP_OUTLINE_STYLE:
+      options->outline_style = g_value_get_enum (value);
+      break;
+    case PROP_OUTLINE_FOREGROUND:
+      color                       = g_value_get_boxed (value);
+      options->outline_foreground = *color;
+      break;
+    case PROP_OUTLINE_PATTERN:
+      {
+        GimpPattern *pattern = g_value_get_object (value);
+
+        if (options->outline_pattern != pattern)
+          {
+            if (options->outline_pattern)
+              g_object_unref (options->outline_pattern);
+
+            options->outline_pattern = pattern ? g_object_ref (pattern) : pattern;
+          }
+        break;
+      }
+    case PROP_OUTLINE_WIDTH:
+      options->outline_width = g_value_get_double (value);
+      break;
+    case PROP_OUTLINE_UNIT:
+      options->outline_unit = g_value_get_int (value);
+      break;
+    case PROP_OUTLINE_CAP_STYLE:
+      options->outline_cap_style = g_value_get_enum (value);
+      break;
+    case PROP_OUTLINE_JOIN_STYLE:
+      options->outline_join_style = g_value_get_enum (value);
+      break;
+    case PROP_OUTLINE_MITER_LIMIT:
+      options->outline_miter_limit = g_value_get_double (value);
+      break;
+    case PROP_OUTLINE_ANTIALIAS:
+      options->outline_antialias = g_value_get_boolean (value);
+      break;
+    case PROP_OUTLINE_DASH_OFFSET:
+      options->outline_dash_offset = g_value_get_double (value);
+      break;
+    case PROP_OUTLINE_DASH_INFO:
+      {
+        GimpValueArray *value_array = g_value_get_boxed (value);
+        options->outline_dash_info = gimp_dash_pattern_from_value_array (value_array);
+      }
+      break;
+
     case PROP_USE_EDITOR:
       options->use_editor = g_value_get_boolean (value);
       break;
+    case PROP_SHOW_ON_CANVAS_EDITOR:
+      options->show_on_canvas = g_value_get_boolean (value);
+      break;
 
     case PROP_FONT_VIEW_TYPE:
       options->font_view_type = g_value_get_enum (value);
@@ -388,7 +594,7 @@ gimp_text_options_set_property (GObject      *object,
       break;
 
     default:
-      gimp_rectangle_options_set_property (object, property_id, value, pspec);
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
     }
 }
@@ -419,7 +625,22 @@ gimp_text_options_reset (GimpConfig *config)
   gimp_config_reset_property (object, "line-spacing");
   gimp_config_reset_property (object, "letter-spacing");
   gimp_config_reset_property (object, "box-mode");
+
+  gimp_config_reset_property (object, "outline");
+  gimp_config_reset_property (object, "outline-custom-style");
+  gimp_config_reset_property (object, "outline-foreground");
+  gimp_config_reset_property (object, "outline-pattern");
+  gimp_config_reset_property (object, "outline-width");
+  gimp_config_reset_property (object, "outline-unit");
+  gimp_config_reset_property (object, "outline-cap-style");
+  gimp_config_reset_property (object, "outline-join-style");
+  gimp_config_reset_property (object, "outline-miter-limit");
+  gimp_config_reset_property (object, "outline-antialias");
+  gimp_config_reset_property (object, "outline-dash-offset");
+  gimp_config_reset_property (object, "outline-dash-info");
+
   gimp_config_reset_property (object, "use-editor");
+  gimp_config_reset_property (object, "use-on-canvas");
 }
 
 static void
@@ -530,108 +751,189 @@ gimp_text_options_connect_text (GimpTextOptions *options,
 GtkWidget *
 gimp_text_options_gui (GimpToolOptions *tool_options)
 {
-  GObject         *config    = G_OBJECT (tool_options);
-  GimpTextOptions *options   = GIMP_TEXT_OPTIONS (tool_options);
-  GtkWidget       *main_vbox = gimp_tool_options_gui (tool_options);
-  GtkWidget       *table;
-  GtkWidget       *vbox;
-  GtkWidget       *hbox;
-  GtkWidget       *button;
-  GtkWidget       *entry;
-  GtkWidget       *box;
-  GtkWidget       *spinbutton;
-  GtkWidget       *combo;
-  GtkSizeGroup    *size_group;
-  gint             row = 0;
+  GObject           *config    = G_OBJECT (tool_options);
+  GimpTextOptions   *options   = GIMP_TEXT_OPTIONS (tool_options);
+  GtkWidget         *main_vbox = gimp_tool_options_gui (tool_options);
+  GimpAsyncSet      *async_set;
+  GtkWidget         *options_vbox;
+  GtkWidget         *grid;
+  GtkWidget         *vbox;
+  GtkWidget         *hbox;
+  GtkWidget         *button;
+  GtkWidget         *entry;
+  GtkWidget         *box;
+  GtkWidget         *spinbutton;
+  GtkWidget         *combo;
+  GtkWidget         *editor;
+  GimpStrokeOptions *stroke_options;
+  GtkWidget         *outline_frame;
+  GtkSizeGroup      *size_group;
+  gint               row = 0;
+
+  async_set =
+    gimp_data_factory_get_async_set (tool_options->tool_info->gimp->font_factory);
+
+  box = gimp_busy_box_new (_("Loading fonts (this may take a while...)"));
+  gtk_container_set_border_width (GTK_CONTAINER (box), 8);
+  gtk_box_pack_start (GTK_BOX (main_vbox), box, FALSE, FALSE, 0);
+
+  g_object_bind_property (async_set, "empty",
+                          box,       "visible",
+                          G_BINDING_SYNC_CREATE |
+                          G_BINDING_INVERT_BOOLEAN);
+
+  options_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL,
+                              gtk_box_get_spacing (GTK_BOX (main_vbox)));
+  gtk_box_pack_start (GTK_BOX (main_vbox), options_vbox, FALSE, FALSE, 0);
+  gtk_widget_show (options_vbox);
+
+  g_object_bind_property (async_set,    "empty",
+                          options_vbox, "sensitive",
+                          G_BINDING_SYNC_CREATE);
 
   hbox = gimp_prop_font_box_new (NULL, GIMP_CONTEXT (tool_options),
                                  _("Font"), 2,
                                  "font-view-type", "font-view-size");
-  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
+  gtk_box_pack_start (GTK_BOX (options_vbox), hbox, FALSE, FALSE, 0);
 
-  table = gtk_table_new (1, 3, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 2);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_box_pack_start (GTK_BOX (main_vbox), table, FALSE, FALSE, 0);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 2);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
+  gtk_box_pack_start (GTK_BOX (options_vbox), grid, FALSE, FALSE, 0);
+  gtk_widget_show (grid);
 
   entry = gimp_prop_size_entry_new (config,
                                     "font-size", FALSE, "font-size-unit", "%p",
                                     GIMP_SIZE_ENTRY_UPDATE_SIZE, 72.0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-                             _("Size:"), 0.0, 0.5,
-                             entry, 2, FALSE);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
+                            _("Size:"), 0.0, 0.5,
+                            entry, 2);
 
   options->size_entry = entry;
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
-  gtk_box_pack_start (GTK_BOX (main_vbox), vbox, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (options_vbox), vbox, FALSE, FALSE, 0);
   gtk_widget_show (vbox);
 
   button = gimp_prop_check_button_new (config, "use-editor", NULL);
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-  gtk_widget_show (button);
+
+  button = gimp_prop_check_button_new (config, "show-on-canvas", NULL);
+  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
   button = gimp_prop_check_button_new (config, "antialias", NULL);
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-  gtk_widget_show (button);
 
-  table = gtk_table_new (6, 3, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 2);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_box_pack_start (GTK_BOX (main_vbox), table, FALSE, FALSE, 0);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 2);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
+  gtk_box_pack_start (GTK_BOX (options_vbox), grid, FALSE, FALSE, 0);
+  gtk_widget_show (grid);
 
   row = 0;
 
   size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
   button = gimp_prop_enum_combo_box_new (config, "hint-style", -1, -1);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-                             _("Hinting:"), 0.0, 0.5,
-                             button, 1, TRUE);
+  gtk_widget_set_halign (button, GTK_ALIGN_START);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
+                            _("Hinting:"), 0.0, 0.5,
+                            button, 1);
   gtk_size_group_add_widget (size_group, button);
 
   button = gimp_prop_color_button_new (config, "foreground", _("Text Color"),
                                        40, 24, GIMP_COLOR_AREA_FLAT);
+  gimp_color_button_set_update (GIMP_COLOR_BUTTON (button), TRUE);
   gimp_color_panel_set_context (GIMP_COLOR_PANEL (button),
                                 GIMP_CONTEXT (options));
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-                             _("Color:"), 0.0, 0.5,
-                             button, 1, TRUE);
+  gtk_widget_set_halign (button, GTK_ALIGN_START);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
+                            _("Color:"), 0.0, 0.5,
+                            button, 1);
   gtk_size_group_add_widget (size_group, button);
 
+  button = gimp_prop_enum_combo_box_new (config, "outline", -1, -1);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
+                             _("Style:"), 0.0, 0.5,
+                             button, 1);
+  gtk_size_group_add_widget (size_group, button);
+
+  outline_frame = gimp_frame_new (_("Outline Options"));
+  gimp_widget_set_identifier (outline_frame, "text-outline-settings");
+  gtk_box_pack_start (GTK_BOX (options_vbox), outline_frame, FALSE, FALSE, 0);
+  gtk_widget_show (outline_frame);
+
+  g_signal_connect (button, "changed",
+                    G_CALLBACK (gimp_text_options_outline_changed),
+                    outline_frame);
+  gimp_text_options_outline_changed (button, outline_frame);
+
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 2);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
+  gtk_box_pack_start (GTK_BOX (options_vbox), grid, FALSE, FALSE, 0);
+  gtk_widget_show (grid);
+
+  row = 0;
+
   box = gimp_prop_enum_icon_box_new (config, "justify", "format-justify", 0, 0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-                             _("Justify:"), 0.0, 0.5,
-                             box, 2, TRUE);
+  gtk_widget_set_halign (box, GTK_ALIGN_START);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
+                            _("Justify:"), 0.0, 0.5,
+                            box, 2);
   gtk_size_group_add_widget (size_group, box);
   g_object_unref (size_group);
 
   spinbutton = gimp_prop_spin_button_new (config, "indent", 1.0, 10.0, 1);
   gtk_entry_set_width_chars (GTK_ENTRY (spinbutton), 5);
-  gimp_table_attach_icon (GTK_TABLE (table), row++,
-                          GIMP_ICON_FORMAT_INDENT_MORE,
-                          spinbutton, 1, TRUE);
+  gtk_widget_set_halign (spinbutton, GTK_ALIGN_START);
+  gimp_grid_attach_icon (GTK_GRID (grid), row++,
+                         GIMP_ICON_FORMAT_INDENT_MORE,
+                         spinbutton, 1);
 
   spinbutton = gimp_prop_spin_button_new (config, "line-spacing", 1.0, 10.0, 1);
   gtk_entry_set_width_chars (GTK_ENTRY (spinbutton), 5);
-  gimp_table_attach_icon (GTK_TABLE (table), row++,
-                          GIMP_ICON_FORMAT_TEXT_SPACING_LINE,
-                          spinbutton, 1, TRUE);
+  gtk_widget_set_halign (spinbutton, GTK_ALIGN_START);
+  gimp_grid_attach_icon (GTK_GRID (grid), row++,
+                         GIMP_ICON_FORMAT_TEXT_SPACING_LINE,
+                         spinbutton, 1);
 
   spinbutton = gimp_prop_spin_button_new (config,
                                           "letter-spacing", 1.0, 10.0, 1);
   gtk_entry_set_width_chars (GTK_ENTRY (spinbutton), 5);
-  gimp_table_attach_icon (GTK_TABLE (table), row++,
-                          GIMP_ICON_FORMAT_TEXT_SPACING_LETTER,
-                          spinbutton, 1, TRUE);
+  gtk_widget_set_halign (spinbutton, GTK_ALIGN_START);
+  gimp_grid_attach_icon (GTK_GRID (grid), row++,
+                         GIMP_ICON_FORMAT_TEXT_SPACING_LETTER,
+                         spinbutton, 1);
 
   combo = gimp_prop_enum_combo_box_new (config, "box-mode", 0, 0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-                             _("Box:"), 0.0, 0.5,
-                             combo, 1, TRUE);
+  gtk_widget_set_halign (combo, GTK_ALIGN_START);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
+                            _("Box:"), 0.0, 0.5,
+                            combo, 1);
+  stroke_options = gimp_stroke_options_new (GIMP_CONTEXT (options)->gimp,
+                                            NULL, FALSE);
+#define BIND(a)                                    \
+  g_object_bind_property (options, "outline-" #a,  \
+                          stroke_options, #a,                           \
+                          G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE)
+  BIND (custom-style);
+  BIND (foreground);
+  BIND (pattern);
+  BIND (width);
+  BIND (unit);
+  BIND (cap-style);
+  BIND (join-style);
+  BIND (miter-limit);
+  BIND (antialias);
+  BIND (dash-offset);
+  BIND (dash-info);
+
+  editor = gimp_stroke_editor_new (stroke_options, 72.0, TRUE, TRUE);
+  gtk_container_add (GTK_CONTAINER (outline_frame), editor);
+  gtk_widget_show (editor);
+
+  g_object_unref (stroke_options);
 
   /*  Only add the language entry if the iso-codes package is available.  */
 
@@ -640,7 +942,7 @@ gimp_text_options_gui (GimpToolOptions *tool_options)
     GtkWidget *label;
 
     vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
-    gtk_box_pack_start (GTK_BOX (main_vbox), vbox, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (options_vbox), vbox, FALSE, FALSE, 0);
     gtk_widget_show (vbox);
 
     hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
@@ -653,7 +955,6 @@ gimp_text_options_gui (GimpToolOptions *tool_options)
 
     entry = gimp_prop_language_entry_new (config, "language");
     gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, FALSE, 0);
-    gtk_widget_show (entry);
   }
 #endif
 
@@ -695,6 +996,20 @@ gimp_text_options_editor_notify_font (GimpTextOptions *options,
   gimp_text_editor_set_font_name (editor, font_name);
 }
 
+static void
+gimp_text_options_outline_changed (GtkWidget *combo,
+                                   GtkWidget *vbox)
+{
+  GimpTextOutline active;
+
+  active = (GimpTextOutline) gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
+
+  if (active == GIMP_TEXT_OUTLINE_NONE)
+    gtk_widget_hide (vbox);
+  else
+    gtk_widget_show (vbox);
+}
+
 GtkWidget *
 gimp_text_options_editor_new (GtkWindow       *parent,
                               Gimp            *gimp,
@@ -738,3 +1053,74 @@ gimp_text_options_editor_new (GtkWindow       *parent,
 
   return editor;
 }
+
+static gboolean
+gimp_text_options_serialize_property (GimpConfig       *config,
+                                      guint             property_id,
+                                      const GValue     *value,
+                                      GParamSpec       *pspec,
+                                      GimpConfigWriter *writer)
+{
+  if (property_id == PROP_OUTLINE_PATTERN)
+    {
+      GimpObject *serialize_obj = g_value_get_object (value);
+
+      gimp_config_writer_open (writer, pspec->name);
+
+      if (serialize_obj)
+        gimp_config_writer_string (writer, gimp_object_get_name (serialize_obj));
+      else
+        gimp_config_writer_print (writer, "NULL", 4);
+
+      gimp_config_writer_close (writer);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static gboolean
+gimp_text_options_deserialize_property (GimpConfig *object,
+                                        guint       property_id,
+                                        GValue     *value,
+                                        GParamSpec *pspec,
+                                        GScanner   *scanner,
+                                        GTokenType *expected)
+{
+  if (property_id == PROP_OUTLINE_PATTERN)
+    {
+      gchar *object_name;
+
+      if (gimp_scanner_parse_identifier (scanner, "NULL"))
+        {
+          g_value_set_object (value, NULL);
+        }
+      else if (gimp_scanner_parse_string (scanner, &object_name))
+        {
+          GimpContext   *context = GIMP_CONTEXT (object);
+          GimpContainer *container;
+          GimpObject    *deserialize_obj;
+
+          if (! object_name)
+            object_name = g_strdup ("");
+
+          container = gimp_data_factory_get_container (context->gimp->pattern_factory);
+
+          deserialize_obj = gimp_container_get_child_by_name (container,
+                                                              object_name);
+
+          g_value_set_object (value, deserialize_obj);
+
+          g_free (object_name);
+        }
+      else
+        {
+          *expected = G_TOKEN_STRING;
+        }
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
