@@ -35,6 +35,8 @@
 #include "core/gimpimage.h"
 #include "core/gimptoolinfo.h"
 
+#include "menus/menus.h"
+
 #include "text/gimptext.h"
 #include "text/gimptextlayout.h"
 
@@ -191,6 +193,9 @@ gimp_text_tool_editor_start (GimpTextTool *text_tool)
   g_signal_connect (options, "notify::use-editor",
                     G_CALLBACK (gimp_text_tool_options_notify),
                     text_tool);
+  g_signal_connect (options, "notify::show-on-canvas",
+                    G_CALLBACK (gimp_text_tool_options_notify),
+                    text_tool);
 
   if (! text_tool->style_overlay)
     {
@@ -221,7 +226,9 @@ gimp_text_tool_editor_start (GimpTextTool *text_tool)
                                                             xres, yres);
       gtk_container_add (GTK_CONTAINER (text_tool->style_overlay),
                          text_tool->style_editor);
-      gtk_widget_show (text_tool->style_editor);
+
+      if (options->show_on_canvas)
+        gtk_widget_show (text_tool->style_editor);
     }
 
   gimp_text_tool_editor_position (text_tool);
@@ -1326,6 +1333,11 @@ gimp_text_tool_options_notify (GimpTextOptions *options,
             gtk_widget_destroy (text_tool->editor_dialog);
         }
     }
+  else if (! strcmp (param_name, "show-on-canvas"))
+    {
+      gtk_widget_set_visible (text_tool->style_editor,
+                              options->show_on_canvas);
+    }
 }
 
 static void
@@ -1352,15 +1364,13 @@ gimp_text_tool_editor_dialog (GimpTextTool *text_tool)
   if (text_tool->image)
     gimp_image_get_resolution (text_tool->image, &xres, &yres);
 
-  text_tool->editor_dialog =
-    gimp_text_options_editor_new (parent, tool->tool_info->gimp, options,
-                                  gimp_dialog_factory_get_menu_factory (dialog_factory),
-                                  _("GIMP Text Editor"),
-                                  text_tool->proxy, text_tool->buffer,
-                                  xres, yres);
-
-  g_object_add_weak_pointer (G_OBJECT (text_tool->editor_dialog),
-                             (gpointer) &text_tool->editor_dialog);
+  g_set_weak_pointer
+    (&text_tool->editor_dialog,
+     gimp_text_options_editor_new (parent, tool->tool_info->gimp, options,
+                                   menus_get_global_menu_factory (tool->tool_info->gimp),
+                                   _("GIMP Text Editor"),
+                                   text_tool->proxy, text_tool->buffer,
+                                   xres, yres));
 
   gimp_dialog_factory_add_foreign (dialog_factory,
                                    "gimp-text-tool-dialog",
