@@ -227,11 +227,11 @@ gimp_warp_tool_init (GimpWarpTool *self)
                                          GIMP_TOOL_CURSOR_WARP);
 
   gimp_tool_control_set_action_pixel_size (tool->control,
-                                           "tools/tools-warp-effect-pixel-size-set");
+                                           "tools-warp-effect-pixel-size-set");
   gimp_tool_control_set_action_size       (tool->control,
-                                           "tools/tools-warp-effect-size-set");
+                                           "tools-warp-effect-size-set");
   gimp_tool_control_set_action_hardness   (tool->control,
-                                           "tools/tools-warp-effect-hardness-set");
+                                           "tools-warp-effect-hardness-set");
 
   self->show_cursor = TRUE;
   self->draw_brush  = TRUE;
@@ -634,8 +634,8 @@ gimp_warp_tool_undo (GimpTool    *tool,
    */
   prev_node = gegl_node_get_producer (to_delete, "input", NULL);
 
-  gegl_node_connect_to (prev_node,       "output",
-                        wt->render_node, "aux");
+  gegl_node_connect (prev_node,       "output",
+                     wt->render_node, "aux");
 
   gimp_warp_tool_update_bounds (wt);
   gimp_warp_tool_update_stroke (wt, to_delete);
@@ -652,8 +652,8 @@ gimp_warp_tool_redo (GimpTool    *tool,
 
   to_add = wt->redo_stack->data;
 
-  gegl_node_connect_to (to_add,          "output",
-                        wt->render_node, "aux");
+  gegl_node_connect (to_add,          "output",
+                     wt->render_node, "aux");
 
   wt->redo_stack = g_list_remove_link (wt->redo_stack, wt->redo_stack);
 
@@ -1065,14 +1065,10 @@ gimp_warp_tool_create_graph (GimpWarpTool *wt)
                                 "abyss-policy", options->abyss_policy,
                                 NULL);
 
-  gegl_node_connect_to (input,  "output",
-                        render, "input");
+  gegl_node_link_many (input, render, output, NULL);
+  gegl_node_connect (coords, "output",
+                     render, "aux");
 
-  gegl_node_connect_to (coords, "output",
-                        render, "aux");
-
-  gegl_node_connect_to (render, "output",
-                        output, "input");
 
   wt->graph       = graph;
   wt->render_node = render;
@@ -1373,10 +1369,9 @@ gimp_warp_tool_add_op (GimpWarpTool *wt,
   last_op = gegl_node_get_producer (wt->render_node, "aux", NULL);
 
   gegl_node_disconnect (wt->render_node, "aux");
-  gegl_node_connect_to (last_op,         "output",
-                        op    ,          "input");
-  gegl_node_connect_to (op,              "output",
-                        wt->render_node, "aux");
+  gegl_node_link (last_op, op);
+  gegl_node_connect (op,              "output",
+                     wt->render_node, "aux");
 }
 
 static void
@@ -1390,8 +1385,8 @@ gimp_warp_tool_remove_op (GimpWarpTool *wt,
   previous = gegl_node_get_producer (op, "input", NULL);
 
   gegl_node_disconnect (op,              "input");
-  gegl_node_connect_to (previous,        "output",
-                        wt->render_node, "aux");
+  gegl_node_connect (previous,        "output",
+                     wt->render_node, "aux");
 
   gegl_node_remove_child (wt->graph, op);
 }
