@@ -15,13 +15,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <string.h>
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -37,25 +38,18 @@
 #include "gimpactionview.h"
 #include "gimpmessagebox.h"
 #include "gimpmessagedialog.h"
-#include "gimpuimanager.h"
 
 #include "gimp-intl.h"
 
 
 /*  local function prototypes  */
 
-static void     gimp_action_view_dispose         (GObject         *object);
 static void     gimp_action_view_finalize        (GObject         *object);
 
 static void     gimp_action_view_select_path     (GimpActionView  *view,
                                                   GtkTreePath     *path);
-static gboolean gimp_action_view_accel_find_func (GtkAccelKey     *key,
-                                                  GClosure        *closure,
-                                                  gpointer         data);
-static void     gimp_action_view_accel_changed   (GtkAccelGroup   *accel_group,
-                                                  guint            unused1,
-                                                  GdkModifierType  unused2,
-                                                  GClosure        *accel_closure,
+static void     gimp_action_view_accels_changed  (GimpAction      *action,
+                                                  gchar          **accels,
                                                   GimpActionView  *view);
 static void     gimp_action_view_accel_edited    (GtkCellRendererAccel *accel,
                                                   const char      *path_string,
@@ -78,7 +72,6 @@ gimp_action_view_class_init (GimpActionViewClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  object_class->dispose  = gimp_action_view_dispose;
   object_class->finalize = gimp_action_view_finalize;
 }
 
@@ -87,232 +80,160 @@ gimp_action_view_init (GimpActionView *view)
 {
 }
 
-static void
-gimp_action_view_dispose (GObject *object)
-{
-  GimpActionView *view = GIMP_ACTION_VIEW (object);
-
-  if (view->manager)
-    {
-      if (view->show_shortcuts)
-        {
-          GtkAccelGroup *group;
-
-          group = gtk_ui_manager_get_accel_group (GTK_UI_MANAGER (view->manager));
-
-          g_signal_handlers_disconnect_by_func (group,
-                                                gimp_action_view_accel_changed,
-                                                view);
-        }
-
-      g_object_unref (view->manager);
-      view->manager = NULL;
-    }
-
-  G_OBJECT_CLASS (parent_class)->dispose (object);
-}
-
 static void
 gimp_action_view_finalize (GObject *object)
 {
   GimpActionView *view = GIMP_ACTION_VIEW (object);
 
-  if (view->filter)
+  if (view->gimp)
     {
-      g_free (view->filter);
-      view->filter = NULL;
+      if (view->show_shortcuts)
+        {
+          gchar **actions;
+
+          actions = g_action_group_list_actions (G_ACTION_GROUP (view->gimp->app));
+
+          for (gint i = 0; actions[i] != NULL; i++)
+            {
+              GAction *action;
+
+              if (gimp_action_is_gui_blacklisted (actions[i]))
+                continue;
+
+              action = g_action_map_lookup_action (G_ACTION_MAP (view->gimp->app), actions[i]);
+              g_signal_handlers_disconnect_by_func (action,
+                                                    gimp_action_view_accels_changed,
+                                                    view);
+            }
+          g_strfreev (actions);
+        }
+
+      g_clear_object (&view->gimp);
     }
 
+  g_clear_pointer (&view->filter, g_free);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static gboolean
-idle_start_editing (GtkTreeView *tree_view)
+static gint
+gimp_action_view_name_compare (const void *name1,
+                               const void *name2)
 {
-  GtkTreePath *path;
-
-  path = g_object_get_data (G_OBJECT (tree_view), "start-editing-path");
-
-  if (path)
-    {
-      gtk_widget_grab_focus (GTK_WIDGET (tree_view));
-
-      gtk_tree_view_set_cursor (tree_view, path,
-                                gtk_tree_view_get_column (tree_view, 1),
-                                TRUE);
-
-      g_object_set_data (G_OBJECT (tree_view), "start-editing-path", NULL);
-    }
-
-  return FALSE;
-}
-
-static gboolean
-gimp_action_view_button_press (GtkWidget      *widget,
-                               GdkEventButton *event)
-{
-  GtkTreeView *tree_view = GTK_TREE_VIEW (widget);
-  GtkTreePath *path;
-
-  if (event->window != gtk_tree_view_get_bin_window (tree_view))
-    return FALSE;
-
-  if (gtk_tree_view_get_path_at_pos (tree_view,
-                                     (gint) event->x,
-                                     (gint) event->y,
-                                     &path, NULL,
-                                     NULL, NULL))
-    {
-      GClosure *closure;
-      GSource  *source;
-
-      if (gtk_tree_path_get_depth (path) == 1)
-        {
-          gtk_tree_path_free (path);
-          return FALSE;
-        }
-
-      g_object_set_data_full (G_OBJECT (tree_view), "start-editing-path",
-                              path, (GDestroyNotify) gtk_tree_path_free);
-
-      g_signal_stop_emission_by_name (tree_view, "button-press-event");
-
-      closure = g_cclosure_new_object (G_CALLBACK (idle_start_editing),
-                                       G_OBJECT (tree_view));
-
-      source = g_idle_source_new ();
-      g_source_set_closure (source, closure);
-      g_source_attach (source, NULL);
-      g_source_unref (source);
-    }
-
-  return TRUE;
+  return strcmp (*(gchar **) name1, *(gchar **) name2);
 }
 
 GtkWidget *
-gimp_action_view_new (GimpUIManager *manager,
-                      const gchar   *select_action,
-                      gboolean       show_shortcuts)
+gimp_action_view_new (Gimp        *gimp,
+                      const gchar *select_action,
+                      gboolean     show_shortcuts)
 {
+  gchar            **actions;
+  gchar             *group_name = NULL;
+
   GtkTreeView       *view;
   GtkTreeViewColumn *column;
   GtkCellRenderer   *cell;
   GtkTreeStore      *store;
   GtkTreeModel      *filter;
-  GtkAccelGroup     *accel_group;
-  GList             *list;
   GtkTreePath       *select_path = NULL;
 
-  g_return_val_if_fail (GIMP_IS_UI_MANAGER (manager), NULL);
+  g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
 
   store = gtk_tree_store_new (GIMP_ACTION_VIEW_N_COLUMNS,
-                              G_TYPE_BOOLEAN,         /* COLUMN_VISIBLE        */
-                              GTK_TYPE_ACTION,        /* COLUMN_ACTION         */
-                              G_TYPE_STRING,          /* COLUMN_STOCK_ID       */
-                              G_TYPE_STRING,          /* COLUMN_LABEL          */
-                              G_TYPE_STRING,          /* COLUMN_LABEL_CASEFOLD */
-                              G_TYPE_STRING,          /* COLUMN_NAME           */
-                              G_TYPE_UINT,            /* COLUMN_ACCEL_KEY      */
-                              GDK_TYPE_MODIFIER_TYPE, /* COLUMN_ACCEL_MASK     */
-                              G_TYPE_CLOSURE);        /* COLUMN_ACCEL_CLOSURE  */
+                              G_TYPE_BOOLEAN,          /* COLUMN_VISIBLE        */
+                              GIMP_TYPE_ACTION,        /* COLUMN_ACTION         */
+                              G_TYPE_STRING,           /* COLUMN_ICON_NAME      */
+                              G_TYPE_STRING,           /* COLUMN_LABEL          */
+                              G_TYPE_STRING,           /* COLUMN_LABEL_CASEFOLD */
+                              G_TYPE_STRING,           /* COLUMN_NAME           */
+                              G_TYPE_UINT,             /* COLUMN_ACCEL_KEY      */
+                              GDK_TYPE_MODIFIER_TYPE); /* COLUMN_ACCEL_MASK     */
 
-  accel_group = gtk_ui_manager_get_accel_group (GTK_UI_MANAGER (manager));
-
-  for (list = gtk_ui_manager_get_action_groups (GTK_UI_MANAGER (manager));
-       list;
-       list = g_list_next (list))
+  actions = g_action_group_list_actions (G_ACTION_GROUP (gimp->app));
+  qsort (actions, g_strv_length (actions), sizeof (gchar *), gimp_action_view_name_compare);
+  for (gint i = 0; actions[i] != NULL; i++)
     {
-      GimpActionGroup *group = list->data;
-      GList           *actions;
-      GList           *list2;
+      gchar          **split_name;
       GtkTreeIter      group_iter;
+      GAction         *action;
+      const gchar     *icon_name;
+      gchar           *label;
+      gchar           *label_casefold;
+      guint            accel_key     = 0;
+      GdkModifierType  accel_mask    = 0;
+      GtkTreeIter      action_iter;
 
-      gtk_tree_store_append (store, &group_iter, NULL);
+      if (gimp_action_is_gui_blacklisted (actions[i]))
+        continue;
 
-      gtk_tree_store_set (store, &group_iter,
-                          GIMP_ACTION_VIEW_COLUMN_STOCK_ID, group->stock_id,
-                          GIMP_ACTION_VIEW_COLUMN_LABEL,    group->label,
-                          -1);
-
-      actions = gtk_action_group_list_actions (GTK_ACTION_GROUP (group));
-
-      actions = g_list_sort (actions, (GCompareFunc) gimp_action_name_compare);
-
-      for (list2 = actions; list2; list2 = g_list_next (list2))
+      split_name = g_strsplit (actions[i], "-", 2);
+      if (group_name == NULL || g_strcmp0 (group_name, split_name[0]) != 0)
         {
-          GtkAction       *action        = list2->data;
-          const gchar     *name          = gtk_action_get_name (action);
-          const gchar     *stock_id      = gtk_action_get_stock_id (action);
-          gchar           *label;
-          gchar           *label_casefold;
-          guint            accel_key     = 0;
-          GdkModifierType  accel_mask    = 0;
-          GClosure        *accel_closure = NULL;
-          GtkTreeIter      action_iter;
+          /* Since we sorted alphabetically and we use the first part of the
+           * action name as group name, we ensure that we create each group only
+           * once.
+           */
+          g_free (group_name);
+          group_name = g_strdup (split_name[0]);
 
-          if (strstr (name, "-menu")  ||
-              strstr (name, "-popup") ||
-              name[0] == '<')
-            continue;
+          gtk_tree_store_append (store, &group_iter, NULL);
 
-          label = gimp_strip_uline (gtk_action_get_label (action));
-
-          if (! (label && strlen (label)))
-            {
-              g_free (label);
-              label = g_strdup (name);
-            }
-
-          label_casefold = g_utf8_casefold (label, -1);
-
-          if (show_shortcuts)
-            {
-              accel_closure = gtk_action_get_accel_closure (action);
-
-              if (accel_closure)
-                {
-                  GtkAccelKey *key;
-
-                  key = gtk_accel_group_find (accel_group,
-                                              gimp_action_view_accel_find_func,
-                                              accel_closure);
-
-                  if (key            &&
-                      key->accel_key &&
-                      key->accel_flags & GTK_ACCEL_VISIBLE)
-                    {
-                      accel_key  = key->accel_key;
-                      accel_mask = key->accel_mods;
-                    }
-                }
-            }
-
-          gtk_tree_store_append (store, &action_iter, &group_iter);
-
-          gtk_tree_store_set (store, &action_iter,
-                              GIMP_ACTION_VIEW_COLUMN_VISIBLE,        TRUE,
-                              GIMP_ACTION_VIEW_COLUMN_ACTION,         action,
-                              GIMP_ACTION_VIEW_COLUMN_STOCK_ID,       stock_id,
-                              GIMP_ACTION_VIEW_COLUMN_LABEL,          label,
-                              GIMP_ACTION_VIEW_COLUMN_LABEL_CASEFOLD, label_casefold,
-                              GIMP_ACTION_VIEW_COLUMN_NAME,           name,
-                              GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,      accel_key,
-                              GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK,     accel_mask,
-                              GIMP_ACTION_VIEW_COLUMN_ACCEL_CLOSURE,  accel_closure,
+          gtk_tree_store_set (store, &group_iter,
+                              /* TODO: get back GimpActionGroup info? */
+                              /*GIMP_ACTION_VIEW_COLUMN_ICON_NAME, group->icon_name,*/
+                              /*GIMP_ACTION_VIEW_COLUMN_LABEL, group->label,*/
+                              GIMP_ACTION_VIEW_COLUMN_LABEL, group_name,
                               -1);
+        }
+      g_strfreev (split_name);
 
+      action = g_action_map_lookup_action (G_ACTION_MAP (gimp->app), actions[i]);
+      g_return_val_if_fail (GIMP_IS_ACTION (action), NULL);
+
+      icon_name = gimp_action_get_icon_name (GIMP_ACTION (action));
+      label     = gimp_strip_uline (gimp_action_get_label (GIMP_ACTION (action)));
+
+      if (! (label && strlen (label)))
+        {
           g_free (label);
-          g_free (label_casefold);
-
-          if (select_action && ! strcmp (select_action, name))
-            {
-              select_path = gtk_tree_model_get_path (GTK_TREE_MODEL (store),
-                                                     &action_iter);
-            }
+          label = g_strdup (actions[i]);
         }
 
-      g_list_free (actions);
+      label_casefold = g_utf8_casefold (label, -1);
+
+      if (show_shortcuts)
+        {
+          const gchar **accels = NULL;
+
+          accels = gimp_action_get_accels (GIMP_ACTION (action));
+
+          /* TODO GAction: support multiple accelerators! */
+          if (accels && accels[0])
+            gtk_accelerator_parse (accels[0], &accel_key, &accel_mask);
+        }
+
+      gtk_tree_store_append (store, &action_iter, &group_iter);
+
+      gtk_tree_store_set (store, &action_iter,
+                          GIMP_ACTION_VIEW_COLUMN_VISIBLE,        TRUE,
+                          GIMP_ACTION_VIEW_COLUMN_ACTION,         action,
+                          GIMP_ACTION_VIEW_COLUMN_ICON_NAME,      icon_name,
+                          GIMP_ACTION_VIEW_COLUMN_LABEL,          label,
+                          GIMP_ACTION_VIEW_COLUMN_LABEL_CASEFOLD, label_casefold,
+                          GIMP_ACTION_VIEW_COLUMN_NAME,           actions[i],
+                          GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,      accel_key,
+                          GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK,     accel_mask,
+                          -1);
+
+      g_free (label);
+      g_free (label_casefold);
+
+      if (select_action && ! strcmp (select_action, actions[i]))
+        select_path = gtk_tree_model_get_path (GTK_TREE_MODEL (store),
+                                               &action_iter);
     }
+  g_free (group_name);
 
   filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (store), NULL);
 
@@ -328,7 +249,7 @@ gimp_action_view_new (GimpUIManager *manager,
   gtk_tree_model_filter_set_visible_column (GTK_TREE_MODEL_FILTER (filter),
                                             GIMP_ACTION_VIEW_COLUMN_VISIBLE);
 
-  GIMP_ACTION_VIEW (view)->manager        = g_object_ref (manager);
+  GIMP_ACTION_VIEW (view)->gimp           = g_object_ref (gimp);
   GIMP_ACTION_VIEW (view)->show_shortcuts = show_shortcuts;
 
   gtk_tree_view_set_search_column (GTK_TREE_VIEW (view),
@@ -340,8 +261,8 @@ gimp_action_view_new (GimpUIManager *manager,
   cell = gtk_cell_renderer_pixbuf_new ();
   gtk_tree_view_column_pack_start (column, cell, FALSE);
   gtk_tree_view_column_set_attributes (column, cell,
-                                       "stock-id",
-                                       GIMP_ACTION_VIEW_COLUMN_STOCK_ID,
+                                       "icon-name",
+                                       GIMP_ACTION_VIEW_COLUMN_ICON_NAME,
                                        NULL);
 
   cell = gtk_cell_renderer_text_new ();
@@ -355,13 +276,18 @@ gimp_action_view_new (GimpUIManager *manager,
 
   if (show_shortcuts)
     {
-      g_signal_connect (view, "button-press-event",
-                        G_CALLBACK (gimp_action_view_button_press),
-                        NULL);
+      for (gint i = 0; actions[i] != NULL; i++)
+        {
+          GAction *action;
 
-      g_signal_connect (accel_group, "accel-changed",
-                        G_CALLBACK (gimp_action_view_accel_changed),
-                        view);
+          if (gimp_action_is_gui_blacklisted (actions[i]))
+            continue;
+
+          action = g_action_map_lookup_action (G_ACTION_MAP (gimp->app), actions[i]);
+          g_signal_connect (action, "accels-changed",
+                            G_CALLBACK (gimp_action_view_accels_changed),
+                            view);
+        }
 
       column = gtk_tree_view_column_new ();
       gtk_tree_view_column_set_title (column, _("Shortcut"));
@@ -388,6 +314,7 @@ gimp_action_view_new (GimpUIManager *manager,
 
       gtk_tree_view_append_column (view, column);
     }
+  g_strfreev (actions);
 
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_set_title (column, _("Name"));
@@ -429,8 +356,7 @@ gimp_action_view_set_filter (GimpActionView *view,
   if (filter && ! strlen (filter))
     filter = NULL;
 
-  g_free (view->filter);
-  view->filter = NULL;
+  g_clear_pointer (&view->filter, g_free);
 
   if (filter)
     view->filter = g_utf8_casefold (filter, -1);
@@ -529,24 +455,15 @@ gimp_action_view_select_path (GimpActionView *view,
   gtk_tree_view_scroll_to_cell (tv, path, NULL, TRUE, 0.5, 0.0);
 }
 
-static gboolean
-gimp_action_view_accel_find_func (GtkAccelKey *key,
-                                  GClosure    *closure,
-                                  gpointer     data)
-{
-  return (GClosure *) data == closure;
-}
-
 static void
-gimp_action_view_accel_changed (GtkAccelGroup   *accel_group,
-                                guint            unused1,
-                                GdkModifierType  unused2,
-                                GClosure        *accel_closure,
-                                GimpActionView  *view)
+gimp_action_view_accels_changed (GimpAction      *action,
+                                 gchar          **accels,
+                                 GimpActionView  *view)
 {
-  GtkTreeModel *model;
+  GtkTreeModel *model, *tmpmodel;
   GtkTreeIter   iter;
   gboolean      iter_valid;
+  gchar        *pathstr = NULL;
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
   if (! model)
@@ -556,6 +473,14 @@ gimp_action_view_accel_changed (GtkAccelGroup   *accel_group,
   if (! model)
     return;
 
+  if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (view)),
+                                       &tmpmodel, &iter))
+    {
+      GtkTreePath *path = gtk_tree_model_get_path (tmpmodel, &iter);
+
+      pathstr = gtk_tree_path_to_string(path);
+    }
+
   for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);
        iter_valid;
        iter_valid = gtk_tree_model_iter_next (model, &iter))
@@ -568,48 +493,53 @@ gimp_action_view_accel_changed (GtkAccelGroup   *accel_group,
            child_valid;
            child_valid = gtk_tree_model_iter_next (model, &child_iter))
         {
-          GClosure *closure;
+          GimpAction *it_action;
 
           gtk_tree_model_get (model, &child_iter,
-                              GIMP_ACTION_VIEW_COLUMN_ACCEL_CLOSURE, &closure,
+                              GIMP_ACTION_VIEW_COLUMN_ACTION, &it_action,
                               -1);
 
-          if (closure)
-            g_closure_unref (closure);
+          if (it_action)
+            g_object_unref (it_action);
 
-          if (accel_closure == closure)
+          if (it_action == action)
             {
-              GtkAccelKey     *key;
-              guint            accel_key  = 0;
-              GdkModifierType  accel_mask = 0;
+              const gchar     **accels;
+              guint             accel_key  = 0;
+              GdkModifierType   accel_mask = 0;
 
-              key = gtk_accel_group_find (accel_group,
-                                          gimp_action_view_accel_find_func,
-                                          accel_closure);
+              accels = gimp_action_get_accels (action);
 
-              if (key            &&
-                  key->accel_key &&
-                  key->accel_flags & GTK_ACCEL_VISIBLE)
-                {
-                  accel_key  = key->accel_key;
-                  accel_mask = key->accel_mods;
-                }
+              if (accels && accels[0])
+                gtk_accelerator_parse (accels[0], &accel_key, &accel_mask);
 
               gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                   GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,  accel_key,
                                   GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK, accel_mask,
                                   -1);
+              gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
+                                  GIMP_ACTION_VIEW_COLUMN_VISIBLE, TRUE,
+                                  -1);
 
+              if (pathstr)
+                {
+                  GtkTreePath *path = gtk_tree_path_new_from_string (pathstr);
+
+                  gimp_action_view_select_path (view, path);
+                  gtk_tree_path_free (path);
+                }
+              g_free (pathstr);
               return;
             }
         }
     }
+  g_free (pathstr);
 }
 
 typedef struct
 {
-  GimpUIManager   *manager;
-  gchar           *accel_path;
+  Gimp            *gimp;
+  GimpAction      *action;
   guint            accel_key;
   GdkModifierType  accel_mask;
 } ConfirmData;
@@ -623,28 +553,37 @@ gimp_action_view_conflict_response (GtkWidget   *dialog,
 
   if (response_id == GTK_RESPONSE_OK)
     {
-      if (! gtk_accel_map_change_entry (confirm_data->accel_path,
-                                        confirm_data->accel_key,
-                                        confirm_data->accel_mask,
-                                        TRUE))
-        {
-          gimp_message_literal (confirm_data->manager->gimp, G_OBJECT (dialog),
-				GIMP_MESSAGE_ERROR,
-				_("Changing shortcut failed."));
-        }
-    }
+      gchar **dup_actions;
+      gchar  *accel;
 
-  g_free (confirm_data->accel_path);
+      accel = gtk_accelerator_name (confirm_data->accel_key,
+                                    confirm_data->accel_mask);
+      dup_actions = gtk_application_get_actions_for_accel (GTK_APPLICATION (confirm_data->gimp->app),
+                                                           accel);
+      for (gint i = 0; dup_actions[i] != NULL; i++)
+        {
+          GAction *conflict_action;
+          gint     start;
+
+          start = g_str_has_prefix (dup_actions[i], "app.") ? 4 : 0;
+          conflict_action = g_action_map_lookup_action (G_ACTION_MAP (confirm_data->gimp->app),
+                                                        dup_actions[i] + start);
+
+          g_return_if_fail (GIMP_IS_ACTION (conflict_action));
+          gimp_action_set_accels (GIMP_ACTION (conflict_action), (const char*[]) { NULL });
+        }
+      gimp_action_set_accels (confirm_data->action, (const char*[]) { accel, NULL });
+    }
 
   g_slice_free (ConfirmData, confirm_data);
 }
 
 static void
 gimp_action_view_conflict_confirm (GimpActionView  *view,
-                                   GtkAction       *action,
+                                   GimpAction      *action,
+                                   GimpAction      *edit_action,
                                    guint            accel_key,
-                                   GdkModifierType  accel_mask,
-                                   const gchar     *accel_path)
+                                   GdkModifierType  accel_mask)
 {
   GimpActionGroup *group;
   gchar           *label;
@@ -653,34 +592,35 @@ gimp_action_view_conflict_confirm (GimpActionView  *view,
   GtkWidget       *dialog;
   GimpMessageBox  *box;
 
-  g_object_get (action, "action-group", &group, NULL);
+  group = gimp_action_get_group (action);
 
-  label = gimp_strip_uline (gtk_action_get_label (action));
+  label = gimp_strip_uline (gimp_action_get_label (action));
 
   accel_string = gtk_accelerator_get_label (accel_key, accel_mask);
 
   confirm_data = g_slice_new (ConfirmData);
 
-  confirm_data->manager    = view->manager;
-  confirm_data->accel_path = g_strdup (accel_path);
+  confirm_data->gimp       = view->gimp;
+  confirm_data->action     = edit_action;
   confirm_data->accel_key  = accel_key;
   confirm_data->accel_mask = accel_mask;
 
   dialog =
     gimp_message_dialog_new (_("Conflicting Shortcuts"),
-                             GIMP_STOCK_WARNING,
+                             GIMP_ICON_DIALOG_WARNING,
                              gtk_widget_get_toplevel (GTK_WIDGET (view)), 0,
                              gimp_standard_help_func, NULL,
 
-                             GTK_STOCK_CANCEL,         GTK_RESPONSE_CANCEL,
-                             _("_Reassign shortcut"),  GTK_RESPONSE_OK,
+                             _("_Cancel"),            GTK_RESPONSE_CANCEL,
+                             _("_Reassign Shortcut"), GTK_RESPONSE_OK,
 
                              NULL);
 
-  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
-                                           -1);
+  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                            GTK_RESPONSE_OK,
+                                            GTK_RESPONSE_CANCEL,
+                                            -1);
+
   g_signal_connect (dialog, "response",
                     G_CALLBACK (gimp_action_view_conflict_response),
                     confirm_data);
@@ -699,15 +639,15 @@ gimp_action_view_conflict_confirm (GimpActionView  *view,
   g_free (label);
   g_free (accel_string);
 
-  g_object_unref (group);
-
   gtk_widget_show (dialog);
 }
 
-static const gchar *
+static void
 gimp_action_view_get_accel_action (GimpActionView  *view,
                                    const gchar     *path_string,
-                                   GtkAction      **action_return)
+                                   GimpAction     **action_return,
+                                   guint           *action_accel_key,
+                                   GdkModifierType *action_accel_mask)
 {
   GtkTreeModel *model;
   GtkTreePath  *path;
@@ -715,33 +655,30 @@ gimp_action_view_get_accel_action (GimpActionView  *view,
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
   if (! model)
-    return NULL;
+    return;
 
   path = gtk_tree_path_new_from_string (path_string);
 
   if (gtk_tree_model_get_iter (model, &iter, path))
     {
-      GtkAction *action;
+      GimpAction *action;
 
       gtk_tree_model_get (model, &iter,
-                          GIMP_ACTION_VIEW_COLUMN_ACTION, &action,
+                          GIMP_ACTION_VIEW_COLUMN_ACTION,     &action,
+                          GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,  action_accel_key,
+                          GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK, action_accel_mask,
                           -1);
 
-      if (! action)
-        goto done;
-
-      gtk_tree_path_free (path);
-      g_object_unref (action);
-
-      *action_return = action;
-
-      return gtk_action_get_accel_path (action);
+      if (action)
+        {
+          g_object_unref (action);
+          *action_return = action;
+        }
     }
 
- done:
   gtk_tree_path_free (path);
 
-  return NULL;
+  return;
 }
 
 static void
@@ -752,95 +689,97 @@ gimp_action_view_accel_edited (GtkCellRendererAccel *accel,
                                guint                 hardware_keycode,
                                GimpActionView       *view)
 {
-  GtkAction   *action;
-  const gchar *accel_path;
+  GimpAction      *action;
+  guint            action_accel_key;
+  GdkModifierType  action_accel_mask;
 
-  accel_path = gimp_action_view_get_accel_action (view, path_string,
-                                                  &action);
+  gimp_action_view_get_accel_action (view, path_string,
+                                     &action,
+                                     &action_accel_key,
+                                     &action_accel_mask);
 
-  if (! accel_path)
+  if (! action)
+    return;
+
+  if (accel_key  == action_accel_key &&
+      accel_mask == action_accel_mask)
     return;
 
   if (! accel_key ||
 
       /* Don't allow arrow keys, they are all swallowed by the canvas
-       * and cannot be invoked anyway
+       * and cannot be invoked anyway, the same applies to space.
        */
       accel_key == GDK_KEY_Left  ||
       accel_key == GDK_KEY_Right ||
       accel_key == GDK_KEY_Up    ||
-      accel_key == GDK_KEY_Down)
+      accel_key == GDK_KEY_Down  ||
+      accel_key == GDK_KEY_space ||
+      accel_key == GDK_KEY_KP_Space)
     {
-      gimp_message_literal (view->manager->gimp,
-			    G_OBJECT (view), GIMP_MESSAGE_ERROR,
-			    _("Invalid shortcut."));
+      gimp_message_literal (view->gimp,
+                            G_OBJECT (view), GIMP_MESSAGE_ERROR,
+                            _("Invalid shortcut."));
     }
-  else if (! gtk_accel_map_change_entry (accel_path,
-                                         accel_key, accel_mask, FALSE))
+  else if (accel_key        == GDK_KEY_F1 ||
+           action_accel_key == GDK_KEY_F1)
     {
-      GtkTreeModel *model;
-      GtkAction    *conflict_action = NULL;
-      GtkTreeIter   iter;
-      gboolean      iter_valid;
+      gimp_message_literal (view->gimp,
+                            G_OBJECT (view), GIMP_MESSAGE_ERROR,
+                            _("F1 cannot be remapped."));
+    }
+  else if (accel_key  >= GDK_KEY_0 &&
+           accel_key  <= GDK_KEY_9 &&
+           accel_mask == GDK_MOD1_MASK)
+    {
+      gimp_message (view->gimp,
+                    G_OBJECT (view), GIMP_MESSAGE_ERROR,
+                    _("Alt+%d is used to switch to display %d and "
+                      "cannot be remapped."),
+                    accel_key - GDK_KEY_0,
+                    accel_key == GDK_KEY_0 ? 10 : accel_key - GDK_KEY_0);
+    }
+  else
+    {
+      gchar **dup_actions;
+      gchar  *accel = gtk_accelerator_name (accel_key, accel_mask);
 
-      model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
-      model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
-
-      for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);
-           iter_valid;
-           iter_valid = gtk_tree_model_iter_next (model, &iter))
+      dup_actions = gtk_application_get_actions_for_accel (GTK_APPLICATION (view->gimp->app),
+                                                           accel);
+      if (dup_actions != NULL && dup_actions[0] != NULL)
         {
-          GtkTreeIter child_iter;
-          gboolean    child_valid;
+          GimpAction *conflict_action = NULL;
 
-          for (child_valid = gtk_tree_model_iter_children (model,
-                                                           &child_iter,
-                                                           &iter);
-               child_valid;
-               child_valid = gtk_tree_model_iter_next (model, &child_iter))
+          for (gint i = 0; dup_actions[i] != NULL; i++)
             {
-              guint           child_accel_key;
-              GdkModifierType child_accel_mask;
+              gint start;
 
-              gtk_tree_model_get (model, &child_iter,
-                                  GIMP_ACTION_VIEW_COLUMN_ACCEL_KEY,
-                                  &child_accel_key,
-                                  GIMP_ACTION_VIEW_COLUMN_ACCEL_MASK,
-                                  &child_accel_mask,
-                                  -1);
+              start = g_str_has_prefix (dup_actions[i], "app.") ? 4 : 0;
+              conflict_action = GIMP_ACTION (g_action_map_lookup_action (G_ACTION_MAP (view->gimp->app),
+                                                                         dup_actions[i] + start));
+              if (! conflict_action)
+                continue;
 
-              if (accel_key  == child_accel_key &&
-                  accel_mask == child_accel_mask)
-                {
-                  gtk_tree_model_get (model, &child_iter,
-                                      GIMP_ACTION_VIEW_COLUMN_ACTION,
-                                      &conflict_action,
-                                      -1);
-                  break;
-                }
+              if (conflict_action != action)
+                break;
+
+              conflict_action = NULL;
             }
 
           if (conflict_action)
-            break;
-        }
-
-      if (conflict_action != action)
-        {
-          if (conflict_action)
-            {
-              gimp_action_view_conflict_confirm (view, conflict_action,
-                                                 accel_key,
-                                                 accel_mask,
-                                                 accel_path);
-              g_object_unref (conflict_action);
-            }
+            gimp_action_view_conflict_confirm (view, conflict_action, action,
+                                               accel_key, accel_mask);
           else
-            {
-              gimp_message_literal (view->manager->gimp,
-				    G_OBJECT (view), GIMP_MESSAGE_ERROR,
-				    _("Changing shortcut failed."));
-            }
+            gimp_message_literal (view->gimp,
+                                  G_OBJECT (view), GIMP_MESSAGE_ERROR,
+                                  _("Changing shortcut failed."));
         }
+      else
+        {
+          gimp_action_set_accels (action, (const char*[]) { accel, NULL });
+        }
+      g_free (accel);
+      g_strfreev (dup_actions);
     }
 }
 
@@ -849,19 +788,25 @@ gimp_action_view_accel_cleared (GtkCellRendererAccel *accel,
                                 const char           *path_string,
                                 GimpActionView       *view)
 {
-  GtkAction   *action;
-  const gchar *accel_path;
+  GimpAction      *action;
+  guint            action_accel_key;
+  GdkModifierType  action_accel_mask;
 
-  accel_path = gimp_action_view_get_accel_action (view, path_string,
-                                                  &action);
+  gimp_action_view_get_accel_action (view, path_string,
+                                     &action,
+                                     &action_accel_key,
+                                     &action_accel_mask);
 
-  if (! accel_path)
+  if (! action)
     return;
 
-  if (! gtk_accel_map_change_entry (accel_path, 0, 0, FALSE))
+  if (action_accel_key == GDK_KEY_F1)
     {
-      gimp_message_literal (view->manager->gimp,
-			    G_OBJECT (view), GIMP_MESSAGE_ERROR,
-			    _("Removing shortcut failed."));
+      gimp_message_literal (view->gimp,
+                            G_OBJECT (view), GIMP_MESSAGE_ERROR,
+                            _("F1 cannot be remapped."));
+      return;
     }
+
+  gimp_action_set_accels (action, (const char*[]) { NULL });
 }
