@@ -710,8 +710,6 @@ gimp_color_dialog_image_changed (GimpContext     *context,
     {
       if (dialog->active_image)
         {
-          g_object_remove_weak_pointer (G_OBJECT (dialog->active_image),
-                                        (gpointer) &dialog->active_image);
           g_signal_handlers_disconnect_by_func (dialog->active_image,
                                                 G_CALLBACK (gimp_color_dialog_update),
                                                 dialog);
@@ -719,11 +717,11 @@ gimp_color_dialog_image_changed (GimpContext     *context,
                                                 gimp_color_dialog_update_simulation,
                                                 dialog);
         }
-      dialog->active_image = image;
+
+      g_set_weak_pointer (&dialog->active_image, image);
+
       if (image)
         {
-          g_object_add_weak_pointer (G_OBJECT (dialog->active_image),
-                                     (gpointer) &dialog->active_image);
           g_signal_connect_swapped (image, "notify::base-type",
                                     G_CALLBACK (gimp_color_dialog_update),
                                     dialog);
@@ -739,6 +737,7 @@ gimp_color_dialog_image_changed (GimpContext     *context,
 
           gimp_color_dialog_update_simulation (image, dialog);
         }
+
       gimp_color_dialog_update (dialog);
     }
 }
