@@ -15,15 +15,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <string.h>
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpcolor/gimpcolor.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
@@ -35,6 +37,7 @@
 #include "core/gimpcontext.h"
 
 #include "gimpcoloreditor.h"
+#include "gimpcolorhistory.h"
 #include "gimpdocked.h"
 #include "gimpfgbgeditor.h"
 #include "gimpfgbgview.h"
@@ -52,6 +55,7 @@ enum
 
 static void   gimp_color_editor_docked_iface_init (GimpDockedInterface  *iface);
 
+static void   gimp_color_editor_constructed     (GObject           *object);
 static void   gimp_color_editor_dispose         (GObject           *object);
 static void   gimp_color_editor_set_property    (GObject           *object,
                                                  guint              property_id,
@@ -62,8 +66,7 @@ static void   gimp_color_editor_get_property    (GObject           *object,
                                                  GValue            *value,
                                                  GParamSpec        *pspec);
 
-static void   gimp_color_editor_style_set       (GtkWidget         *widget,
-                                                 GtkStyle          *prev_style);
+static void   gimp_color_editor_style_updated   (GtkWidget         *widget);
 
 static void   gimp_color_editor_set_aux_info    (GimpDocked        *docked,
                                                  GList             *aux_info);
@@ -89,14 +92,15 @@ static void   gimp_color_editor_tab_toggled     (GtkWidget         *widget,
 static void   gimp_color_editor_fg_bg_notify    (GtkWidget         *widget,
                                                  GParamSpec        *pspec,
                                                  GimpColorEditor   *editor);
-#ifndef GDK_WINDOWING_QUARTZ
 static void   gimp_color_editor_color_picked    (GtkWidget         *widget,
                                                  const GimpRGB     *rgb,
                                                  GimpColorEditor   *editor);
-#endif
 static void   gimp_color_editor_entry_changed   (GimpColorHexEntry *entry,
                                                  GimpColorEditor   *editor);
 
+static void  gimp_color_editor_history_selected (GimpColorHistory *history,
+                                                 const GimpRGB    *rgb,
+                                                 GimpColorEditor  *editor);
 
 G_DEFINE_TYPE_WITH_CODE (GimpColorEditor, gimp_color_editor, GIMP_TYPE_EDITOR,
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_DOCKED,
@@ -113,16 +117,18 @@ gimp_color_editor_class_init (GimpColorEditorClass* klass)
   GObjectClass   *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-  object_class->dispose      = gimp_color_editor_dispose;
-  object_class->set_property = gimp_color_editor_set_property;
-  object_class->get_property = gimp_color_editor_get_property;
+  object_class->constructed   = gimp_color_editor_constructed;
+  object_class->dispose       = gimp_color_editor_dispose;
+  object_class->set_property  = gimp_color_editor_set_property;
+  object_class->get_property  = gimp_color_editor_get_property;
 
-  widget_class->style_set    = gimp_color_editor_style_set;
+  widget_class->style_updated = gimp_color_editor_style_updated;
 
   g_object_class_install_property (object_class, PROP_CONTEXT,
                                    g_param_spec_object ("context",
                                                         NULL, NULL,
                                                         GIMP_TYPE_CONTEXT,
+                                                        G_PARAM_CONSTRUCT |
                                                         GIMP_PARAM_READWRITE));
 }
 
@@ -145,7 +151,6 @@ gimp_color_editor_init (GimpColorEditor *editor)
 {
   GtkWidget   *notebook;
   GtkWidget   *hbox;
-  GtkWidget   *vbox;
   GtkWidget   *button;
   gint         content_spacing;
   gint         button_spacing;
@@ -154,6 +159,8 @@ gimp_color_editor_init (GimpColorEditor *editor)
   GimpHSV      hsv;
   GList       *list;
   GSList      *group;
+  gint         icon_width  = 40;
+  gint         icon_height = 38;
 
   editor->context = NULL;
   editor->edit_bg = FALSE;
@@ -174,7 +181,7 @@ gimp_color_editor_init (GimpColorEditor *editor)
 
   editor->notebook = gimp_color_selector_new (GIMP_TYPE_COLOR_NOTEBOOK,
                                               &rgb, &hsv,
-                                              GIMP_COLOR_SELECTOR_HUE);
+                                              GIMP_COLOR_SELECTOR_RED);
   gimp_color_selector_set_show_alpha (GIMP_COLOR_SELECTOR (editor->notebook),
                                       FALSE);
   gtk_box_pack_start (GTK_BOX (editor), editor->notebook,
@@ -185,7 +192,7 @@ gimp_color_editor_init (GimpColorEditor *editor)
                     G_CALLBACK (gimp_color_editor_color_changed),
                     editor);
 
-  notebook = GIMP_COLOR_NOTEBOOK (editor->notebook)->notebook;
+  notebook = gimp_color_notebook_get_notebook (GIMP_COLOR_NOTEBOOK (editor->notebook));
 
   gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
   gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
@@ -195,7 +202,7 @@ gimp_color_editor_init (GimpColorEditor *editor)
 
   group = NULL;
 
-  for (list = GIMP_COLOR_NOTEBOOK (editor->notebook)->selectors;
+  for (list = gimp_color_notebook_get_selectors (GIMP_COLOR_NOTEBOOK (editor->notebook));
        list;
        list = g_list_next (list))
     {
@@ -213,8 +220,8 @@ gimp_color_editor_init (GimpColorEditor *editor)
       gtk_box_pack_start (GTK_BOX (editor->hbox), button, TRUE, TRUE, 0);
       gtk_widget_show (button);
 
-      image = gtk_image_new_from_stock (selector_class->stock_id,
-                                        GTK_ICON_SIZE_BUTTON);
+      image = gtk_image_new_from_icon_name (selector_class->icon_name,
+                                            GTK_ICON_SIZE_MENU);
       gtk_container_add (GTK_CONTAINER (button), image);
       gtk_widget_show (image);
 
@@ -230,41 +237,35 @@ gimp_color_editor_init (GimpColorEditor *editor)
     }
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-  gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);
+  gtk_box_set_homogeneous (GTK_BOX (hbox), FALSE);
   gtk_box_pack_start (GTK_BOX (editor), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
   /*  FG/BG editor  */
   editor->fg_bg = gimp_fg_bg_editor_new (NULL);
-  gtk_box_pack_start (GTK_BOX (hbox), editor->fg_bg, TRUE, TRUE, 0);
+  gtk_icon_size_lookup (button_icon_size, &icon_width, &icon_height);
+  gtk_widget_set_size_request (editor->fg_bg,
+                               (gint) (icon_width * 1.75),
+                               (gint) (icon_height * 1.75));
+  gtk_box_pack_start (GTK_BOX (hbox), editor->fg_bg, FALSE, FALSE, 0);
   gtk_widget_show (editor->fg_bg);
 
   g_signal_connect (editor->fg_bg, "notify::active-color",
                     G_CALLBACK (gimp_color_editor_fg_bg_notify),
                     editor);
 
-  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 3);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
-  gtk_widget_show (vbox);
-
-#ifndef GDK_WINDOWING_QUARTZ
   /*  The color picker  */
   button = gimp_pick_button_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
 
   g_signal_connect (button, "color-picked",
                     G_CALLBACK (gimp_color_editor_color_picked),
                     editor);
-#endif
 
   /*  The hex triplet entry  */
   editor->hex_entry = gimp_color_hex_entry_new ();
-  gimp_help_set_help_data (editor->hex_entry,
-                           _("Hexadecimal color notation as used in HTML and "
-                             "CSS.  This entry also accepts CSS color names."),
-                           NULL);
-  gtk_box_pack_end (GTK_BOX (vbox), editor->hex_entry, FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (hbox), editor->hex_entry, TRUE, TRUE, 0);
   gtk_widget_show (editor->hex_entry);
 
   g_signal_connect (editor->hex_entry, "color-changed",
@@ -272,6 +273,24 @@ gimp_color_editor_init (GimpColorEditor *editor)
                     editor);
 }
 
+static void
+gimp_color_editor_constructed (GObject *object)
+{
+  GimpColorEditor *editor = GIMP_COLOR_EDITOR (object);
+  GtkWidget       *history;
+
+  G_OBJECT_CLASS (parent_class)->constructed (object);
+
+  /* The color history */
+  history = gimp_color_history_new (editor->context, 12);
+  gtk_box_pack_end (GTK_BOX (editor), history, FALSE, FALSE, 0);
+  gtk_widget_show (history);
+
+  g_signal_connect (history, "color-selected",
+                    G_CALLBACK (gimp_color_editor_history_selected),
+                    editor);
+}
+
 static void
 gimp_color_editor_dispose (GObject *object)
 {
@@ -325,14 +344,13 @@ gimp_color_editor_get_preview (GimpDocked  *docked,
                                GimpContext *context,
                                GtkIconSize  size)
 {
-  GtkSettings *settings = gtk_widget_get_settings (GTK_WIDGET (docked));
-  GtkWidget   *preview;
-  gint         width;
-  gint         height;
+  GtkWidget *preview;
+  gint       width;
+  gint       height;
 
   preview = gimp_fg_bg_view_new (context);
 
-  if (gtk_icon_size_lookup_for_settings (settings, size, &width, &height))
+  if (gtk_icon_size_lookup (size, &width, &height))
     gtk_widget_set_size_request (preview, width, height);
 
   return preview;
@@ -345,9 +363,11 @@ gimp_color_editor_set_aux_info (GimpDocked *docked,
                                 GList      *aux_info)
 {
   GimpColorEditor *editor   = GIMP_COLOR_EDITOR (docked);
-  GtkWidget       *notebook = GIMP_COLOR_NOTEBOOK (editor->notebook)->notebook;
+  GtkWidget       *notebook;
   GList           *list;
 
+  notebook = gimp_color_notebook_get_notebook (GIMP_COLOR_NOTEBOOK (editor->notebook));
+
   parent_docked_iface->set_aux_info (docked, aux_info);
 
   for (list = aux_info; list; list = g_list_next (list))
@@ -387,16 +407,19 @@ gimp_color_editor_get_aux_info (GimpDocked *docked)
 {
   GimpColorEditor    *editor   = GIMP_COLOR_EDITOR (docked);
   GimpColorNotebook  *notebook = GIMP_COLOR_NOTEBOOK (editor->notebook);
+  GimpColorSelector  *current;
   GList              *aux_info;
 
   aux_info = parent_docked_iface->get_aux_info (docked);
 
-  if (notebook->cur_page)
+  current = gimp_color_notebook_get_current_selector (notebook);
+
+  if (current)
     {
       GimpSessionInfoAux *aux;
 
       aux = gimp_session_info_aux_new (AUX_INFO_CURRENT_PAGE,
-                                       G_OBJECT_TYPE_NAME (notebook->cur_page));
+                                       G_OBJECT_TYPE_NAME (current));
       aux_info = g_list_append (aux_info, aux);
     }
 
@@ -421,6 +444,10 @@ gimp_color_editor_set_context (GimpDocked  *docked,
                                             gimp_color_editor_bg_changed,
                                             editor);
 
+      g_signal_handlers_disconnect_by_func (editor->context->gimp->config,
+                                            G_CALLBACK (gimp_color_editor_style_updated),
+                                            editor);
+
       g_object_unref (editor->context);
     }
 
@@ -439,6 +466,19 @@ gimp_color_editor_set_context (GimpDocked  *docked,
                         G_CALLBACK (gimp_color_editor_bg_changed),
                         editor);
 
+      g_signal_connect_object (editor->context->gimp->config,
+                               "notify::theme",
+                               G_CALLBACK (gimp_color_editor_style_updated),
+                               editor, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+      g_signal_connect_object (context->gimp->config,
+                               "notify::override-theme-icon-size",
+                               G_CALLBACK (gimp_color_editor_style_updated),
+                               editor, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+      g_signal_connect_object (context->gimp->config,
+                               "notify::custom-icon-size",
+                               G_CALLBACK (gimp_color_editor_style_updated),
+                               editor, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+
       if (editor->edit_bg)
         {
           gimp_context_get_background (editor->context, &rgb);
@@ -472,15 +512,25 @@ gimp_color_editor_new (GimpContext *context)
 }
 
 static void
-gimp_color_editor_style_set (GtkWidget *widget,
-                             GtkStyle  *prev_style)
+gimp_color_editor_style_updated (GtkWidget *widget)
 {
   GimpColorEditor *editor = GIMP_COLOR_EDITOR (widget);
+  GtkIconSize      button_icon_size;
+  gint             icon_width  = 40;
+  gint             icon_height = 38;
 
-  GTK_WIDGET_CLASS (parent_class)->style_set (widget, prev_style);
+  GTK_WIDGET_CLASS (parent_class)->style_updated (widget);
 
   if (editor->hbox)
     gimp_editor_set_box_style (GIMP_EDITOR (editor), GTK_BOX (editor->hbox));
+
+  gtk_widget_style_get (GTK_WIDGET (editor),
+                        "button-icon-size", &button_icon_size,
+                        NULL);
+  gtk_icon_size_lookup (button_icon_size, &icon_width, &icon_height);
+  gtk_widget_set_size_request (editor->fg_bg,
+                               (gint) (icon_width * 1.75),
+                               (gint) (icon_height * 1.75));
 }
 
 
@@ -594,7 +644,7 @@ gimp_color_editor_tab_toggled (GtkWidget       *widget,
           GtkWidget *notebook;
           gint       page_num;
 
-          notebook = GIMP_COLOR_NOTEBOOK (editor->notebook)->notebook;
+          notebook = gimp_color_notebook_get_notebook (GIMP_COLOR_NOTEBOOK (editor->notebook));
 
           page_num = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), selector);
 
@@ -636,7 +686,6 @@ gimp_color_editor_fg_bg_notify (GtkWidget       *widget,
     }
 }
 
-#ifndef GDK_WINDOWING_QUARTZ
 static void
 gimp_color_editor_color_picked (GtkWidget       *widget,
                                 const GimpRGB   *rgb,
@@ -650,7 +699,6 @@ gimp_color_editor_color_picked (GtkWidget       *widget,
         gimp_context_set_foreground (editor->context, rgb);
     }
 }
-#endif
 
 static void
 gimp_color_editor_entry_changed (GimpColorHexEntry *entry,
@@ -668,3 +716,17 @@ gimp_color_editor_entry_changed (GimpColorHexEntry *entry,
         gimp_context_set_foreground (editor->context, &rgb);
     }
 }
+
+static void
+gimp_color_editor_history_selected (GimpColorHistory *history,
+                                    const GimpRGB    *rgb,
+                                    GimpColorEditor  *editor)
+{
+  if (editor->context)
+    {
+      if (editor->edit_bg)
+        gimp_context_set_background (editor->context, rgb);
+      else
+        gimp_context_set_foreground (editor->context, rgb);
+    }
+}
