@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_EDITOR_H__
@@ -58,17 +58,26 @@ void        gimp_editor_create_menu       (GimpEditor           *editor,
 gboolean    gimp_editor_popup_menu        (GimpEditor           *editor,
                                            GimpMenuPositionFunc  position_func,
                                            gpointer              position_data);
+gboolean    gimp_editor_popup_menu_at_pointer
+                                          (GimpEditor           *editor,
+                                           const GdkEvent       *trigger_event);
+gboolean   gimp_editor_popup_menu_at_rect (GimpEditor           *editor,
+                                           GdkWindow            *window,
+                                           const GdkRectangle   *rect,
+                                           GdkGravity            rect_anchor,
+                                           GdkGravity            menu_anchor,
+                                           const GdkEvent       *trigger_event);
 
 GtkWidget * gimp_editor_add_button        (GimpEditor           *editor,
-                                           const gchar          *stock_id,
+                                           const gchar          *icon_name,
                                            const gchar          *tooltip,
                                            const gchar          *help_id,
                                            GCallback             callback,
                                            GCallback             extended_callback,
-                                           gpointer              callback_data);
-GtkWidget * gimp_editor_add_stock_box     (GimpEditor           *editor,
+                                           GObject              *callback_data);
+GtkWidget * gimp_editor_add_icon_box      (GimpEditor           *editor,
                                            GType                 enum_type,
-                                           const gchar          *stock_prefix,
+                                           const gchar          *icon_prefix,
                                            GCallback             callback,
                                            gpointer              callback_data);
 
