@@ -165,6 +165,7 @@
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_USE_SRGB    "gimp-image-color-profile-use-srgb"
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_SAVE        "gimp-image-color-profile-save"
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_IMPORT      "gimp-image-color-profile-import"
+#define GIMP_HELP_IMAGE_METADATA_ROTATION_IMPORT  "gimp-image-metadata-rotation-import"
 #define GIMP_HELP_IMAGE_GRID                      "gimp-image-grid"
 #define GIMP_HELP_IMAGE_PROPERTIES                "gimp-image-properties"
 
@@ -225,7 +226,8 @@
 #define GIMP_HELP_LAYER_RESIZE                    "gimp-layer-resize"
 #define GIMP_HELP_LAYER_RESIZE_TO_IMAGE           "gimp-layer-resize-to-image"
 #define GIMP_HELP_LAYER_SCALE                     "gimp-layer-scale"
-#define GIMP_HELP_LAYER_CROP                      "gimp-layer-crop"
+#define GIMP_HELP_LAYER_RESIZE_TO_SELECTION       "gimp-layer-resize-to-selection"
+#define GIMP_HELP_LAYER_CROP_TO_CONTENT           "gimp-layer-crop-to-content"
 #define GIMP_HELP_LAYER_EDIT                      "gimp-layer-edit"
 
 #define GIMP_HELP_CHANNEL_DIALOG                  "gimp-channel-dialog"
@@ -370,7 +372,7 @@
 #define GIMP_HELP_FILTER_DISPLACE                 "gimp-filter-displace"
 #define GIMP_HELP_FILTER_DISTANCE_MAP             "gimp-filter-distance-map"
 #define GIMP_HELP_FILTER_DITHER                   "gimp-filter-dither"
-#define GIMP_HELP_FILTER_DROPSHADOW               "gimp-filter-dropshadow"
+#define GIMP_HELP_FILTER_DROPSHADOW               "gimp-filter-drop-shadow"
 #define GIMP_HELP_FILTER_EDGE                     "gimp-filter-edge"
 #define GIMP_HELP_FILTER_EDGE_LAPLACE             "gimp-filter-edge-laplace"
 #define GIMP_HELP_FILTER_EDGE_NEON                "gimp-filter-edge-neon"
@@ -400,7 +402,7 @@
 #define GIMP_HELP_FILTER_LINEAR_SINUSOID          "gimp-filter-linear-sinusoid"
 #define GIMP_HELP_FILTER_LITTLE_PLANET            "gimp-filter-little-planet"
 #define GIMP_HELP_FILTER_LONG_SHADOW              "gimp-filter-long-shadow"
-#define GIMP_HELP_FILTER_MANTIUK_2006             "gimp-filter-manituk-2006"
+#define GIMP_HELP_FILTER_MANTIUK_2006             "gimp-filter-mantiuk-2006"
 #define GIMP_HELP_FILTER_MAZE                     "gimp-filter-maze"
 #define GIMP_HELP_FILTER_MEAN_CURVATURE_BLUR      "gimp-filter-mean-curvature-blur"
 #define GIMP_HELP_FILTER_MEDIAN_BLUR              "gimp-filter-median-blur"
@@ -698,7 +700,6 @@
 #define GIMP_HELP_DOCK_IMAGE_MENU                 "gimp-dock-image-menu"
 #define GIMP_HELP_DOCK_AUTO_BUTTON                "gimp-dock-auto-button"
 #define GIMP_HELP_DOCK_CHANGE_SCREEN              "gimp-dock-change-screen"
-#define GIMP_HELP_DOCK_SEPARATOR                  "gimp-dock-separator"
 
 #define GIMP_HELP_DOCK_TAB_ADD                    "gimp-dock-tab-add"
 #define GIMP_HELP_DOCK_TAB_CLOSE                  "gimp-dock-tab-close"
@@ -717,7 +718,6 @@
 #define GIMP_HELP_DEVICE_STATUS_DIALOG            "gimp-device-status-dialog"
 #define GIMP_HELP_DISPLAY_FILTER_DIALOG           "gimp-display-filter-dialog"
 #define GIMP_HELP_HISTOGRAM_DIALOG                "gimp-histogram-dialog"
-#define GIMP_HELP_INFO_DIALOG                     "gimp-info-dialog"
 #define GIMP_HELP_MODULE_DIALOG                   "gimp-module-dialog"
 #define GIMP_HELP_NAVIGATION_DIALOG               "gimp-navigation-dialog"
 #define GIMP_HELP_SYMMETRY_DIALOG                 "gimp-symmetry-dialog"
@@ -728,7 +728,6 @@
 
 #define GIMP_HELP_EXPORT_DIALOG                   "gimp-export-dialog"
 #define GIMP_HELP_EXPORT_CONFIRM_DIALOG           "gimp-export-confirm-dialog"
-#define GIMP_HELP_UNIT_DIALOG                     "gimp-unit-dialog"
 
 #define GIMP_HELP_WINDOWS_SHOW_DOCK               "gimp-windows-show-dock"
 #define GIMP_HELP_WINDOWS_HIDE_DOCKS              "gimp-windows-hide-docks"
@@ -740,20 +739,29 @@
 #define GIMP_HELP_HELP                            "gimp-help"
 #define GIMP_HELP_HELP_CONTEXT                    "gimp-help-context"
 
+/* Most of the colordisplay, colorselector and controller id's below are not
+   used since they are needed in /libgimpwidgets/ or /modules/ where this can't
+   be used. Instead the strings are used there directly.
+   We define them here anyway, to make gimp-help aware of them.
+ */
+#define GIMP_HELP_COLORDISPLAY_ACES_RRT           "gimp-colordisplay-aces-rrt"
+#define GIMP_HELP_COLORDISPLAY_CLIP               "gimp-colordisplay-clip"
 #define GIMP_HELP_COLORDISPLAY_COLORBLIND         "gimp-colordisplay-colorblind"
 #define GIMP_HELP_COLORDISPLAY_CONTRAST           "gimp-colordisplay-contrast"
 #define GIMP_HELP_COLORDISPLAY_GAMMA              "gimp-colordisplay-gamma"
-#define GIMP_HELP_COLORDISPLAY_GIMP               "gimp-colordisplay-gimp"
-#define GIMP_HELP_COLORDISPLAY_PROOF              "gimp-colordisplay-proof"
 
 #define GIMP_HELP_COLORSELECTOR_CMYK              "gimp-colorselector-cmyk"
+#define GIMP_HELP_COLORSELECTOR_GIMP              "gimp-colorselector-gimp"
+#define GIMP_HELP_COLORSELECTOR_NOTEBOOK          "gimp-colorselector-notebook"
+#define GIMP_HELP_COLORSELECTOR_PALETTE           "gimp-colorselector-palette"
+#define GIMP_HELP_COLORSELECTOR_SCALES            "gimp-colorselector-scales"
 #define GIMP_HELP_COLORSELECTOR_TRIANGLE          "gimp-colorselector-triangle"
-#define GIMP_HELP_COLORSELECTOR_WATER             "gimp-colorselector-water"
+#define GIMP_HELP_COLORSELECTOR_WATERCOLOR        "gimp-colorselector-watercolor"
 
 #define GIMP_HELP_CONTROLLER_KEYBOARD             "gimp-controller-keyboard"
+#define GIMP_HELP_CONTROLLER_DIRECTX_DIRECTINPUT  "gimp-controller-directx-directinput"
 #define GIMP_HELP_CONTROLLER_LINUX_INPUT          "gimp-controller-linux-input"
 #define GIMP_HELP_CONTROLLER_MIDI                 "gimp-controller-midi"
-#define GIMP_HELP_CONTROLLER_MOUSE                "gimp-controller-mouse"
 #define GIMP_HELP_CONTROLLER_WHEEL                "gimp-controller-wheel"
 
 #define GIMP_HELP_EXTENSIONS_DIALOG               "gimp-extensions-dialog"
