@@ -125,13 +125,16 @@ typedef struct _GimpToolPresetFactoryView    GimpToolPresetFactoryView;
 /*  menus  */
 
 typedef struct _GimpAction                   GimpAction;
+typedef struct _GimpActionImpl               GimpActionImpl;
 typedef struct _GimpActionFactory            GimpActionFactory;
 typedef struct _GimpActionGroup              GimpActionGroup;
 typedef struct _GimpDoubleAction             GimpDoubleAction;
 typedef struct _GimpEnumAction               GimpEnumAction;
 typedef struct _GimpMenuFactory              GimpMenuFactory;
 typedef struct _GimpProcedureAction          GimpProcedureAction;
+typedef struct _GimpRadioAction              GimpRadioAction;
 typedef struct _GimpStringAction             GimpStringAction;
+typedef struct _GimpToggleAction             GimpToggleAction;
 typedef struct _GimpUIManager                GimpUIManager;
 
 
@@ -210,6 +213,10 @@ typedef struct _GimpLanguageStore            GimpLanguageStore;
 typedef struct _GimpLayerModeBox             GimpLayerModeBox;
 typedef struct _GimpLayerModeComboBox        GimpLayerModeComboBox;
 typedef struct _GimpMessageBox               GimpMessageBox;
+typedef struct _GimpMenu                     GimpMenu;
+typedef struct _GimpMenuBar                  GimpMenuBar;
+typedef struct _GimpMenuModel                GimpMenuModel;
+typedef struct _GimpMenuShell                GimpMenuShell;
 typedef struct _GimpMeter                    GimpMeter;
 typedef struct _GimpModifiersEditor          GimpModifiersEditor;
 typedef struct _GimpOverlayBox               GimpOverlayBox;
@@ -232,6 +239,7 @@ typedef struct _GimpTagPopup                 GimpTagPopup;
 typedef struct _GimpTemplateEditor           GimpTemplateEditor;
 typedef struct _GimpTextStyleEditor          GimpTextStyleEditor;
 typedef struct _GimpThumbBox                 GimpThumbBox;
+typedef struct _GimpToolbar                  GimpToolbar;
 typedef struct _GimpToolButton               GimpToolButton;
 typedef struct _GimpToolPalette              GimpToolPalette;
 typedef struct _GimpTranslationStore         GimpTranslationStore;
