@@ -49,7 +49,8 @@
 #define WAIT_ANY -1
 #endif
 
-#include <gtk/gtk.h> /* need GDK_WINDOWING_FOO defines */
+#include <glib.h>
+#include <glib/gstdio.h>
 
 #ifndef G_OS_WIN32
 #include "libgimpbase/gimpsignal.h"
@@ -68,10 +69,6 @@
 #include <sys/select.h>
 #endif
 
-#ifdef GDK_WINDOWING_QUARTZ
-#include <Cocoa/Cocoa.h>
-#endif
-
 #if defined(G_OS_WIN32) || defined(G_WITH_CYGWIN)
 #  ifdef STRICT
 #  undef STRICT
@@ -199,15 +196,7 @@ gimp_main (GType  plug_in_type,
   gint i, j, k;
 
   /* Reduce risks */
-  {
-    typedef BOOL (WINAPI *t_SetDllDirectoryA) (LPCSTR lpPathName);
-    t_SetDllDirectoryA p_SetDllDirectoryA;
-
-    p_SetDllDirectoryA = (t_SetDllDirectoryA) GetProcAddress (GetModuleHandle ("kernel32.dll"),
-                                                              "SetDllDirectoryA");
-    if (p_SetDllDirectoryA)
-      (*p_SetDllDirectoryA) ("");
-  }
+  SetDllDirectoryW (L"");
 
   /* On Windows, set DLL search path to $INSTALLDIR/bin so that GEGL
    * file operations can find their respective file library DLLs (such
@@ -217,38 +206,28 @@ gimp_main (GType  plug_in_type,
     const gchar *install_dir;
     gchar       *bin_dir;
     LPWSTR       w_bin_dir;
-    int          n;
 
     w_bin_dir = NULL;
     install_dir = gimp_installation_directory ();
     bin_dir = g_build_filename (install_dir, "bin", NULL);
 
-    n = MultiByteToWideChar (CP_UTF8, MB_ERR_INVALID_CHARS,
-                             bin_dir, -1, NULL, 0);
-    if (n == 0)
-      goto out;
-
-    w_bin_dir = g_malloc_n (n + 1, sizeof (wchar_t));
-    n = MultiByteToWideChar (CP_UTF8, MB_ERR_INVALID_CHARS,
-                             bin_dir, -1,
-                             w_bin_dir, (n + 1) * sizeof (wchar_t));
-    if (n == 0)
-      goto out;
-
-    SetDllDirectoryW (w_bin_dir);
-
-  out:
+    w_bin_dir = g_utf8_to_utf16 (bin_dir, -1, NULL, NULL, NULL);
     if (w_bin_dir)
-      g_free (w_bin_dir);
+      {
+        SetDllDirectoryW (w_bin_dir);
+        g_free (w_bin_dir);
+      }
+
     g_free (bin_dir);
   }
 
 #ifdef HAVE_EXCHNDL
   /* Use Dr. Mingw (dumps backtrace on crash) if it is available. */
   {
-    time_t  t;
-    gchar  *filename;
-    gchar  *dir;
+    time_t   t;
+    gchar   *filename;
+    gchar   *dir;
+    wchar_t *plug_in_backtrace_path_utf16;
 
     /* This has to be the non-roaming directory (i.e., the local
      * directory) as backtraces correspond to the binaries on this
@@ -274,7 +253,14 @@ gimp_main (GType  plug_in_type,
       _prevExceptionFilter = SetUnhandledExceptionFilter (gimp_plugin_sigfatal_handler);
 
     ExcHndlInit ();
-    ExcHndlSetLogFileNameA (plug_in_backtrace_path);
+
+    plug_in_backtrace_path_utf16 = g_utf8_to_utf16 (plug_in_backtrace_path,
+                                                    -1, NULL, NULL, NULL);
+    if (plug_in_backtrace_path_utf16)
+      {
+        ExcHndlSetLogFileNameW (plug_in_backtrace_path_utf16);
+        g_free (plug_in_backtrace_path_utf16);
+      }
   }
 #endif /* HAVE_EXCHNDL */
 
@@ -283,7 +269,7 @@ gimp_main (GType  plug_in_type,
     typedef BOOL (WINAPI *t_SetProcessDEPPolicy) (DWORD dwFlags);
     t_SetProcessDEPPolicy p_SetProcessDEPPolicy;
 
-    p_SetProcessDEPPolicy = GetProcAddress (GetModuleHandle ("kernel32.dll"),
+    p_SetProcessDEPPolicy = GetProcAddress (GetModuleHandleW (L"kernel32.dll"),
                                             "SetProcessDEPPolicy");
     if (p_SetProcessDEPPolicy)
       (*p_SetProcessDEPPolicy) (PROCESS_DEP_ENABLE|PROCESS_DEP_DISABLE_ATL_THUNK_EMULATION);
@@ -295,7 +281,7 @@ gimp_main (GType  plug_in_type,
     typedef HRESULT (WINAPI *t_SetCurrentProcessExplicitAppUserModelID) (PCWSTR lpPathName);
     t_SetCurrentProcessExplicitAppUserModelID p_SetCurrentProcessExplicitAppUserModelID;
 
-    p_SetCurrentProcessExplicitAppUserModelID = (t_SetCurrentProcessExplicitAppUserModelID) GetProcAddress (GetModuleHandle ("shell32.dll"),
+    p_SetCurrentProcessExplicitAppUserModelID = (t_SetCurrentProcessExplicitAppUserModelID) GetProcAddress (GetModuleHandleW (L"shell32.dll"),
                                                                                                             "SetCurrentProcessExplicitAppUserModelID");
     if (p_SetCurrentProcessExplicitAppUserModelID)
       (*p_SetCurrentProcessExplicitAppUserModelID) (L"gimp.GimpApplication");
@@ -440,8 +426,9 @@ gimp_main (GType  plug_in_type,
       G_TYPE_STRING,           G_TYPE_PARAM_STRING,
       G_TYPE_STRV,             G_TYPE_PARAM_BOXED,
 
+      G_TYPE_BYTES,            G_TYPE_PARAM_BOXED,
+
       GIMP_TYPE_ARRAY,         GIMP_TYPE_PARAM_ARRAY,
-      GIMP_TYPE_UINT8_ARRAY,   GIMP_TYPE_PARAM_UINT8_ARRAY,
       GIMP_TYPE_INT32_ARRAY,   GIMP_TYPE_PARAM_INT32_ARRAY,
       GIMP_TYPE_FLOAT_ARRAY,   GIMP_TYPE_PARAM_FLOAT_ARRAY,
       GIMP_TYPE_RGB_ARRAY,     GIMP_TYPE_PARAM_RGB_ARRAY,
@@ -455,7 +442,13 @@ gimp_main (GType  plug_in_type,
       GIMP_TYPE_CHANNEL,       GIMP_TYPE_PARAM_CHANNEL,
       GIMP_TYPE_LAYER_MASK,    GIMP_TYPE_PARAM_LAYER_MASK,
       GIMP_TYPE_SELECTION,     GIMP_TYPE_PARAM_SELECTION,
-      GIMP_TYPE_VECTORS,       GIMP_TYPE_PARAM_VECTORS
+      GIMP_TYPE_VECTORS,       GIMP_TYPE_PARAM_VECTORS,
+
+      GIMP_TYPE_BRUSH,         GIMP_TYPE_PARAM_BRUSH,
+      GIMP_TYPE_FONT,          GIMP_TYPE_PARAM_FONT,
+      GIMP_TYPE_GRADIENT,      GIMP_TYPE_PARAM_GRADIENT,
+      GIMP_TYPE_PALETTE,       GIMP_TYPE_PARAM_PALETTE,
+      GIMP_TYPE_PATTERN,       GIMP_TYPE_PARAM_PATTERN
     };
 
     gint i;
@@ -495,7 +488,7 @@ gimp_main (GType  plug_in_type,
   /*  initialize i18n support  */
   setlocale (LC_ALL, "");
 
-  bindtextdomain (GETTEXT_PACKAGE"-libgimp", gimp_locale_directory ());
+  gimp_bind_text_domain (GETTEXT_PACKAGE"-libgimp", gimp_locale_directory ());
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
   bind_textdomain_codeset (GETTEXT_PACKAGE"-libgimp", "UTF-8");
 #endif
@@ -541,6 +534,8 @@ gimp_main (GType  plug_in_type,
   _gimp_plug_in_run (PLUG_IN);
 
   gimp_close ();
+  g_io_channel_unref (read_channel);
+  g_io_channel_unref (write_channel);
 
   return EXIT_SUCCESS;
 }
@@ -988,7 +983,7 @@ gimp_plugin_sigfatal_handler (PEXCEPTION_POINTERS pExceptionInfo)
       guchar  buffer[256];
       size_t  read_len;
 
-      stream = fopen (plug_in_backtrace_path, "r");
+      stream = g_fopen (plug_in_backtrace_path, "r");
       do
         {
           /* Just read and output directly the file content. */
