@@ -152,6 +152,7 @@ struct _GimpPlugInPrivate
   GHashTable *displays;
   GHashTable *images;
   GHashTable *items;
+  GHashTable *resources;
 };
 
 
@@ -326,9 +327,10 @@ gimp_plug_in_finalize (GObject *object)
 
   g_clear_pointer (&plug_in->priv->menu_branches, g_list_free);
 
-  gimp_plug_in_destroy_proxies (plug_in->priv->displays, TRUE);
-  gimp_plug_in_destroy_proxies (plug_in->priv->images,   TRUE);
-  gimp_plug_in_destroy_proxies (plug_in->priv->items,    TRUE);
+  gimp_plug_in_destroy_proxies (plug_in->priv->displays,  TRUE);
+  gimp_plug_in_destroy_proxies (plug_in->priv->images,    TRUE);
+  gimp_plug_in_destroy_proxies (plug_in->priv->items,     TRUE);
+  gimp_plug_in_destroy_proxies (plug_in->priv->resources, TRUE);
 
   gimp_plug_in_destroy_hashes (plug_in);
 
@@ -898,15 +900,17 @@ _gimp_plug_in_set_i18n (GimpPlugIn   *plug_in,
     }
   else
     {
+      gchar *utf8_catalog_dir = NULL;
+
       use_gettext = GIMP_PLUG_IN_GET_CLASS (plug_in)->set_i18n (plug_in,
                                                                 procedure_name,
                                                                 gettext_domain,
-                                                                catalog_dir);
+                                                                &utf8_catalog_dir);
       if (use_gettext)
         {
           gboolean reserved = FALSE;
 
-          if (! (*gettext_domain))
+          if (*gettext_domain == NULL)
             {
               *gettext_domain = g_strdup (plug_in->priv->translation_domain_name);
             }
@@ -919,17 +923,34 @@ _gimp_plug_in_set_i18n (GimpPlugIn   *plug_in,
                * set by the lib itself; devs must set NULL). See docs of
                * set_i18n() method.
                */
-              if (*catalog_dir)
-                {
-                  g_printerr ("[%s] Do not set a catalog directory with set_i18n() with reserved domain: %s\n",
-                              procedure_name, *gettext_domain);
-                  g_clear_pointer (catalog_dir, g_free);
-                }
+              if (utf8_catalog_dir != NULL)
+                g_printerr ("[%s] Do not set a catalog directory with set_i18n() with reserved domain: %s\n",
+                            procedure_name, *gettext_domain);
 
               *catalog_dir = g_strdup (gimp_locale_directory ());
               reserved = TRUE;
             }
 
+          if (utf8_catalog_dir != NULL && *catalog_dir == NULL)
+            {
+              GError *error = NULL;
+
+              /* The passed-on catalog directory is in UTF-8 because this is
+               * usually hardcoded (in which case it's easier to request a
+               * specific encoding, chosen at development time, rather than the
+               * "OS encoding", depending on runtime).
+               * But now we want to transform it to the encoding used for
+               * filenames by GLib.
+               */
+              *catalog_dir = g_filename_from_utf8 (utf8_catalog_dir, -1, NULL, NULL, &error);
+
+              if (*catalog_dir == NULL)
+                g_printerr ("[%s] provided catalog directory is not proper UTF-8: %s\n",
+                            procedure_name, error ? error->message : "(N/A)");
+
+              g_clear_error (&error);
+            }
+
           if (*catalog_dir && ! reserved)
             {
               if (g_path_is_absolute (*catalog_dir))
@@ -982,6 +1003,8 @@ _gimp_plug_in_set_i18n (GimpPlugIn   *plug_in,
               *catalog_dir = g_file_get_path (plug_in->priv->translation_domain_path);
             }
         }
+
+      g_clear_pointer (&utf8_catalog_dir, g_free);
     }
 
   if (use_gettext && ! g_file_test (*catalog_dir, G_FILE_TEST_IS_DIR))
@@ -1016,7 +1039,7 @@ _gimp_plug_in_create_procedure (GimpPlugIn  *plug_in,
 
   if (_gimp_plug_in_set_i18n (plug_in, procedure_name, &gettext_domain, &catalog_dir))
     {
-      bindtextdomain (gettext_domain, catalog_dir);
+      gimp_bind_text_domain (gettext_domain, catalog_dir);
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
       bind_textdomain_codeset (gettext_domain, "UTF-8");
 #endif
@@ -1346,7 +1369,7 @@ gimp_plug_in_proc_run_internal (GimpPlugIn    *plug_in,
   if (_gimp_plug_in_set_i18n (plug_in, gimp_procedure_get_name (procedure),
                               &gettext_domain, &catalog_dir))
     {
-      bindtextdomain (gettext_domain, catalog_dir);
+      gimp_bind_text_domain (gettext_domain, catalog_dir);
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
       bind_textdomain_codeset (gettext_domain, "UTF-8");
 #endif
@@ -1418,15 +1441,17 @@ gimp_plug_in_pop_procedure (GimpPlugIn    *plug_in,
 
   _gimp_procedure_destroy_proxies (procedure);
 
-  gimp_plug_in_destroy_proxies (plug_in->priv->displays, FALSE);
-  gimp_plug_in_destroy_proxies (plug_in->priv->images,   FALSE);
-  gimp_plug_in_destroy_proxies (plug_in->priv->items,    FALSE);
+  gimp_plug_in_destroy_proxies (plug_in->priv->displays,  FALSE);
+  gimp_plug_in_destroy_proxies (plug_in->priv->images,    FALSE);
+  gimp_plug_in_destroy_proxies (plug_in->priv->items,     FALSE);
+  gimp_plug_in_destroy_proxies (plug_in->priv->resources, FALSE);
 
   if (! plug_in->priv->procedure_stack)
     {
-      gimp_plug_in_destroy_proxies (plug_in->priv->displays, TRUE);
-      gimp_plug_in_destroy_proxies (plug_in->priv->images,   TRUE);
-      gimp_plug_in_destroy_proxies (plug_in->priv->items,    TRUE);
+      gimp_plug_in_destroy_proxies (plug_in->priv->displays,  TRUE);
+      gimp_plug_in_destroy_proxies (plug_in->priv->images,    TRUE);
+      gimp_plug_in_destroy_proxies (plug_in->priv->items,     TRUE);
+      gimp_plug_in_destroy_proxies (plug_in->priv->resources, TRUE);
 
       gimp_plug_in_destroy_hashes (plug_in);
     }
@@ -1562,12 +1587,73 @@ _gimp_plug_in_get_item (GimpPlugIn *plug_in,
   return item;
 }
 
+GimpResource *
+_gimp_plug_in_get_resource (GimpPlugIn *plug_in,
+                            gint32      resource_id)
+{
+  GimpResource *resource = NULL;
+
+  g_return_val_if_fail (GIMP_IS_PLUG_IN (plug_in), NULL);
+
+  if (G_UNLIKELY (! plug_in->priv->resources))
+    plug_in->priv->resources =
+      g_hash_table_new_full (g_direct_hash,
+                             g_direct_equal,
+                             NULL,
+                             (GDestroyNotify) g_object_unref);
+
+  resource = g_hash_table_lookup (plug_in->priv->resources,
+                                  GINT_TO_POINTER (resource_id));
+
+  if (! resource)
+    {
+      if (gimp_resource_id_is_brush (resource_id))
+        {
+          resource = g_object_new (GIMP_TYPE_BRUSH,
+                                   "id", resource_id,
+                                   NULL);
+        }
+      else if (gimp_resource_id_is_pattern (resource_id))
+        {
+          resource = g_object_new (GIMP_TYPE_PATTERN,
+                                   "id", resource_id,
+                                   NULL);
+        }
+      else if (gimp_resource_id_is_gradient (resource_id))
+        {
+          resource = g_object_new (GIMP_TYPE_GRADIENT,
+                                   "id", resource_id,
+                                   NULL);
+        }
+      else if (gimp_resource_id_is_palette (resource_id))
+        {
+          resource = g_object_new (GIMP_TYPE_PALETTE,
+                                   "id", resource_id,
+                                   NULL);
+        }
+      else if (gimp_resource_id_is_font (resource_id))
+        {
+          resource = g_object_new (GIMP_TYPE_FONT,
+                                   "id", resource_id,
+                                   NULL);
+        }
+
+      if (resource)
+        g_hash_table_insert (plug_in->priv->resources,
+                             GINT_TO_POINTER (resource_id),
+                             g_object_ref (resource) /* add debug ref */);
+    }
+
+  return resource;
+}
+
 static void
 gimp_plug_in_destroy_hashes (GimpPlugIn *plug_in)
 {
-  g_clear_pointer (&plug_in->priv->displays, g_hash_table_unref);
-  g_clear_pointer (&plug_in->priv->images,   g_hash_table_unref);
-  g_clear_pointer (&plug_in->priv->items,    g_hash_table_unref);
+  g_clear_pointer (&plug_in->priv->displays,  g_hash_table_unref);
+  g_clear_pointer (&plug_in->priv->images,    g_hash_table_unref);
+  g_clear_pointer (&plug_in->priv->items,     g_hash_table_unref);
+  g_clear_pointer (&plug_in->priv->resources, g_hash_table_unref);
 }
 
 static void
