@@ -16,11 +16,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpwidgets/gimpwidgets.h"
@@ -49,22 +50,21 @@ enum
   PROP_MODEL
 };
 
-
-typedef struct GimpZoomPreviewPrivate
-{
-  GimpDrawable  *drawable;
-  GimpZoomModel *model;
-  GdkRectangle   extents;
-} GimpZoomPreviewPrivate;
-
 typedef struct
 {
   gboolean  update;
 } PreviewSettings;
 
 
-#define GIMP_ZOOM_PREVIEW_GET_PRIVATE(obj) \
-  ((GimpZoomPreviewPrivate *) ((GimpZoomPreview *) (obj))->priv)
+struct _GimpZoomPreviewPrivate
+{
+  GimpDrawable  *drawable;
+  GimpZoomModel *model;
+  GdkRectangle   extents;
+};
+
+#define GET_PRIVATE(obj) (((GimpZoomPreview *) (obj))->priv)
+
 
 static void     gimp_zoom_preview_constructed     (GObject         *object);
 static void     gimp_zoom_preview_finalize        (GObject         *object);
@@ -84,8 +84,7 @@ static void     gimp_zoom_preview_set_adjustments (GimpZoomPreview *preview,
 static void     gimp_zoom_preview_size_allocate   (GtkWidget       *widget,
                                                    GtkAllocation   *allocation,
                                                    GimpZoomPreview *preview);
-static void     gimp_zoom_preview_style_set       (GtkWidget       *widget,
-                                                   GtkStyle        *prev_style);
+static void     gimp_zoom_preview_style_updated   (GtkWidget       *widget);
 static gboolean gimp_zoom_preview_scroll_event    (GtkWidget       *widget,
                                                    GdkEventScroll  *event,
                                                    GimpZoomPreview *preview);
@@ -121,7 +120,8 @@ static void     gimp_zoom_preview_get_source_area (GimpPreview     *preview,
                                                    gint            *h);
 
 
-G_DEFINE_TYPE (GimpZoomPreview, gimp_zoom_preview, GIMP_TYPE_SCROLLED_PREVIEW)
+G_DEFINE_TYPE_WITH_PRIVATE (GimpZoomPreview, gimp_zoom_preview,
+                            GIMP_TYPE_SCROLLED_PREVIEW)
 
 #define parent_class gimp_zoom_preview_parent_class
 
@@ -135,44 +135,47 @@ gimp_zoom_preview_class_init (GimpZoomPreviewClass *klass)
   GtkWidgetClass   *widget_class  = GTK_WIDGET_CLASS (klass);
   GimpPreviewClass *preview_class = GIMP_PREVIEW_CLASS (klass);
 
-  object_class->constructed  = gimp_zoom_preview_constructed;
-  object_class->finalize     = gimp_zoom_preview_finalize;
-  object_class->dispose      = gimp_zoom_preview_dispose;
-  object_class->get_property = gimp_zoom_preview_get_property;
-  object_class->set_property = gimp_zoom_preview_set_property;
+  object_class->constructed   = gimp_zoom_preview_constructed;
+  object_class->finalize      = gimp_zoom_preview_finalize;
+  object_class->dispose       = gimp_zoom_preview_dispose;
+  object_class->get_property  = gimp_zoom_preview_get_property;
+  object_class->set_property  = gimp_zoom_preview_set_property;
 
-  widget_class->style_set    = gimp_zoom_preview_style_set;
+  widget_class->style_updated = gimp_zoom_preview_style_updated;
 
-  preview_class->draw        = gimp_zoom_preview_draw;
-  preview_class->draw_buffer = gimp_zoom_preview_draw_buffer;
-  preview_class->draw_thumb  = gimp_zoom_preview_draw_thumb;
-  preview_class->set_cursor  = gimp_zoom_preview_set_cursor;
-  preview_class->transform   = gimp_zoom_preview_transform;
-  preview_class->untransform = gimp_zoom_preview_untransform;
-
-  g_type_class_add_private (object_class, sizeof (GimpZoomPreviewPrivate));
+  preview_class->draw         = gimp_zoom_preview_draw;
+  preview_class->draw_buffer  = gimp_zoom_preview_draw_buffer;
+  preview_class->draw_thumb   = gimp_zoom_preview_draw_thumb;
+  preview_class->set_cursor   = gimp_zoom_preview_set_cursor;
+  preview_class->transform    = gimp_zoom_preview_transform;
+  preview_class->untransform  = gimp_zoom_preview_untransform;
 
   /**
-   * GimpZoomPreview:drawable:
+   * GimpZoomPreview:drawable-id:
    *
    * The drawable the #GimpZoomPreview is attached to.
    *
-   * Since: GIMP 2.4
+   * Since: 2.10
    */
   g_object_class_install_property (object_class, PROP_DRAWABLE,
-                                   g_param_spec_pointer ("drawable", NULL, NULL,
-                                                         GIMP_PARAM_READWRITE |
-                                                         G_PARAM_CONSTRUCT_ONLY));
+                                   g_param_spec_object ("drawable",
+                                                        "Drawable",
+                                                        "The drawable this preview is attached to",
+                                                        GIMP_TYPE_DRAWABLE,
+                                                        GIMP_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT_ONLY));
 
   /**
    * GimpZoomPreview:model:
    *
    * The #GimpZoomModel used by this #GimpZoomPreview.
    *
-   * Since: GIMP 2.4
+   * Since: 2.4
    */
   g_object_class_install_property (object_class, PROP_MODEL,
-                                   g_param_spec_object ("model", NULL, NULL,
+                                   g_param_spec_object ("model",
+                                                        "Model",
+                                                        "The zoom preview's GimpZoomModel",
                                                         GIMP_TYPE_ZOOM_MODEL,
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT_ONLY));
@@ -181,20 +184,22 @@ gimp_zoom_preview_class_init (GimpZoomPreviewClass *klass)
 static void
 gimp_zoom_preview_init (GimpZoomPreview *preview)
 {
-  preview->priv = G_TYPE_INSTANCE_GET_PRIVATE (preview,
-                                               GIMP_TYPE_ZOOM_PREVIEW,
-                                               GimpZoomPreviewPrivate);
+  GtkWidget *area = gimp_preview_get_area (GIMP_PREVIEW (preview));
 
-  g_signal_connect (GIMP_PREVIEW (preview)->area, "size-allocate",
+  preview->priv = gimp_zoom_preview_get_instance_private (preview);
+
+  g_signal_connect (area, "size-allocate",
                     G_CALLBACK (gimp_zoom_preview_size_allocate),
                     preview);
-  g_signal_connect (GIMP_PREVIEW (preview)->area, "scroll-event",
+  g_signal_connect (area, "scroll-event",
                     G_CALLBACK (gimp_zoom_preview_scroll_event),
                     preview);
 
-  g_object_set (GIMP_PREVIEW (preview)->area,
+  g_object_set (area,
                 "check-size", gimp_check_size (),
                 "check-type", gimp_check_type (),
+                "check-custom-color1", gimp_check_custom_color1 (),
+                "check-custom-color2", gimp_check_custom_color2 (),
                 NULL);
 
   gimp_scrolled_preview_set_policy (GIMP_SCROLLED_PREVIEW (preview),
@@ -204,12 +209,11 @@ gimp_zoom_preview_init (GimpZoomPreview *preview)
 static void
 gimp_zoom_preview_constructed (GObject *object)
 {
-  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW_GET_PRIVATE (object);
+  GimpZoomPreviewPrivate *priv = GET_PRIVATE (object);
   gchar                  *data_name;
   PreviewSettings         settings;
 
-  if (G_OBJECT_CLASS (parent_class)->constructed)
-    G_OBJECT_CLASS (parent_class)->constructed (object);
+  G_OBJECT_CLASS (parent_class)->constructed (object);
 
   data_name = g_strdup_printf ("%s-zoom-preview-%d",
                                g_get_prgname (),
@@ -239,13 +243,10 @@ gimp_zoom_preview_constructed (GObject *object)
 static void
 gimp_zoom_preview_finalize (GObject *object)
 {
-  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW_GET_PRIVATE (object);
+  GimpZoomPreviewPrivate *priv = GET_PRIVATE (object);
 
-  if (priv->model)
-    {
-      g_object_unref (priv->model);
-      priv->model = NULL;
-    }
+  g_clear_object (&priv->model);
+  g_clear_object (&priv->drawable);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -280,7 +281,7 @@ gimp_zoom_preview_get_property (GObject    *object,
   switch (property_id)
     {
     case PROP_DRAWABLE:
-      g_value_set_pointer (value, gimp_zoom_preview_get_drawable (preview));
+      g_value_set_object (value, gimp_zoom_preview_get_drawable (preview));
       break;
 
     case PROP_MODEL:
@@ -304,7 +305,7 @@ gimp_zoom_preview_set_property (GObject      *object,
   switch (property_id)
     {
     case PROP_DRAWABLE:
-      gimp_zoom_preview_set_drawable (preview, g_value_get_pointer (value));
+      gimp_zoom_preview_set_drawable (preview, g_value_dup_object (value));
       break;
 
     case PROP_MODEL:
@@ -323,21 +324,22 @@ gimp_zoom_preview_set_adjustments (GimpZoomPreview *preview,
                                    gdouble          new_factor)
 {
   GimpScrolledPreview *scrolled_preview = GIMP_SCROLLED_PREVIEW (preview);
-  GtkAdjustment       *adj;
-  gdouble              width;
-  gdouble              height;
+  GtkAdjustment       *hadj;
+  GtkAdjustment       *vadj;
+  gint                 width;
+  gint                 height;
   gdouble              ratio;
 
   gimp_scrolled_preview_freeze (scrolled_preview);
 
-  width  = GIMP_PREVIEW (preview)->width;
-  height = GIMP_PREVIEW (preview)->height;
+  gimp_preview_get_size (GIMP_PREVIEW (preview), &width, &height);
 
   ratio = new_factor / old_factor;
 
-  adj = gtk_range_get_adjustment (GTK_RANGE (scrolled_preview->hscr));
-  gtk_adjustment_configure (adj,
-                            (gtk_adjustment_get_value (adj) + width / 2.0) * ratio
+  gimp_scrolled_preview_get_adjustments (scrolled_preview, &hadj, &vadj);
+
+  gtk_adjustment_configure (hadj,
+                            (gtk_adjustment_get_value (vadj) + width / 2.0) * ratio
                             - width / 2.0,
                             0,
                             width * new_factor,
@@ -345,9 +347,8 @@ gimp_zoom_preview_set_adjustments (GimpZoomPreview *preview,
                             MAX (width / 2.0, new_factor),
                             width);
 
-  adj = gtk_range_get_adjustment (GTK_RANGE (scrolled_preview->vscr));
-  gtk_adjustment_configure (adj,
-                            (gtk_adjustment_get_value (adj) + height / 2.0) * ratio
+  gtk_adjustment_configure (vadj,
+                            (gtk_adjustment_get_value (vadj) + height / 2.0) * ratio
                             - height / 2.0,
                             0,
                             height * new_factor,
@@ -363,61 +364,67 @@ gimp_zoom_preview_size_allocate (GtkWidget       *widget,
                                  GtkAllocation   *allocation,
                                  GimpZoomPreview *preview)
 {
-  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
-  gdouble                 zoom;
+  gdouble zoom;
 
+#if 0
   gint width  = GIMP_PREVIEW (preview)->xmax - GIMP_PREVIEW (preview)->xmin;
   gint height = GIMP_PREVIEW (preview)->ymax - GIMP_PREVIEW (preview)->ymin;
 
   GIMP_PREVIEW (preview)->width  = MIN (width,  allocation->width);
   GIMP_PREVIEW (preview)->height = MIN (height, allocation->height);
+#endif
 
-  zoom = gimp_zoom_model_get_factor (priv->model);
+  zoom = gimp_zoom_model_get_factor (preview->priv->model);
 
   gimp_zoom_preview_set_adjustments (preview, zoom, zoom);
 }
 
 static void
-gimp_zoom_preview_style_set (GtkWidget *widget,
-                             GtkStyle  *prev_style)
+gimp_zoom_preview_style_updated (GtkWidget *widget)
 {
-  GimpPreview            *preview  = GIMP_PREVIEW (widget);
-  GimpZoomPreviewPrivate *priv     = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
-  GimpDrawable           *drawable = priv->drawable;
-  gint                    size;
-  gint                    width, height;
-  gint                    x1, y1;
-  gint                    x2, y2;
+  GimpPreview *preview = GIMP_PREVIEW (widget);
+  GtkWidget   *area    = gimp_preview_get_area (preview);
 
-  if (GTK_WIDGET_CLASS (parent_class)->style_set)
-    GTK_WIDGET_CLASS (parent_class)->style_set (widget, prev_style);
+  GTK_WIDGET_CLASS (parent_class)->style_updated (widget);
 
-  gtk_widget_style_get (widget, "size", &size, NULL);
-
-  if (_gimp_drawable_preview_get_bounds (drawable, &x1, &y1, &x2, &y2))
+  if (area)
     {
-      width  = x2 - x1;
-      height = y2 - y1;
-    }
-  else
-    {
-      width  = gimp_drawable_width  (drawable->drawable_id);
-      height = gimp_drawable_height (drawable->drawable_id);
-    }
+      GimpZoomPreviewPrivate *priv = GET_PRIVATE (preview);
+      gint                    size;
+      gint                    width;
+      gint                    height;
+      gint                    preview_width;
+      gint                    preview_height;
+      gint                    x1, y1;
+      gint                    x2, y2;
 
-  if (width > height)
-    {
-      preview->width  = MIN (width, size);
-      preview->height = (height * preview->width) / width;
-    }
-  else
-    {
-      preview->height = MIN (height, size);
-      preview->width  = (width * preview->height) / height;
-    }
+      gtk_widget_style_get (widget, "size", &size, NULL);
 
-  gtk_widget_set_size_request (preview->area,
-                               preview->width, preview->height);
+      if (_gimp_drawable_preview_get_bounds (priv->drawable,
+                                             &x1, &y1, &x2, &y2))
+        {
+          width  = x2 - x1;
+          height = y2 - y1;
+        }
+      else
+        {
+          width  = gimp_drawable_get_width  (priv->drawable);
+          height = gimp_drawable_get_height (priv->drawable);
+        }
+
+      if (width > height)
+        {
+          preview_width  = MIN (width, size);
+          preview_height = (height * preview_width) / width;
+        }
+      else
+        {
+          preview_height = MIN (height, size);
+          preview_width  = (width * preview_height) / height;
+        }
+
+      gimp_preview_set_size (preview, preview_width, preview_height);
+    }
 }
 
 static gboolean
@@ -427,7 +434,8 @@ gimp_zoom_preview_scroll_event (GtkWidget       *widget,
 {
   if (event->state & GDK_CONTROL_MASK)
     {
-      GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
+      GimpZoomPreviewPrivate *priv = GET_PRIVATE (preview);
+      gdouble                 delta;
 
       gimp_scrolled_preview_freeze (GIMP_SCROLLED_PREVIEW (preview));
 
@@ -441,6 +449,11 @@ gimp_zoom_preview_scroll_event (GtkWidget       *widget,
           gimp_zoom_model_zoom (priv->model, GIMP_ZOOM_OUT, 0.0);
           break;
 
+        case GDK_SCROLL_SMOOTH:
+          gdk_event_get_scroll_deltas ((GdkEvent *) event, NULL, &delta);
+          gimp_zoom_model_zoom (priv->model, GIMP_ZOOM_SMOOTH, delta);
+          break;
+
         default:
           break;
         }
@@ -454,8 +467,7 @@ gimp_zoom_preview_scroll_event (GtkWidget       *widget,
 static void
 gimp_zoom_preview_draw (GimpPreview *preview)
 {
-  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
-  GimpDrawable           *drawable;
+  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW (preview)->priv;
   guchar                 *data;
   gint                    width;
   gint                    height;
@@ -464,8 +476,7 @@ gimp_zoom_preview_draw (GimpPreview *preview)
   if (! priv->model)
     return;
 
-  drawable = priv->drawable;
-  if (! drawable)
+  if (priv->drawable == NULL)
     return;
 
   data = gimp_zoom_preview_get_source (GIMP_ZOOM_PREVIEW (preview),
@@ -473,9 +484,11 @@ gimp_zoom_preview_draw (GimpPreview *preview)
 
   if (data)
     {
-      gimp_preview_area_draw (GIMP_PREVIEW_AREA (preview->area),
+      GtkWidget *area = gimp_preview_get_area (preview);
+
+      gimp_preview_area_draw (GIMP_PREVIEW_AREA (area),
                               0, 0, width, height,
-                              gimp_drawable_type (drawable->drawable_id),
+                              gimp_drawable_type (priv->drawable),
                               data, width * bpp);
       g_free (data);
     }
@@ -486,65 +499,66 @@ gimp_zoom_preview_draw_buffer (GimpPreview  *preview,
                                const guchar *buffer,
                                gint          rowstride)
 {
-  GimpZoomPreviewPrivate *priv     = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
-  GimpDrawable           *drawable = priv->drawable;
-  gint32                  image_id;
+  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW (preview)->priv;
+  GtkWidget              *area = gimp_preview_get_area (preview);
+  GimpImage              *image;
+  gint                    width;
+  gint                    height;
 
-  image_id = gimp_item_get_image (drawable->drawable_id);
+  gimp_preview_get_size (preview, &width, &height);
+  image = gimp_item_get_image (GIMP_ITEM (priv->drawable));
 
-  if (gimp_selection_is_empty (image_id))
+  if (gimp_selection_is_empty (image))
     {
-      gimp_preview_area_draw (GIMP_PREVIEW_AREA (preview->area),
+      gimp_preview_area_draw (GIMP_PREVIEW_AREA (area),
                               0, 0,
-                              preview->width, preview->height,
-                              gimp_drawable_type (drawable->drawable_id),
+                              width, height,
+                              gimp_drawable_type (priv->drawable),
                               buffer,
                               rowstride);
     }
   else
     {
-      guchar  *sel;
-      guchar  *src;
-      gint     selection_id;
-      gint     width, height;
-      gint     bpp;
-      gint     src_x;
-      gint     src_y;
-      gint     src_width;
-      gint     src_height;
-      gint     offsx = 0;
-      gint     offsy = 0;
+      GBytes        *sel;
+      GBytes        *src;
+      GimpSelection *selection;
+      gint           w, h;
+      gint           bpp;
+      gint           src_x;
+      gint           src_y;
+      gint           src_width;
+      gint           src_height;
+      gint           offsx = 0;
+      gint           offsy = 0;
 
-      selection_id = gimp_image_get_selection (image_id);
-
-      width  = preview->width;
-      height = preview->height;
+      selection = gimp_image_get_selection (image);
 
       gimp_zoom_preview_get_source_area (preview,
                                          &src_x, &src_y,
                                          &src_width, &src_height);
 
-      src = gimp_drawable_get_sub_thumbnail_data (drawable->drawable_id,
+      src = gimp_drawable_get_sub_thumbnail_data (priv->drawable,
                                                   src_x, src_y,
                                                   src_width, src_height,
-                                                  &width, &height, &bpp);
-      gimp_drawable_offsets (drawable->drawable_id, &offsx, &offsy);
-      sel = gimp_drawable_get_sub_thumbnail_data (selection_id,
+                                                  width, height,
+                                                  &w, &h, &bpp);
+      gimp_drawable_get_offsets (priv->drawable, &offsx, &offsy);
+      sel = gimp_drawable_get_sub_thumbnail_data (GIMP_DRAWABLE (selection),
                                                   src_x + offsx, src_y + offsy,
                                                   src_width, src_height,
-                                                  &width, &height, &bpp);
+                                                  width, height,
+                                                  &w, &h, &bpp);
 
-      gimp_preview_area_mask (GIMP_PREVIEW_AREA (preview->area),
-                              0, 0, preview->width, preview->height,
-                              gimp_drawable_type (drawable->drawable_id),
-                              src, width * drawable->bpp,
+      gimp_preview_area_mask (GIMP_PREVIEW_AREA (area),
+                              0, 0, width, height,
+                              gimp_drawable_type (priv->drawable),
+                              g_bytes_get_data (src, NULL), width * gimp_drawable_get_bpp (priv->drawable),
                               buffer, rowstride,
-                              sel, width);
+                              g_bytes_get_data (sel, NULL), width);
 
-      g_free (sel);
-      g_free (src);
+      g_bytes_unref (sel);
+      g_bytes_unref (src);
     }
-
 }
 
 static void
@@ -553,28 +567,35 @@ gimp_zoom_preview_draw_thumb (GimpPreview     *preview,
                               gint             width,
                               gint             height)
 {
-  GimpZoomPreviewPrivate *priv     = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
-  GimpDrawable           *drawable = priv->drawable;
+  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW (preview)->priv;
 
-  if (drawable)
-    _gimp_drawable_preview_area_draw_thumb (area, drawable, width, height);
+  if (priv->drawable != NULL)
+    _gimp_drawable_preview_area_draw_thumb (area, priv->drawable,
+                                            width, height);
 }
 
 static void
 gimp_zoom_preview_set_cursor (GimpPreview *preview)
 {
-  if (! gtk_widget_get_realized (preview->area))
+  GtkWidget *area = gimp_preview_get_area (preview);
+
+  if (! gtk_widget_get_realized (area))
     return;
 
   if (gimp_zoom_preview_get_factor (GIMP_ZOOM_PREVIEW (preview)) > 1.0)
     {
-      gdk_window_set_cursor (gtk_widget_get_window (preview->area),
-                             GIMP_SCROLLED_PREVIEW (preview)->cursor_move);
+      GdkDisplay *display = gtk_widget_get_display (GTK_WIDGET (preview));
+      GdkCursor  *cursor;
+
+      cursor = gdk_cursor_new_for_display (display, GDK_HAND1);
+      gdk_window_set_cursor (gtk_widget_get_window (area),
+                             cursor);
+      g_object_unref (cursor);
     }
   else
     {
-      gdk_window_set_cursor (gtk_widget_get_window (preview->area),
-                             preview->default_cursor);
+      gdk_window_set_cursor (gtk_widget_get_window (area),
+                             gimp_preview_get_default_cursor (preview));
     }
 }
 
@@ -585,15 +606,23 @@ gimp_zoom_preview_transform (GimpPreview *preview,
                              gint        *dest_x,
                              gint        *dest_y)
 {
-  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
+  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW (preview)->priv;
+  gint                    width;
+  gint                    height;
+  gint                    xoff;
+  gint                    yoff;
+  gdouble                 zoom;
 
-  gdouble zoom = gimp_zoom_preview_get_factor (GIMP_ZOOM_PREVIEW (preview));
+  gimp_preview_get_size (preview, &width, &height);
+  gimp_preview_get_offsets (preview, &xoff, &yoff);
+
+  zoom = gimp_zoom_preview_get_factor (GIMP_ZOOM_PREVIEW (preview));
 
   *dest_x = ((gdouble) (src_x - priv->extents.x) *
-             preview->width / priv->extents.width * zoom) - preview->xoff;
+             width / priv->extents.width * zoom) - xoff;
 
   *dest_y = ((gdouble) (src_y - priv->extents.y) *
-             preview->height / priv->extents.height * zoom) - preview->yoff;
+             height / priv->extents.height * zoom) - yoff;
 }
 
 static void
@@ -603,42 +632,49 @@ gimp_zoom_preview_untransform (GimpPreview *preview,
                                gint        *dest_x,
                                gint        *dest_y)
 {
-  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
+  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW (preview)->priv;
+  gint                    width;
+  gint                    height;
+  gint                    xoff;
+  gint                    yoff;
+  gdouble                 zoom;
 
-  gdouble zoom = gimp_zoom_preview_get_factor (GIMP_ZOOM_PREVIEW (preview));
+  gimp_preview_get_size (preview, &width, &height);
+  gimp_preview_get_offsets (preview, &xoff, &yoff);
+
+  zoom = gimp_zoom_preview_get_factor (GIMP_ZOOM_PREVIEW (preview));
 
   *dest_x = (priv->extents.x +
-             ((gdouble) (src_x + preview->xoff) *
-              priv->extents.width / preview->width / zoom));
+             ((gdouble) (src_x + xoff) *
+              priv->extents.width / width / zoom));
 
   *dest_y = (priv->extents.y +
-             ((gdouble) (src_y + preview->yoff) *
-              priv->extents.height / preview->height / zoom));
+             ((gdouble) (src_y + yoff) *
+              priv->extents.height / height / zoom));
 }
 
 static void
 gimp_zoom_preview_set_drawable (GimpZoomPreview *preview,
                                 GimpDrawable    *drawable)
 {
-  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
+  GimpZoomPreviewPrivate *priv = preview->priv;
   gint                    x, y;
   gint                    width, height;
   gint                    max_width, max_height;
 
-  g_return_if_fail (priv->drawable == NULL);
+  g_return_if_fail (preview->priv->drawable == NULL);
 
   priv->drawable = drawable;
 
-  if (gimp_drawable_mask_intersect (drawable->drawable_id,
-                                    &x, &y, &width, &height))
+  if (gimp_drawable_mask_intersect (drawable, &x, &y, &width, &height))
     {
       priv->extents.x = x;
       priv->extents.y = y;
     }
   else
     {
-      width  = gimp_drawable_width  (drawable->drawable_id);
-      height = gimp_drawable_height (drawable->drawable_id);
+      width  = gimp_drawable_get_width  (drawable);
+      height = gimp_drawable_get_height (drawable);
 
       priv->extents.x = 0;
       priv->extents.y = 0;
@@ -661,7 +697,7 @@ gimp_zoom_preview_set_drawable (GimpZoomPreview *preview,
   gimp_preview_set_bounds (GIMP_PREVIEW (preview),
                            0, 0, max_width, max_height);
 
-  g_object_set (GIMP_PREVIEW (preview)->frame,
+  g_object_set (gimp_preview_get_frame (GIMP_PREVIEW (preview)),
                 "ratio", (gdouble) width / (gdouble) height,
                 NULL);
 }
@@ -670,7 +706,7 @@ static void
 gimp_zoom_preview_set_model (GimpZoomPreview *preview,
                              GimpZoomModel   *model)
 {
-  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
+  GimpZoomPreviewPrivate *priv = GET_PRIVATE (preview);
   GtkWidget              *button_bar;
   GtkWidget              *button;
   GtkWidget              *box;
@@ -713,7 +749,7 @@ gimp_zoom_preview_get_source_area (GimpPreview *preview,
                                    gint        *w,
                                    gint        *h)
 {
-  GimpZoomPreviewPrivate *priv = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
+  GimpZoomPreviewPrivate *priv = GET_PRIVATE (preview);
   gdouble                 zoom = gimp_zoom_model_get_factor (priv->model);
 
   gimp_zoom_preview_untransform (preview, 0, 0, x, y);
@@ -724,19 +760,20 @@ gimp_zoom_preview_get_source_area (GimpPreview *preview,
 
 
 /**
- * gimp_zoom_preview_new:
- * @drawable: a #GimpDrawable
+ * gimp_zoom_preview_new_from_drawable:
+ * @drawable: (transfer none): a drawable
  *
  * Creates a new #GimpZoomPreview widget for @drawable.
  *
- * Since: GIMP 2.4
+ * Since: 3.0
  *
  * Returns: a new #GimpZoomPreview.
  **/
 GtkWidget *
-gimp_zoom_preview_new (GimpDrawable *drawable)
+gimp_zoom_preview_new_from_drawable (GimpDrawable *drawable)
 {
-  g_return_val_if_fail (drawable != NULL, NULL);
+  g_return_val_if_fail (GIMP_IS_DRAWABLE (drawable), NULL);
+  g_return_val_if_fail (gimp_item_is_valid (GIMP_ITEM (drawable)), NULL);
 
   return g_object_new (GIMP_TYPE_ZOOM_PREVIEW,
                        "drawable", drawable,
@@ -744,28 +781,28 @@ gimp_zoom_preview_new (GimpDrawable *drawable)
 }
 
 /**
- * gimp_zoom_preview_new_with_model:
+ * gimp_zoom_preview_new_with_model_from_drawable:
  * @drawable: a #GimpDrawable
  * @model:    a #GimpZoomModel
  *
  * Creates a new #GimpZoomPreview widget for @drawable using the
  * given @model.
  *
- * This variant of gimp_zoom_preview_new() allows you to create a
- * preview using an existing zoom model. This may be useful if for
- * example you want to have two zoom previews that keep their zoom
- * factor in sync.
+ * This variant of gimp_zoom_preview_new_from_drawable() allows you
+ * to create a preview using an existing zoom model. This may be
+ * useful if for example you want to have two zoom previews that keep
+ * their zoom factor in sync.
  *
- * Since: GIMP 2.4
+ * Since: 2.10
  *
  * Returns: a new #GimpZoomPreview.
  **/
 GtkWidget *
-gimp_zoom_preview_new_with_model (GimpDrawable  *drawable,
-                                  GimpZoomModel *model)
-
+gimp_zoom_preview_new_with_model_from_drawable (GimpDrawable  *drawable,
+                                                GimpZoomModel *model)
 {
-  g_return_val_if_fail (drawable != NULL, NULL);
+  g_return_val_if_fail (GIMP_IS_DRAWABLE (drawable), NULL);
+  g_return_val_if_fail (gimp_item_is_valid (GIMP_ITEM (drawable)), NULL);
   g_return_val_if_fail (GIMP_IS_ZOOM_MODEL (model), NULL);
 
   return g_object_new (GIMP_TYPE_ZOOM_PREVIEW,
@@ -774,23 +811,23 @@ gimp_zoom_preview_new_with_model (GimpDrawable  *drawable,
                        NULL);
 }
 
-
 /**
  * gimp_zoom_preview_get_drawable:
  * @preview: a #GimpZoomPreview widget
  *
- * Returns the #GimpDrawable the #GimpZoomPreview is attached to.
+ * Returns the drawable the #GimpZoomPreview is attached to.
  *
- * Return Value: the #GimpDrawable that was passed to gimp_zoom_preview_new().
+ * Returns: (transfer none): the drawable that was passed to
+ *          gimp_zoom_preview_new_from_drawable().
  *
- * Since: GIMP 2.4
+ * Since: 3.0
  **/
 GimpDrawable *
 gimp_zoom_preview_get_drawable (GimpZoomPreview *preview)
 {
   g_return_val_if_fail (GIMP_IS_ZOOM_PREVIEW (preview), NULL);
 
-  return GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview)->drawable;
+  return GET_PRIVATE (preview)->drawable;
 }
 
 /**
@@ -799,16 +836,17 @@ gimp_zoom_preview_get_drawable (GimpZoomPreview *preview)
  *
  * Returns the #GimpZoomModel the preview is using.
  *
- * Return Value: a pointer to the #GimpZoomModel owned by the @preview
+ * Returns: (transfer none): a pointer to the #GimpZoomModel owned
+ *          by the @preview
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 GimpZoomModel *
 gimp_zoom_preview_get_model (GimpZoomPreview *preview)
 {
   g_return_val_if_fail (GIMP_IS_ZOOM_PREVIEW (preview), NULL);
 
-  return GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview)->model;
+  return GET_PRIVATE (preview)->model;
 }
 
 /**
@@ -817,9 +855,9 @@ gimp_zoom_preview_get_model (GimpZoomPreview *preview)
  *
  * Returns the zoom factor the preview is currently using.
  *
- * Return Value: the current zoom factor
+ * Returns: the current zoom factor
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 gdouble
 gimp_zoom_preview_get_factor (GimpZoomPreview *preview)
@@ -828,7 +866,7 @@ gimp_zoom_preview_get_factor (GimpZoomPreview *preview)
 
   g_return_val_if_fail (GIMP_IS_ZOOM_PREVIEW (preview), 1.0);
 
-  priv = GIMP_ZOOM_PREVIEW_GET_PRIVATE (preview);
+  priv = GET_PRIVATE (preview);
 
   return priv->model ? gimp_zoom_model_get_factor (priv->model) : 1.0;
 }
@@ -836,21 +874,21 @@ gimp_zoom_preview_get_factor (GimpZoomPreview *preview)
 /**
  * gimp_zoom_preview_get_source:
  * @preview: a #GimpZoomPreview widget
- * @width: a pointer to an int where the current width of the zoom widget
+ * @width: (out): a pointer to an int where the current width of the zoom widget
  *         will be put.
- * @height: a pointer to an int where the current width of the zoom widget
+ * @height: (out): a pointer to an int where the current width of the zoom widget
  *          will be put.
- * @bpp: return location for the number of bytes per pixel
+ * @bpp: (out): return location for the number of bytes per pixel
  *
  * Returns the scaled image data of the part of the drawable the
  * #GimpZoomPreview is currently showing, as a newly allocated array of guchar.
  * This function also allow to get the current width, height and bpp of the
  * #GimpZoomPreview.
  *
- * Return Value: newly allocated data that should be released using g_free()
- *               when it is not any longer needed
+ * Returns: (transfer full) (array): newly allocated data that should be
+ *          released using g_free() when it is not any longer needed
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  */
 guchar *
 gimp_zoom_preview_get_source (GimpZoomPreview *preview,
@@ -872,18 +910,21 @@ gimp_zoom_preview_get_source (GimpZoomPreview *preview,
       gint         src_y;
       gint         src_width;
       gint         src_height;
+      GBytes      *src_bytes;
+      gsize        src_bytes_size;
 
-      *width  = gimp_preview->width;
-      *height = gimp_preview->height;
+      gimp_preview_get_size (gimp_preview, width, height);
 
       gimp_zoom_preview_get_source_area (gimp_preview,
                                          &src_x, &src_y,
                                          &src_width, &src_height);
 
-      return gimp_drawable_get_sub_thumbnail_data (drawable->drawable_id,
-                                                   src_x, src_y,
-                                                   src_width, src_height,
-                                                   width, height, bpp);
+      src_bytes =  gimp_drawable_get_sub_thumbnail_data (drawable,
+                                                         src_x, src_y,
+                                                         src_width, src_height,
+                                                         *width, *height,
+                                                         width, height, bpp);
+      return g_bytes_unref_to_data (src_bytes, &src_bytes_size);
     }
   else
     {
