@@ -16,13 +16,14 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <cairo.h>
-#include <glib-object.h>
+#include <gegl.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "libgimpbase/gimpbase.h"
 #include "libgimpcolor/gimpcolor.h"
@@ -31,6 +32,7 @@
 #include "gimpconfigtypes.h"
 
 #include "gimpconfigwriter.h"
+#include "gimpconfig-array.h"
 #include "gimpconfig-iface.h"
 #include "gimpconfig-deserialize.h"
 #include "gimpconfig-params.h"
@@ -56,41 +58,50 @@
  *  couldn't parse it.
  */
 
-static GTokenType  gimp_config_deserialize_value       (GValue     *value,
-                                                        GimpConfig *config,
-                                                        GParamSpec *prop_spec,
-                                                        GScanner   *scanner);
-static GTokenType  gimp_config_deserialize_fundamental (GValue     *value,
-                                                        GParamSpec *prop_spec,
-                                                        GScanner   *scanner);
-static GTokenType  gimp_config_deserialize_enum        (GValue     *value,
-                                                        GParamSpec *prop_spec,
-                                                        GScanner   *scanner);
-static GTokenType  gimp_config_deserialize_memsize     (GValue     *value,
-                                                        GParamSpec *prop_spec,
-                                                        GScanner   *scanner);
-static GTokenType  gimp_config_deserialize_path        (GValue     *value,
-                                                        GParamSpec *prop_spec,
-                                                        GScanner   *scanner);
-static GTokenType  gimp_config_deserialize_rgb         (GValue     *value,
-                                                        GParamSpec *prop_spec,
-                                                        GScanner   *scanner);
-static GTokenType  gimp_config_deserialize_matrix2     (GValue     *value,
-                                                        GParamSpec *prop_spec,
-                                                        GScanner   *scanner);
-static GTokenType  gimp_config_deserialize_object      (GValue     *value,
-                                                        GimpConfig *config,
-                                                        GParamSpec *prop_spec,
-                                                        GScanner   *scanner,
-                                                        gint        nest_level);
-static GTokenType  gimp_config_deserialize_value_array (GValue     *value,
-                                                        GimpConfig *config,
-                                                        GParamSpec *prop_spec,
-                                                        GScanner   *scanner);
-static GTokenType  gimp_config_deserialize_any         (GValue     *value,
-                                                        GParamSpec *prop_spec,
-                                                        GScanner   *scanner);
-static GTokenType  gimp_config_skip_unknown_property   (GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_value          (GValue     *value,
+                                                           GimpConfig *config,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_fundamental    (GValue     *value,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_enum           (GValue     *value,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_memsize        (GValue     *value,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_path           (GValue     *value,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_rgb            (GValue     *value,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_matrix2        (GValue     *value,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_object         (GValue     *value,
+                                                           GimpConfig *config,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner,
+                                                           gint        nest_level);
+static GTokenType  gimp_config_deserialize_value_array    (GValue     *value,
+                                                           GimpConfig *config,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_unit           (GValue     *value,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_file_value     (GValue     *value,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_parasite_value (GValue     *value,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_deserialize_any            (GValue     *value,
+                                                           GParamSpec *prop_spec,
+                                                           GScanner   *scanner);
+static GTokenType  gimp_config_skip_unknown_property      (GScanner   *scanner);
 
 static inline gboolean  scanner_string_utf8_valid (GScanner    *scanner,
                                                    const gchar *token_name);
@@ -117,9 +128,9 @@ scanner_string_utf8_valid (GScanner    *scanner,
  *
  * This function uses the @scanner to configure the properties of @config.
  *
- * Return value: %TRUE on success, %FALSE otherwise.
+ * Returns: %TRUE on success, %FALSE otherwise.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 gboolean
 gimp_config_deserialize_properties (GimpConfig *config,
@@ -133,7 +144,6 @@ gimp_config_deserialize_properties (GimpConfig *config,
   guint          scope_id;
   guint          old_scope_id;
   GTokenType     token;
-  GTokenType     next;
 
   g_return_val_if_fail (GIMP_IS_CONFIG (config), FALSE);
 
@@ -165,7 +175,10 @@ gimp_config_deserialize_properties (GimpConfig *config,
 
   while (TRUE)
     {
-      next = g_scanner_peek_next_token (scanner);
+      GTokenType next = g_scanner_peek_next_token (scanner);
+
+      if (next == G_TOKEN_EOF)
+        break;
 
       if (G_UNLIKELY (next != token &&
                       ! (token == G_TOKEN_SYMBOL &&
@@ -220,11 +233,11 @@ gimp_config_deserialize_properties (GimpConfig *config,
  * shouldn't need to call this function directly. If possible, use
  * gimp_config_deserialize_properties() instead.
  *
- * Return value: %G_TOKEN_RIGHT_PAREN on success, otherwise the
+ * Returns: %G_TOKEN_RIGHT_PAREN on success, otherwise the
  * expected #GTokenType or %G_TOKEN_NONE if the expected token was
  * found but couldn't be parsed.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 GTokenType
 gimp_config_deserialize_property (GimpConfig *config,
@@ -234,8 +247,8 @@ gimp_config_deserialize_property (GimpConfig *config,
   GimpConfigInterface *config_iface = NULL;
   GimpConfigInterface *parent_iface = NULL;
   GParamSpec          *prop_spec;
-  GTokenType           token = G_TOKEN_RIGHT_PAREN;
-  GValue               value = { 0, };
+  GTokenType           token        = G_TOKEN_RIGHT_PAREN;
+  GValue               value        = G_VALUE_INIT;
   guint                old_scope_id;
 
   old_scope_id = g_scanner_set_scope (scanner, 0);
@@ -286,13 +299,18 @@ gimp_config_deserialize_property (GimpConfig *config,
     }
   else
     {
-      if (G_VALUE_HOLDS_OBJECT (&value))
-        token = gimp_config_deserialize_object (&value,
-                                                config, prop_spec,
-                                                scanner, nest_level);
+      if (G_VALUE_HOLDS_OBJECT (&value) &&
+          G_VALUE_TYPE (&value) != G_TYPE_FILE)
+        {
+          token = gimp_config_deserialize_object (&value,
+                                                  config, prop_spec,
+                                                  scanner, nest_level);
+        }
       else
-        token = gimp_config_deserialize_value (&value,
-                                               config, prop_spec, scanner);
+        {
+          token = gimp_config_deserialize_value (&value,
+                                                 config, prop_spec, scanner);
+        }
     }
 
   if (token == G_TOKEN_RIGHT_PAREN &&
@@ -350,11 +368,27 @@ gimp_config_deserialize_value (GValue     *value,
     {
       return gimp_config_deserialize_matrix2 (value, prop_spec, scanner);
     }
-  else if (prop_spec->value_type == G_TYPE_VALUE_ARRAY)
+  else if (prop_spec->value_type == GIMP_TYPE_VALUE_ARRAY)
     {
       return gimp_config_deserialize_value_array (value,
                                                   config, prop_spec, scanner);
     }
+  else if (prop_spec->value_type == G_TYPE_STRV)
+    {
+      return gimp_config_deserialize_strv (value, scanner);
+    }
+  else if (prop_spec->value_type == GIMP_TYPE_UNIT)
+    {
+      return gimp_config_deserialize_unit (value, prop_spec, scanner);
+    }
+  else if (prop_spec->value_type == G_TYPE_FILE)
+    {
+      return gimp_config_deserialize_file_value (value, prop_spec, scanner);
+    }
+  else if (prop_spec->value_type == GIMP_TYPE_PARASITE)
+    {
+      return gimp_config_deserialize_parasite_value (value, prop_spec, scanner);
+    }
 
   /*  This fallback will only work for value_types that
    *  can be transformed from a string value.
@@ -368,6 +402,7 @@ gimp_config_deserialize_fundamental (GValue     *value,
                                      GScanner   *scanner)
 {
   GTokenType token;
+  GTokenType next_token;
   GType      value_type;
   gboolean   negate = FALSE;
 
@@ -413,7 +448,13 @@ gimp_config_deserialize_fundamental (GValue     *value,
       break;
     }
 
-  if (g_scanner_peek_next_token (scanner) != token)
+  next_token = g_scanner_peek_next_token (scanner);
+
+  /* we parse integers into floats too, because g_ascii_dtostr()
+   * serialized whole number without decimal point
+   */
+  if (next_token != token &&
+      ! (token == G_TOKEN_FLOAT && next_token == G_TOKEN_INT))
     {
       return token;
     }
@@ -424,7 +465,7 @@ gimp_config_deserialize_fundamental (GValue     *value,
     {
     case G_TYPE_STRING:
       if (scanner_string_utf8_valid (scanner, prop_spec->name))
-        g_value_set_static_string (value, scanner->value.v_string);
+        g_value_set_string (value, scanner->value.v_string);
       else
         return G_TOKEN_NONE;
       break;
@@ -449,8 +490,8 @@ gimp_config_deserialize_fundamental (GValue     *value,
 
     case G_TYPE_INT:
       g_value_set_int (value, (negate ?
-                               - scanner->value.v_int64 :
-                               scanner->value.v_int64));
+                               - (gint) scanner->value.v_int64 :
+                                 (gint) scanner->value.v_int64));
       break;
     case G_TYPE_UINT:
       g_value_set_uint (value, scanner->value.v_int64);
@@ -458,8 +499,8 @@ gimp_config_deserialize_fundamental (GValue     *value,
 
     case G_TYPE_LONG:
       g_value_set_long (value, (negate ?
-                                - scanner->value.v_int64 :
-                                scanner->value.v_int64));
+                                - (glong) scanner->value.v_int64 :
+                                  (glong) scanner->value.v_int64));
       break;
     case G_TYPE_ULONG:
       g_value_set_ulong (value, scanner->value.v_int64);
@@ -467,20 +508,33 @@ gimp_config_deserialize_fundamental (GValue     *value,
 
     case G_TYPE_INT64:
       g_value_set_int64 (value, (negate ?
-                                 - scanner->value.v_int64 :
-                                 scanner->value.v_int64));
+                                 - (gint64) scanner->value.v_int64 :
+                                   (gint64) scanner->value.v_int64));
       break;
     case G_TYPE_UINT64:
       g_value_set_uint64 (value, scanner->value.v_int64);
       break;
 
     case G_TYPE_FLOAT:
-      g_value_set_float (value, negate ?
-                         - scanner->value.v_float : scanner->value.v_float);
+      if (next_token == G_TOKEN_FLOAT)
+        g_value_set_float (value, negate ?
+                           - scanner->value.v_float :
+                             scanner->value.v_float);
+      else
+        g_value_set_float (value, negate ?
+                           - (gfloat) scanner->value.v_int :
+                             (gfloat) scanner->value.v_int);
       break;
+
     case G_TYPE_DOUBLE:
-      g_value_set_double (value, negate ?
-                          - scanner->value.v_float: scanner->value.v_float);
+      if (next_token == G_TOKEN_FLOAT)
+        g_value_set_double (value, negate ?
+                            - scanner->value.v_float:
+                              scanner->value.v_float);
+      else
+        g_value_set_double (value, negate ?
+                            - (gdouble) scanner->value.v_int:
+                              (gdouble) scanner->value.v_int);
       break;
 
     default:
@@ -508,11 +562,39 @@ gimp_config_deserialize_enum (GValue     *value,
 
       enum_value = g_enum_get_value_by_nick (enum_class,
                                              scanner->value.v_identifier);
-      if (!enum_value)
+      if (! enum_value)
         enum_value = g_enum_get_value_by_name (enum_class,
                                                scanner->value.v_identifier);
+      if (! enum_value)
+        {
+          /*  if the value was not found, check if we have a compat
+           *  enum to find the ideitifier
+           */
+          GQuark quark       = g_quark_from_static_string ("gimp-compat-enum");
+          GType  compat_type = (GType) g_type_get_qdata (G_VALUE_TYPE (value),
+                                                         quark);
 
-      if (!enum_value)
+          if (compat_type)
+            {
+              GEnumClass *compat_class = g_type_class_ref (compat_type);
+
+              enum_value = g_enum_get_value_by_nick (compat_class,
+                                                     scanner->value.v_identifier);
+              if (! enum_value)
+                enum_value = g_enum_get_value_by_name (compat_class,
+                                                       scanner->value.v_identifier);
+
+              /*  finally, if we found a compat value, make sure the
+               *  same value exists in the original enum
+               */
+              if (enum_value)
+                enum_value = g_enum_get_value (enum_class, enum_value->value);
+
+              g_type_class_unref (compat_class);
+           }
+        }
+
+      if (! enum_value)
         {
           g_scanner_error (scanner,
                            _("invalid value '%s' for token %s"),
@@ -527,7 +609,7 @@ gimp_config_deserialize_enum (GValue     *value,
       enum_value = g_enum_get_value (enum_class,
                                      (gint) scanner->value.v_int64);
 
-      if (!enum_value)
+      if (! enum_value)
         {
           g_scanner_error (scanner,
                            _("invalid value '%ld' for token %s"),
@@ -671,6 +753,13 @@ gimp_config_deserialize_object (GValue     *value,
           if (! gimp_scanner_parse_string (scanner, &type_name))
             return G_TOKEN_STRING;
 
+          if (! (type_name && *type_name))
+            {
+              g_scanner_error (scanner, "Type name is empty");
+              g_free (type_name);
+              return G_TOKEN_NONE;
+            }
+
           type = g_type_from_name (type_name);
           g_free (type_name);
 
@@ -687,7 +776,7 @@ gimp_config_deserialize_object (GValue     *value,
         }
     }
 
-  config_iface = GIMP_CONFIG_GET_INTERFACE (prop_object);
+  config_iface = GIMP_CONFIG_GET_IFACE (prop_object);
 
   if (! config_iface)
     return gimp_config_deserialize_any (value, prop_spec, scanner);
@@ -704,19 +793,19 @@ gimp_config_deserialize_value_array (GValue     *value,
                                      GParamSpec *prop_spec,
                                      GScanner   *scanner)
 {
-  GParamSpecValueArray *array_spec;
-  GValueArray          *array;
-  GValue                array_value = { 0, };
-  gint                  n_values;
-  GTokenType            token;
-  gint                  i;
+  GimpParamSpecValueArray *array_spec;
+  GimpValueArray          *array;
+  GValue                   array_value = G_VALUE_INIT;
+  gint                     n_values;
+  GTokenType               token;
+  gint                     i;
 
-  array_spec = G_PARAM_SPEC_VALUE_ARRAY (prop_spec);
+  array_spec = GIMP_PARAM_SPEC_VALUE_ARRAY (prop_spec);
 
   if (! gimp_scanner_parse_int (scanner, &n_values))
     return G_TOKEN_INT;
 
-  array = g_value_array_new (n_values);
+  array = gimp_value_array_new (n_values);
 
   for (i = 0; i < n_values; i++)
     {
@@ -728,12 +817,15 @@ gimp_config_deserialize_value_array (GValue     *value,
                                              scanner);
 
       if (token == G_TOKEN_RIGHT_PAREN)
-        g_value_array_append (array, &array_value);
+        gimp_value_array_append (array, &array_value);
 
       g_value_unset (&array_value);
 
       if (token != G_TOKEN_RIGHT_PAREN)
-        return token;
+        {
+          gimp_value_array_unref (array);
+          return token;
+        }
     }
 
   g_value_take_boxed (value, array);
@@ -741,27 +833,210 @@ gimp_config_deserialize_value_array (GValue     *value,
   return G_TOKEN_RIGHT_PAREN;
 }
 
+/* This function is entirely sick, so is our method of serializing
+ * units, which we write out as (unit foo bar) instead of
+ * (unit "foo bar"). The assumption that caused this shit was that a
+ * unit's "identifier" is really an identifier in the C-ish sense,
+ * when in fact it's just a random user entered string.
+ *
+ * Here, we try to parse at least the default units shipped with gimp,
+ * and we add code to parse (unit "foo bar") in order to be compatible
+ * with future correct unit serializing.
+ */
+static GTokenType
+gimp_config_deserialize_unit (GValue     *value,
+                              GParamSpec *prop_spec,
+                              GScanner   *scanner)
+{
+  gchar      *old_cset_skip_characters;
+  gchar      *old_cset_identifier_first;
+  gchar      *old_cset_identifier_nth;
+  GString    *buffer;
+  GValue      src = G_VALUE_INIT;
+  GTokenType  token;
+
+  /* parse the next token *before* reconfiguring the scanner, so it
+   * skips whitespace first
+   */
+  token = g_scanner_peek_next_token (scanner);
+
+  if (token == G_TOKEN_STRING)
+    return gimp_config_deserialize_any (value, prop_spec, scanner);
+
+  old_cset_skip_characters  = scanner->config->cset_skip_characters;
+  old_cset_identifier_first = scanner->config->cset_identifier_first;
+  old_cset_identifier_nth   = scanner->config->cset_identifier_nth;
+
+  scanner->config->cset_skip_characters  = "";
+  scanner->config->cset_identifier_first = ( G_CSET_a_2_z G_CSET_A_2_Z "." );
+  scanner->config->cset_identifier_nth   = ( G_CSET_a_2_z G_CSET_A_2_Z
+                                             G_CSET_DIGITS "-_." );
+
+  buffer = g_string_new ("");
+
+  while (g_scanner_peek_next_token (scanner) != G_TOKEN_RIGHT_PAREN)
+    {
+      token = g_scanner_peek_next_token (scanner);
+
+      if (token == G_TOKEN_IDENTIFIER)
+        {
+          g_scanner_get_next_token (scanner);
+          g_string_append (buffer, scanner->value.v_identifier);
+        }
+      else if (token == G_TOKEN_CHAR)
+        {
+          g_scanner_get_next_token (scanner);
+          g_string_append_c (buffer, scanner->value.v_char);
+        }
+      else if (token == ' ')
+        {
+          g_scanner_get_next_token (scanner);
+          g_string_append_c (buffer, token);
+        }
+      else
+        {
+          token = G_TOKEN_IDENTIFIER;
+          goto cleanup;
+        }
+    }
+
+  g_value_init (&src, G_TYPE_STRING);
+  g_value_set_static_string (&src, buffer->str);
+  g_value_transform (&src, value);
+  g_value_unset (&src);
+
+  token = G_TOKEN_RIGHT_PAREN;
+
+ cleanup:
+
+  g_string_free (buffer, TRUE);
+
+  scanner->config->cset_skip_characters  = old_cset_skip_characters;
+  scanner->config->cset_identifier_first = old_cset_identifier_first;
+  scanner->config->cset_identifier_nth   = old_cset_identifier_nth;
+
+  return token;
+}
+
+static GTokenType
+gimp_config_deserialize_file_value (GValue     *value,
+                                    GParamSpec *prop_spec,
+                                    GScanner   *scanner)
+{
+  GTokenType token;
+
+  token = g_scanner_peek_next_token (scanner);
+
+  if (token != G_TOKEN_IDENTIFIER &&
+      token != G_TOKEN_STRING)
+    {
+      return G_TOKEN_STRING;
+    }
+
+  g_scanner_get_next_token (scanner);
+
+  if (token == G_TOKEN_IDENTIFIER)
+    {
+      /* this is supposed to parse a literal "NULL" only, but so what... */
+      g_value_set_object (value, NULL);
+    }
+  else
+    {
+      gchar *path = gimp_config_path_expand (scanner->value.v_string, TRUE,
+                                             NULL);
+
+      if (path)
+        {
+          GFile *file = g_file_new_for_path (path);
+
+          g_value_take_object (value, file);
+          g_free (path);
+        }
+      else
+        {
+          g_value_set_object (value, NULL);
+        }
+    }
+
+  return G_TOKEN_RIGHT_PAREN;
+}
+
+/*
+ * Note: this is different from gimp_config_deserialize_parasite()
+ * which is a public API to deserialize random properties into a config
+ * object from a parasite. Here we are deserializing the contents of a
+ * parasite itself in @scanner.
+ */
+static GTokenType
+gimp_config_deserialize_parasite_value (GValue     *value,
+                                        GParamSpec *prop_spec,
+                                        GScanner   *scanner)
+{
+  GimpParasite *parasite;
+  gchar        *name;
+  guint8       *data;
+  gint          data_length;
+  gint64        flags;
+
+  if (! gimp_scanner_parse_string (scanner, &name))
+    return G_TOKEN_STRING;
+
+  if (! (name && *name))
+    {
+      g_scanner_error (scanner, "Parasite name is empty");
+      g_free (name);
+      return G_TOKEN_NONE;
+    }
+
+  if (! gimp_scanner_parse_int64 (scanner, &flags))
+    return G_TOKEN_INT;
+
+  if (! gimp_scanner_parse_int (scanner, &data_length))
+    return G_TOKEN_INT;
+
+  if (! gimp_scanner_parse_data (scanner, data_length, &data))
+    return G_TOKEN_STRING;
+
+  parasite = gimp_parasite_new (name, flags, data_length, data);
+
+  g_value_take_boxed (value, parasite);
+
+  return G_TOKEN_RIGHT_PAREN;
+}
+
 static GTokenType
 gimp_config_deserialize_any (GValue     *value,
                              GParamSpec *prop_spec,
                              GScanner   *scanner)
 {
-  GValue src = { 0, };
+  GValue     src = G_VALUE_INIT;
+  GTokenType token;
 
   if (!g_value_type_transformable (G_TYPE_STRING, prop_spec->value_type))
     {
-      g_warning ("%s: %s can not be transformed from a string",
-                 G_STRFUNC, g_type_name (prop_spec->value_type));
+      g_scanner_error (scanner,
+                       "%s can not be transformed from a string",
+                       g_type_name (prop_spec->value_type));
       return G_TOKEN_NONE;
     }
 
-  if (g_scanner_peek_next_token (scanner) != G_TOKEN_IDENTIFIER)
-    return G_TOKEN_IDENTIFIER;
+  token = g_scanner_peek_next_token (scanner);
+
+  if (token != G_TOKEN_IDENTIFIER &&
+      token != G_TOKEN_STRING)
+    {
+      return G_TOKEN_IDENTIFIER;
+    }
 
   g_scanner_get_next_token (scanner);
 
   g_value_init (&src, G_TYPE_STRING);
-  g_value_set_static_string (&src, scanner->value.v_identifier);
+
+  if (token == G_TOKEN_IDENTIFIER)
+    g_value_set_static_string (&src, scanner->value.v_identifier);
+  else
+    g_value_set_static_string (&src, scanner->value.v_string);
+
   g_value_transform (&src, value);
   g_value_unset (&src);
 
