@@ -204,9 +204,10 @@ gimp_input_device_store_init (GimpInputDeviceStore *store)
   gtk_list_store_set_column_types (GTK_LIST_STORE (store),
                                    G_N_ELEMENTS (types), types);
 
-  if (!GetModuleHandleEx (GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS,
-                          (LPCTSTR) &gimp_input_device_store_init,
-                          &thismodule))
+  if (!GetModuleHandleExW (GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS |
+                           GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT,
+                           (LPCTSTR) &gimp_input_device_store_init,
+                           &thismodule))
     return;
 
   if ((store->window = create_aux_window (store)) == NULL)
@@ -216,7 +217,7 @@ gimp_input_device_store_init (GimpInputDeviceStore *store)
       return;
     }
 
-  if ((dinput8 = LoadLibrary ("dinput8.dll")) == NULL)
+  if ((dinput8 = LoadLibraryW (L"dinput8.dll")) == NULL)
     {
       g_set_error_literal (&store->error, GIMP_MODULE_ERROR, GIMP_MODULE_FAILED,
 			   "Could not load dinput8.dll");
