@@ -237,6 +237,66 @@ gimp_param_spec_image ("$name",
                        "$blurb",
                        $none_ok,
                        $flags)
+CODE
+    }
+    elsif ($pdbtype eq 'resource') {
+	$none_ok = exists $arg->{none_ok} ? 'TRUE' : 'FALSE';
+	$pspec = <<CODE;
+gimp_param_spec_resource ("$name",
+                          "$nick",
+                          "$blurb",
+                          $none_ok,
+                          $flags)
+CODE
+    }
+    elsif ($pdbtype eq 'brush') {
+	$none_ok = exists $arg->{none_ok} ? 'TRUE' : 'FALSE';
+	$pspec = <<CODE;
+gimp_param_spec_brush ("$name",
+                       "$nick",
+                       "$blurb",
+                       $none_ok,
+                       $flags)
+CODE
+    }
+    elsif ($pdbtype eq 'font') {
+  $none_ok = exists $arg->{none_ok} ? 'TRUE' : 'FALSE';
+  $pspec = <<CODE;
+gimp_param_spec_font ("$name",
+                      "$nick",
+                      "$blurb",
+                      $none_ok,
+                      $flags)
+CODE
+    }
+    elsif ($pdbtype eq 'gradient') {
+  $none_ok = exists $arg->{none_ok} ? 'TRUE' : 'FALSE';
+  $pspec = <<CODE;
+gimp_param_spec_gradient ("$name",
+                          "$nick",
+                          "$blurb",
+                          $none_ok,
+                          $flags)
+CODE
+    }
+    elsif ($pdbtype eq 'palette') {
+  $none_ok = exists $arg->{none_ok} ? 'TRUE' : 'FALSE';
+  $pspec = <<CODE;
+gimp_param_spec_palette ("$name",
+                         "$nick",
+                         "$blurb",
+                         $none_ok,
+                         $flags)
+CODE
+    }
+    elsif ($pdbtype eq 'pattern') {
+  $none_ok = exists $arg->{none_ok} ? 'TRUE' : 'FALSE';
+  $pspec = <<CODE;
+gimp_param_spec_pattern ("$name",
+                         "$nick",
+                         "$blurb",
+                         $none_ok,
+                         $flags)
 CODE
     }
     elsif ($pdbtype eq 'item') {
@@ -257,6 +317,16 @@ gimp_param_spec_drawable ("$name",
                           "$blurb",
                           $none_ok,
                           $flags)
+CODE
+    }
+    elsif ($pdbtype eq 'text_layer') {
+	$none_ok = exists $arg->{none_ok} ? 'TRUE' : 'FALSE';
+	$pspec = <<CODE;
+gimp_param_spec_text_layer ("$name",
+                            "$nick",
+                            "$blurb",
+                            $none_ok,
+                            $flags)
 CODE
     }
     elsif ($pdbtype eq 'layer') {
@@ -544,14 +614,6 @@ gimp_param_spec_int32_array ("$name",
                              "$nick",
                              "$blurb",
                              $flags)
-CODE
-    }
-    elsif ($pdbtype eq 'int8array') {
-	$pspec = <<CODE;
-gimp_param_spec_uint8_array ("$name",
-                             "$nick",
-                             "$blurb",
-                             $flags)
 CODE
     }
     elsif ($pdbtype eq 'floatarray') {
@@ -560,6 +622,15 @@ gimp_param_spec_float_array ("$name",
                              "$nick",
                              "$blurb",
                              $flags)
+CODE
+    }
+    elsif ($pdbtype eq 'bytes') {
+	$pspec = <<CODE;
+g_param_spec_boxed ("$name",
+                    "$nick",
+                    "$blurb",
+                    G_TYPE_BYTES,
+                    $flags)
 CODE
     }
     elsif ($pdbtype eq 'strv') {
@@ -783,7 +854,7 @@ CODE
 	}
 	else {
 	    my $invoker = "";
-	
+
 	    $invoker .= ' ' x 2 . "GimpValueArray *return_vals;\n" if scalar @outargs;
 	    $invoker .= &declare_args($proc, $out, 0, qw(inargs));
 	    $invoker .= &declare_args($proc, $out, 1, qw(outargs));
