@@ -491,8 +491,6 @@ load_image (GFile   *file,
       return NULL;
     }
 
-  gimp_image_set_file (image, file);
-
   /* Create an indexed-alpha layer to hold the image... */
   if (bpp == 32)
     layer = gimp_layer_new (image, _("Background"), width, height,
@@ -867,7 +865,7 @@ save_image (GFile         *file,
   /* Work out whether to save as 8bit or 4bit */
   if (bpp < 32)
     {
-      g_free (gimp_image_get_colormap (image, &colors));
+      g_free (gimp_image_get_colormap (image, NULL, &colors));
 
       if (colors > 15)
         {
