@@ -129,7 +129,7 @@ DEFINE_STD_SET_I18N
 
 
 static guchar        used_cmap[3][256];
-static guchar        highest_used_index;
+static guint         highest_used_index;
 static gboolean      promote_to_rgb   = FALSE;
 static guchar        gimp_cmap[768];
 static GimpParasite *comment_parasite = NULL;
@@ -255,7 +255,7 @@ gif_load (GimpProcedure        *procedure,
 
   if (! promote_to_rgb)
     gimp_image_set_colormap (image,
-                             gimp_cmap, highest_used_index + 1);
+                             gimp_cmap, highest_used_index);
 
   return_vals = gimp_procedure_new_return_values (procedure,
                                                   GIMP_PDB_SUCCESS,
@@ -287,7 +287,7 @@ gif_load_thumb (GimpProcedure        *procedure,
                                              error);
 
   if (! promote_to_rgb)
-    gimp_image_set_colormap (image, gimp_cmap, highest_used_index + 1);
+    gimp_image_set_colormap (image, gimp_cmap, highest_used_index);
 
   return_vals = gimp_procedure_new_return_values (procedure,
                                                   GIMP_PDB_SUCCESS,
@@ -304,6 +304,9 @@ gif_load_thumb (GimpProcedure        *procedure,
 
 
 #define MAXCOLORMAPSIZE  256
+/* Max size of data buffer. Count is one byte so maximum read is 255. In
+ * GetCode we use 2 extra bytes, so let's set it to 260 to be safe. */
+#define MAXDATABLOCKSIZE 260
 
 #define CM_RED           0
 #define CM_GREEN         1
@@ -315,6 +318,7 @@ gif_load_thumb (GimpProcedure        *procedure,
 #define LOCALCOLORMAP      0x80
 #define BitSet(byte, bit)  (((byte) & (bit)) == (bit))
 
+#define gif_message(...) (g_message("%s. %s", __VA_ARGS__))
 #define ReadOK(file,buffer,len) (fread(buffer, len, 1, file) != 0)
 #define LM_to_uint(a,b)         (((b)<<8)|(a))
 
@@ -346,6 +350,9 @@ static struct
   gint disposal;
 } Gif89 = { -1, -1, -1, 0 };
 
+static void     read_error   (const gchar *error_type,
+                              GimpImage   *image,
+                              GError     **error);
 static gboolean ReadColorMap (FILE        *fd,
                               gint         number,
                               CMap         buffer,
@@ -373,8 +380,8 @@ static gboolean ReadImage    (FILE        *fd,
                               guint        toppos,
                               guint        screenwidth,
                               guint        screenheight,
-                              GimpImage  **image);
-
+                              GimpImage  **image,
+                              GError     **error);
 
 static GimpImage *
 load_image (GFile     *file,
@@ -407,29 +414,30 @@ load_image (GFile     *file,
 
   if (! ReadOK (fd, buf, 6))
     {
-      g_message ("Error reading magic number");
+      read_error (_("GIF magic code"), NULL, error);
       fclose (fd);
       return NULL;
     }
 
   if (strncmp ((gchar *) buf, "GIF", 3) != 0)
     {
-      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                   "%s", _("This is not a GIF file"));
+      g_set_error (error, GIMP_PLUG_IN_ERROR, 0,
+                   _("This is not a GIF file: incorrect magic code"));
       fclose (fd);
       return NULL;
     }
 
   if ((strncmp ((gchar *) buf + 3, "87a", 3) != 0) && (strncmp ((gchar *) buf + 3, "89a", 3) != 0))
     {
-      g_message ("Bad version number, not '87a' or '89a'");
+      g_set_error (error, GIMP_PLUG_IN_ERROR, 0,
+                   _("Incorrect GIF version: not '87a' or '89a'"));
       fclose (fd);
       return NULL;
     }
 
   if (! ReadOK (fd, buf, 7))
     {
-      g_message ("Failed to read screen descriptor");
+      read_error (_("screen descriptor"), NULL, error);
       fclose (fd);
       return NULL;
     }
@@ -447,7 +455,7 @@ load_image (GFile     *file,
       if (! ReadColorMap (fd, GifScreen.BitPixel, GifScreen.ColorMap,
                           &GifScreen.GrayScale))
         {
-          g_message ("Error reading global colormap");
+          read_error (_("global colormap"), NULL, error);
           fclose (fd);
           return NULL;
         }
@@ -458,14 +466,13 @@ load_image (GFile     *file,
       g_message (_("Non-square pixels.  Image might look squashed."));
     }
 
-
-  highest_used_index = 0;
+  highest_used_index = GifScreen.BitPixel;
 
   while (TRUE)
     {
       if (! ReadOK (fd, &c, 1))
         {
-          g_message ("EOF / read error on image data");
+          read_error (_("image data"), image, error);
           fclose (fd);
           return image; /* will be NULL if failed on first image! */
         }
@@ -482,7 +489,7 @@ load_image (GFile     *file,
           /* Extension */
           if (! ReadOK (fd, &c, 1))
             {
-              g_message ("EOF / read error on extension function code");
+              read_error (_("extension data"), image, error);
               fclose (fd);
               return image; /* will be NULL if failed on first image! */
             }
@@ -502,7 +509,7 @@ load_image (GFile     *file,
 
       if (! ReadOK (fd, buf, 9))
         {
-          g_message ("Couldn't read left/top/width/height");
+          read_error (_("frame info"), image, error);
           fclose (fd);
           return image; /* will be NULL if failed on first image! */
         }
@@ -515,10 +522,11 @@ load_image (GFile     *file,
         {
           if (! ReadColorMap (fd, bitPixel, localColorMap, &grayScale))
             {
-              g_message ("Error reading local colormap");
+              read_error (_("local colormap"), image, error);
               fclose (fd);
               return image; /* will be NULL if failed on first image! */
             }
+          highest_used_index = bitPixel;
 
           status = ReadImage (fd, file, LM_to_uint (buf[4], buf[5]),
                               LM_to_uint (buf[6], buf[7]),
@@ -529,7 +537,7 @@ load_image (GFile     *file,
                               (guint) LM_to_uint (buf[2], buf[3]),
                               GifScreen.Width,
                               GifScreen.Height,
-                              &image);
+                              &image, error);
         }
       else
         {
@@ -542,7 +550,7 @@ load_image (GFile     *file,
                               (guint) LM_to_uint (buf[2], buf[3]),
                               GifScreen.Width,
                               GifScreen.Height,
-                              &image);
+                              &image, error);
         }
 
       if (!status)
@@ -569,6 +577,22 @@ load_image (GFile     *file,
   return image;
 }
 
+static void
+read_error (const gchar *error_type, GimpImage *image, GError **error)
+{
+  if (! image)
+    {
+      g_set_error (error, GIMP_PLUG_IN_ERROR, 0,
+                   _("Error reading %s. %s"),
+                   error_type, _("Image may be corrupt!"));
+    }
+  else
+    {
+      g_message (_("Error reading %s. %s"),
+                 error_type, _("Image may be corrupt!"));
+    }
+}
+
 static gboolean
 ReadColorMap (FILE *fd,
               gint  number,
@@ -602,7 +626,8 @@ static gint
 DoExtension (FILE *fd,
              gint  label)
 {
-  static guchar  buf[256];
+  static guchar  buf[MAXDATABLOCKSIZE];
+  gint           bufsize;
 #ifdef GIFDEBUG
   gchar         *str;
 #endif
@@ -650,10 +675,13 @@ DoExtension (FILE *fd,
 #ifdef GIFDEBUG
       str = "Comment Extension";
 #endif
-      while (GetDataBlock (fd, (guchar *) buf) > 0)
+      while ((bufsize = GetDataBlock (fd, (guchar *) buf)) > 0)
         {
           gchar *comment = (gchar *) buf;
 
+          /* Make sure we end with a NULL */
+          comment[bufsize] = '\0';
+
           if (! g_utf8_validate (comment, -1, NULL))
             continue;
 
@@ -712,7 +740,7 @@ GetDataBlock (FILE   *fd,
 
   if (! ReadOK (fd, &count, 1))
     {
-      g_message ("Error in getting DataBlock size");
+      gif_message (_("Error reading data size"), _("Image may be corrupt!"));
       return -1;
     }
 
@@ -720,7 +748,7 @@ GetDataBlock (FILE   *fd,
 
   if ((count != 0) && (! ReadOK (fd, buf, count)))
     {
-      g_message ("Error in reading DataBlock");
+      gif_message (_("Error reading data"), _("Image may be corrupt!"));
       return -1;
     }
 
@@ -732,7 +760,7 @@ GetCode (FILE     *fd,
          gint      code_size,
          gboolean  flag)
 {
-  static guchar buf[280];
+  static guchar buf[MAXDATABLOCKSIZE];
   static gint   curbit, lastbit, done, last_byte;
   gint          i, j, ret, count;
 
@@ -750,9 +778,12 @@ GetCode (FILE     *fd,
       if (done)
         {
           if (curbit >= lastbit)
-            g_message ("Ran off the end of my bits");
+            {
+              gif_message (_("Not enough compressed data"), _("Image may be corrupt!"));
+              return -1;
+            }
 
-          return -1;
+          return -2;
         }
 
       buf[0] = buf[last_byte - 2];
@@ -802,7 +833,7 @@ LZWReadByte (FILE *fd,
     {
       if (input_code_size > MAX_LZW_BITS || input_code_size <= 1)
         {
-          g_message ("Value out of range for code size (corrupted file?)");
+          gif_message (_("Value out of range for code size"), _("Image may be corrupt!"));
           return -1;
         }
 
@@ -813,9 +844,9 @@ LZWReadByte (FILE *fd,
       max_code_size = 2 * clear_code;
       max_code      = clear_code + 2;
 
-      if (GetCode (fd, 0, TRUE) < 0)
+      if ((code = GetCode (fd, 0, TRUE)) < 0)
         {
-          return -1;
+          return code;
         }
 
       fresh = TRUE;
@@ -846,7 +877,7 @@ LZWReadByte (FILE *fd,
 
       if (firstcode < 0)
         {
-          return -1;
+          return firstcode;
         }
 
       return firstcode & 255;
@@ -878,26 +909,43 @@ LZWReadByte (FILE *fd,
 
           if (firstcode < 0)
             {
-              return -1;
+              return firstcode;
             }
 
           return firstcode & 255;
         }
       else if (code == end_code || code > max_code)
         {
-          gint   count;
-          guchar buf[260];
+          gint     count;
+          guchar   buf[MAXDATABLOCKSIZE];
+          gboolean extra_data = FALSE;
 
+          if (code != end_code)
+            {
+              gif_message (_("Invalid code, expecting end of data marker"),
+                           _("Image may be corrupt!"));
+              code = -1;
+            }
+          else
+            {
+              code = -2;
+            }
           if (ZeroDataBlock)
-            return -2;
+            {
+              return code;
+            }
 
           while ((count = GetDataBlock (fd, buf)) > 0)
-            ;
+            {
+              extra_data = TRUE;
+            }
 
-          if (count != 0)
-            g_print ("GIF: missing EOD in data stream (common occurrence)");
+          if (count != 0 || extra_data)
+            {
+              gif_message (_("Unexpected extra data"), _("Image may be corrupt!"));
+            }
 
-          return -2;
+          return -1;
         }
 
       incode = code;
@@ -913,14 +961,14 @@ LZWReadByte (FILE *fd,
         {
           if (code >= (1 << MAX_LZW_BITS))
             {
-              g_message ("Invalid table entry.  Corrupt file.");
+              gif_message (_("Invalid table entry"), _("Image is corrupt!"));
               return -1;
             }
           *sp++ = table[1][code];
           if (code == table[0][code])
             {
-              g_message ("Circular table entry.  Corrupt file.");
-              gimp_quit ();
+              gif_message (_("Circular table entry"), _("Image is corrupt!"));
+              return -1;
             }
           code = table[0][code];
         }
@@ -949,7 +997,7 @@ LZWReadByte (FILE *fd,
 
   if (code < 0)
     {
-      return -1;
+      return code;
     }
 
   return code & 255;
@@ -969,7 +1017,8 @@ ReadImage (FILE        *fd,
            guint        toppos,
            guint        screenwidth,
            guint        screenheight,
-           GimpImage  **image)
+           GimpImage  **image,
+           GError     **error)
 {
   static gint   frame_number = 1;
 
@@ -987,10 +1036,19 @@ ReadImage (FILE        *fd,
   static gint   previous_disposal;
 
   /* Guard against bogus frame size */
-  if (len < 1 || height < 1)
+  if (len < 1 || height < 1 ||
+      leftpos + len > screenwidth || toppos + height > screenheight)
     {
-      g_message ("Bogus frame dimensions");
-      *image = NULL;
+      if (frame_number == 1)
+        {
+          g_set_error (error, GIMP_PLUG_IN_ERROR, 0,
+                       _("%s. %s"),
+                       _("Invalid frame dimensions"), _("Image may be corrupt!"));
+        }
+      else
+        {
+          gif_message (_("Invalid frame dimensions"), _("Image may be corrupt!"));
+        }
       return FALSE;
     }
 
@@ -999,15 +1057,13 @@ ReadImage (FILE        *fd,
    */
   if (! ReadOK (fd, &c, 1))
     {
-      g_message ("EOF / read error on image data");
-      *image = NULL;
+      read_error (_("image data"), *image, error);
       return FALSE;
     }
 
   if (LZWReadByte (fd, TRUE, c) < 0)
     {
-      g_message ("Error while reading");
-      *image = NULL;
+      read_error (_("compressed image data"), *image, error);
       return FALSE;
     }
 
@@ -1015,13 +1071,18 @@ ReadImage (FILE        *fd,
     {
       /* Guard against bogus logical screen size values */
       if (screenwidth == 0)
-        screenwidth = len;
+        {
+          g_printerr ("Invalid zero screenwidth changed to %d", len);
+          screenwidth = len;
+        }
 
       if (screenheight == 0)
-        screenheight = height;
+        {
+          g_printerr ("Invalid zero screenheight changed to %d", height);
+          screenheight = height;
+        }
 
       *image = gimp_image_new (screenwidth, screenheight, GIMP_INDEXED);
-      gimp_image_set_file (*image, file);
 
       for (i = 0, j = 0; i < ncols; i++)
         {
@@ -1130,7 +1191,8 @@ ReadImage (FILE        *fd,
                      previous_disposal);
           break;
         default:
-          g_message ("Disposal word got corrupted.  Bug.");
+          g_message (_("Unknown composite type %d. %s"),
+                     previous_disposal, _("Image may be corrupt!"));
           break;
         }
       previous_disposal = Gif89.disposal;
@@ -1162,11 +1224,6 @@ ReadImage (FILE        *fd,
       return FALSE;
     }
 
-  if (alpha_frame)
-    dest = (guchar *) g_malloc ((gsize)len * (gsize)height * (promote_to_rgb ? 4 : 2));
-  else
-    dest = (guchar *) g_malloc ((gsize)len * (gsize)height);
-
 #ifdef GIFDEBUG
     g_print ("GIF: reading %d by %d%s GIF image, ncols=%d\n",
              len, height, interlace ? " interlaced" : "", ncols);
@@ -1176,18 +1233,24 @@ ReadImage (FILE        *fd,
     {
       /* I don't see how one would easily construct a GIF in which
          this could happen, but it's a mad mad world. */
-      g_message ("Ouch!  Can't handle non-alpha RGB frames.\n"
-                 "Please file a bug report at "
-                 "https://gitlab.gnome.org/GNOME/gimp/issues");
-      gimp_quit ();
+      g_set_error (error, GIMP_PLUG_IN_ERROR, 0,
+                   _("Can't handle non-alpha RGB frames.\n"
+                     "Please file a bug report at "
+                     "https://gitlab.gnome.org/GNOME/gimp/issues"));
+      gimp_image_delete (*image);
+      *image = NULL;
+      return FALSE;
     }
 
+  if (alpha_frame)
+    dest = (guchar *) g_malloc ((gsize)len * (gsize)height * (promote_to_rgb ? 4 : 2));
+  else
+    dest = (guchar *) g_malloc ((gsize)len * (gsize)height);
+
   while ((v = LZWReadByte (fd, FALSE, c)) >= 0)
     {
       if (alpha_frame)
         {
-          if (((guchar) v > highest_used_index) && !(v == Gif89.transparent))
-            highest_used_index = (guchar) v;
 
           if (promote_to_rgb)
             {
@@ -1206,8 +1269,6 @@ ReadImage (FILE        *fd,
         }
       else
         {
-          if ((guchar) v > highest_used_index)
-            highest_used_index = (guchar) v;
 
           temp = dest + (ypos * len) + xpos;
           *temp = (guchar) v;
@@ -1270,10 +1331,10 @@ ReadImage (FILE        *fd,
         break;
     }
 
-  if (v < 0)
-    {
-      return FALSE;
-    }
+  /* v can be < 0 here, indicating an error or encountering an end_code (-2).
+   * At this point we have already safely handled any corruption, so we might
+   * as well load any partial data we can instead of returning false.
+   */
 
  fini:
   buffer = gimp_drawable_get_buffer (GIMP_DRAWABLE (layer));
@@ -1287,11 +1348,20 @@ ReadImage (FILE        *fd,
 
   gimp_progress_update (1.0);
 
-  if (LZWReadByte (fd, FALSE, c) >= 0)
-    {
-      g_print ("GIF: too much input data, ignoring extra...\n");
-      return FALSE;
-    }
+  {
+    gboolean first = TRUE;
+
+    /* Skip remaining compressed data if any. */
+
+    while (LZWReadByte (fd, FALSE, c) >= 0)
+      {
+        if (first)
+          {
+            g_message (_("Too much compressed data, ignoring extra..."));
+            first = FALSE;
+          }
+      }
+  }
 
   return TRUE;
 }
