@@ -35,12 +35,6 @@
 #define SAVE_PROC_AV1  "file-heif-av1-save"
 #define PLUG_IN_BINARY "file-heif"
 
-typedef struct
-{
-  gchar *tag;
-  gint  type;
-} XmpStructs;
-
 typedef enum _HeifpluginEncoderSpeed
 {
   HEIFPLUGIN_ENCODER_SPEED_SLOW = 0,
@@ -56,24 +50,24 @@ typedef enum _HeifpluginExportFormat
   HEIFPLUGIN_EXPORT_FORMAT_YUV420 = 3
 } HeifpluginExportFormat;
 
-typedef struct _Heif      Heif;
-typedef struct _HeifClass HeifClass;
+typedef struct _GimpHeif      GimpHeif;
+typedef struct _GimpHeifClass GimpHeifClass;
 
-struct _Heif
+struct _GimpHeif
 {
   GimpPlugIn      parent_instance;
 };
 
-struct _HeifClass
+struct _GimpHeifClass
 {
   GimpPlugInClass parent_class;
 };
 
 
-#define HEIF_TYPE  (heif_get_type ())
-#define HEIF (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), HEIF_TYPE, Heif))
+#define GIMP_HEIF_TYPE  (gimp_heif_get_type ())
+#define GIMP_HEIF (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GIMP_HEIF_TYPE, GimpHeif))
 
-GType                   heif_get_type         (void) G_GNUC_CONST;
+GType                   gimp_heif_get_type    (void) G_GNUC_CONST;
 
 static GList          * heif_init_procedures  (GimpPlugIn           *plug_in);
 static GimpProcedure  * heif_create_procedure (GimpPlugIn           *plug_in,
@@ -123,14 +117,14 @@ static gboolean         save_dialog           (GimpProcedure        *procedure,
                                                GimpImage            *image);
 
 
-G_DEFINE_TYPE (Heif, heif, GIMP_TYPE_PLUG_IN)
+G_DEFINE_TYPE (GimpHeif, gimp_heif, GIMP_TYPE_PLUG_IN)
 
-GIMP_MAIN (HEIF_TYPE)
+GIMP_MAIN (GIMP_HEIF_TYPE)
 DEFINE_STD_SET_I18N
 
 
 static void
-heif_class_init (HeifClass *klass)
+gimp_heif_class_init (GimpHeifClass *klass)
 {
   GimpPlugInClass *plug_in_class = GIMP_PLUG_IN_CLASS (klass);
 
@@ -140,7 +134,7 @@ heif_class_init (HeifClass *klass)
 }
 
 static void
-heif_init (Heif *heif)
+gimp_heif_init (GimpHeif *heif)
 {
 }
 
@@ -149,6 +143,10 @@ heif_init_procedures (GimpPlugIn *plug_in)
 {
   GList *list = NULL;
 
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+  heif_init (NULL);
+#endif
+
   if (heif_have_decoder_for_format (heif_compression_HEVC))
     {
       list = g_list_append (list, g_strdup (LOAD_PROC));
@@ -169,6 +167,11 @@ heif_init_procedures (GimpPlugIn *plug_in)
       list = g_list_append (list, g_strdup (SAVE_PROC_AV1));
     }
 #endif
+
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+  heif_deinit ();
+#endif
+
   return list;
 }
 
@@ -225,6 +228,8 @@ heif_create_procedure (GimpPlugIn  *plug_in,
       gimp_procedure_set_image_types (procedure, "RGB*");
 
       gimp_procedure_set_menu_label (procedure, _("HEIF/HEIC"));
+      gimp_file_procedure_set_format_name (GIMP_FILE_PROCEDURE (procedure),
+                                           "HEIF");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Exports HEIF images"),
@@ -244,52 +249,52 @@ heif_create_procedure (GimpPlugIn  *plug_in,
                                           "heif,heic");
 
       GIMP_PROC_ARG_INT (procedure, "quality",
-                         "Quality",
-                         "Quality factor (0 = worst, 100 = best)",
+                         _("_Quality"),
+                         _("Quality factor (0 = worst, 100 = best)"),
                          0, 100, 50,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "lossless",
-                             "Lossless",
-                             "Use lossless compression",
+                             _("L_ossless"),
+                             _("Use lossless compression"),
                              FALSE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-color-profile",
-                                 "Save color profile",
-                                 "Save the image's color profile",
+                                 _("Save color prof_ile"),
+                                 _("Save the image's color profile"),
                                  gimp_export_color_profile (),
                                  G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "save-bit-depth",
-                         "Bit depth",
-                         "Bit depth of exported image",
+                         _("_Bit depth"),
+                         _("Bit depth of exported image"),
                          8, 12, 8,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "pixel-format",
-                         "Pixel format",
-                         "Format of color sub-sampling",
+                         _("_Pixel format"),
+                         _("Format of color sub-sampling"),
                          HEIFPLUGIN_EXPORT_FORMAT_RGB, HEIFPLUGIN_EXPORT_FORMAT_YUV420,
                          HEIFPLUGIN_EXPORT_FORMAT_YUV420,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "encoder-speed",
-                         "Encoder speed",
-                         "Tradeoff between speed and compression",
+                         _("Enco_der speed"),
+                         _("Tradeoff between speed and compression"),
                          HEIFPLUGIN_ENCODER_SPEED_SLOW, HEIFPLUGIN_ENCODER_SPEED_FASTER,
                          HEIFPLUGIN_ENCODER_SPEED_BALANCED,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "save-exif",
-                             "Save Exif",
-                             "Toggle saving Exif data",
+                             _("Save Exi_f"),
+                             _("Toggle saving Exif data"),
                              gimp_export_exif (),
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "save-xmp",
-                             "Save XMP",
-                             "Toggle saving XMP data",
+                             _("Save _XMP"),
+                             _("Toggle saving XMP data"),
                              gimp_export_xmp (),
                              G_PARAM_READWRITE);
     }
@@ -332,6 +337,8 @@ heif_create_procedure (GimpPlugIn  *plug_in,
       gimp_procedure_set_image_types (procedure, "RGB*");
 
       gimp_procedure_set_menu_label (procedure, "HEIF/AVIF");
+      gimp_file_procedure_set_format_name (GIMP_FILE_PROCEDURE (procedure),
+                                           "AVIF");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Exports AVIF images"),
@@ -352,52 +359,52 @@ heif_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_priority (GIMP_FILE_PROCEDURE (procedure), 100);
 
       GIMP_PROC_ARG_INT (procedure, "quality",
-                         "Quality",
-                         "Quality factor (0 = worst, 100 = best)",
+                         _("_Quality"),
+                         _("Quality factor (0 = worst, 100 = best)"),
                          0, 100, 50,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "lossless",
-                             "Lossless",
-                             "Use lossless compression",
+                             _("L_ossless"),
+                             _("Use lossless compression"),
                              FALSE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-color-profile",
-                                 "Save color profile",
-                                 "Save the image's color profile",
+                                 _("Save color prof_ile"),
+                                 _("Save the image's color profile"),
                                  gimp_export_color_profile (),
                                  G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "save-bit-depth",
-                         "Bit depth",
-                         "Bit depth of exported image",
+                         _("_Bit depth"),
+                         _("Bit depth of exported image"),
                          8, 12, 8,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "pixel-format",
-                         "Pixel format",
-                         "Format of color sub-sampling",
+                         _("_Pixel format"),
+                         _("Format of color sub-sampling"),
                          HEIFPLUGIN_EXPORT_FORMAT_RGB, HEIFPLUGIN_EXPORT_FORMAT_YUV420,
                          HEIFPLUGIN_EXPORT_FORMAT_YUV420,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "encoder-speed",
-                         "Encoder speed",
-                         "Tradeoff between speed and compression",
+                         _("Enco_der speed"),
+                         _("Tradeoff between speed and compression"),
                          HEIFPLUGIN_ENCODER_SPEED_SLOW, HEIFPLUGIN_ENCODER_SPEED_FASTER,
                          HEIFPLUGIN_ENCODER_SPEED_BALANCED,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "save-exif",
-                             "Save Exif",
-                             "Toggle saving Exif data",
+                             _("Save Exi_f"),
+                             _("Toggle saving Exif data"),
                              gimp_export_exif (),
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "save-xmp",
-                             "Save XMP",
-                             "Toggle saving XMP data",
+                             _("Save _XMP"),
+                             _("Toggle saving XMP data"),
                              gimp_export_xmp (),
                              G_PARAM_READWRITE);
     }
@@ -425,8 +432,16 @@ heif_load (GimpProcedure        *procedure,
   if (interactive)
     gimp_ui_init (PLUG_IN_BINARY);
 
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+  heif_init (NULL);
+#endif
+
   image = load_image (file, interactive, &status, &error);
 
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+  heif_deinit ();
+#endif
+
   if (! image)
     return gimp_procedure_new_return_values (procedure, status, error);
 
@@ -502,12 +517,20 @@ heif_save (GimpProcedure        *procedure,
 
       metadata = gimp_image_metadata_save_prepare (image, "image/heif", &metadata_flags);
 
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+      heif_init (NULL);
+#endif
+
       if (! save_image (file, image, drawables[0], G_OBJECT (config),
                         &error, heif_compression_HEVC, metadata))
         {
           status = GIMP_PDB_EXECUTION_ERROR;
         }
 
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+      heif_deinit ();
+#endif
+
       if (metadata)
         {
           g_object_unref (metadata);
@@ -590,12 +613,20 @@ heif_av1_save (GimpProcedure        *procedure,
 
       metadata = gimp_image_metadata_save_prepare (image, "image/avif", &metadata_flags);
 
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+      heif_init (NULL);
+#endif
+
       if (! save_image (file, image, drawables[0], G_OBJECT (config),
                         &error, heif_compression_AV1, metadata))
         {
           status = GIMP_PDB_EXECUTION_ERROR;
         }
 
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+      heif_deinit ();
+#endif
+
       if (metadata)
         {
           g_object_unref (metadata);
@@ -1131,7 +1162,6 @@ load_image (GFile              *file,
     }
 
   image = gimp_image_new_with_precision (width, height, GIMP_RGB, precision);
-  gimp_image_set_file (image, file);
 
   if (profile)
     {
@@ -1406,6 +1436,7 @@ save_image (GFile                        *file,
   struct heif_image                    *h_image = NULL;
   struct heif_context                  *context = heif_context_alloc ();
   struct heif_encoder                  *encoder = NULL;
+  struct heif_encoding_options         *encoder_options = NULL;
   const struct heif_encoder_descriptor *encoder_descriptor;
   const char                           *encoder_name;
   struct heif_image_handle             *handle  = NULL;
@@ -1936,11 +1967,25 @@ save_image (GFile                        *file,
     }
 #endif
 
+#if LIBHEIF_HAVE_VERSION(1,10,0)
+  if (pixel_format == HEIFPLUGIN_EXPORT_FORMAT_RGB)
+    {
+      encoder_options = heif_encoding_options_alloc ();
+      encoder_options->save_two_colr_boxes_when_ICC_and_nclx_available = 1;
+    }
+#endif
+
   err = heif_context_encode_image (context,
                                    h_image,
                                    encoder,
-                                   NULL,
+                                   encoder_options,
                                    &handle);
+
+  if (encoder_options)
+    {
+      heif_encoding_options_free (encoder_options);
+    }
+
   if (err.code != 0)
     {
       g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
@@ -2442,118 +2487,72 @@ save_dialog (GimpProcedure *procedure,
              GimpImage     *image)
 {
   GtkWidget *dialog;
-  GtkWidget *main_vbox;
-  GtkWidget *grid;
-  GtkWidget *button;
-  GtkWidget *scale;
-  GtkWidget *frame;
-#if LIBHEIF_HAVE_VERSION(1,8,0)
-  GtkWidget *grid2;
-  GtkListStore  *store;
-  GtkWidget     *combo;
+  GtkWidget *quality_scale;
+#if LIBHEIF_HAVE_VERSION(1, 10, 0)
+  GtkListStore *store_pixelformats;
+#endif
+#if LIBHEIF_HAVE_VERSION(1, 8, 0)
+  GtkListStore *store_bitdepths;
+  GtkListStore *store_speeds;
 #endif
   gboolean   run;
 
-  dialog = gimp_procedure_dialog_new (procedure,
-                                      GIMP_PROCEDURE_CONFIG (config),
-                                      g_strcmp0 (gimp_procedure_get_name (procedure), SAVE_PROC_AV1) == 0 ?
-                                      _("Export Image as AVIF") : _("Export Image as HEIF"));
+  dialog = gimp_save_procedure_dialog_new (GIMP_SAVE_PROCEDURE (procedure),
+                                           GIMP_PROCEDURE_CONFIG (config),
+                                           image);
 
-  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
-  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      main_vbox, FALSE, FALSE, 0);
-  gtk_widget_show (main_vbox);
+  gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                    "lossless", GTK_TYPE_CHECK_BUTTON);
 
-  frame = gimp_frame_new (NULL);
-  gtk_box_pack_start (GTK_BOX (main_vbox), frame, FALSE, FALSE, 0);
-  gtk_widget_show (frame);
+  quality_scale = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                                    "quality",
+                                                    GIMP_TYPE_SCALE_ENTRY);
 
-  button = gimp_prop_check_button_new (config, "lossless",
-                                       _("_Lossless"));
-  gtk_frame_set_label_widget (GTK_FRAME (frame), button);
-
-  grid = gtk_grid_new ();
-  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
-  gtk_grid_set_row_spacing (GTK_GRID (grid), 2);
-  gtk_container_add (GTK_CONTAINER (frame), grid);
-  gtk_widget_show (grid);
-
-  g_object_bind_property (config, "lossless",
-                          grid,   "sensitive",
+  g_object_bind_property (config,        "lossless",
+                          quality_scale, "sensitive",
                           G_BINDING_SYNC_CREATE |
                           G_BINDING_INVERT_BOOLEAN);
 
-  scale = gimp_prop_scale_entry_new (config, "quality",
-                                     NULL, 1.0, FALSE, 0, 0);
-  gtk_widget_hide (gimp_labeled_get_label (GIMP_LABELED (scale)));
-  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 1,
-                            _("_Quality"),
-                            0.0, 0.5, scale, 2);
+#if LIBHEIF_HAVE_VERSION(1, 10, 0)
+  store_pixelformats = gimp_int_store_new (_("RGB"), HEIFPLUGIN_EXPORT_FORMAT_RGB,
+                                           _("YUV444"), HEIFPLUGIN_EXPORT_FORMAT_YUV444,
+                                           _("YUV420"), HEIFPLUGIN_EXPORT_FORMAT_YUV420,
+                                           NULL);
 
-#if LIBHEIF_HAVE_VERSION(1,10,0)
-  store = gimp_int_store_new (_("RGB"), HEIFPLUGIN_EXPORT_FORMAT_RGB,
-                              _("YUV444"), HEIFPLUGIN_EXPORT_FORMAT_YUV444,
-                              _("YUV420"), HEIFPLUGIN_EXPORT_FORMAT_YUV420,
-                              NULL);
-
-  combo = gimp_prop_int_combo_box_new (config, "pixel-format",
-                                       GIMP_INT_STORE (store));
-  g_object_unref (store);
-  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 2,
-                            _("Pixel format:"), 0.0, 0.5,
-                            combo, 2);
+  gimp_procedure_dialog_get_int_combo (GIMP_PROCEDURE_DIALOG (dialog),
+                                       "pixel-format", GIMP_INT_STORE (store_pixelformats));
 #endif
 
-#if LIBHEIF_HAVE_VERSION(1,8,0)
-  grid2 = gtk_grid_new ();
-  gtk_grid_set_column_spacing (GTK_GRID (grid2), 6);
-  gtk_grid_set_row_spacing (GTK_GRID (grid2), 2);
-  gtk_box_pack_start (GTK_BOX (main_vbox), grid2, FALSE, FALSE, 0);
-  gtk_widget_show (grid2);
+#if LIBHEIF_HAVE_VERSION(1, 8, 0)
+  store_bitdepths = gimp_int_store_new (_("8 bit/channel"),   8,
+                                        _("10 bit/channel"), 10,
+                                        _("12 bit/channel"), 12,
+                                        NULL);
 
-  store = gimp_int_store_new (_("8 bit/channel"),   8,
-                              _("10 bit/channel"), 10,
-                              _("12 bit/channel"), 12,
-                              NULL);
+  gimp_procedure_dialog_get_int_combo (GIMP_PROCEDURE_DIALOG (dialog),
+                                       "save-bit-depth", GIMP_INT_STORE (store_bitdepths));
 
-  combo = gimp_prop_int_combo_box_new (config, "save-bit-depth",
-                                       GIMP_INT_STORE (store));
-  g_object_unref (store);
-  gimp_grid_attach_aligned (GTK_GRID (grid2), 0, 1,
-                            _("Bit depth:"), 0.0, 0.5,
-                            combo, 2);
+  store_speeds = gimp_int_store_new (_("Slow"), HEIFPLUGIN_ENCODER_SPEED_SLOW,
+                                     _("Balanced"), HEIFPLUGIN_ENCODER_SPEED_BALANCED,
+                                     _("Fast"), HEIFPLUGIN_ENCODER_SPEED_FASTER,
+                                     NULL);
 
-  store = gimp_int_store_new (_("Slow"), HEIFPLUGIN_ENCODER_SPEED_SLOW,
-                              _("Balanced"), HEIFPLUGIN_ENCODER_SPEED_BALANCED,
-                              _("Fast"), HEIFPLUGIN_ENCODER_SPEED_FASTER,
-                              NULL);
-
-  combo = gimp_prop_int_combo_box_new (config, "encoder-speed",
-                                       GIMP_INT_STORE (store));
-  g_object_unref (store);
-  gimp_grid_attach_aligned (GTK_GRID (grid2), 0, 2,
-                            _("Speed:"), 0.0, 0.5,
-                            combo, 2);
+  gimp_procedure_dialog_get_int_combo (GIMP_PROCEDURE_DIALOG (dialog),
+                                       "encoder-speed", GIMP_INT_STORE (store_speeds));
 #endif
 
-#if LIBHEIF_HAVE_VERSION(1,4,0)
-  button = gimp_prop_check_button_new (config, "save-color-profile",
-                                       _("Save color _profile"));
-  gtk_box_pack_start (GTK_BOX (main_vbox), button, FALSE, FALSE, 0);
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
+                              "lossless", "quality",
+#if LIBHEIF_HAVE_VERSION(1, 10, 0)
+                              "pixel-format",
 #endif
-
-  /* Save EXIF data */
-  button = gimp_prop_check_button_new (config, "save-exif",
-                                       _("_Save Exif data"));
-  gtk_box_pack_start (GTK_BOX (main_vbox), button, FALSE, FALSE, 0);
-
-  /* XMP metadata */
-  button = gimp_prop_check_button_new (config, "save-xmp",
-                                       _("Save _XMP data"));
-  gtk_box_pack_start (GTK_BOX (main_vbox), button, FALSE, FALSE, 0);
-
-  gtk_widget_show (dialog);
+#if LIBHEIF_HAVE_VERSION(1, 8, 0)
+                              "save-bit-depth", "encoder-speed",
+#endif
+#if LIBHEIF_HAVE_VERSION(1, 4, 0)
+                              "save-color-profile",
+#endif
+                              "save-exif", "save-xmp", NULL);
 
   run = gimp_procedure_dialog_run (GIMP_PROCEDURE_DIALOG (dialog));
 
