@@ -206,13 +206,14 @@ static gint             save_index              (FILE                *ofp,
                                                  GimpImage           *image,
                                                  GimpDrawable        *drawable,
                                                  gboolean             grey,
-                                                 gboolean             rle);
+                                                 gint                 rle);
 static gint             save_rgb                (FILE                *ofp,
                                                  GimpImage           *image,
                                                  GimpDrawable        *drawable,
-                                                 gboolean             rle);
+                                                 gint                 rle);
 
-static gboolean         save_dialog             (GimpProcedure       *procedure,
+static gboolean         save_dialog             (GimpImage           *image,
+                                                 GimpProcedure       *procedure,
                                                  GObject             *config);
 
 /* Portability kludge */
@@ -310,16 +311,18 @@ sunras_create_procedure (GimpPlugIn  *plug_in,
                                       "Peter Kirchgessner",
                                       "1996");
 
+      gimp_file_procedure_set_format_name (GIMP_FILE_PROCEDURE (procedure),
+                                           _("SUNRAS"));
       gimp_file_procedure_set_mime_types (GIMP_FILE_PROCEDURE (procedure),
                                           "image/x-sun-raster");
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "im1,im8,im24,im32,rs,ras,sun");
 
-      GIMP_PROC_ARG_BOOLEAN (procedure, "rle",
-                             "RLE",
-                             "Use RLE output",
-                             TRUE,
-                             G_PARAM_READWRITE);
+      GIMP_PROC_ARG_INT (procedure, "rle",
+                         _("Data Formatting"),
+                         _("Use standard (0) or Run-Length Encoded (1) output"),
+                         0, 1, 1,
+                         G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -410,7 +413,7 @@ sunras_save (GimpProcedure        *procedure,
 
   if (run_mode == GIMP_RUN_INTERACTIVE)
     {
-      if (! save_dialog (procedure, G_OBJECT (config)))
+      if (! save_dialog (image, procedure, G_OBJECT (config)))
         status = GIMP_PDB_CANCEL;
     }
 
@@ -594,7 +597,7 @@ save_image (GFile         *file,
 {
   FILE          *ofp;
   GimpImageType  drawable_type;
-  gboolean       rle;
+  gint           rle;
   gboolean       retval;
 
   drawable_type = gimp_drawable_type (drawable);
@@ -1076,7 +1079,6 @@ create_new_image (GFile              *file,
     }
 
   image = gimp_image_new (width, height, type);
-  gimp_image_set_file (image, file);
 
   *layer = gimp_layer_new (image, _("Background"), width, height,
                            gdtype,
@@ -1480,7 +1482,7 @@ save_index (FILE         *ofp,
             GimpImage    *image,
             GimpDrawable *drawable,
             gboolean      grey,
-            gboolean      rle)
+            gint          rle)
 {
   int             height, width, linepad, i, j;
   int             ncols, bw, is_bw, is_wb, bpl;
@@ -1527,7 +1529,7 @@ save_index (FILE         *ofp,
     }
   else
     {
-      cmap = gimp_image_get_colormap (image, &ncols);
+      cmap = gimp_image_get_colormap (image, NULL, &ncols);
 
       for (j = 0; j < ncols; j++)
         {
@@ -1658,7 +1660,7 @@ static gboolean
 save_rgb (FILE         *ofp,
           GimpImage    *image,
           GimpDrawable *drawable,
-          gboolean      rle)
+          gint          rle)
 {
   int              height, width, tile_height, linepad;
   int              i, j, bpp;
@@ -1778,25 +1780,26 @@ save_rgb (FILE         *ofp,
 /*  Save interface functions  */
 
 static gboolean
-save_dialog (GimpProcedure *procedure,
+save_dialog (GimpImage     *image,
+             GimpProcedure *procedure,
              GObject       *config)
 {
-  GtkWidget *dialog;
-  GtkWidget *frame;
-  gboolean   run;
+  GtkWidget    *dialog;
+  GtkListStore *store;
+  gboolean      run;
 
-  dialog = gimp_procedure_dialog_new (procedure,
-                                      GIMP_PROCEDURE_CONFIG (config),
-                                      _("Export Image as SUNRAS"));
+  dialog = gimp_save_procedure_dialog_new (GIMP_SAVE_PROCEDURE (procedure),
+                                           GIMP_PROCEDURE_CONFIG (config),
+                                           image);
 
-  frame = gimp_prop_boolean_radio_frame_new (config, "rle",
-                                             _("Data Formatting"),
-                                             _("_RunLength Encoded"),
-                                             _("_Standard"));
+  store = gimp_int_store_new (_("S_tandard"),           FALSE,
+                              _("R_un-Length Encoded"), TRUE,
+                              NULL);
+  gimp_procedure_dialog_get_int_radio (GIMP_PROCEDURE_DIALOG (dialog),
+                                       "rle", GIMP_INT_STORE (store));
 
-  gtk_container_set_border_width (GTK_CONTAINER (frame), 12);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      frame, TRUE, TRUE, 0);
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
+                              NULL);
 
   gtk_widget_show (dialog);
 
