@@ -378,7 +378,6 @@ load_image (GFile            *file,
   image = gimp_image_new (width, height, GIMP_RGB);
   gimp_image_undo_disable (image);
 
-  gimp_image_set_file (image, file);
   gimp_image_set_resolution (image,
                              load_vals.resolution, load_vals.resolution);
 
@@ -393,7 +392,20 @@ load_image (GFile            *file,
   return image;
 }
 
-/*  This is the callback used from load_rsvg_pixbuf().  */
+#if ! LIBRSVG_CHECK_VERSION(2, 46, 0)
+/* XXX Why we keep old deprecated implementation next to newer librsvg API is
+ * because librsvg uses Rust since version 2.41.0. There are 2 raised problems:
+ * 1. Rust is still not available or well tested on every architecture/OS out
+ *    there yet so unless we decide to have SVG support as optional again, it
+ *    would make GIMP non-available on these.
+ * 2. There are some technical-ideological position against Rust for bundling
+ *    and linking dependencies statically.
+ * While the second point may or may not be a problem we want to take into
+ * account (I guess that it mostly depends on the amount of additional
+ * maintenance work it would imply), the first is definitely enough of a reason
+ * to keep an old version requirement.
+ * See also report #6821.
+ */
 static void
 load_set_size_callback (gint     *width,
                         gint     *height,
@@ -440,6 +452,7 @@ load_set_size_callback (gint     *width,
       vals->height = *height;
     }
 }
+#endif /* ! LIBRSVG_CHECK_VERSION(2, 46, 0) */
 
 /*  This function renders a pixbuf from an SVG file according to vals.  */
 static GdkPixbuf *
@@ -452,6 +465,14 @@ load_rsvg_pixbuf (GFile            *file,
   GdkPixbuf  *pixbuf  = NULL;
   RsvgHandle *handle;
 
+#if LIBRSVG_CHECK_VERSION(2, 46, 0)
+  cairo_surface_t *surf = NULL;
+  cairo_t         *cr = NULL;
+  RsvgRectangle    viewport = { 0, };
+  guchar          *src;
+  gint             y;
+#endif
+
   handle = rsvg_handle_new_from_gfile_sync (file, rsvg_flags, NULL, error);
 
   if (! handle)
@@ -465,9 +486,44 @@ load_rsvg_pixbuf (GFile            *file,
     }
 
   rsvg_handle_set_dpi (handle, vals->resolution);
-  rsvg_handle_set_size_callback (handle, load_set_size_callback, vals, NULL);
+#if LIBRSVG_CHECK_VERSION(2, 46, 0)
+  pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
+                           vals->width, vals->height);
+  surf = cairo_image_surface_create_for_data (gdk_pixbuf_get_pixels (pixbuf),
+                                              CAIRO_FORMAT_ARGB32,
+                                              gdk_pixbuf_get_width (pixbuf),
+                                              gdk_pixbuf_get_height (pixbuf),
+                                              gdk_pixbuf_get_rowstride (pixbuf));
+  cr = cairo_create (surf);
 
+  viewport.width = vals->width;
+  viewport.height = vals->height;
+
+  rsvg_handle_render_document (handle, cr, &viewport, NULL);
+
+  cairo_destroy (cr);
+  cairo_surface_destroy (surf);
+
+  /* un-premultiply the data */
+  src = gdk_pixbuf_get_pixels (pixbuf);
+
+  for (y = 0; y < vals->height; y++)
+    {
+      guchar *s = src;
+      gint w = vals->width;
+
+      while (w--)
+        {
+          GIMP_CAIRO_ARGB32_GET_PIXEL (s, s[0], s[1], s[2], s[3]);
+          s += 4;
+        }
+
+      src += gdk_pixbuf_get_rowstride (pixbuf);
+    }
+#else
+  rsvg_handle_set_size_callback (handle, load_set_size_callback, vals, NULL);
   pixbuf = rsvg_handle_get_pixbuf (handle);
+#endif
 
   g_object_unref (handle);
 
@@ -484,8 +540,11 @@ load_rsvg_size (GFile            *file,
                 GError          **error)
 {
   RsvgHandle        *handle;
-  RsvgDimensionData  dim;
   gboolean           has_size;
+  gdouble            width, height;
+#if ! LIBRSVG_CHECK_VERSION(2, 52, 0)
+  RsvgDimensionData  dim;
+#endif
 
   handle = rsvg_handle_new_from_gfile_sync (file, rsvg_flags, NULL, error);
 
@@ -494,12 +553,18 @@ load_rsvg_size (GFile            *file,
 
   rsvg_handle_set_dpi (handle, vals->resolution);
 
+#if LIBRSVG_CHECK_VERSION(2, 52, 0)
+  rsvg_handle_get_intrinsic_size_in_pixels (handle, &width, &height);
+#else
   rsvg_handle_get_dimensions (handle, &dim);
+  width  = (gdouble) dim.width;
+  height = (gdouble) dim.height;
+#endif
 
-  if (dim.width > 0 && dim.height > 0)
+  if (width > 0.0 && height > 0.0)
     {
-      vals->width  = dim.width;
-      vals->height = dim.height;
+      vals->width  = ceil (width);
+      vals->height = ceil (height);
       has_size = TRUE;
     }
   else
@@ -509,21 +574,21 @@ load_rsvg_size (GFile            *file,
       has_size = FALSE;
     }
 
-    if (size_label)
-      {
-        if (has_size)
-          {
-            gchar *text = g_strdup_printf (_("%d × %d"),
-                                           vals->width, vals->height);
-            gtk_label_set_text (GTK_LABEL (size_label), text);
-            g_free (text);
-          }
-        else
-          {
-            gtk_label_set_text (GTK_LABEL (size_label),
-                                _("SVG file does not\nspecify a size!"));
-          }
-      }
+  if (size_label)
+    {
+      if (has_size)
+        {
+          gchar *text = g_strdup_printf (_("%d × %d"),
+                                         vals->width, vals->height);
+          gtk_label_set_text (GTK_LABEL (size_label), text);
+          g_free (text);
+        }
+      else
+        {
+          gtk_label_set_text (GTK_LABEL (size_label),
+                              _("SVG file does not\nspecify a size!"));
+        }
+    }
 
   g_object_unref (handle);
 
@@ -670,8 +735,8 @@ load_dialog (GFile            *file,
   SvgLoadVals    vals  =
   {
     SVG_DEFAULT_RESOLUTION,
-    - SVG_PREVIEW_SIZE,
-    - SVG_PREVIEW_SIZE
+    SVG_PREVIEW_SIZE,
+    SVG_PREVIEW_SIZE,
   };
 
   preview = load_rsvg_pixbuf (file, &vals, *rsvg_flags, &allow_retry, &error);
