@@ -196,7 +196,8 @@ static gboolean         save_image           (GFile                *file,
                                               GObject              *config,
                                               GError              **error);
 
-static gboolean         save_dialog          (GimpProcedure        *procedure,
+static gboolean         save_dialog          (GimpImage            *image,
+                                              GimpProcedure        *procedure,
                                               GObject              *config);
 
 static GimpImage      * ReadImage            (FILE                 *fp,
@@ -293,20 +294,22 @@ tga_create_procedure (GimpPlugIn  *plug_in,
                                       "Raphael FRANCOIS, Gordon Matzigkeit",
                                       "1997,2000");
 
+      gimp_file_procedure_set_format_name (GIMP_FILE_PROCEDURE (procedure),
+                                           _("TGA"));
       gimp_file_procedure_set_mime_types (GIMP_FILE_PROCEDURE (procedure),
                                           "image/x-tga");
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "tga");
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "rle",
-                             "RLE",
-                             "Use RLE compression",
+                             _("_Use RLE compression"),
+                             _("Use RLE compression"),
                              TRUE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "origin",
-                         "Origin",
-                         "Image origin (0 = top-left, 1 = bottom-left)",
+                         _("Ori_gin"),
+                         _("Image origin (0 = top-left, 1 = bottom-left)"),
                          0, 1, ORIGIN_BOTTOM_LEFT,
                          G_PARAM_READWRITE);
     }
@@ -397,7 +400,7 @@ tga_save (GimpProcedure        *procedure,
 
   if (run_mode == GIMP_RUN_INTERACTIVE)
     {
-      if (! save_dialog (procedure, G_OBJECT (config)))
+      if (! save_dialog (image, procedure, G_OBJECT (config)))
         status = GIMP_PDB_CANCEL;
     }
 
@@ -556,6 +559,11 @@ load_image (GFile   *file,
       if (info.imageType == TGA_TYPE_GRAY && info.bpp == 16)
         info.alphaBits = 8;
     }
+  else if (info.alphaBits == 4 && info.imageType == TGA_TYPE_COLOR && info.bpp == 32)
+    {
+      /* Incorrect TGA saved by Krita, see issue #9067*/
+      info.alphaBits = 8;
+    }
 
   switch (info.imageType)
     {
@@ -1101,7 +1109,6 @@ ReadImage (FILE     *fp,
     }
 
   image = gimp_image_new (info->width, info->height, itype);
-  gimp_image_set_file (image, file);
 
   if (gimp_cmap)
     gimp_image_set_colormap (image, gimp_cmap, info->colorMapLength);
@@ -1237,7 +1244,7 @@ save_image (GFile         *file,
 
   if (dtype == GIMP_INDEXED_IMAGE)
     {
-      gimp_cmap = gimp_image_get_colormap (image, &num_colors);
+      gimp_cmap = gimp_image_get_colormap (image, NULL, &num_colors);
 
       header[1] = 1; /* cmap type */
       header[2] = rle ? 9 : 1;
@@ -1248,7 +1255,7 @@ save_image (GFile         *file,
     }
   else if (dtype == GIMP_INDEXEDA_IMAGE)
     {
-      gimp_cmap = gimp_image_get_colormap (image, &num_colors);
+      gimp_cmap = gimp_image_get_colormap (image, NULL, &num_colors);
 
       header[1] = 1; /* cmap type */
       header[2] = rle ? 9 : 1;
@@ -1430,50 +1437,35 @@ save_image (GFile         *file,
 }
 
 static gboolean
-save_dialog (GimpProcedure *procedure,
+save_dialog (GimpImage     *image,
+             GimpProcedure *procedure,
              GObject       *config)
 {
   GtkWidget    *dialog;
-  GtkWidget    *label;
-  GtkWidget    *toggle;
   GtkListStore *store;
-  GtkWidget    *combo;
   GtkWidget    *vbox;
-  GtkWidget    *hbox;
   gboolean      run;
 
-  dialog = gimp_procedure_dialog_new (procedure,
-                                      GIMP_PROCEDURE_CONFIG (config),
-                                      _("Export Image as TGA"));
-
-  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      vbox, TRUE, TRUE, 0);
-  gtk_widget_show (vbox);
-
-  /*  rle  */
-  toggle = gimp_prop_check_button_new (config, "rle",
-                                       _("_RLE compression"));
-  gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);
-
-  /*  origin  */
-  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
-
-  label = gtk_label_new_with_mnemonic (_("Or_igin:"));
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
+  dialog = gimp_save_procedure_dialog_new (GIMP_SAVE_PROCEDURE (procedure),
+                                           GIMP_PROCEDURE_CONFIG (config),
+                                           image);
 
   store = gimp_int_store_new (_("Bottom left"), ORIGIN_BOTTOM_LEFT,
                               _("Top left"),    ORIGIN_TOP_LEFT,
                               NULL);
-  combo = gimp_prop_int_combo_box_new (config, "origin",
-                                       GIMP_INT_STORE (store));
-  gtk_box_pack_start (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
+  gimp_procedure_dialog_get_int_combo (GIMP_PROCEDURE_DIALOG (dialog),
+                                       "origin", GIMP_INT_STORE (store));
 
-  gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
+  vbox = gimp_procedure_dialog_fill_box (GIMP_PROCEDURE_DIALOG (dialog),
+                                         "tga-save-vbox",
+                                         "rle",
+                                         "origin",
+                                         NULL);
+  gtk_box_set_spacing (GTK_BOX (vbox), 12);
+
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
+                              "tga-save-vbox",
+                              NULL);
 
   gtk_widget_show (dialog);
 
