@@ -129,7 +129,8 @@ static gboolean         save_image           (GFile                *file,
                                               GimpDrawable         *drawable,
                                               GObject              *config,
                                               GError              **error);
-static gboolean         save_dialog          (GimpProcedure        *procedure,
+static gboolean         save_dialog          (GimpImage            *image,
+                                              GimpProcedure        *procedure,
                                               GObject              *config);
 
 
@@ -247,14 +248,16 @@ xpm_create_procedure (GimpPlugIn  *plug_in,
                                       "Spencer Kimball & Peter Mattis",
                                       "1997");
 
+      gimp_file_procedure_set_format_name (GIMP_FILE_PROCEDURE (procedure),
+                                           _("XPM"));
       gimp_file_procedure_set_mime_types (GIMP_FILE_PROCEDURE (procedure),
                                           "image/x-pixmap");
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "xpm");
 
       GIMP_PROC_ARG_INT (procedure, "threshold",
-                         "Threshold",
-                         "Alpha threshold",
+                         _("_Threshold"),
+                         _("Alpha threshold"),
                          0, 255, 127,
                          G_PARAM_READWRITE);
     }
@@ -346,7 +349,7 @@ xpm_save (GimpProcedure        *procedure,
   if (run_mode == GIMP_RUN_INTERACTIVE)
     {
       if (gimp_drawable_has_alpha (drawables[0]))
-        if (! save_dialog (procedure, G_OBJECT (config)))
+        if (! save_dialog (image, procedure, G_OBJECT (config)))
           status = GIMP_PDB_CANCEL;
     }
 
@@ -409,8 +412,6 @@ load_image (GFile   *file,
                           xpm_image.height,
                           GIMP_RGB);
 
-  gimp_image_set_file (image, file);
-
   /* fill it */
   parse_image (image, &xpm_image, cmap);
 
@@ -445,11 +446,11 @@ parse_colors (XpmImage *xpm_image)
   for (i = 0, j = 0; i < xpm_image->ncolors; i++)
     {
       gchar     *colorspec = "None";
-      XpmColor *xpm_color;
+      XpmColor  *xpm_color;
 #ifndef XPM_NO_X
-      XColor    xcolor;
+      XColor     xcolor;
 #else
-      GdkColor  xcolor;
+      GdkRGBA    xcolor;
 #endif
 
       xpm_color = &(xpm_image->colorTable[i]);
@@ -469,12 +470,15 @@ parse_colors (XpmImage *xpm_image)
         {
 #ifndef XPM_NO_X
           XParseColor (display, colormap, colorspec, &xcolor);
-#else
-          gdk_color_parse (colorspec, &xcolor);
-#endif
           cmap[j++] = xcolor.red >> 8;
           cmap[j++] = xcolor.green >> 8;
           cmap[j++] = xcolor.blue >> 8;
+#else
+          gdk_rgba_parse (&xcolor, colorspec);
+          cmap[j++] = CLAMP (xcolor.red * G_MAXUINT8, 0, G_MAXUINT8);
+          cmap[j++] = CLAMP (xcolor.green * G_MAXUINT8, 0, G_MAXUINT8);
+          cmap[j++] = CLAMP (xcolor.blue * G_MAXUINT8, 0, G_MAXUINT8);
+#endif
           cmap[j++] = ~0;
         }
       else
@@ -786,7 +790,7 @@ save_image (GFile         *file,
 
   if (indexed)
     {
-      guchar *cmap = gimp_image_get_colormap (image, &ncolors);
+      guchar *cmap = gimp_image_get_colormap (image, NULL, &ncolors);
       guchar *c;
 
       c = cmap;
@@ -872,21 +876,21 @@ save_image (GFile         *file,
 }
 
 static gboolean
-save_dialog (GimpProcedure *procedure,
+save_dialog (GimpImage     *image,
+             GimpProcedure *procedure,
              GObject       *config)
 {
   GtkWidget *dialog;
-  GtkWidget *scale;
   gboolean   run;
 
-  dialog = gimp_procedure_dialog_new (procedure,
-                                      GIMP_PROCEDURE_CONFIG (config),
-                                      _("Export Image as XPM"));
+  dialog = gimp_save_procedure_dialog_new (GIMP_SAVE_PROCEDURE (procedure),
+                                           GIMP_PROCEDURE_CONFIG (config),
+                                           image);
 
-  scale = gimp_prop_scale_entry_new (config, "threshold", NULL, 1.0, FALSE, 0, 0);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      scale, TRUE, TRUE, 6);
-  gtk_widget_show (scale);
+  gimp_procedure_dialog_get_scale_entry (GIMP_PROCEDURE_DIALOG (dialog),
+                                         "threshold", 1.0);
+
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog), NULL);
   gtk_widget_show (dialog);
 
   run = gimp_procedure_dialog_run (GIMP_PROCEDURE_DIALOG (dialog));
