@@ -413,14 +413,14 @@ grid (GimpImage    *image,
       GimpDrawable *drawable,
       GimpPreview  *preview)
 {
-  GeglBuffer *src_buffer;
-  GeglBuffer *dest_buffer;
+  GeglBuffer *src_buffer  = NULL;
+  GeglBuffer *dest_buffer = NULL;
   const Babl *format;
   gint        bytes;
   gint        x_offset;
   gint        y_offset;
-  guchar     *dest;
-  guchar     *buffer = NULL;
+  guchar     *dest        = NULL;
+  guchar     *buffer      = NULL;
   gint        x, y;
   gboolean    alpha;
   gboolean    blend;
@@ -463,7 +463,7 @@ grid (GimpImage    *image,
       break;
 
     case GIMP_INDEXED:
-      cmap = gimp_image_get_colormap (image, &ncolors);
+      cmap = gimp_image_get_colormap (image, NULL, &ncolors);
 
       hcolor[0] = best_cmap_match (cmap, ncolors, &grid_cfg.hcolor);
       vcolor[0] = best_cmap_match (cmap, ncolors, &grid_cfg.vcolor);
