@@ -52,16 +52,6 @@
 #define PLUG_IN_ROLE   "gimp-mail"
 
 
-typedef struct
-{
-  gchar filename[BUFFER_SIZE];
-  gchar receipt[BUFFER_SIZE];
-  gchar from[BUFFER_SIZE];
-  gchar subject[BUFFER_SIZE];
-  gchar comment[BUFFER_SIZE];
-} m_info;
-
-
 typedef struct _Mail      Mail;
 typedef struct _MailClass MailClass;
 
@@ -93,24 +83,21 @@ static GimpValueArray * mail_run              (GimpProcedure        *procedure,
                                                const GimpValueArray *args,
                                                gpointer              run_data);
 
-static GimpPDBStatusType  send_image          (const gchar          *filename,
+static GimpPDBStatusType  send_image          (GObject              *config,
                                                GimpImage            *image,
                                                gint                  n_drawables,
                                                GimpDrawable        **drawables,
                                                gint32                run_mode);
 
-static gboolean           send_dialog             (void);
-static void               mail_entry_callback     (GtkWidget        *widget,
-                                                   gchar            *data);
+static gboolean           send_dialog             (GimpProcedure    *procedure,
+                                                   GObject          *config);
 static gboolean           valid_file              (GFile            *file);
 static gchar            * find_extension          (const gchar      *filename);
 
 #ifdef SENDMAIL
-static void               mesg_body_callback      (GtkTextBuffer    *buffer,
-                                                   gpointer          data);
-
 static gchar            * sendmail_content_type   (const gchar      *filename);
-static void               sendmail_create_headers (FILE             *mailpipe);
+static void               sendmail_create_headers (FILE             *mailpipe,
+                                                   GObject          *config);
 static gboolean           sendmail_to64           (const gchar      *filename,
                                                    FILE             *outfile,
                                                    GError          **error);
@@ -125,12 +112,9 @@ GIMP_MAIN (MAIL_TYPE)
 DEFINE_STD_SET_I18N
 
 
-static m_info mail_info =
-{
-  "", "", "", "", ""
-};
-
+#ifdef SENDMAIL
 static gchar *mesg_body = NULL;
+#endif
 
 
 static void
@@ -203,17 +187,17 @@ mail_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_procedure_set_menu_label (procedure, _("Send by E_mail..."));
       gimp_procedure_set_icon_name (procedure, GIMP_ICON_EDIT);
-      gimp_procedure_add_menu_path (procedure, "<Image>/File/Send");
+      gimp_procedure_add_menu_path (procedure, "<Image>/File");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Send the image by email"),
 #ifdef SENDMAIL
-                                        "Sendmail is used to send emails "
-                                        "and must be properly configured.",
+                                        _("Sendmail is used to send emails "
+                                          "and must be properly configured."),
 #else /* xdg-email */
-                                        "The preferred email composer is "
-                                        "used to send emails and must be "
-                                        "properly configured.",
+                                        _("The preferred email composer is "
+                                          "used to send emails and must be "
+                                          "properly configured."),
 #endif
                                         name);
       gimp_procedure_set_attribution (procedure,
@@ -224,32 +208,32 @@ mail_create_procedure (GimpPlugIn  *plug_in,
                                       "1995-1997");
 
       GIMP_PROC_ARG_STRING (procedure, "filename",
-                            "Filename",
-                            "The name of the file to save the image in",
+                            _("File_name"),
+                            _("The name of the file to save the image in"),
                             NULL,
                             G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_STRING (procedure, "to-address",
-                            "To address",
-                            "The email address to send to",
-                            NULL,
+                            _("_To"),
+                            _("The email address to send to"),
+                            "",
                             G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_STRING (procedure, "from-address",
-                            "From address",
-                            "The email address for the From: field",
-                            NULL,
+                            _("_From"),
+                            _("The email address for the From: field"),
+                            "",
                             G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_STRING (procedure, "subject",
-                            "Subject",
-                            "The subject",
-                            NULL,
+                            _("Su_bject"),
+                            _("The subject"),
+                            "",
                             G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_STRING (procedure, "comment",
-                            "Comment",
-                            "The comment",
+                            _("Co_mment"),
+                            _("The comment"),
                             NULL,
                             G_PARAM_READWRITE);
     }
@@ -266,69 +250,60 @@ mail_run (GimpProcedure        *procedure,
           const GimpValueArray *args,
           gpointer              run_data)
 {
-  GimpPDBStatusType status = GIMP_PDB_SUCCESS;
+  GimpProcedureConfig *config;
+  GimpPDBStatusType    status = GIMP_PDB_SUCCESS;
+
+  config = gimp_procedure_create_config (procedure);
+  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
 
   switch (run_mode)
     {
     case GIMP_RUN_INTERACTIVE:
-      gimp_get_data (PLUG_IN_PROC, &mail_info);
       {
         gchar *filename = g_file_get_path (gimp_image_get_file (image));
 
         if (filename)
           {
             gchar *basename = g_filename_display_basename (filename);
+            gchar  buffername[BUFFER_SIZE];
 
-            g_strlcpy (mail_info.filename, basename, BUFFER_SIZE);
+            g_strlcpy (buffername, basename, BUFFER_SIZE);
+
+            g_object_set (config,
+                          "filename", buffername,
+                          NULL);
             g_free (basename);
             g_free (filename);
           }
       }
 
-      if (! send_dialog ())
-        return gimp_procedure_new_return_values (procedure, GIMP_PDB_CANCEL,
-                                                 NULL);
-      break;
+      if (! send_dialog (procedure, G_OBJECT (config)))
+        {
+          gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
+          g_object_unref (config);
 
-    case GIMP_RUN_NONINTERACTIVE:
-      g_strlcpy (mail_info.filename, GIMP_VALUES_GET_STRING (args, 0),
-                 BUFFER_SIZE);
-      g_strlcpy (mail_info.receipt,  GIMP_VALUES_GET_STRING (args, 1),
-                 BUFFER_SIZE);
-      g_strlcpy (mail_info.from,     GIMP_VALUES_GET_STRING (args, 2),
-                 BUFFER_SIZE);
-      g_strlcpy (mail_info.subject,  GIMP_VALUES_GET_STRING (args, 3),
-                 BUFFER_SIZE);
-      g_strlcpy (mail_info.comment,  GIMP_VALUES_GET_STRING (args, 4),
-                 BUFFER_SIZE);
-      break;
-
-    case GIMP_RUN_WITH_LAST_VALS:
-      gimp_get_data (PLUG_IN_PROC, &mail_info);
+          return gimp_procedure_new_return_values (procedure, GIMP_PDB_CANCEL,
+                                                   NULL);
+        }
       break;
 
     default:
       break;
     }
 
-  status = send_image (mail_info.filename,
+  status = send_image (G_OBJECT (config),
                        image,
                        n_drawables, drawables,
                        run_mode);
 
-  if (status == GIMP_PDB_SUCCESS)
-    {
-      if (mesg_body)
-        g_strlcpy (mail_info.comment, mesg_body, BUFFER_SIZE);
-
-      gimp_set_data (PLUG_IN_PROC, &mail_info, sizeof (m_info));
-    }
+  gimp_procedure_config_end_run (config, status);
+  g_object_unref (config);
 
   return gimp_procedure_new_return_values (procedure, status, NULL);
 }
 
 static GimpPDBStatusType
-send_image (const gchar   *filename,
+send_image (GObject       *config,
             GimpImage     *image,
             gint           n_drawables,
             GimpDrawable **drawables,
@@ -349,7 +324,20 @@ send_image (const gchar   *filename,
   GPid               mailpid;
   FILE              *mailpipe = NULL;
 #endif
-  GError            *error = NULL;
+  GError            *error    = NULL;
+  gchar             *filename = NULL;
+  gchar             *receipt  = NULL;
+  gchar             *from     = NULL;
+  gchar             *subject  = NULL;
+  gchar             *comment  = NULL;
+
+  g_object_get (config,
+                "filename",     &filename,
+                "to-address",   &receipt,
+                "from-address", &from,
+                "subject",      &subject,
+                "comment",      &comment,
+                NULL);
 
   ext = find_extension (filename);
 
@@ -417,8 +405,8 @@ send_image (const gchar   *filename,
       g_object_unref (enumerator);
     }
 
-  filepath = g_build_filename (gimp_file_get_utf8_name (tmp_dir),
-                               mail_info.filename, NULL);
+  filepath = g_build_filename (gimp_file_get_utf8_name (tmp_dir), filename,
+                               NULL);
   if (g_rename (tmpname, filepath) == -1)
     {
       /* But on some system, I got an 'Invalid cross-device link' errno
@@ -443,19 +431,19 @@ send_image (const gchar   *filename,
   mailcmd[1] = "--attach";
   mailcmd[2] = filepath;
   i = 3;
-  if (strlen (mail_info.subject) > 0)
+  if (subject != NULL && strlen (subject) > 0)
     {
       mailcmd[i++] = "--subject";
-      mailcmd[i++] = mail_info.subject;
+      mailcmd[i++] = subject;
     }
-  if (strlen (mail_info.comment) > 0)
+  if (comment != NULL && strlen (comment) > 0)
     {
       mailcmd[i++] = "--body";
-      mailcmd[i++] = mail_info.comment;
+      mailcmd[i++] = comment;
     }
-  if (strlen (mail_info.receipt) > 0)
+  if (receipt != NULL && strlen (receipt) > 0)
     {
-      mailcmd[i++] = mail_info.receipt;
+      mailcmd[i++] = receipt;
     }
   mailcmd[i] = NULL;
 
@@ -475,7 +463,7 @@ send_image (const gchar   *filename,
   else
     mailcmd[0] = g_build_filename (SENDMAIL, "sendmail", NULL);
 
-  mailcmd[1] = mail_info.receipt;
+  mailcmd[1] = receipt;
   mailcmd[2] = NULL;
 
   /* create a pipe to sendmail */
@@ -484,7 +472,7 @@ send_image (const gchar   *filename,
   if (mailpipe == NULL)
     return GIMP_PDB_EXECUTION_ERROR;
 
-  sendmail_create_headers (mailpipe);
+  sendmail_create_headers (mailpipe, config);
 
   fflush (mailpipe);
 
@@ -526,6 +514,12 @@ cleanup:
     g_free (filepath);
 #endif
 
+  g_free (filename);
+  g_free (receipt);
+  g_free (from);
+  g_free (subject);
+  g_free (comment);
+
   g_free (mailcmd[0]);
   g_free (tmpname);
   g_object_unref (tmpfile);
@@ -535,19 +529,20 @@ cleanup:
 
 
 static gboolean
-send_dialog (void)
+send_dialog (GimpProcedure *procedure,
+             GObject       *config)
 {
   GtkWidget     *dlg;
   GtkWidget     *main_vbox;
   GtkWidget     *entry;
+  GtkWidget     *real_entry;
   GtkWidget     *grid;
+  GtkWidget     *button;
 #ifdef SENDMAIL
   GtkWidget     *scrolled_window;
   GtkWidget     *text_view;
-  GtkTextBuffer *text_buffer;
 #endif
   gchar         *gump_from;
-  gint           row = 0;
   gboolean       run;
 
   gimp_ui_init (PLUG_IN_BINARY);
@@ -557,23 +552,25 @@ send_dialog (void)
 
   if (gump_from)
     {
-      g_strlcpy (mail_info.from, gump_from, BUFFER_SIZE);
+      g_object_set (config,
+                    "from-address", gump_from,
+                    NULL);
       g_free (gump_from);
     }
 
-  dlg = gimp_dialog_new (_("Send by Email"), PLUG_IN_ROLE,
-                         NULL, 0,
-                         gimp_standard_help_func, PLUG_IN_PROC,
+  dlg = gimp_procedure_dialog_new (procedure,
+                                   GIMP_PROCEDURE_CONFIG (config),
+                                   _("Send by Email"));
+  /* Change "Ok" button to "Send" */
+  button = gtk_dialog_get_widget_for_response (GTK_DIALOG (dlg),
+                                               GTK_RESPONSE_OK);
+  gtk_button_set_label (GTK_BUTTON (button), _("Send"));
 
-                         _("_Cancel"), GTK_RESPONSE_CANCEL,
-                         _("_Send"),   GTK_RESPONSE_OK,
-
-                         NULL);
 
   gimp_dialog_set_alternative_button_order (GTK_DIALOG (dlg),
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
-                                           -1);
+                                            GTK_RESPONSE_OK,
+                                            GTK_RESPONSE_CANCEL,
+                                            -1);
 
   gimp_window_set_transient (GTK_WINDOW (dlg));
 
@@ -592,86 +589,84 @@ send_dialog (void)
   gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
 
   /* Filename entry */
-  entry = gtk_entry_new ();
-  gtk_widget_set_size_request (entry, 200, -1);
-  gtk_entry_set_max_length (GTK_ENTRY (entry), BUFFER_SIZE - 1);
-  gtk_entry_set_text (GTK_ENTRY (entry), mail_info.filename);
-  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
-                            _("_Filename:"), 0.0, 0.5,
-                            entry, 1);
-  g_signal_connect (entry, "changed",
-                    G_CALLBACK (mail_entry_callback),
-                    mail_info.filename);
-  gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+  entry = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dlg),
+                                            "filename", GIMP_TYPE_LABEL_ENTRY);
+  real_entry = gimp_label_entry_get_entry (GIMP_LABEL_ENTRY (entry));
+  gtk_widget_set_size_request (real_entry, 200, -1);
+  gtk_entry_set_activates_default (GTK_ENTRY (real_entry), TRUE);
+
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dlg),
+                              "filename",
+                              NULL);
+  gtk_entry_set_max_length (GTK_ENTRY (real_entry),
+                            BUFFER_SIZE - 1);
 
 #ifdef SENDMAIL
   /* To entry */
-  entry = gtk_entry_new ();
-  gtk_widget_set_size_request (entry, 200, -1);
-  gtk_entry_set_max_length (GTK_ENTRY (entry), BUFFER_SIZE - 1);
-  gtk_entry_set_text (GTK_ENTRY (entry), mail_info.receipt);
-  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
-                            C_("email-address", "_To:"), 0.0, 0.5,
-                            entry, 1);
-  g_signal_connect (entry, "changed",
-                    G_CALLBACK (mail_entry_callback),
-                    mail_info.receipt);
+  entry = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dlg),
+                                            "to-address",
+                                            GIMP_TYPE_LABEL_ENTRY);
+  real_entry = gimp_label_entry_get_entry (GIMP_LABEL_ENTRY (entry));
+  gtk_widget_set_size_request (real_entry, 200, -1);
+  gtk_entry_set_max_length (GTK_ENTRY (real_entry), BUFFER_SIZE - 1);
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dlg),
+                              "to-address",
+                              NULL);
+  gtk_entry_set_max_length (GTK_ENTRY (real_entry),
+                            BUFFER_SIZE - 1);
 
-  gtk_widget_grab_focus (entry);
+  gtk_widget_grab_focus (real_entry);
 
   /* From entry */
-  entry = gtk_entry_new ();
-  gtk_widget_set_size_request (entry, 200, -1);
-  gtk_entry_set_max_length (GTK_ENTRY (entry), BUFFER_SIZE - 1);
-  gtk_entry_set_text (GTK_ENTRY (entry), mail_info.from);
-  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
-                            C_("email-address", "_From:"), 0.0, 0.5,
-                            entry, 1);
-  g_signal_connect (entry, "changed",
-                    G_CALLBACK (mail_entry_callback),
-                    mail_info.from);
+  entry = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dlg),
+                                            "from-address",
+                                            GIMP_TYPE_LABEL_ENTRY);
+  real_entry = gimp_label_entry_get_entry (GIMP_LABEL_ENTRY (entry));
+  gtk_widget_set_size_request (real_entry, 200, -1);
+  gtk_entry_set_max_length (GTK_ENTRY (real_entry), BUFFER_SIZE - 1);
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dlg),
+                              "from-address",
+                              NULL);
+  gtk_entry_set_max_length (GTK_ENTRY (real_entry),
+                            BUFFER_SIZE - 1);
 
   /* Subject entry */
-  entry = gtk_entry_new ();
-  gtk_widget_set_size_request (entry, 200, -1);
-  gtk_entry_set_max_length (GTK_ENTRY (entry), BUFFER_SIZE - 1);
-  gtk_entry_set_text (GTK_ENTRY (entry), mail_info.subject);
-  gimp_grid_attach_aligned (GTK_GRID (grid), 0, row++,
-                            _("S_ubject:"), 0.0, 0.5,
-                            entry, 1);
-  g_signal_connect (entry, "changed",
-                    G_CALLBACK (mail_entry_callback),
-                    mail_info.subject);
+  entry = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dlg),
+                                            "subject",
+                                            GIMP_TYPE_LABEL_ENTRY);
+  real_entry = gimp_label_entry_get_entry (GIMP_LABEL_ENTRY (entry));
+  gtk_widget_set_size_request (real_entry, 200, -1);
+  gtk_entry_set_max_length (GTK_ENTRY (real_entry), BUFFER_SIZE - 1);
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dlg),
+                              "subject",
+                              NULL);
+  gtk_entry_set_max_length (GTK_ENTRY (real_entry),
+                            BUFFER_SIZE - 1);
 
   /* Body  */
-  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+  text_view = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dlg),
+                                                "comment",
+                                                GTK_TYPE_TEXT_VIEW);
+  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_WORD);
+
+  scrolled_window =
+    gimp_procedure_dialog_fill_scrolled_window (GIMP_PROCEDURE_DIALOG (dlg),
+                                                "comment-scrolled",
+                                                "comment");
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
                                        GTK_SHADOW_IN);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
-  gtk_box_pack_start (GTK_BOX (main_vbox), scrolled_window, TRUE, TRUE, 0);
-  gtk_widget_show (scrolled_window);
 
-  text_buffer = gtk_text_buffer_new (NULL);
-
-  g_signal_connect (text_buffer, "changed",
-                    G_CALLBACK (mesg_body_callback),
-                    NULL);
-
-  gtk_text_buffer_set_text (text_buffer, mail_info.comment, -1);
-
-  text_view = gtk_text_view_new_with_buffer (text_buffer);
-  g_object_unref (text_buffer);
-
-  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_WORD);
-  gtk_container_add (GTK_CONTAINER (scrolled_window), text_view);
-  gtk_widget_show (text_view);
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dlg),
+                              "comment-scrolled",
+                              NULL);
 #endif
 
   gtk_widget_show (dlg);
 
-  run = (gimp_dialog_run (GIMP_DIALOG (dlg)) == GTK_RESPONSE_OK);
+  run = gimp_procedure_dialog_run (GIMP_PROCEDURE_DIALOG (dlg));
 
   gtk_widget_destroy (dlg);
 
@@ -729,27 +724,7 @@ find_extension (const gchar *filename)
   return ext;
 }
 
-static void
-mail_entry_callback (GtkWidget *widget,
-                     gchar     *data)
-{
-  g_strlcpy (data, gtk_entry_get_text (GTK_ENTRY (widget)), BUFFER_SIZE);
-}
-
 #ifdef SENDMAIL
-static void
-mesg_body_callback (GtkTextBuffer *buffer,
-                    gpointer       data)
-{
-  GtkTextIter start_iter;
-  GtkTextIter end_iter;
-
-  gtk_text_buffer_get_bounds (buffer, &start_iter, &end_iter);
-
-  g_free (mesg_body);
-  mesg_body = gtk_text_buffer_get_text (buffer, &start_iter, &end_iter, FALSE);
-}
-
 static gchar *
 sendmail_content_type (const gchar *filename)
 {
@@ -796,17 +771,32 @@ sendmail_content_type (const gchar *filename)
 }
 
 static void
-sendmail_create_headers (FILE *mailpipe)
+sendmail_create_headers (FILE    *mailpipe,
+                         GObject *config)
 {
+  gchar *filename = NULL;
+  gchar *receipt  = NULL;
+  gchar *from     = NULL;
+  gchar *subject  = NULL;
+  gchar *comment  = NULL;
+
+  g_object_get (config,
+                "filename",     &filename,
+                "to-address",   &receipt,
+                "from-address", &from,
+                "subject",      &subject,
+                "comment",      &comment,
+                NULL);
+
   /* create all the mail header stuff. Feel free to add your own */
   /* It is advisable to leave the X-Mailer header though, as     */
   /* there is a possibility of a Gimp mail scanner/reader in the  */
   /* future. It will probably need that header.                 */
 
-  fprintf (mailpipe, "To: %s \n", mail_info.receipt);
-  fprintf (mailpipe, "Subject: %s \n", mail_info.subject);
-  if (strlen (mail_info.from) > 0)
-    fprintf (mailpipe, "From: %s \n", mail_info.from);
+  fprintf (mailpipe, "To: %s \n", receipt);
+  fprintf (mailpipe, "Subject: %s \n", subject);
+  if (from != NULL && strlen (from) > 0)
+    fprintf (mailpipe, "From: %s \n", from);
 
   fprintf (mailpipe, "X-Mailer: GIMP Useless Mail plug-in %s\n", GIMP_VERSION);
 
@@ -825,17 +815,23 @@ sendmail_create_headers (FILE *mailpipe)
   fprintf (mailpipe, "\n\n");
 
   {
-    gchar *content = sendmail_content_type (mail_info.filename);
+    gchar *content = sendmail_content_type (filename);
 
     fprintf (mailpipe, "--GUMP-MIME-boundary\n");
     fprintf (mailpipe, "Content-type: %s\n", content);
     fprintf (mailpipe, "Content-transfer-encoding: base64\n");
     fprintf (mailpipe, "Content-disposition: attachment; filename=\"%s\"\n",
-             mail_info.filename);
-    fprintf (mailpipe, "Content-description: %s\n\n", mail_info.filename);
+             filename);
+    fprintf (mailpipe, "Content-description: %s\n\n", filename);
 
     g_free (content);
   }
+
+  g_free (filename);
+  g_free (receipt);
+  g_free (from);
+  g_free (subject);
+  g_free (comment);
 }
 
 static gboolean
