@@ -608,13 +608,13 @@ ico_load_layer (FILE        *fp,
                 guchar      *buf,
                 gint         maxsize,
                 gint32       file_offset,
+                gchar       *layer_prefix,
                 IcoLoadInfo *info)
 {
   gint        width, height;
   GimpLayer  *layer;
   guint32     first_bytes;
   GeglBuffer *buffer;
-  gchar       name[ICO_MAXBUF];
 
   if (fseek (fp, info->offset + file_offset, SEEK_SET) < 0 ||
       ! ico_read_int32 (fp, &first_bytes, 1))
@@ -636,10 +636,8 @@ ico_load_layer (FILE        *fp,
     }
 
   /* read successfully. add to image */
-  g_snprintf (name, sizeof (name), _("Icon #%i"), icon_num+1);
-  layer = gimp_layer_new (image, name, width, height,
-                          GIMP_RGBA_IMAGE,
-                          100,
+  layer = gimp_layer_new (image, layer_prefix, width, height,
+                          GIMP_RGBA_IMAGE, 100,
                           gimp_image_get_default_new_layer_mode (image));
   gimp_image_insert_layer (image, layer, NULL, icon_num);
 
@@ -657,6 +655,7 @@ ico_load_layer (FILE        *fp,
 GimpImage *
 ico_load_image (GFile        *file,
                 gint32       *file_offset,
+                gint          frame_num,
                 GError      **error)
 {
   FILE          *fp;
@@ -721,16 +720,42 @@ ico_load_image (GFile        *file,
   D(("image size: %ix%i\n", max_width, max_height));
 
   image = gimp_image_new (max_width, max_height, GIMP_RGB);
-  if (! file_offset)
-    gimp_image_set_file (image, file);
 
   maxsize = max_width * max_height * 4;
   buf = g_new (guchar, max_width * max_height * 4);
   for (i = 0; i < icon_count; i++)
     {
       GimpLayer *layer;
+      gchar     *layer_prefix;
+      gchar     *icon_metadata;
 
-      layer = ico_load_layer (fp, image, i, buf, maxsize, file_offset ? *file_offset : 0, info+i);
+      if (info[i].bpp)
+        icon_metadata = g_strdup_printf ("(%dx%d, %dbpp)", info[i].width,
+                                         info[i].height, info[i].bpp);
+      else
+        icon_metadata = g_strdup_printf ("(%dx%d)", info[i].width,
+                                         info[i].height);
+
+      if (frame_num > -1)
+        {
+          layer_prefix = g_strdup_printf ("Cursor %s Frame #%i", icon_metadata,
+                                          frame_num);
+        }
+      else
+        {
+          if (header.resource_type == 1)
+            layer_prefix = g_strdup_printf ("Icon #%i %s ", i + 1,
+                                            icon_metadata);
+          else
+            layer_prefix = g_strdup_printf ("Cursor #%i %s ", i + 1,
+                                            icon_metadata);
+        }
+
+      layer = ico_load_layer (fp, image, i + 1, buf, maxsize,
+                              file_offset ? *file_offset : 0,
+                              layer_prefix, info + i);
+      g_free (icon_metadata);
+      g_free (layer_prefix);
 
       /* Save CUR hot spot information */
       if (header.resource_type == 2)
@@ -777,10 +802,10 @@ ani_load_image (GFile   *file,
   guint32       size;
   guint8        padding;
   gint32        file_offset;
-  gint          frame = 1;
+  guint         frame = 1;
   AniFileHeader header;
-  gchar         inam[G_MAXSHORT] = {0};
-  gchar         iart[G_MAXSHORT] = {0};
+  gchar        *inam  = NULL;
+  gchar        *iart  = NULL;
   gchar        *str;
 
   gimp_progress_init_printf (_("Opening '%s'"),
@@ -823,9 +848,26 @@ ani_load_image (GFile   *file,
         }
       else if (memcmp (id, "INAM", 4) == 0)
         {
+          gint n_read = -1;
+
           fread (&size, sizeof (size), 1, fp);
-          fread (&inam, sizeof (char), size, fp);
-          inam[size] = '\0';
+          if (size > 0)
+            {
+              if (inam)
+                g_free (inam);
+
+              inam = g_new0 (gchar, size + 1);
+              n_read = fread (inam, sizeof (gchar), size, fp);
+              inam[size] = '\0';
+            }
+
+          if (n_read < 1 || (inam && ! g_utf8_validate (inam, -1, NULL)))
+            {
+              fclose (fp);
+              g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
+                           _("Invalid ANI metadata"));
+              return NULL;
+            }
 
           /* Metadata length must be even. If data itself is odd,
            * then an extra 0x00 is added for padding. We read in
@@ -837,9 +879,26 @@ ani_load_image (GFile   *file,
         }
       else if (memcmp (id, "IART", 4) == 0)
         {
+          gint n_read = -1;
+
           fread (&size, sizeof (size), 1, fp);
-          fread (&iart, sizeof (char), size, fp);
-          iart[size] = '\0';
+          if (size > 0)
+            {
+              if (iart)
+                g_free (iart);
+
+              iart = g_new0 (gchar, size + 1);
+              n_read = fread (iart, sizeof (gchar), size, fp);
+              iart[size] = '\0';
+            }
+
+          if (n_read < 1 || (iart && ! g_utf8_validate (iart, -1, NULL)))
+            {
+              fclose (fp);
+              g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
+                           _("Invalid ANI metadata"));
+              return NULL;
+            }
 
           if (size % 2 != 0)
             fread (&padding, sizeof (padding), 1, fp);
@@ -857,7 +916,7 @@ ani_load_image (GFile   *file,
              {
                if (! image)
                  {
-                   image = ico_load_image (file, &file_offset, error);
+                   image = ico_load_image (file, &file_offset, 1, error);
                  }
                else
                  {
@@ -866,7 +925,8 @@ ani_load_image (GFile   *file,
                    GimpLayer    *new_layer;
                    gint          nlayers;
 
-                   temp_image = ico_load_image (file, &file_offset, error);
+                   temp_image = ico_load_image (file, &file_offset, frame + 1,
+                                                error);
                    layers = gimp_image_get_layers (temp_image, &nlayers);
                    if (layers)
                      {
@@ -901,29 +961,30 @@ ani_load_image (GFile   *file,
   gimp_parasite_free (parasite);
 
   /* Saving INFO block */
-  if (strlen (inam) > 0)
+  if (inam && strlen (inam) > 0)
     {
       str = g_strdup_printf ("%s", inam);
       parasite = gimp_parasite_new ("ani-info-inam",
                                     GIMP_PARASITE_PERSISTENT,
                                     strlen (str) + 1, (gpointer) str);
       g_free (str);
+      g_free (inam);
       gimp_image_attach_parasite (image, parasite);
       gimp_parasite_free (parasite);
     }
 
-  if (strlen (iart) > 0)
+  if (iart && strlen (iart) > 0)
     {
       str = g_strdup_printf ("%s", iart);
       parasite = gimp_parasite_new ("ani-info-iart",
                                     GIMP_PARASITE_PERSISTENT,
                                     strlen (str) + 1, (gpointer) str);
       g_free (str);
+      g_free (iart);
       gimp_image_attach_parasite (image, parasite);
       gimp_parasite_free (parasite);
     }
 
-  gimp_image_set_file (image, file);
   gimp_progress_update (1.0);
 
   return image;
@@ -1008,7 +1069,8 @@ ico_load_thumbnail_image (GFile   *file,
 
   image = gimp_image_new (w, h, GIMP_RGB);
   buf = g_new (guchar, w*h*4);
-  ico_load_layer (fp, image, match, buf, w*h*4, file_offset, info+match);
+  ico_load_layer (fp, image, match, buf, w*h*4, file_offset,
+                  "Thumbnail", info + match);
   g_free (buf);
 
   *width  = w;
