@@ -254,7 +254,7 @@ ico_save_init (GimpImage   *image,
     {
       num_colors = ico_get_layer_num_colors (iter->data, &uses_alpha_values);
 
-      if (!uses_alpha_values)
+      if (! uses_alpha_values)
         {
           if (num_colors <= 2)
             {
@@ -648,6 +648,15 @@ ico_get_layer_num_colors (GimpLayer *layer,
 
   num_colors = g_hash_table_size (hash);
 
+  /* Icons with 1 bit masks are converted to
+   * full transparency when loaded in GIMP.
+   * If the layer is not semi-transparent, we
+   * can subtract the "mask color" out to fix
+   * the default depth value for 1/4/8 bpp icons.
+   */
+  if (! *uses_alpha_levels)
+    num_colors--;
+
   g_hash_table_destroy (hash);
 
   g_free (colors);
@@ -737,7 +746,7 @@ ico_image_get_reduced_buf (GimpDrawable *layer,
           guchar *cmap;
           gint    num_colors;
 
-          cmap = gimp_image_get_colormap (image, &num_colors);
+          cmap = gimp_image_get_colormap (image, NULL, &num_colors);
           gimp_image_set_colormap (tmp_image, cmap, num_colors);
           g_free (cmap);
         }
@@ -768,7 +777,7 @@ ico_image_get_reduced_buf (GimpDrawable *layer,
                                       GIMP_CONVERT_PALETTE_GENERATE,
                                       1 << bpp, TRUE, FALSE, "dummy");
 
-          cmap = gimp_image_get_colormap (tmp_image, num_colors);
+          cmap = gimp_image_get_colormap (tmp_image, NULL, num_colors);
 
           if (*num_colors == (1 << bpp) &&
               ! ico_cmap_contains_black (cmap, *num_colors))
@@ -780,7 +789,7 @@ ico_image_get_reduced_buf (GimpDrawable *layer,
               if (gimp_drawable_is_indexed (layer))
                 {
                   g_free (cmap);
-                  cmap = gimp_image_get_colormap (image, num_colors);
+                  cmap = gimp_image_get_colormap (image, NULL, num_colors);
                   gimp_image_set_colormap (tmp_image, cmap, *num_colors);
                 }
               else if (gimp_drawable_is_gray (layer))
@@ -807,7 +816,7 @@ ico_image_get_reduced_buf (GimpDrawable *layer,
                                           GIMP_CONVERT_PALETTE_GENERATE,
                                           (1<<bpp) - 1, TRUE, FALSE, "dummy");
               g_free (cmap);
-              cmap = gimp_image_get_colormap (tmp_image, num_colors);
+              cmap = gimp_image_get_colormap (tmp_image, NULL, num_colors);
             }
 
           gimp_image_convert_rgb (tmp_image);
