@@ -181,8 +181,8 @@ ico_create_procedure (GimpPlugIn  *plug_in,
                                           "image/x-ico");
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "ico");
-      gimp_file_procedure_set_magics (GIMP_FILE_PROCEDURE (procedure),
-                                      "0,string,\\0\\0\\1\\0");
+      /* We do not set magics here, since that interferes with certain types
+         of TGA images. */
 
       gimp_load_procedure_set_thumbnail_loader (GIMP_LOAD_PROCEDURE (procedure),
                                                 LOAD_THUMB_PROC);
@@ -211,8 +211,8 @@ ico_create_procedure (GimpPlugIn  *plug_in,
                                           "image/vnd.microsoft.icon");
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "cur");
-      gimp_file_procedure_set_magics (GIMP_FILE_PROCEDURE (procedure),
-                                      "0,string,\\0\\0\\2\\0");
+      /* We do not set magics here, since that interferes with certain types
+         of TGA images. */
 
       gimp_load_procedure_set_thumbnail_loader (GIMP_LOAD_PROCEDURE (procedure),
                                                 LOAD_THUMB_PROC);
@@ -435,7 +435,7 @@ ico_load (GimpProcedure        *procedure,
 
   gegl_init (NULL, NULL);
 
-  image = ico_load_image (file, NULL, &error);
+  image = ico_load_image (file, NULL, -1, &error);
 
   if (! image)
     return gimp_procedure_new_return_values (procedure,
@@ -502,7 +502,7 @@ ico_load_thumb (GimpProcedure        *procedure,
   image = ico_load_thumbnail_image (file,
                                     &width, &height, 0, &error);
 
-  if (image)
+  if (! image)
     return gimp_procedure_new_return_values (procedure,
                                              GIMP_PDB_EXECUTION_ERROR,
                                              error);
@@ -541,7 +541,7 @@ ani_load_thumb (GimpProcedure        *procedure,
   image = ani_load_image (file, TRUE,
                           &width, &height, &error);
 
-  if (image)
+  if (! image)
     return gimp_procedure_new_return_values (procedure,
                                              GIMP_PDB_EXECUTION_ERROR,
                                              error);
