@@ -224,7 +224,7 @@ jpeg_create_procedure (GimpPlugIn  *plug_in,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "cmyk",
-                             "Export as _CMYK",
+                             "Export as CM_YK",
                              "Create a CMYK JPEG image using the soft-proofing color profile",
                              FALSE,
                              G_PARAM_READWRITE);
@@ -232,9 +232,9 @@ jpeg_create_procedure (GimpPlugIn  *plug_in,
       GIMP_PROC_ARG_INT (procedure, "sub-sampling",
                          _("Su_bsampling"),
                          "Sub-sampling type { 0 == 4:2:0 (chroma quartered), "
-                         "1 == 4:2:2 Horizontal (chroma halved), "
+                         "1 == 4:2:2 (chroma halved horizontally), "
                          "2 == 4:4:4 (best quality), "
-                         "3 == 4:2:2 Vertical (chroma halved)",
+                         "3 == 4:4:0 (chroma halved vertically)",
                          JPEG_SUBSAMPLING_2x2_1x1_1x1,
                          JPEG_SUBSAMPLING_1x2_1x1_1x1,
                          JPEG_SUBSAMPLING_1x1_1x1_1x1,
@@ -322,8 +322,9 @@ jpeg_load (GimpProcedure        *procedure,
 {
   GimpValueArray *return_vals;
   GimpImage      *image;
-  gboolean        resolution_loaded = FALSE;
-  GError         *error             = NULL;
+  gboolean        resolution_loaded  = FALSE;
+  gboolean        ps_metadata_loaded = FALSE;
+  GError         *error              = NULL;
 
   gegl_init (NULL, NULL);
 
@@ -342,7 +343,7 @@ jpeg_load (GimpProcedure        *procedure,
     }
 
   image = load_image (file, run_mode, FALSE,
-                      &resolution_loaded, &error);
+                      &resolution_loaded, &ps_metadata_loaded, &error);
 
   if (image)
     {
@@ -464,20 +465,8 @@ jpeg_save (GimpProcedure        *procedure,
       switch (export)
         {
         case GIMP_EXPORT_EXPORT:
-          {
-            gchar *tmp = g_filename_from_utf8 (_("Export Preview"), -1,
-                                               NULL, NULL, NULL);
-            if (tmp)
-              {
-                GFile *file = g_file_new_for_path (tmp);
-                gimp_image_set_file (image, file);
-                g_object_unref (file);
-                g_free (tmp);
-              }
-
-            display = NULL;
-            separate_display = TRUE;
-          }
+          display = NULL;
+          separate_display = TRUE;
           break;
 
         case GIMP_EXPORT_IGNORE:
