@@ -243,6 +243,11 @@ static gint     load_resource_2000     (const PSDimageres     *res_a,
                                         GInputStream          *input,
                                         GError               **error);
 
+static gint     load_resource_2999     (const PSDimageres     *res_a,
+                                        GimpImage             *image,
+                                        GInputStream          *input,
+                                        GError               **error);
+
 /* Public Functions */
 gint
 get_image_resource_header (PSDimageres   *res_a,
@@ -410,6 +415,16 @@ load_image_resource (PSDimageres  *res_a,
             load_resource_1077 (res_a, image, img_a, input, error);
             break;
 
+          case PSD_CLIPPING_PATH:
+            load_resource_2999 (res_a, image, input, error);
+            break;
+
+          case PSD_LAYER_COMPS:
+            img_a->unsupported_features->layer_comp = TRUE;
+            img_a->unsupported_features->show_gui = TRUE;
+            load_resource_unknown (res_a, image, input, error);
+            break;
+
           default:
             if (res_a->id >= 2000 &&
                 res_a->id <  2999)
@@ -1215,7 +1230,8 @@ load_resource_1045 (const PSDimageres  *res_a,
   block_rem = res_a->data_len;
   while (block_rem > 1)
     {
-      str = fread_unicode_string (&read_len, &write_len, 1, input, error);
+      str = fread_unicode_string (&read_len, &write_len, 1, FALSE, input,
+                                  error);
       if (*error)
         return -1;
 
@@ -1254,7 +1270,7 @@ load_resource_1046 (const PSDimageres  *res_a,
   /* FIXME - check that we have indexed image */
   if (index_count && index_count < 256)
     {
-      cmap = gimp_image_get_colormap (image, &cmap_count);
+      cmap = gimp_image_get_colormap (image, NULL, &cmap_count);
       if (cmap && index_count < cmap_count)
         gimp_image_set_colormap (image, cmap, index_count);
       g_free (cmap);
@@ -1685,3 +1701,59 @@ load_resource_2000 (const PSDimageres  *res_a,
 
   return 0;
 }
+
+static gint
+load_resource_2999 (const PSDimageres  *res_a,
+                    GimpImage          *image,
+                    GInputStream       *input,
+                    GError            **error)
+{
+  gchar        *path_name;
+  gint16        path_flatness_int;
+  gint16        path_flatness_fixed;
+  gfloat        path_flatness;
+  GimpParasite *parasite;
+  gint32        read_len;
+  gint32        write_len;
+
+  path_name = fread_pascal_string (&read_len, &write_len, 2, input, error);
+  if (*error)
+    return -1;
+
+  /* Convert from fixed to floating point */
+  if (psd_read (input, &path_flatness_int, 1, error) < 1 ||
+      psd_read (input, &path_flatness_fixed, 1, error) < 1)
+    {
+      psd_set_error (error);
+      return -1;
+    }
+  path_flatness_fixed = GINT16_FROM_BE (path_flatness_fixed);
+  /* Converting from Adobe fixed point value to float */
+  path_flatness = (path_flatness_fixed - 0.5f) / 65536.0;
+  path_flatness += path_flatness_int;
+
+  /* Adobe path flatness range is 0.2 to 100.0 */
+  path_flatness = CLAMP (path_flatness, 0.2f, 100.0f);
+
+  /* Save to image parasite */
+  parasite = gimp_parasite_new (PSD_PARASITE_CLIPPING_PATH, 0,
+                                read_len, path_name);
+  gimp_image_attach_parasite (image, parasite);
+  gimp_parasite_free (parasite);
+
+  parasite = gimp_parasite_new (PSD_PARASITE_PATH_FLATNESS, 0,
+                                sizeof (gfloat), &path_flatness);
+  gimp_image_attach_parasite (image, parasite);
+  gimp_parasite_free (parasite);
+
+  /* Adobe says they ignore the last two bytes, the fill rule */
+  if (psd_read (input, &path_flatness_fixed, 1, error) < 1)
+    {
+      psd_set_error (error);
+      return -1;
+    }
+
+  g_free (path_name);
+
+  return 0;
+}
