@@ -74,6 +74,11 @@ static GimpValueArray * psd_save             (GimpProcedure        *procedure,
                                               GFile                *file,
                                               const GimpValueArray *args,
                                               gpointer              run_data);
+static GimpValueArray * psd_load_metadata    (GimpProcedure        *procedure,
+                                              GimpRunMode           run_mode,
+                                              GFile                *file,
+                                              const GimpValueArray *args,
+                                              gpointer              run_data);
 
 
 G_DEFINE_TYPE (Psd, psd, GIMP_TYPE_PLUG_IN)
@@ -106,6 +111,7 @@ psd_query_procedures (GimpPlugIn *plug_in)
   list = g_list_append (list, g_strdup (LOAD_PROC));
   list = g_list_append (list, g_strdup (LOAD_MERGED_PROC));
   list = g_list_append (list, g_strdup (SAVE_PROC));
+  list = g_list_append (list, g_strdup (LOAD_METADATA_PROC));
 
   return list;
 }
@@ -224,10 +230,77 @@ psd_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "psd");
 
+      GIMP_PROC_ARG_BOOLEAN (procedure, "clippingpath",
+                             _("Assign a Clipping _Path"),
+                             _("Select a path to be the "
+                             "clipping path"),
+                             FALSE,
+                             G_PARAM_READWRITE);
+
+      GIMP_PROC_ARG_STRING (procedure, "clippingpathname",
+                            _("Clipping Path _Name"),
+                            _("Clipping path name\n"
+                            "(ignored if no clipping path)"),
+                            NULL,
+                            G_PARAM_READWRITE);
+
+      GIMP_PROC_ARG_DOUBLE (procedure, "clippingpathflatness",
+                            _("Path _Flatness"),
+                            _("Clipping path flatness in device pixels\n"
+                            "(ignored if no clipping path)"),
+                            0.0, 100.0, 0.2,
+                            G_PARAM_READWRITE);
+
+      GIMP_PROC_ARG_BOOLEAN (procedure, "cmyk",
+                             _("Export as _CMYK"),
+                             _("Export a CMYK PSD image using the soft-proofing color profile"),
+                             FALSE,
+                             G_PARAM_READWRITE);
+
       GIMP_PROC_ARG_BOOLEAN (procedure, "duotone",
-                             "Export as _Duotone",
-                             "Export as a Duotone PSD file if Duotone color space information "
-                             "was attached to the image when originally imported.",
+                             _("Export as _Duotone"),
+                             _("Export as a Duotone PSD file if Duotone color space information "
+                             "was attached to the image when originally imported."),
+                             FALSE,
+                             G_PARAM_READWRITE);
+    }
+  else if (! strcmp (name, LOAD_METADATA_PROC))
+    {
+      procedure = gimp_load_procedure_new (plug_in, name,
+                                           GIMP_PDB_PROC_TYPE_PLUGIN,
+                                           psd_load_metadata, NULL, NULL);
+
+      gimp_procedure_set_documentation (procedure,
+                                        "Loads Photoshop-format metadata "
+                                        "from other file formats.",
+                                        "Loads Photoshop-format metadata "
+                                        "from other file formats.",
+                                        name);
+      gimp_procedure_set_attribution (procedure,
+                                      "John Marshall",
+                                      "John Marshall",
+                                      "2007");
+      GIMP_PROC_ARG_INT (procedure, "size",
+                         "Metadata size",
+                         NULL,
+                         0, G_MAXINT, 0,
+                         G_PARAM_READWRITE);
+      gimp_procedure_add_argument (procedure,
+                                   gimp_param_spec_image ("image",
+                                                          "image",
+                                                          "The image",
+                                                          FALSE,
+                                                          GIMP_PARAM_READWRITE));
+      GIMP_PROC_ARG_BOOLEAN (procedure, "metadata-type",
+                             "Metadata type",
+                             "If the metadata contains image or "
+                             "layer PSD resources.",
+                             FALSE,
+                             G_PARAM_READWRITE);
+      GIMP_PROC_ARG_BOOLEAN (procedure, "cmyk",
+                             "CMYK",
+                             "If the layer metadata needs to be "
+                             "converted from CMYK colorspace.",
                              FALSE,
                              G_PARAM_READWRITE);
     }
@@ -249,6 +322,7 @@ psd_load (GimpProcedure        *procedure,
   GimpMetadata   *metadata;
   GimpParasite   *parasite = NULL;
   GError         *error = NULL;
+  PSDSupport      unsupported_features;
 
   gegl_init (NULL, NULL);
 
@@ -267,6 +341,7 @@ psd_load (GimpProcedure        *procedure,
                               LOAD_MERGED_PROC) == 0,
                       &resolution_loaded,
                       &profile_loaded,
+                      &unsupported_features,
                       &error);
 
   if (! image)
@@ -279,10 +354,14 @@ psd_load (GimpProcedure        *procedure,
     {
       parasite = gimp_image_get_parasite (image, PSD_PARASITE_DUOTONE_DATA);
       if (parasite)
-        {
-          load_dialog ();
-          gimp_parasite_free (parasite);
-        }
+        unsupported_features.duotone_mode = TRUE;
+
+      if (unsupported_features.duotone_mode ||
+          unsupported_features.show_gui)
+        load_dialog (_("Import PSD"), &unsupported_features);
+
+      if (parasite)
+        gimp_parasite_free (parasite);
     }
 
   metadata = gimp_image_metadata_load_prepare (image, "image/x-psd",
@@ -360,17 +439,14 @@ psd_save (GimpProcedure        *procedure,
   GimpProcedureConfig   *config;
   GimpPDBStatusType      status = GIMP_PDB_SUCCESS;
   GimpMetadata          *metadata;
-  GimpMetadataSaveFlags  metadata_flags;
   GimpExportReturn       export = GIMP_EXPORT_IGNORE;
-  GimpParasite          *parasite = NULL;
   GError                *error = NULL;
 
   gegl_init (NULL, NULL);
 
   config = gimp_procedure_create_config (procedure);
-  metadata = gimp_image_metadata_save_prepare (image,
-                                               "image/x-psd",
-                                               &metadata_flags);
+  metadata = gimp_procedure_config_begin_export (config, image, run_mode,
+                                                 args, "image/x-psd");
 
   switch (run_mode)
     {
@@ -397,47 +473,111 @@ psd_save (GimpProcedure        *procedure,
 
   if (run_mode == GIMP_RUN_INTERACTIVE)
     {
-      /* Only show dialog if image is grayscale and duotone color space
-       * information was attached from the original image imported
-       */
-      parasite = gimp_image_get_parasite (image, PSD_PARASITE_DUOTONE_DATA);
-      if (parasite)
-        {
-          if (gimp_image_get_base_type (image) == GIMP_GRAY)
-            {
-              if (! save_dialog (image, procedure, G_OBJECT (config)))
-                return gimp_procedure_new_return_values (procedure, GIMP_PDB_CANCEL,
-                                                         NULL);
-            }
-          gimp_parasite_free (parasite);
-        }
+      if (! save_dialog (image, procedure, G_OBJECT (config)))
+        return gimp_procedure_new_return_values (procedure, GIMP_PDB_CANCEL,
+                                                 NULL);
     }
 
   if (save_image (file, image, G_OBJECT (config), &error))
     {
       if (metadata)
-        {
-          gimp_metadata_set_bits_per_sample (metadata, 8);
-
-          gimp_image_metadata_save_finish (image,
-                                           "image/x-psd",
-                                           metadata, metadata_flags,
-                                           file, NULL);
-        }
+        gimp_metadata_set_bits_per_sample (metadata, 8);
     }
   else
     {
       status = GIMP_PDB_EXECUTION_ERROR;
     }
 
+  gimp_procedure_config_end_export (config, image, file, status);
+  g_object_unref (config);
+
   if (export == GIMP_EXPORT_EXPORT)
     {
       gimp_image_delete (image);
       g_free (drawables);
     }
 
-  if (metadata)
-    g_object_unref (metadata);
-
   return gimp_procedure_new_return_values (procedure, status, error);
 }
+
+static GimpValueArray *
+psd_load_metadata (GimpProcedure        *procedure,
+                   GimpRunMode           run_mode,
+                   GFile                *file,
+                   const GimpValueArray *args,
+                   gpointer              run_data)
+{
+  GimpValueArray *return_vals;
+  GimpImage      *image;
+  gint            data_length;
+  PSDSupport      unsupported_features;
+  gboolean        is_layer = FALSE;
+  gboolean        is_cmyk  = FALSE;
+  GError         *error    = NULL;
+
+  gegl_init (NULL, NULL);
+
+  /* Retrieve image */
+  if (gimp_value_array_length (args) > 1)
+    {
+      data_length = g_value_get_int (gimp_value_array_index (args, 0));
+      image       = g_value_get_object (gimp_value_array_index (args, 1));
+    }
+  else
+    {
+      return gimp_procedure_new_return_values (procedure,
+                                               GIMP_PDB_EXECUTION_ERROR,
+                                               error);
+    }
+
+  if (gimp_value_array_length (args) > 2)
+    is_layer = g_value_get_boolean (gimp_value_array_index (args, 2));
+  if (gimp_value_array_length (args) > 3)
+    is_cmyk = g_value_get_boolean (gimp_value_array_index (args, 3));
+
+  image = load_image_metadata (file, data_length, image, is_layer, is_cmyk,
+                               &unsupported_features, &error);
+
+  /* Check for unsupported layers */
+
+  if (is_layer && unsupported_features.show_gui)
+    {
+      /* Metadata doesn't store rasterized versions of fill layers,
+       * (unlike PSDs) so we can't display them for now.
+       */
+      if (unsupported_features.fill_layer ||
+          unsupported_features.text_layer)
+        {
+          unsupported_features.psd_metadata = TRUE;
+          unsupported_features.fill_layer   = FALSE;
+          unsupported_features.text_layer   = FALSE;
+        }
+
+      switch (run_mode)
+        {
+        case GIMP_RUN_INTERACTIVE:
+          gimp_ui_init (PLUG_IN_BINARY);
+          load_dialog (_("Import PSD metadata"), &unsupported_features);
+          break;
+        default:
+          g_printerr ("[%s] %s\n", "file-psd-load-metadata",
+                      _("Metadata fill layers are not supported "
+                        "and will be dropped."));
+          break;
+        }
+
+    }
+
+  if (! image)
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_EXECUTION_ERROR,
+                                             error);
+
+  return_vals = gimp_procedure_new_return_values (procedure,
+                                                  GIMP_PDB_SUCCESS,
+                                                  NULL);
+
+  GIMP_VALUES_SET_IMAGE (return_vals, 1, image);
+
+  return return_vals;
+}
