@@ -35,12 +35,15 @@
 #define LOAD_MERGED_PROC                "file-psd-load-merged"
 #define LOAD_THUMB_PROC                 "file-psd-load-thumb"
 #define SAVE_PROC                       "file-psd-save"
+#define LOAD_METADATA_PROC              "file-psd-load-metadata"
 #define PLUG_IN_BINARY                  "file-psd"
 #define PLUG_IN_ROLE                    "gimp-file-psd"
 
 #define GIMP_PARASITE_COMMENT           "gimp-comment"
 
 #define PSD_PARASITE_DUOTONE_DATA       "psd-duotone-data"
+#define PSD_PARASITE_CLIPPING_PATH      "psd-clipping-path"
+#define PSD_PARASITE_PATH_FLATNESS      "psd-path-flatness"
 
 /* Copied from app/base/gimpimage-quick-mask.h - internal identifier for quick mask channel */
 #define GIMP_IMAGE_QUICK_MASK_NAME      "Qmask"
@@ -135,13 +138,15 @@
 #define PSD_LFX_BEVEL           "bevl"          /* Effects layer - bevel (PS5) */
 
 /* Placed Layer */
-#define PSD_LPL_PLACE_LAYER     "plLd"          /* Placed layer (?) */
+#define PSD_LPL_PLACE_LAYER     "PlLd"          /* Placed layer (?) (based on PSD files, not specification) */
 #define PSD_LPL_PLACE_LAYER_NEW "SoLd"          /* Placed layer (PS10) */
+#define PSD_SMART_OBJECT_LAYER  "SoLE"          /* Smart Object Layer (CC2015) */
 
 /* Linked Layer */
-#define PSD_LLL_LINKED_LAYER    "lnkD"          /* Linked layer (?) */
-#define PSD_LLL_LINKED_LAYER_2  "lnk2"          /* Linked layer 2nd key */
-#define PSD_LLL_LINKED_LAYER_3  "lnk3"          /* Linked layer 3rd key */
+#define PSD_LLL_LINKED_LAYER     "lnkD"         /* Linked layer (?) */
+#define PSD_LLL_LINKED_LAYER_2   "lnk2"         /* Linked layer 2nd key */
+#define PSD_LLL_LINKED_LAYER_3   "lnk3"         /* Linked layer 3rd key */
+#define PSD_LLL_LINKED_LAYER_EXT "lnkE"         /* Linked layer external */
 
 /* Merged Transparency */
 #define PSD_LMT_MERGE_TRANS     "Mtrn"          /* Merged transparency save flag (?) */
@@ -575,6 +580,24 @@ typedef struct
   gchar                 *info; /* Text information */
 } PSDText;
 
+/* Partially or Unsupported Features */
+typedef struct
+{
+  gboolean show_gui;
+  gboolean duotone_mode;
+
+  gboolean adjustment_layer;
+  gboolean fill_layer;
+  gboolean text_layer;
+  gboolean linked_layer;
+  gboolean vector_mask;
+  gboolean smart_object;
+  gboolean stroke;
+  gboolean layer_effect;
+  gboolean layer_comp;
+  gboolean psd_metadata;
+} PSDSupport;
+
 /* PSD Layer data structure */
 typedef struct
 {
@@ -602,6 +625,8 @@ typedef struct
   guint32               id;                     /* Layer ID (Tattoo) */
   guchar                group_type;             /* 0 -> not a group; 1 -> open folder; 2 -> closed folder; 3 -> end of group */
   guint16               color_tag[4];           /* 4 * 16 bit color components */
+
+  PSDSupport           *unsupported_features;
 } PSDlayer;
 
 /* PSD Channel data structure */
@@ -642,6 +667,7 @@ typedef struct
   gchar         key[4];                 /* Layer resource key */
   guint64       data_start;             /* Layer resource data start */
   guint64       data_len;               /* Layer resource data length */
+  gboolean      ibm_pc_format;          /* If layers are saved in little endian format */
 } PSDlayerres;
 
 /* PSD File data structures */
@@ -683,6 +709,9 @@ typedef struct
   GimpColorProfile     *cmyk_profile;
   gpointer              cmyk_transform;
   gpointer              cmyk_transform_alpha;
+
+  gboolean              ibm_pc_format;          /* If layers are saved in little endian format */
+  PSDSupport           *unsupported_features;
 } PSDimage;
 
 /* Public functions */
