@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -77,12 +76,16 @@ static void      print_preview_finalize             (GObject          *object);
 
 static void      print_preview_realize              (GtkWidget        *widget);
 static void      print_preview_unrealize            (GtkWidget        *widget);
-static void      print_preview_size_request         (GtkWidget        *widget,
-                                                     GtkRequisition   *requisition);
+static void      print_preview_get_preferred_width  (GtkWidget        *widget,
+                                                     gint             *minimum_width,
+                                                     gint             *natural_width);
+static void      print_preview_get_preferred_height (GtkWidget        *widget,
+                                                     gint             *minimum_height,
+                                                     gint             *natural_height);
 static void      print_preview_size_allocate        (GtkWidget        *widget,
                                                      GtkAllocation    *allocation);
-static gboolean  print_preview_expose_event         (GtkWidget        *widget,
-                                                     GdkEventExpose   *event);
+static gboolean  print_preview_draw                 (GtkWidget        *widget,
+                                                     cairo_t          *cr);
 static gboolean  print_preview_button_press_event   (GtkWidget        *widget,
                                                      GdkEventButton   *event);
 static gboolean  print_preview_button_release_event (GtkWidget        *widget,
@@ -181,9 +184,10 @@ print_preview_class_init (PrintPreviewClass *klass)
 
   widget_class->realize              = print_preview_realize;
   widget_class->unrealize            = print_preview_unrealize;
-  widget_class->size_request         = print_preview_size_request;
+  widget_class->get_preferred_width  = print_preview_get_preferred_width;
+  widget_class->get_preferred_height = print_preview_get_preferred_height;
   widget_class->size_allocate        = print_preview_size_allocate;
-  widget_class->expose_event         = print_preview_expose_event;
+  widget_class->draw                 = print_preview_draw;
   widget_class->button_press_event   = print_preview_button_press_event;
   widget_class->button_release_event = print_preview_button_release_event;
   widget_class->motion_notify_event  = print_preview_motion_notify_event;
@@ -209,12 +213,6 @@ print_preview_finalize (GObject *object)
 {
   PrintPreview *preview = PRINT_PREVIEW (object);
 
-  if (preview->drawable)
-    {
-      gimp_drawable_detach (preview->drawable);
-      preview->drawable = NULL;
-    }
-
   if (preview->thumbnail)
     {
       cairo_surface_destroy (preview->thumbnail);
@@ -247,7 +245,7 @@ print_preview_unrealize (GtkWidget *widget)
   PrintPreview *preview = PRINT_PREVIEW (widget);
 
   if (preview->cursor)
-    gdk_cursor_unref (preview->cursor);
+    g_object_unref (preview->cursor);
 
   GTK_WIDGET_CLASS (print_preview_parent_class)->unrealize (widget);
 }
@@ -259,7 +257,9 @@ print_preview_size_request (GtkWidget      *widget,
   PrintPreview *preview = PRINT_PREVIEW (widget);
   gdouble       paper_width;
   gdouble       paper_height;
-  gint          border  = GTK_CONTAINER (widget)->border_width + 1;
+  gint          border;
+
+  border = gtk_container_get_border_width (GTK_CONTAINER (widget)) + 1;
 
   print_preview_get_page_size (preview, &paper_width, &paper_height);
 
@@ -280,6 +280,30 @@ print_preview_size_request (GtkWidget      *widget,
   requisition->height += 2 * border;
 }
 
+static void
+print_preview_get_preferred_width (GtkWidget *widget,
+                                   gint      *minimum_width,
+                                   gint      *natural_width)
+{
+  GtkRequisition requisition;
+
+  print_preview_size_request (widget, &requisition);
+
+  *minimum_width = *natural_width = requisition.width;
+}
+
+static void
+print_preview_get_preferred_height (GtkWidget *widget,
+                                    gint      *minimum_height,
+                                    gint      *natural_height)
+{
+  GtkRequisition requisition;
+
+  print_preview_size_request (widget, &requisition);
+
+  *minimum_height = *natural_height = requisition.height;
+}
+
 static void
 print_preview_size_allocate (GtkWidget     *widget,
                              GtkAllocation *allocation)
@@ -304,15 +328,18 @@ print_preview_button_press_event (GtkWidget      *widget,
 
   if (event->type == GDK_BUTTON_PRESS && event->button == 1 && preview->inside)
     {
-      GdkCursor *cursor;
+      GdkDisplay *display = gtk_widget_get_display (widget);
+      GdkSeat    *seat    = gdk_display_get_default_seat (display);
+      GdkCursor  *cursor;
 
       cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
                                            GDK_FLEUR);
 
-      if (gdk_pointer_grab (event->window, FALSE,
-                            (GDK_BUTTON1_MOTION_MASK |
-                             GDK_BUTTON_RELEASE_MASK),
-                            NULL, cursor, event->time) == GDK_GRAB_SUCCESS)
+      if (gdk_seat_grab (seat, gdk_event_get_window ((GdkEvent *) event),
+                         GDK_SEAT_CAPABILITY_ALL_POINTING, FALSE,
+                         cursor,
+                         (GdkEvent *) event,
+                         NULL, NULL) == GDK_GRAB_SUCCESS)
         {
           preview->orig_offset_x = preview->image_offset_x;
           preview->orig_offset_y = preview->image_offset_y;
@@ -323,7 +350,7 @@ print_preview_button_press_event (GtkWidget      *widget,
           preview->dragging = TRUE;
         }
 
-      gdk_cursor_unref (cursor);
+      g_object_unref (cursor);
     }
 
   return FALSE;
@@ -337,8 +364,11 @@ print_preview_button_release_event (GtkWidget      *widget,
 
   if (preview->dragging)
     {
-      gdk_display_pointer_ungrab (gtk_widget_get_display (widget),
-                                  event->time);
+      GdkDisplay *display = gtk_widget_get_display (widget);
+      GdkSeat    *seat    = gdk_display_get_default_seat (display);
+
+      gdk_seat_ungrab (seat);
+
       preview->dragging = FALSE;
 
       print_preview_set_inside (preview,
@@ -403,20 +433,24 @@ print_preview_leave_notify_event (GtkWidget        *widget,
 }
 
 static gboolean
-print_preview_expose_event (GtkWidget      *widget,
-                            GdkEventExpose *event)
+print_preview_draw (GtkWidget *widget,
+                    cairo_t   *cr)
 {
-  PrintPreview *preview = PRINT_PREVIEW (widget);
-  GtkStyle     *style   = gtk_widget_get_style (widget);
-  cairo_t      *cr;
-  gdouble       paper_width;
-  gdouble       paper_height;
-  gdouble       left_margin;
-  gdouble       right_margin;
-  gdouble       top_margin;
-  gdouble       bottom_margin;
-  gdouble       scale;
-  gint          border = GTK_CONTAINER (widget)->border_width + 1;
+  PrintPreview  *preview = PRINT_PREVIEW (widget);
+  GtkAllocation  allocation;
+  GdkRGBA        color;
+  gdouble        paper_width;
+  gdouble        paper_height;
+  gdouble        left_margin;
+  gdouble        right_margin;
+  gdouble        top_margin;
+  gdouble        bottom_margin;
+  gdouble        scale;
+  gint           border;
+
+  gtk_widget_get_allocation (widget, &allocation);
+
+  border = gtk_container_get_border_width (GTK_CONTAINER (widget)) + 1;
 
   print_preview_get_page_size (preview, &paper_width, &paper_height);
   print_preview_get_page_margins (preview,
@@ -425,15 +459,11 @@ print_preview_expose_event (GtkWidget      *widget,
 
   scale = print_preview_get_scale (preview);
 
-  cr = gdk_cairo_create (widget->window);
-
-  cairo_translate (cr,
-                   widget->allocation.x + border,
-                   widget->allocation.y + border);
+  cairo_translate (cr, border, border);
 
   if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
     {
-      gint width = widget->allocation.width - 2 * border;
+      gint width = allocation.width - 2 * border;
 
       cairo_translate (cr, width - scale * paper_width, 0);
     }
@@ -443,10 +473,12 @@ print_preview_expose_event (GtkWidget      *widget,
   /* draw page background */
   cairo_rectangle (cr, 0, 0, scale * paper_width, scale * paper_height);
 
-  gdk_cairo_set_source_color (cr, &style->black);
+  color = (GdkRGBA) { 0.0, 0.0, 0.0, 1.0 };
+  gdk_cairo_set_source_rgba (cr, &color);
   cairo_stroke_preserve (cr);
 
-  gdk_cairo_set_source_color (cr, &style->white);
+  color = (GdkRGBA) { 1.0, 1.0, 1.0, 1.0 };
+  gdk_cairo_set_source_rgba (cr, &color);
   cairo_fill (cr);
 
   /* draw page_margins */
@@ -456,7 +488,8 @@ print_preview_expose_event (GtkWidget      *widget,
                    scale * (paper_width - left_margin - right_margin),
                    scale * (paper_height - top_margin - bottom_margin));
 
-  gdk_cairo_set_source_color (cr, &style->mid[widget->state]);
+  color = (GdkRGBA) { 0.0, 0.0, 0.0, 0.3 };
+  gdk_cairo_set_source_rgba (cr, &color);
   cairo_stroke (cr);
 
   cairo_translate (cr,
@@ -470,17 +503,18 @@ print_preview_expose_event (GtkWidget      *widget,
                        scale * preview->image_width,
                        scale * preview->image_height);
 
-      gdk_cairo_set_source_color (cr, &style->black);
+      color = (GdkRGBA) { 0.0, 0.0, 0.0, 1.0 };
+      gdk_cairo_set_source_rgba (cr, &color);
       cairo_stroke (cr);
     }
 
   if (preview->thumbnail == NULL &&
-      gimp_drawable_is_valid (preview->drawable->drawable_id))
+      gimp_item_is_valid (GIMP_ITEM (preview->drawable)))
     {
       preview->thumbnail =
         print_preview_get_thumbnail (preview->drawable,
-                                     MIN (widget->allocation.width,  1024),
-                                     MIN (widget->allocation.height, 1024));
+                                     MIN (allocation.width,  1024),
+                                     MIN (allocation.height, 1024));
     }
 
   if (preview->thumbnail != NULL)
@@ -501,23 +535,21 @@ print_preview_expose_event (GtkWidget      *widget,
       cairo_fill (cr);
     }
 
-  cairo_destroy (cr);
-
   return FALSE;
 }
 
 /**
  * print_preview_new:
  * @page: page setup
- * @drawable_id: the drawable to print
+ * @drawable: the drawable to print
  *
  * Creates a new #PrintPreview widget.
  *
- * Return value: the new #PrintPreview widget.
+ * Returns: the new #PrintPreview widget.
  **/
 GtkWidget *
 print_preview_new (GtkPageSetup *page,
-                   gint32        drawable_id)
+                   GimpDrawable *drawable)
 {
   PrintPreview *preview;
 
@@ -525,7 +557,7 @@ print_preview_new (GtkPageSetup *page,
 
   preview = g_object_new (PRINT_TYPE_PREVIEW, NULL);
 
-  preview->drawable = gimp_drawable_get (drawable_id);
+  preview->drawable = drawable;
 
   print_preview_set_page_setup (preview, page);
 
@@ -552,8 +584,8 @@ print_preview_set_image_dpi (PrintPreview *preview,
   g_return_if_fail (PRINT_IS_PREVIEW (preview));
   g_return_if_fail (xres > 0.0 && yres > 0.0);
 
-  width  = preview->drawable->width  * 72.0 / xres;
-  height = preview->drawable->height * 72.0 / yres;
+  width  = gimp_drawable_get_width  (preview->drawable) * 72.0 / xres;
+  height = gimp_drawable_get_height (preview->drawable) * 72.0 / yres;
 
   if (width != preview->image_width || height != preview->image_height)
     {
@@ -654,13 +686,18 @@ print_preview_is_inside (PrintPreview *preview,
                          gdouble       x,
                          gdouble       y)
 {
-  GtkWidget *widget = GTK_WIDGET (preview);
-  gdouble    left_margin;
-  gdouble    right_margin;
-  gdouble    top_margin;
-  gdouble    bottom_margin;
-  gdouble    scale;
-  gint       border = GTK_CONTAINER (widget)->border_width + 1;
+  GtkWidget     *widget = GTK_WIDGET (preview);
+  GtkAllocation  allocation;
+  gdouble        left_margin;
+  gdouble        right_margin;
+  gdouble        top_margin;
+  gdouble        bottom_margin;
+  gdouble        scale;
+  gint           border;
+
+  gtk_widget_get_allocation (widget, &allocation);
+
+  border = gtk_container_get_border_width (GTK_CONTAINER (widget)) + 1;
 
   x -= border;
 
@@ -670,7 +707,7 @@ print_preview_is_inside (PrintPreview *preview,
     {
       gdouble paper_width;
       gdouble paper_height;
-      gint    width = widget->allocation.width - 2 * border;
+      gint    width = allocation.width - 2 * border;
 
       print_preview_get_page_size (preview, &paper_width, &paper_height);
 
@@ -700,8 +737,8 @@ print_preview_set_inside (PrintPreview *preview,
 
       preview->inside = inside;
 
-      if (GTK_WIDGET_DRAWABLE (widget))
-        gdk_window_set_cursor (widget->window,
+      if (gtk_widget_is_drawable (widget))
+        gdk_window_set_cursor (gtk_widget_get_window (widget),
                                inside ? preview->cursor : NULL);
 
       gtk_widget_queue_draw (widget);
@@ -711,17 +748,22 @@ print_preview_set_inside (PrintPreview *preview,
 static gdouble
 print_preview_get_scale (PrintPreview *preview)
 {
-  GtkWidget *widget = GTK_WIDGET (preview);
-  gdouble    paper_width;
-  gdouble    paper_height;
-  gdouble    scale_x;
-  gdouble    scale_y;
-  gint       border = GTK_CONTAINER (widget)->border_width + 1;
+  GtkWidget     *widget = GTK_WIDGET (preview);
+  GtkAllocation  allocation;
+  gdouble        paper_width;
+  gdouble        paper_height;
+  gdouble        scale_x;
+  gdouble        scale_y;
+  gint           border;
+
+  gtk_widget_get_allocation (widget, &allocation);
+
+  border = gtk_container_get_border_width (GTK_CONTAINER (widget)) + 1;
 
   print_preview_get_page_size (preview, &paper_width, &paper_height);
 
-  scale_x = (gdouble) (widget->allocation.width  - 2 * border) / paper_width;
-  scale_y = (gdouble) (widget->allocation.height - 2 * border) / paper_height;
+  scale_x = (gdouble) (allocation.width  - 2 * border) / paper_width;
+  scale_y = (gdouble) (allocation.height - 2 * border) / paper_height;
 
   return MIN (scale_x, scale_y);
 }
@@ -774,7 +816,7 @@ print_preview_get_thumbnail (GimpDrawable *drawable,
 {
   cairo_surface_t *surface;
   cairo_format_t   format;
-  guchar          *data;
+  GBytes          *data;
   guchar          *dest;
   const guchar    *src;
   gint             src_stride;
@@ -785,7 +827,8 @@ print_preview_get_thumbnail (GimpDrawable *drawable,
   g_return_val_if_fail (width  > 0 && width  <= 1024, NULL);
   g_return_val_if_fail (height > 0 && height <= 1024, NULL);
 
-  data = gimp_drawable_get_thumbnail_data (drawable->drawable_id,
+  data = gimp_drawable_get_thumbnail_data (drawable,
+                                           width, height,
                                            &width, &height, &bpp);
 
   switch (bpp)
@@ -807,7 +850,7 @@ print_preview_get_thumbnail (GimpDrawable *drawable,
 
   surface = cairo_image_surface_create (format, width, height);
 
-  src         = data;
+  src         = g_bytes_get_data (data, NULL);
   src_stride  = width * bpp;
 
   dest        = cairo_image_surface_get_data (surface);
@@ -862,7 +905,9 @@ print_preview_get_thumbnail (GimpDrawable *drawable,
       dest += dest_stride;
     }
 
-  g_free (data);
+  g_bytes_unref (data);
+
+  cairo_surface_mark_dirty (surface);
 
   return surface;
 }
