@@ -166,7 +166,7 @@ print_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_procedure_set_menu_label (procedure, _("_Print..."));
       gimp_procedure_set_icon_name (procedure, GIMP_ICON_DOCUMENT_PRINT);
-      gimp_procedure_add_menu_path (procedure, "<Image>/File/Send");
+      gimp_procedure_add_menu_path (procedure, "<Image>/File");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Print the image"),
@@ -190,7 +190,7 @@ print_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_procedure_set_menu_label (procedure, _("Page Set_up..."));
       gimp_procedure_set_icon_name (procedure, GIMP_ICON_DOCUMENT_PAGE_SETUP);
-      gimp_procedure_add_menu_path (procedure, "<Image>/File/Send");
+      gimp_procedure_add_menu_path (procedure, "<Image>/File");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Adjust page size and orientation "
@@ -518,7 +518,8 @@ print_temp_proc_run (GimpProcedure        *procedure,
 static gchar *
 print_temp_proc_name (GimpImage *image)
 {
-  return g_strdup_printf (PRINT_TEMP_PROC_NAME "-%d", image);
+  return g_strdup_printf (PRINT_TEMP_PROC_NAME "-%d",
+                          gimp_image_get_id (image));
 }
 
 static gchar *
