@@ -28,7 +28,6 @@
 
 #include "screenshot.h"
 #include "screenshot-freedesktop.h"
-#include "screenshot-icon.h"
 #include "screenshot-osx.h"
 #include "screenshot-x11.h"
 #include "screenshot-win32.h"
@@ -138,7 +137,7 @@ screenshot_create_procedure (GimpPlugIn  *plug_in,
                                       screenshot_run, NULL, NULL);
 
       gimp_procedure_set_menu_label (procedure, _("_Screenshot..."));
-      gimp_procedure_add_menu_path (procedure, "<Image>/File/Create/Acquire");
+      gimp_procedure_add_menu_path (procedure, "<Image>/File/Create");
 
       gimp_procedure_set_documentation
         (procedure,
@@ -166,10 +165,8 @@ screenshot_create_procedure (GimpPlugIn  *plug_in,
                                       "1998 - 2008",
                                       "v1.1 (2008/04)");
 
-
       gimp_procedure_set_icon_pixbuf (procedure,
-                                      gdk_pixbuf_new_from_inline (-1, screenshot_icon,
-                                                                  FALSE, NULL));
+                                      gdk_pixbuf_new_from_resource ("/org/gimp/screenshot-icons/screenshot-icon.png", NULL));
 
       GIMP_PROC_ARG_ENUM (procedure, "run-mode",
                           "Run mode",
