@@ -12,18 +12,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __SCRIPT_FU_EVAL_H__
 #define __SCRIPT_FU_EVAL_H__
 
 
-void  script_fu_eval_run (const gchar      *name,
-                          gint              nparams,
-                          const GimpParam  *params,
-                          gint             *nreturn_vals,
-                          GimpParam       **return_vals);
+GimpValueArray * script_fu_eval_run (GimpProcedure        *procedure,
+                                     GimpRunMode           run_mode,
+                                     const gchar          *code,
+                                     const GimpValueArray *args);
 
 
 #endif /*  __SCRIPT_FU_EVAL_H__  */
